#import "ConnectionDataItem.h"
#import "ConvertExtension.h"

@implementation ConnectionDataItem
//------------------------------------
// initWithURL
//------------------------------------
- (id)initWithURL:(NSString*)urlString
		userInfo:(NSDictionary*)userInfo
		notifyTarget:(id)notifyTarget
		params:(NSDictionary*)params
{
    if (self = [super init])
	{
		[self setUrlString:urlString];
		[self setError:nil];
		[self setConnection:nil];
		[self setStatus:CONNECTION_INIT];
		[self setParams:params];
		notifyTarget_ = notifyTarget;

		if([self getDataWithURL:urlString
						userInfo:userInfo
						notifyTarget:notifyTarget
			] == NO){
			return nil;
		}
		[self notifyConnectionDataItemStatus:self];

	}
    return self;
}

//------------------------------------
// getDataWithURL
//------------------------------------
- (BOOL)getDataWithURL:(NSString*)urlString
				userInfo:(NSDictionary*)userInfo
				notifyTarget:(id)notifyTarget
{

	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:notifyTarget selector:@selector(handleDataItemStatus:) name:CONNECTION_NOTIF_NAME_STATUS object:self];
	[nc addObserver:notifyTarget selector:@selector(handleDataItemDidLoad:) name:CONNECTION_NOTIF_NAME_LOAD object:self];

	return [self createConnection:urlString userInfo:userInfo];
}

//------------------------------------
// createConnection
//------------------------------------
- (BOOL)createConnection:(NSString*)urlString userInfo:(NSDictionary*)userInfo
{

	urlString = [self decodeToPercentEscapesString:urlString];
//	NSLog(@"urlString=%@", urlString);

	NSString *referer = @"";
	NSString *userAgent = @"";

	if(userInfo != nil){
		if([userInfo valueForKey:@"referer"]){
			referer = [userInfo valueForKey:@"referer"];
		}
		if([userInfo valueForKey:@"userAgent"]){
			userAgent = [userInfo valueForKey:@"userAgent"];
		}
	}

	NSURL *url = [NSURL URLWithString:urlString];

	NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url
															cachePolicy:NSURLRequestUseProtocolCachePolicy
															timeoutInterval:10
									];
	// set referer
	if(![referer isEqualToString:@""]){
		[request setValue:referer forHTTPHeaderField:@"Referer"];
	}
	if(![userAgent isEqualToString:@""]){
		[request setValue:userAgent forHTTPHeaderField:@"User-Agent"];
	}

	[request setHTTPMethod:@"GET"];

//	[[[NSURLConnection alloc] initWithRequest:request delegate:self] autorelease];
//	[[NSURLConnection alloc] initWithRequest:request delegate:self];
	[self setConnection:[[[NSURLConnection alloc] initWithRequest:request delegate:self] autorelease]];

	if([self connection] != nil){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// didReceiveResponse
//------------------------------------
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
//	NSLog(@"didReceiveResponse");
//	NSLog(@"response=%@", [[response URL] absoluteString]);
//	[self setStatus:CONNECTION_STARTED];
//	[self notifyConnectionDataItemStatus:self];
}
//------------------------------------
// redirectResponse
//------------------------------------
-(NSURLRequest *)connection:(NSURLConnection *)connection
            willSendRequest:(NSURLRequest *)request
           redirectResponse:(NSURLResponse *)redirectResponse
{
//	NSLog(@"redirectResponse");
//	NSLog(@"response=%@", [[redirectResponse URL] absoluteString]);
	return request;
}
//------------------------------------
// didReceiveData
//------------------------------------
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	if(!data_) data_ = [[NSMutableData data] retain];
	[data_ appendData:data];
}
//------------------------------------
// didFailWithError
//------------------------------------
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	[self setStatus:CONNECTION_FAILED];
	[self setError:error];

	[self notifyConnectionDataItemStatus:self];
	[self notifyConnectionDataItemDidLoad:self];
}
//------------------------------------
// connectionDidFinishLoading
//------------------------------------
- (void)connectionDidFinishLoading:(NSURLConnection*)connection
{
	[self setStatus:CONNECTION_COMPLETED];

	[self notifyConnectionDataItemStatus:self];
	[self notifyConnectionDataItemDidLoad:self];
}
//------------------------------------
// notifyConnectionDataItemStatus
//------------------------------------
- (void)notifyConnectionDataItemStatus:(ConnectionDataItem*)item
{
//	NSLog(@"notifyConnectionDataItemStatus");
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:CONNECTION_NOTIF_NAME_STATUS object:item];

}
//------------------------------------
// notifyConnectionDataItemDidLoad
//------------------------------------
- (void)notifyConnectionDataItemDidLoad:(ConnectionDataItem*)item
{
//	NSLog(@"notifyConnectionDataItemDidLoad");
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:CONNECTION_NOTIF_NAME_LOAD object:item];

	// remove observer
	[self removeItemObserver:item];
}
//------------------------------------
// cancelConnection
//------------------------------------
- (void)cancelConnection
{
	if(connection_ != nil){
		[connection_ cancel];
	}

	[self setStatus:CONNECTION_CANCELED];

	[self notifyConnectionDataItemStatus:self];

	// remove observer
	[self removeItemObserver:self];
	
}
//------------------------------------
// removeItemObserver
//------------------------------------
- (void)removeItemObserver:(ConnectionDataItem*)item
{
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	[nc removeObserver:notifyTarget_ name:CONNECTION_NOTIF_NAME_STATUS object:self];
	[nc removeObserver:notifyTarget_ name:CONNECTION_NOTIF_NAME_LOAD object:self];
}
//------------------------------------
// connection
//------------------------------------
- (void)setConnection:(NSURLConnection*)connection
{
	[connection retain];
	[connection_ release];
	connection_ = connection;
}
- (NSURLConnection*)connection
{
	return connection_;
}
//------------------------------------
// error
//------------------------------------
- (void)setError:(NSError*)error
{
	[error retain];
	[error_ release];
	error_ = error;
}
- (NSError*)error
{
	return error_;
}
//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
	[params_ release];
	params_ = params;
}
- (NSDictionary*)params
{
	return params_;
}
//------------------------------------
// setStatus
//------------------------------------
- (void)setStatus:(int)status
{
	status_ = status;
}
- (int)status
{
	return status_;
}
//------------------------------------
// urlString
//------------------------------------
- (void)setUrlString:(NSString*)urlString
{
	[urlString retain];
	[urlString_ release];
	urlString_ = urlString;
}
- (NSString*)urlString
{
	return urlString_;
}
//------------------------------------
// data
//------------------------------------
- (NSData*)data
{
	return data_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
//	NSLog(@"dealloc=%@", urlString_);
	// remove observer
	[self removeItemObserver:self];

	[urlString_ release];
	[data_ release];
	[error_ release];
	[connection_ release];
	[params_ release];
    [super dealloc];
}

@end
