#import "ConvertExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

@implementation NSObject(convertExtension_)
//------------------------------------
// convertToBaseURL
//------------------------------------
- (NSString*)convertToBaseURL
{
	return [self defaultStringValue:@"optNicoVideoBaseURL"];
}
//------------------------------------
// convertToSecureBaseURL
//------------------------------------
- (NSString*)convertToSecureBaseURL
{
	return [self defaultStringValue:@"optNicoVideoSecureBaseURL"];
}
//------------------------------------
// convertToSecureLoginURL
//------------------------------------
- (NSString*)convertToSecureLoginURL
{
	return [NSString stringWithFormat:@"%@/secure/login_form", [self convertToSecureBaseURL]];
}

//------------------------------------
// convertToChannelBaseURL
//------------------------------------
- (NSString*)convertToChannelBaseURL
{
	return [self defaultStringValue:@"optNicoVideoChannelBaseURL"];
}
//------------------------------------
// convertToExternalBaseURL
//------------------------------------
- (NSString*)convertToExternalBaseURL
{
	return [self defaultStringValue:@"optNicoVideoExternalBaseURL"];
}
//------------------------------------
// convertToDictionaryBaseURL
//------------------------------------
- (NSString*)convertToDictionaryBaseURL
{
	return [self defaultStringValue:@"optNicoVideoDictionaryBaseURL"];
}
//------------------------------------
// convertToThumbWatchBaseURL
//------------------------------------
- (NSString*)convertToThumbWatchBaseURL
{
	return [NSString stringWithFormat:@"%@/thumb_watch", [self convertToExternalBaseURL]];
}
//------------------------------------
// convertToNicoPlayerSwfURL
//------------------------------------
- (NSString*)convertToNicoPlayerSwfURL
{
	return [NSString stringWithFormat:@"%@/swf/nicoplayer.swf", [self convertToBaseURL]];
}
//------------------------------------
// convertToExternalNicoPlayerSwfURL
//------------------------------------
- (NSString*)convertToExternalNicoPlayerSwfURL
{
	return [NSString stringWithFormat:@"%@/swf/player/thumbwatch.swf", [self convertToExternalBaseURL]];
}
//------------------------------------
// convertToExternalThumbWatchSwfURL
//------------------------------------
- (NSString*)convertToExternalThumbWatchSwfURL
{
	return [NSString stringWithFormat:@"%@/swf/thumbwatch.swf", [self convertToExternalBaseURL]];
}
//------------------------------------
// convertToLoginURL
//------------------------------------
- (NSString*)convertToLoginURL:(NSString*)account password:(NSString*)password
{
	return [NSString stringWithFormat:@"https://secure.nicovideo.jp/secure/login?site=niconico&mail=%@&password=%@", account, password];
}
//------------------------------------
// convertToSearchKeywordURL
//------------------------------------
- (NSString*)convertToSearchKeywordURL:(NSString*)searchString
{
	return [NSString stringWithFormat:@"%@/search/%@",[self convertToBaseURL], searchString];
}
//------------------------------------
// convertToSearchTagURL
//------------------------------------
- (NSString*)convertToSearchTagURL:(NSString*)searchString
{
	return [NSString stringWithFormat:@"%@/tag/%@", [self convertToBaseURL], searchString];
}
//------------------------------------
// convertToRssURL
//------------------------------------
- (NSString*)convertToRssURL:(NSString*)urlString rssType:(int)rssType
{
	return [NSString stringWithFormat:@"%@&rss=%@", urlString, [self getRssTypeName:rssType]];
}
//------------------------------------
// convertToSearchURL
//------------------------------------
- (NSString*)convertToSearchURL:(NSString*)urlString
{
	return [NSString stringWithFormat:@"%@/%@",[self convertToBaseURL], urlString];
}
//------------------------------------
// convertToCategoryURL
//------------------------------------
- (NSString*)convertToCategoryURL:(NSString*)categoryName adult:(BOOL)adult
{
	// allow adult content
	if(adult == YES){
		categoryName = [NSString stringWithFormat:@"%@?adult=1", categoryName];
	}

	return [NSString stringWithFormat:@"%@/top/%@", [self convertToBaseURL], categoryName];	
}
//------------------------------------
// convertToCategoryRankingURL
//------------------------------------
- (NSString*)convertToCategoryRankingURL:(NSString*)categoryName target:(NSString*)target period:(NSString*)period withRss:(BOOL)withRss rssType:(int)rssType
{
	NSString *url = [NSString stringWithFormat:@"%@/ranking/%@/%@/%@", [self convertToBaseURL], target, period, categoryName];
	if(withRss == YES){
		url = [NSString stringWithFormat:@"%@?rss=%@", url, [self getRssTypeName:rssType]];
	}
	return url;
}
//------------------------------------
// convertToContentsURL
//------------------------------------
- (NSString*)convertToContentsURL:(NSString*)contentsName withRss:(BOOL)withRss rssType:(int)rssType
{
	NSString *url = [NSString stringWithFormat:@"%@/%@", [self convertToBaseURL], contentsName];
	if(withRss == YES){
		url = [NSString stringWithFormat:@"%@?rss=%@", url, [self getRssTypeName:rssType]];
	}
	return url;
}
//------------------------------------
// convertToMylistURL
//------------------------------------
- (NSString*)convertToMylistURL:(NSString*)mylistId withRss:(BOOL)withRss rssType:(int)rssType
{
	NSString *url = [NSString stringWithFormat:@"%@/mylist/%@", [self convertToBaseURL], mylistId];
	if(withRss == YES){
		url = [NSString stringWithFormat:@"%@?rss=%@", url, [self getRssTypeName:rssType]];
	}
	return url;
}
//------------------------------------
// convertToPostUserURL
//------------------------------------
- (NSString*)convertToPostUserURL:(NSString*)postUserId withRss:(BOOL)withRss rssType:(int)rssType
{
	NSString *url = [NSString stringWithFormat:@"%@/user/%@/video", [self convertToBaseURL], postUserId];
	if(withRss == YES){
		url = [NSString stringWithFormat:@"%@?rss=%@", url, [self getRssTypeName:rssType]];
	}
	return url;
}

//------------------------------------
// convertToChannelURL
//------------------------------------
- (NSString*)convertToChannelURL:(NSString*)channelId withRss:(BOOL)withRss rssType:(int)rssType
{
	NSString *url = [NSString stringWithFormat:@"%@/video/%@", [self convertToChannelBaseURL], channelId];
	if(withRss == YES){
		url = [NSString stringWithFormat:@"%@?rss=%@", url, [self getRssTypeName:rssType]];
	}
	return url;
}

//------------------------------------
// convertToRelatedVideoURL
//------------------------------------
- (NSString*)convertToRelatedVideoURL:(NSString*)videoId
{
	return [NSString stringWithFormat:@"%@/api/getrelation?video=%@", [self convertToBaseURL], videoId];
}
//------------------------------------
// convertToRelatedTagURL
//------------------------------------
- (NSString*)convertToRelatedTagURL:(NSString*)searchString
{
	return [NSString stringWithFormat:@"%@/related_tag/%@", [self convertToBaseURL], searchString];
}
//------------------------------------
// convertToDictionaryURL
//------------------------------------
- (NSString*)convertToDictionaryURL:(NSString*)searchString
{
	return [NSString stringWithFormat:@"%@/a/%@", [self convertToDictionaryBaseURL], searchString];
}
//------------------------------------
// convertToWatchURL
//------------------------------------
- (NSString*)convertToWatchURL:(NSString*)videoId
{
	return [NSString stringWithFormat:@"%@/watch/%@", [self convertToBaseURL], videoId];
}
//------------------------------------
// convertToWatchQPlayerURL
//------------------------------------
- (NSString*)convertToWatchQPlayerURL:(NSString*)videoId
{
	return [NSString stringWithFormat:@"%@/watch/%@?ver=q9", [self convertToBaseURL], videoId];
}

//------------------------------------
// convertToPlayerURL
//------------------------------------
- (NSString*)convertToPlayerURL:(NSString*)playerURL videoId:(NSString*)videoId width:(int)width height:(int)height
{
	return [NSString stringWithFormat:@"%@?video_id=%@&width=%d&height=%d", playerURL, videoId, width, height];
//	return [NSString stringWithFormat:@"%@?%@?%d?%d", playerURL, videoId, width, height];
}
//------------------------------------
// convertToExternalThumbWatchURL
//------------------------------------
- (NSString*)convertToExternalThumbWatchURL:(NSString*)videoId width:(int)width height:(int)height
{
	return [NSString stringWithFormat:@"%@/%@?w=%d&h=%d", [self convertToThumbWatchBaseURL], videoId, width, height];
}
//------------------------------------
// convertToPlayerHTML
//------------------------------------
- (NSString*)convertToPlayerHTML:(NSString*)string
{
	return [NSString stringWithFormat:@"<html><body bgcolor=black style=\"margin:0; text-align:center;\"><script type=\"text/javascript\">\n%@\n</script></body></html>", string];
}
//------------------------------------
// convertToThumbInfoURL
//------------------------------------
- (NSString*)convertToThumbInfoURL:(NSString*)videoId
{

//	return [NSString stringWithFormat:@"%@/api/getthumbinfo/%@", [self convertToBaseURL], videoId];
	// external
	return [NSString stringWithFormat:@"%@/api/getthumbinfo/%@", [self convertToExternalBaseURL], videoId];

}
//------------------------------------
// convertToGetFileURL
//------------------------------------
- (NSString*)convertToGetFileURL:(NSString*)videoId
{
//	return [NSString stringWithFormat:@"%@/api/getflv?v=%@", [self convertToBaseURL], videoId];
	return [NSString stringWithFormat:@"%@/api/getflv/%@", [self convertToBaseURL], videoId];
}

//------------------------------------
// appendStringToURL
//------------------------------------
- (NSString*)appendStringToURL:(NSString*)url string:(NSString*)string isAdded:(BOOL*)isAdded
{

	if(![string isEqualToString:@""]){
		if(*isAdded == NO){
			url = [NSString stringWithFormat:@"%@?%@", url, string];
		}else{
			url = [NSString stringWithFormat:@"%@&%@", url, string];
		}
		*isAdded = YES;
	}

	return url;

}
//------------------------------------
// appendEcoModeStringToURL
//------------------------------------
- (NSString*)appendEcoModeStringToURL:(NSString*)url
{
	NSRange rng = [url rangeOfString:@"?"];
	if(rng.length > 0){
		return [url stringByAppendingString:@"&eco=1"];
	}else{
		return [url stringByAppendingString:@"?eco=1"];
	}

}

//------------------------------------
// convertToResultString
//------------------------------------
- (NSString*)convertToResultString:(int)startIndex
						lastIndex:(int)lastIndex
						totalResults:(int)totalResults
						totalString:(NSString*)totalString
						showTotal:(int)showTotal
{

	NSString *string = @"";

	if(showTotal == 0){
		 string = [NSString stringWithFormat:@"%@ - %@",
									[self convertToComma:startIndex],
									[self convertToComma:lastIndex]
				];
	}
	else if(showTotal == 1){
		 string = [NSString stringWithFormat:@"%@ - %@ of %@",
									[self convertToComma:startIndex],
									[self convertToComma:lastIndex],
									[self convertToComma:totalResults]
				];
	}
	else if(showTotal == 2){
		 string = [NSString stringWithFormat:@"%@ - %@ of %@",
									[self convertToComma:startIndex],
									[self convertToComma:lastIndex],
									totalString
				];
	}
	return string;

}
//------------------------------------
// convertToMylistHeadString
//------------------------------------
- (NSString*)convertToMylistHeadString:(NSString*)title author:(NSString*)author
{
	
	NSString *string = @"";
	if(![title isEqualToString:@""] && ![author isEqualToString:@""]){
		string = [NSString stringWithFormat:@"%@ - %@", title, author];
	}
	else{
		if(![title isEqualToString:@""]){
			string = title;
		}
		if(![author isEqualToString:@""]){
			string = author;
		}
	}

	return string;
}
//------------------------------------
// decodeToPercentEscapesString
//------------------------------------
- (NSString*)decodeToPercentEscapesString:(NSString*)string
{
	// %XX -> char
	NSString *encodedStr = [self encodeFromPercentEscapesString:string];

	// char -> %XX
	NSString *decodedStr = (NSString *) CFURLCreateStringByAddingPercentEscapes(
				kCFAllocatorDefault,
				(CFStringRef) encodedStr,
				nil,
				nil,
				kCFStringEncodingUTF8);

	return decodedStr;
}
//------------------------------------
// encodeFromPercentEscapesString
//------------------------------------
- (NSString*)encodeFromPercentEscapesString:(NSString*)string
{
	// %XX -> char
	NSString *encodedStr = (NSString *) CFURLCreateStringByReplacingPercentEscapesUsingEncoding(
				kCFAllocatorDefault,
				(CFStringRef) string,
				CFSTR(""),
				kCFStringEncodingUTF8);

	return encodedStr;
}
//------------------------------------
// convertToURIDecodedString
//------------------------------------
- (NSString*)convertToURIDecodedString:(NSString*)string
{
	NSArray *escapeChars = [self escapeChars];
	NSArray *percentChars = [self percentChars];

	int len = [escapeChars count];

	NSMutableString *temp = [string mutableCopy];

	int i;
	for(i = 0; i < len; i++){
		[temp replaceOccurrencesOfString: [escapeChars objectAtIndex:i]
									withString:[percentChars objectAtIndex:i]
									options:NSLiteralSearch
									range:NSMakeRange(0, [temp length])];
	}

	NSString *out = [NSString stringWithString: temp];

	return out;
}
//------------------------------------
// convertToURIEncodedString
//------------------------------------
- (NSString*)convertToURIEncodedString:(NSString*)string
{

	NSArray *percentChars = [self percentChars];
	NSArray *escapeChars = [self escapeChars];

	int len = [percentChars count];

	NSMutableString *temp = [string mutableCopy];

	int i;
	for(i = 0; i < len; i++){
		[temp replaceOccurrencesOfString: [percentChars objectAtIndex:i]
									withString:[escapeChars objectAtIndex:i]
									options:NSLiteralSearch
									range:NSMakeRange(0, [temp length])];
	}

	NSString *out = [NSString stringWithString: temp];

	return out;
}
//------------------------------------
// percentChars
//------------------------------------
- (NSArray*)percentChars
{
	return [NSArray arrayWithObjects:@"%3B" , @"%2F" , @"%3F" , @"%3A" , 
									@"%40" , @"%26" , @"%3D" , @"%2B" , 
									@"%24" , @"%2C" , @"%5B" , @"%5D", 
									@"%23", @"%21", @"%27", @"%28", 
									@"%29", @"%2A", nil];
}
//------------------------------------
// escapeChars
//------------------------------------
- (NSArray*)escapeChars
{
	return [NSArray arrayWithObjects:@";" , @"/" , @"?" , @":" ,
									@"@" , @"&" , @"=" , @"+" ,
									@"$" , @"," , @"[" , @"]",
									@"#", @"!", @"'", @"(", 
									@")", @"*", nil];
}

/*
//------------------------------------
// convertToURIEncodedString
//------------------------------------
- (NSString*)convertToURIEncodedString:(NSString*)string
{
    NSMutableString *afterString = [NSMutableString string];
    const char *str_chars = [string cStringUsingEncoding: NSUTF8StringEncoding];
    int i = [string length];
    
    while (i--) {
        if (*str_chars == ' ') {
            [afterString appendString:@" "];
        }
        else if (*str_chars == '%' && isxdigit((int)*(str_chars + 1)) && isxdigit((int)*(str_chars+2)))
        {
            unsigned char buff = 0x00;
            buff = (char)HexFromChar((str_chars + 1));
            [afterString appendFormat:@"%c",buff,nil];
            i -= 2;
            str_chars += 2;
        }
        else
        {
            [afterString appendFormat:@"%c",*str_chars,nil];
        }
        str_chars ++;
    }
    
    return afterString;
}
*/
//------------------------------------
// convertToShiftJISString
//------------------------------------
- (NSString*)convertToShiftJISString:(char*)string
{
	return [NSString stringWithCString:string encoding:NSShiftJISStringEncoding];
}

//------------------------------------
// convertDownloadStatusToString
//------------------------------------
- (NSString*)convertDownloadStatusToString:(int)status
{
	NSString *string = @"";

	if(status == DOWNLOAD_INIT){
		string = @"Initializing..";
	}
	else if(status == DOWNLOAD_STARTED ||
			status == DOWNLOAD_COMPLETED){
		// none
	}
	else if(status == DOWNLOAD_CANCELED){
		string = @"Canceled";
	}
	else if(status == DOWNLOAD_FAILED){
		string = @"Failed";
	}

	return string;

}
//------------------------------------
// convertDownloadStatusToImage
//------------------------------------
- (NSImage*)convertDownloadStatusToImage:(int)status
{
	NSImage *image;

	// init, started
	if( status == DOWNLOAD_INIT ||
		status == DOWNLOAD_STARTED){
		image = [NSImage imageNamed:@"icon_download_cancel"];
	}
	// completed
	else if(status == DOWNLOAD_COMPLETED){
		image = [NSImage imageNamed:@"icon_download_completed"];
	}
	// canceled
	else if(status == DOWNLOAD_CANCELED){
		image = [NSImage imageNamed:@"icon_download_restart"];
	}
	// failed, other
	else{
		image = [NSImage imageNamed:@"icon_download_failed"];
	}
	return image;
}
//------------------------------------
// convertDownloadSearchToImage
//------------------------------------
- (NSImage*)convertDownloadSearchToImage:(BOOL)isExist
{
	NSImage *image = nil;
	if(isExist == YES){
		image = [NSImage imageNamed:@"icon_download_search"];
	}
	return image;
}
//------------------------------------
// convert file size string
//------------------------------------
- (NSString*)convertFileSizeToString:(double)fileSize
{
	NSString *fileSizeName = @"";

	// fileSize < 1MB
/*
	if(fileSize < 1024){
		fileSize = fileSize / 1024;
		fileSizeName = @"KB";
	}
*/
	if(fileSize < (1024 * 1024)){
		fileSize = fileSize / 1024;
		fileSizeName = @"KB";
	}
	// fileSize < 1GB
	else if(fileSize < (1024 * 1024 * 1024)){
		fileSize = fileSize / (1024 * 1024);
		fileSizeName = @"MB";
	}
	else{
		fileSize = fileSize / (1024 * 1024 * 1024);
		fileSizeName = @"GB";
	}

	return [NSString stringWithFormat: @"%.1f %@",fileSize,fileSizeName];

}
//------------------------------------
// convertTimeToString
//------------------------------------
- (NSString*)convertTimeToString:(int)timeSec
{

/*
	output format
	 1:11:11
	   11:11
	    1:11
	    0:11
	    0:01
*/

	NSString *timeStr;
	NSString *hourStr;
	NSString *minStr;
	NSString *secStr;

	int hour = 0;
	int min = 0;
	int sec = 0;

	if(timeSec >= 3600){
		hour = timeSec / 3600;
		timeSec = timeSec % 3600;
	}

	if(timeSec >= 60){
		min = timeSec / 60;
		timeSec = timeSec % 60;
	}

	sec = timeSec;

	// hour format
	hourStr = [NSString stringWithFormat:@"%d",hour];

	// min format
	if(hour > 0){
		if(min < 10){
			minStr = [NSString stringWithFormat:@"0%d",min];
		}else{
			minStr = [NSString stringWithFormat:@"%d",min];
		}
	}else{
		minStr = [NSString stringWithFormat:@"%d",min];
	}

	// sec format
	if(sec < 10){
		secStr = [NSString stringWithFormat:@"0%d",sec];
	}else{
		secStr = [NSString stringWithFormat:@"%d",sec];
	}

	// time format
	if(hour > 0){
		timeStr = [NSString stringWithFormat:@"%@:%@:%@",hourStr, minStr, secStr];
	}else{
		timeStr = [NSString stringWithFormat:@"%@:%@", minStr, secStr];
	}

	return timeStr;

}
//------------------------------------
// convertToComma
//------------------------------------
- (NSString*)convertToComma:(int)value
{
	NSNumber *number=[NSNumber numberWithInt:value];
	NSNumberFormatter *formatter=[[[NSNumberFormatter alloc] init] autorelease];

//	[formatter setThousandSeparator:@","];
//	[formatter setHasThousandSeparators:YES];
	[formatter setFormat:@"#,##0;0;-#,##0"];

	return [formatter stringForObjectValue:number];
}
//------------------------------------
// convertToZeroFormat
//------------------------------------
- (NSString*)convertToZeroFormat:(int)value
{
	NSNumber *number=[NSNumber numberWithInt:value];
	NSNumberFormatter *formatter=[[[NSNumberFormatter alloc] init] autorelease];

	[formatter setFormat:@"00000000;00000000;-00000000"];

	return [formatter stringForObjectValue:number];
}


//------------------------------------
// convertStringToIntValue
//------------------------------------
- (int)convertStringToIntValue:(NSString*)string
{
	int value = 0;
	if(string && ![string isEqualToString:@""]){
		NSMutableString *muString = [NSMutableString stringWithString:string];
		[muString replaceOccurrencesOfString:@" " withString:@"" options:0 range:NSMakeRange(0,[muString length])];
		[muString replaceOccurrencesOfString:@"," withString:@"" options:0 range:NSMakeRange(0,[muString length])];
		if([self checkIsNumberString:muString] == YES){
			value = [muString intValue];
		}
	}

	return value;
}
//------------------------------------
// replaceCharacter
//------------------------------------
- (NSString*)replaceCharacter:(NSString*)string str1:(NSString*)str1 str2:(NSString*)str2
{
	NSMutableString *muString = [NSMutableString stringWithString:string];

	[muString replaceOccurrencesOfString:str1 withString:str2 options:0 range:NSMakeRange(0,[muString length])];

	return muString;
}
//------------------------------------
// replaceSizeCharacter
//------------------------------------
- (NSString*)replaceSizeCharacter:(NSString*)string size1:(int)size1 size2:(int)size2
{
	return [self replaceCharacter:string str1:[NSString stringWithFormat:@"'%d'", size1] str2:[NSString stringWithFormat:@"'%d'", size2]];
}
//------------------------------------
// replaceWebCharacters
//------------------------------------
- (NSString*)replaceWebCharacters:(NSString*)string
{

	string = [self replaceCharacter:string str1:@"&amp;" str2:@"&"];
	string = [self replaceCharacter:string str1:@"amp;" str2:@""];
	string = [self replaceCharacter:string str1:@"&lt;" str2:@"<"];
	string = [self replaceCharacter:string str1:@"lt;" str2:@""];
	string = [self replaceCharacter:string str1:@"&gt;" str2:@">"];
	string = [self replaceCharacter:string str1:@"gt;" str2:@""];
	string = [self replaceCharacter:string str1:@"&quot;" str2:@"\""];
	string = [self replaceCharacter:string str1:@"quot;" str2:@""];
	string = [self replaceCharacter:string str1:@"&apos;" str2:@"'"];
	string = [self replaceCharacter:string str1:@"apos;" str2:@""];
	string = [self replaceCharacter:string str1:@"&#039;" str2:@"'"];

	return string;
}
//------------------------------------
// replaceDateCharacter
//------------------------------------
- (NSString*)replaceDateCharacter:(NSString*)string
{

	string = [self replaceCharacter:string str1:[self convertToShiftJISString:"yVz"] str2:@""];
	string = [self replaceCharacter:string str1:[self convertToShiftJISString:"N"] str2:@"/"];
	string = [self replaceCharacter:string str1:[self convertToShiftJISString:""] str2:@"/"];
	string = [self replaceCharacter:string str1:[self convertToShiftJISString:""] str2:@""];
	string = [self replaceCharacter:string str1:[self convertToShiftJISString:"F"] str2:@":"];

	// triming
	string = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];

	return string;
}
//------------------------------------
// replaceBreakCharacters
//------------------------------------
- (NSString*)replaceBreakCharacters:(NSString*)string
{

	string = [self replaceCharacter:string str1:@"\n" str2:@" "];
	string = [self replaceCharacter:string str1:@"\r" str2:@" "];

	return string;
}

//------------------------------------
// convertFormatedDateString
//------------------------------------
- (NSString*)convertFormatedDateString:(NSString*)string
{
	NSString *dateString = @"";

	// 2009-07-01T16:07:10+09:00
	if(string && [string length] > 0){
		NSString *year = [string substringWithRange:NSMakeRange(0,4)];
		NSString *month = [string substringWithRange:NSMakeRange(5,2)];
		NSString *date = [string substringWithRange:NSMakeRange(8,2)];
		NSString *time = [string substringWithRange:NSMakeRange(11,5)];
		dateString = [NSString stringWithFormat:@"%@/%@/%@ %@", year, month, date, time];
	}
	return dateString;
}
//------------------------------------
// convertCalenderDateString
//------------------------------------
- (NSString*)convertCalenderDateString:(NSString*)string
{
	NSString *dateString = @"";

	// Sat, 24 Oct 2009 14:20:33 +0900
/*
	if(string && [string length] > 0){
		NSCalendarDate *calDate = [NSCalendarDate dateWithString:string calendarFormat:@"%a, %d %b %Y %H:%M:%S %Z"];
		if(calDate != nil){
			dateString = [calDate descriptionWithCalendarFormat:@"%Y/%m/%d %H:%M"];
		}
	}
*/
//	NSLog(@"string=%@", string); 

	// input date format
	NSDateFormatter *inputDateFormatter = [[[NSDateFormatter alloc] init] autorelease]; 
	[inputDateFormatter setDateFormat:@"EEE, d MMM yyyy HH:mm:ss zzzz"];
	[inputDateFormatter setLocale:[[[NSLocale alloc] initWithLocaleIdentifier:@"ja_JP"] autorelease]];

	NSDate *date = [inputDateFormatter dateFromString:string];
//	NSLog(@"date=%@", [date description]);

	// error
	if(date == nil){
		return dateString;
	}

	// output date format
	NSDateFormatter *outputDateFormatter = [[[NSDateFormatter alloc] 
									initWithDateFormat:@"%Y/%m/%d %H:%M" 
									allowNaturalLanguage:NO
								] autorelease];
	[outputDateFormatter setLocale:[[[NSLocale alloc] initWithLocaleIdentifier:@"ja_JP"] autorelease]];

	dateString = [outputDateFormatter stringFromDate:date];

//	NSLog(@"dateString=%@", dateString); 

	return dateString;
}

//------------------------------------
// tranceteSpecialCharacter
//------------------------------------
- (NSString*)tranceteSpecialCharacter:(NSString*)string
{

	string = [self replaceCharacter:string str1:@"\r" str2:@""];
	string = [self replaceCharacter:string str1:@"\n" str2:@""];
	string = [self replaceCharacter:string str1:@"\t" str2:@""];

	return string;
}

//------------------------------------
// removeTagCharacters
//------------------------------------
- (NSString*)removeTagCharacters:(NSString*)string
{
	string = [self replaceCharacter:string str1:@"<nobr>" str2:@""];
	string = [self replaceCharacter:string str1:@"</nobr>" str2:@""];
	string = [self replaceCharacter:string str1:@"<span>" str2:@""];
	string = [self replaceCharacter:string str1:@"</span>" str2:@""];
	return string;
}
//------------------------------------
// createLineBreakArray
//------------------------------------
- (NSArray*)createLineBreakArray:(NSString*)string
{
	string = [self replaceCharacter:string str1:@"><" str2:@">\n<"];
	return [string componentsSeparatedByString:@"\n"];
}
//------------------------------------
// getRssTypeName
//------------------------------------
- (NSString*)getRssTypeName:(int)rssType
{
	NSString *rssTypeName = @"";
	switch (rssType){
		case PARSE_RSS_TYPE_NONE:
			rssTypeName = PARSE_RSS_TYPE_NAME_NONE;
			break;
		case PARSE_RSS_TYPE_ATOM:
			rssTypeName = PARSE_RSS_TYPE_NAME_ATOM;
			break;
		case PARSE_RSS_TYPE_2_0:
			rssTypeName = PARSE_RSS_TYPE_NAME_2_0;
			break;
	}
	return rssTypeName;
}
//------------------------------------
// getVideoIdFromURL
//------------------------------------
- (NSString*)getVideoIdFromURL:(NSString*)string setDefault:(BOOL)setDefault
{

	return [self getTargetIdFromURL:string targetString:@"watch/" setDefault:setDefault];

}
//------------------------------------
// getMylistIdFromURL
//------------------------------------
- (NSString*)getMylistIdFromURL:(NSString*)string setDefault:(BOOL)setDefault
{

	return [self getTargetIdFromURL:string targetString:@"mylist/" setDefault:setDefault];

}
//------------------------------------
// getChannelIdFromURL
//------------------------------------
- (NSString*)getChannelIdFromURL:(NSString*)string setDefault:(BOOL)setDefault
{

	return [self getTargetIdFromURL:string targetString:@"channel/" setDefault:setDefault];

}
//------------------------------------
// getTargetIdFromURL
//------------------------------------
- (NSString*)getTargetIdFromURL:(NSString*)string targetString:(NSString*)targetString setDefault:(BOOL)setDefault
{

	NSString *targetId = [self getEnclosedString:string
								beginString:targetString
								endString:@"&?\""
								withEndSet:YES
						];
	// null
	if(setDefault == YES && [targetId isEqualToString:@""]){
		targetId = string;
	}
	return targetId;

}

//------------------------------------
// getLastSeparatedString
//------------------------------------
- (NSString*)getLastSeparatedString:(NSString*)str sep:(NSString*)sep
{
	NSString *retStr = str;

	NSArray *cols = [str componentsSeparatedByString:sep];
	if([cols count] > 0){
		retStr = [cols objectAtIndex:[cols count] - 1];
	}

	return retStr;
}
//------------------------------------
// checkIsNumberString
//------------------------------------
- (BOOL)checkIsNumberString:(NSString*)string
{
	BOOL ret = YES;
	NSCharacterSet *invertedSet = [[NSCharacterSet characterSetWithCharactersInString:@"0123456789"] invertedSet];
//	NSRange rng = [string rangeOfCharacterFromSet:invertedSet options:nil range:NSMakeRange(0,[string length])];
	NSRange rng = [string rangeOfCharacterFromSet:invertedSet];
	if(rng.length > 0){
		ret = NO;
	}
	return ret;
}
//------------------------------------
// checkIsAlphabetString
//------------------------------------
- (BOOL)checkIsAlphabetString:(NSString*)string
{
	BOOL ret = YES;
	NSCharacterSet *invertedSet = [[NSCharacterSet characterSetWithCharactersInString:@"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"] invertedSet];
	NSRange rng = [string rangeOfCharacterFromSet:invertedSet];
	if(rng.length > 0){
		ret = NO;
	}
	return ret;
}
//------------------------------------
// checkIsAlphabetOrNumberString
//------------------------------------
- (BOOL)checkIsAlphabetOrNumberString:(NSString*)string
{
	BOOL ret = YES;
	NSCharacterSet *invertedSet = [[NSCharacterSet characterSetWithCharactersInString:@"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"] invertedSet];
	NSRange rng = [string rangeOfCharacterFromSet:invertedSet];
	if(rng.length > 0){
		ret = NO;
	}
	return ret;
}
//------------------------------------
// checkIsURLString
//------------------------------------
- (BOOL)checkIsURLString:(NSString*)string
{
	//  http: or ttp:
	if([string length] >= 5 &&
		([[string substringToIndex:5] isEqualToString:@"http:"] ||
		[[string substringToIndex:4] isEqualToString:@"ttp:"])
	){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// checkIsChannelString
//------------------------------------
- (BOOL)checkIsChannelString:(NSString*)string
{
	// chXXX
	if( [string length] >= 3 &&
		[[string substringToIndex:2] isEqualToString:@"ch"] &&
		[self checkIsNumberString:[string substringFromIndex:2]] == YES
	){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// checkIsMylistNoString
//------------------------------------
- (BOOL)checkIsMylistNoString:(NSString*)string
{

	NSString *value = @"";
	BOOL ret = NO;

	string = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];

	if([string length] > 5){
		if([[string substringToIndex:5] isEqualToString:@"mylist"]){
			value = [[string substringFromIndex:5] stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"/: "]];
			if([self checkIsNumberString:value] == YES){
				ret = YES;
			}
		}
	}

	return YES;

}
//------------------------------------
// checkIsPostUserIdString
//------------------------------------
- (BOOL)checkIsPostUserIdString:(NSString*)string;
{

	NSString *value = @"";
	BOOL ret = NO;

	string = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];

	if([string length] > 4){
		if([[string substringToIndex:4] isEqualToString:@"user"]){
			value = [[string substringFromIndex:4] stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"/: "]];
			if([self checkIsNumberString:value] == YES){
				ret = YES;
			}
		}
	}

	return YES;

}
//------------------------------------
// checkIsExistString
//------------------------------------
- (BOOL)checkIsExistString:(NSString*)string searchString:(NSString*)searchString
{

	NSRange rng = [string rangeOfString:searchString];
	if(rng.length > 0){
		return YES;
	}else{
		return NO;
	}

}

//------------------------------------
// calcPageMaxIndex
//------------------------------------
- (int)calcPageMaxIndex:(int)totalResults maxResults:(int)maxResults
{
	int pageMaxIndex = 0;
	// calc from totalResults
	if(maxResults > 0){
		pageMaxIndex = totalResults / maxResults;
		if(totalResults % maxResults > 0){
			pageMaxIndex++;
		}
	}
	return pageMaxIndex;
}
@end