#import "DialogExtension.h"
#import "ConvertExtension.h"

@implementation NSObject(dialogExtension_)

//------------------------------------
// displayMessage
//------------------------------------
- (int)displayMessage:(NSString *)messageStyle
					messageText:(NSString *)messageText
					infoText:(NSString *)infoText
					btnList:(NSString *)btnList
{
	NSAlert *alert;
    int i,result;
	NSArray *btnArray;

    alert = [[[NSAlert alloc] init] autorelease];

	[alert setAlertStyle:NSWarningAlertStyle];
	[alert setMessageText:messageText];
	if(![infoText isEqualToString:@""]){
		[alert setInformativeText:infoText];
	}

	if([messageStyle isEqualToString:@"alert"]){
		[alert setIcon:[NSImage imageNamed:@"icon_alert"]];
	}
	else if([messageStyle isEqualToString:@"error"]){
		[alert setIcon:[NSImage imageNamed:@"icon_stop"]];
	}
	else if([messageStyle isEqualToString:@"info"]){
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}
	else{
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}

	btnArray = [btnList componentsSeparatedByString:@","];
	for(i = 0; i < [btnArray count]; i++){
		[alert addButtonWithTitle:[btnArray objectAtIndex:i]];
	}

	result = [alert runModal];
	return result;

}
//------------------------------------
// displayMessageWithIcon
//------------------------------------
- (int)displayMessageWithIcon:(NSString *)iconName
					messageText:(NSString *)messageText
					infoText:(NSString *)infoText
					btnList:(NSString *)btnList
{
	NSAlert *alert;
    int i,result;
	NSArray *btnArray;

    alert = [[[NSAlert alloc] init] autorelease];

	[alert setAlertStyle:NSWarningAlertStyle];
	[alert setMessageText:messageText];
	if(![infoText isEqualToString:@""]){
		[alert setInformativeText:infoText];
	}

	if(![iconName isEqualToString:@""]){
		[alert setIcon:[NSImage imageNamed:iconName]];
	}else{
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}

	btnArray = [btnList componentsSeparatedByString:@","];
	for(i = 0; i < [btnArray count]; i++){
		[alert addButtonWithTitle:[btnArray objectAtIndex:i]];
	}

	result = [alert runModal];
	return result;

}
//------------------------------------
// displayMessageAlertOpenVideo 
//------------------------------------
- (int)displayMessageAlertOpenVideo
{
	return [self displayMessage:@"alert"
						messageText:@"Can not open video."
						infoText:@"Please open URL with browser."
						btnList:@"Cancel,Open URL"
			];
}
//------------------------------------
// displayMessageAlertOpenVideoWithError 
//------------------------------------
- (int)displayMessageAlertOpenVideoWithError:(NSString*)errorMessage
{
	return [self displayMessage:@"alert"
						messageText:[NSString stringWithFormat:@"Can not open video.\n%@", errorMessage]
						infoText:@"Please open URL with browser"
						btnList:@"Cancel,Open URL"
			];
}
//------------------------------------
// displayMessageAlertOpenURLWithBrowser 
//------------------------------------
- (int)displayMessageAlertOpenURLWithBrowser:(NSString*)url
{
	return [self displayMessage:@"alert"
						messageText:[NSString stringWithFormat:@"Can not open url with browser url=%@.", url]
						infoText:@""
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertGetData 
//------------------------------------
- (int)displayMessageAlertGetData
{
	return [self displayMessage:@"alert"
						messageText:@"Can not get data."
						infoText:@"Please check error log"
						btnList:@"Cancel,Log"
			];
}

//------------------------------------
// displayMessageAlertNeedsLogin 
//------------------------------------
- (int)displayMessageAlertNeedsLogin
{
	return [self displayMessageWithIcon:@"icon_alert_nico"
						messageText:[self convertToShiftJISString:"OCKv܂B"]
						infoText:[self convertToShiftJISString:"OCy[WŃOCĉB"]
						btnList:@"Login,Cancel"
			];
}
//------------------------------------
// displayMessageAlertAccessDenied 
//------------------------------------
- (int)displayMessageAlertAccessDenied
{
	return [self displayMessageWithIcon:@"icon_alert_nico"
						messageText:[self convertToShiftJISString:"ZԂł̘AANZX͂B"]
						infoText:[self convertToShiftJISString:"Gɘâ߁AFl̂͂肢܂B"]
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertAccessBusy 
//------------------------------------
- (int)displayMessageAlertAccessBusy
{
	return [self displayMessageWithIcon:@"icon_alert_nico"
						messageText:[self convertToShiftJISString:"GłB"]
						infoText:[self convertToShiftJISString:"Ԃo߂ĂANZXĉB"]
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertAccessFailed 
//------------------------------------
- (int)displayMessageAlertAccessFailed
{
	return [self displayMessageWithIcon:@"icon_alert_nico"
						messageText:[self convertToShiftJISString:"ʂ̎擾Ɏs܂B"]
						infoText:[self convertToShiftJISString:"SafariŊmFĂ݂ĉB"]
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertNotLogin 
//------------------------------------
- (int)displayMessageAlertNotLogin
{
	return [self displayMessageWithIcon:@"icon_alert_nico"
						messageText:[self convertToShiftJISString:"OCĂ܂"]
						infoText:[self convertToShiftJISString:"SafariŃOCĂ邩mFĂ݂ĉB"]
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertNotFound 
//------------------------------------
- (int)displayMessageAlertNotFound
{
	return [self displayMessageWithIcon:@"icon_alert_nico"
						messageText:[self convertToShiftJISString:"y[W܂B"]
						infoText:[self convertToShiftJISString:"T̃y[ẂAłɍ폜ꂽ\\ĂȂ\\܂B"]
						btnList:@"Cancel,Open URL"
			];
}

//------------------------------------
// selectFilePathWithOpenPanel
//------------------------------------
- (NSArray*)selectFilePathWithOpenPanel:(BOOL)isFile
						isDir:(BOOL)isDir
						isMultiSel:(BOOL)isMultiSel
						isPackage:(BOOL)isPackage
						isAlias:(BOOL)isAlias
						canCreateDir:(BOOL)canCreateDir
						defaultPath:(NSString*)defaultPath
{
	NSOpenPanel* openPanel;
	int	result;
	NSArray* filePaths = [[[NSArray alloc] init] autorelease];
//	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	openPanel = [NSOpenPanel openPanel];
	[openPanel setCanChooseFiles:isFile];
	[openPanel setCanChooseDirectories:isDir];
	[openPanel setAllowsMultipleSelection:isMultiSel];
	[openPanel setTreatsFilePackagesAsDirectories:isPackage];
	[openPanel setResolvesAliases:isAlias];
	[openPanel setCanCreateDirectories:canCreateDir];

	result = [openPanel runModalForDirectory:defaultPath file:nil types:nil];

	if (result == NSFileHandlingPanelOKButton) {
		filePaths = [openPanel filenames];
	}

	return filePaths;

}
//------------------------------------
// selectFilePathWithSavePanel
//------------------------------------
- (NSString*)selectFilePathWithSavePanel:(NSString*)fileName
						canCreateDir:(BOOL)canCreateDir
						defaultPath:(NSString*)defaultPath
{

	NSSavePanel* savePanel;
	int	result;
	NSString *filePath = @"";

	savePanel = [NSSavePanel savePanel];
	[savePanel setCanCreateDirectories:canCreateDir];

	result = [savePanel runModalForDirectory:defaultPath file:fileName];

	if (result == NSFileHandlingPanelOKButton) {
		filePath = [savePanel filename];
	}

	return filePath;
}

@end