#import "DownloadItem.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"

@implementation DownloadItem

//------------------------------------
// initWithURLString
//------------------------------------
- (id)initWithURLString:(NSString*)urlString
				videoId:(NSString*)videoId
				fileName:(NSString*)fileName
{

    if (self = [super init])
	{
		if([self startDownload:urlString
						videoId:videoId
						fileName:fileName
		] == NO){
			return nil;
		}
	}
    return self;

}
//------------------------------------
// startDownload
//------------------------------------
- (BOOL)startDownload:(NSString*)urlString
				videoId:(NSString*)videoId
				fileName:(NSString*)fileName
{

	// init parameters
	[self setTargetURL:urlString];
	[self setVideoId:videoId];
	[self setFileName:fileName];
	[self setFilePath:@""];
	[self setFileExt:@""];
	[self setIconImage:nil];
	[self setErrorLog:@""];
	[self setReceivedLength:0];
	[self setTotalLength:0];
	[self setStatus:DOWNLOAD_INIT];
	[self setNotifyTimer:nil];
	[self clearNotifyTimer];

	// create download instance
	[self setDownload:[self createDownload:urlString]];

	if([self download] != nil){
		return YES;
	}else{
		return NO;
	}
	
}

//------------------------------------
// createDownload
//------------------------------------
- (NSURLDownload*)createDownload:(NSString*)urlString
{
	// download
	NSURL *url = [NSURL URLWithString:urlString];
	if(url == nil){
		return nil;
	}
	NSURLRequest *req = [NSURLRequest requestWithURL:url cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:[self defaultRequestTimeOut]];
/*
	NSMutableURLRequest* req = [NSMutableURLRequest requestWithURL:url
															cachePolicy:NSURLRequestUseProtocolCachePolicy
															timeoutInterval:[self defaultRequestTimeOut]
									];
	[req setValue:[self convertToBaseURL] forHTTPHeaderField:@"Referer"];
*/
	if(req == nil){
		return nil;
	}

	return [[[NSURLDownload alloc] initWithRequest:req delegate:self] autorelease];

}
//------------------------------------
// cancelDownload
//------------------------------------
- (void)cancelDownload
{
	if([self status] != DOWNLOAD_STARTED){
		return;
	}

	[download_ cancel];
	[self setStatus:DOWNLOAD_CANCELED];
	[self notifyDownloadItemChange:self];
	[self clearNotifyTimer];
}
//------------------------------------
// didReceiveResponse
//------------------------------------
- (void)download:(NSURLDownload *)download didReceiveResponse:(NSURLResponse *)response
{

	// set file extension
	NSString *MIMEType = [response MIMEType];
	if([MIMEType isEqualToString:@"video/flv"]){
		[self setFileExt:@"flv"];
	}
	else if([MIMEType isEqualToString:@"video/mp4"]){
		[self setFileExt:@"mp4"];
	}
	else if([MIMEType isEqualToString:@"application/x-shockwave-flash"]){
		[self setFileExt:@"swf"];
	}
	else{
		[self setFileExt:@"flv"];
	}

	long long totalLength = [response expectedContentLength];
	if(totalLength != NSURLResponseUnknownLength){
		[self setTotalLength:totalLength];
	}else{
		[self setTotalLength:0];
	}
	[self setReceivedLength:0];
//	[self notifyDownloadItemChange:self];
}
//------------------------------------
// decideDestinationWithSuggestedFilename
//------------------------------------
- (void)download:(NSURLDownload *)download decideDestinationWithSuggestedFilename:(NSString *)filename
{

	// create destination path
	// path => @"/Users/foo/Desktop/filename"
	NSString *path = [self defaultStringValue:@"optDownloadFolderPath"];

	// check folder is exist
	BOOL isDir;
	if( [path isEqualToString:@""] ||
		[[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isDir] == NO ||
		isDir == NO
		){
		// change desktop path
		path = [@"~/Desktop" stringByExpandingTildeInPath];
	}

	// change file name
	NSMutableString *muFileName;
	if([self fileName] && ![[self fileName] isEqualToString:@""]){
		muFileName = [NSMutableString stringWithString:[self fileName]];
	}else{
		muFileName = [NSMutableString stringWithString:filename];
	}
	// replace "/" -> "-"
	[muFileName replaceOccurrencesOfString:@"/" withString:@"-" options:0 range:NSMakeRange(0,[muFileName length])];

	// return replaced string
	filename = muFileName;

	// add extension with file format
	filename = [filename stringByAppendingFormat:@".%@", [self fileExt]];

	// add file name
	path = [path stringByAppendingPathComponent:filename];

//	NSLog(@"path=%@", path);

	// allow overwrite
	[download setDestination:path allowOverwrite:NO];
 
}
//------------------------------------
// didCreateDestination
//------------------------------------
- (void)download:(NSURLDownload *)download didCreateDestination:(NSString *)filePath
{
	// save file path
	[self setFilePath:filePath];

	// set file icon image
	if([[NSFileManager defaultManager] fileExistsAtPath:[self filePath]] == YES){
		NSImage *image = [[NSWorkspace sharedWorkspace] iconForFile:[self filePath]];
		[image setSize:NSMakeSize(32,32)];
		[image setScalesWhenResized:YES];
		[self setIconImage:image];
	}
}
//------------------------------------
// didReceiveDataOfLength
//------------------------------------
- (void)download:(NSURLDownload *)download didReceiveDataOfLength:(unsigned)length
{
	[self setReceivedLength:[self receivedLength] + length];
//	if([self receivedLength] != NSURLResponseUnknownLength){
//		NSLog(@"length=%.0f/%.0f",(double)receivedLength_, (double)totalLength_);
//	}
}
//------------------------------------
// downloadDidBegin
//------------------------------------
-(void)downloadDidBegin:(NSURLDownload *)download
{
	[self setStatus:DOWNLOAD_STARTED];
	[self notifyDownloadItemChange:self];
	[self createNotifyTimer];
}
//------------------------------------
// downloadDidFinish
//------------------------------------
- (void)downloadDidFinish:(NSURLDownload *)download
{

	[self setStatus:DOWNLOAD_COMPLETED];
	[self notifyDownloadItemChange:self];
	[self clearNotifyTimer];

}

//------------------------------------
// didFailWithError
//------------------------------------
- (void)download:(NSURLDownload *)download didFailWithError:(NSError *)error
{

	[self setErrorLog:[NSString stringWithFormat: @"Download failed Error - %@ %@"
										, [error localizedDescription]
										, [[error userInfo] objectForKey:NSErrorFailingURLStringKey]
						]
	];

	[self setStatus:DOWNLOAD_FAILED];
	[self clearNotifyTimer];
	[self notifyDownloadItemChange:self];
}
//------------------------------------
// shouldDecodeSourceDataOfMIMEType
//------------------------------------
- (BOOL)download:(NSURLDownload *)download shouldDecodeSourceDataOfMIMEType:(NSString *)aType
{

	return NO;
  
}
//------------------------------------
// notifyDownloadItemChange
//------------------------------------
- (void)notifyDownloadItemChange:(DownloadItem*)item
{
//	NSLog(@"notifyDownloadItemChange");
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:DOWNLOAD_ITEM_NOTIF_NAME object:item];
}
//------------------------------------
// download
//------------------------------------
- (void)setDownload:(NSURLDownload*)download
{
	[download retain];
	[download_ release];
	download_ = download;
}
- (NSURLDownload*)download
{
	return download_;
}
//------------------------------------
// setTotalLength
//------------------------------------
- (void)setTotalLength:(long long)totalLength
{
	totalLength_ = totalLength;
}
- (long long)totalLength
{
	return totalLength_;
}

//------------------------------------
// setReceivedLength
//------------------------------------
- (void)setReceivedLength:(long long)receivedLength
{
	receivedLength_ = receivedLength;
}
- (long long)receivedLength
{
	return receivedLength_;
}
//------------------------------------
// setStatus
//------------------------------------
- (void)setStatus:(int)status
{
	status_ = status;
}
- (int)status
{
	return status_;
}
//------------------------------------
// targetURL
//------------------------------------
- (void)setTargetURL:(NSString*)targetURL
{
	[targetURL retain];
	[targetURL_ release];
	targetURL_ = targetURL;
}
- (NSString*)targetURL
{
	return targetURL_;
}
//------------------------------------
// videoId
//------------------------------------
- (void)setVideoId:(NSString*)videoId
{
	[videoId retain];
	[videoId_ release];
	videoId_ = videoId;
}
- (NSString*)videoId
{
	return videoId_;
}
//------------------------------------
// fileName
//------------------------------------
- (void)setFileName:(NSString*)fileName
{
	[fileName retain];
	[fileName_ release];
	fileName_ = fileName;
}
- (NSString*)fileName
{
	return fileName_;
}
//------------------------------------
// filePath
//------------------------------------
- (void)setFilePath:(NSString*)filePath
{
	[filePath retain];
	[filePath_ release];
	filePath_ = filePath;
}
- (NSString*)filePath
{
	return filePath_;
}
//------------------------------------
// fileExt
//------------------------------------
- (void)setFileExt:(NSString*)fileExt
{
	[fileExt retain];
	[fileExt_ release];
	fileExt_ = fileExt;
}
- (NSString*)fileExt
{
	return fileExt_;
}
//------------------------------------
// iconImage
//------------------------------------
- (void)setIconImage:(NSImage*)iconImage
{
	[iconImage retain];
	[iconImage_ release];
	iconImage_ = iconImage;
}
- (NSImage*)iconImage
{
	return iconImage_;
}
//------------------------------------
// errorLog
//------------------------------------
- (void)setErrorLog:(NSString*)errorLog
{
	[errorLog retain];
	[errorLog_ release];
	errorLog_ = errorLog;
}
- (NSString*)errorLog
{
	return errorLog_;
}
//------------------------------------
// notifyTimer
//------------------------------------
- (void)setNotifyTimer:(NSTimer*)notifyTimer
{
	[notifyTimer retain];
	[notifyTimer_ release];
	notifyTimer_ = notifyTimer;
}
- (NSTimer*)notifyTimer
{
	return notifyTimer_;
}
//------------------------------------
// createNotifyTimer
//------------------------------------
- (void)createNotifyTimer
{
//	NSLog(@"createNotifyTimer");
	// set timer for update
	[self setNotifyTimer:[NSTimer scheduledTimerWithTimeInterval:1.0
							target: self 
							selector:@selector(sendNotifyWithTimer:)
							userInfo:nil
							repeats: YES]
	];
	[[NSRunLoop currentRunLoop] addTimer:[self notifyTimer] forMode:(NSString*)kCFRunLoopCommonModes];

}
//------------------------------------
// clearNotifyTimer
//------------------------------------
- (void)clearNotifyTimer
{
	if([[self notifyTimer] isValid] == YES){
		[[self notifyTimer] invalidate];
//		NSLog(@"clearNotifyTimer");
	}
	
}
//------------------------------------
// sendNotifyWithTimer
//------------------------------------
- (void)sendNotifyWithTimer:(NSTimer*)aTimer
{
	[self notifyDownloadItemChange:self];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{

	[targetURL_ release];
	[videoId_ release];
	[fileName_ release];
	[filePath_ release];
	[fileExt_ release];
	[iconImage_ release];
	[errorLog_ release];
	[notifyTimer_ release];
	[download_ release];
	[super dealloc];
}
@end
