#import "DownloadManager.h"
#import "DownloadItem.h"
#import "LogStatusController.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "CellAttributeExtension.h"
#import "UserDefaultsExtension.h"

@implementation DownloadManager

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// alloc downloadList
	downloadList_ = [[NSMutableArray alloc] init];

	// window rect
	[self setWindowRect:downloadWindow key:@"rectWindowDownload"];

	[txtStatus setStringValue:@""];

	// set notification
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(downloadWindowWillClose:) name:NSWindowWillCloseNotification object:downloadWindow];
	// Register to get notified when the download manager's list changes
	[nc addObserver:self selector:@selector(handleDownloadsChange:) name:DOWNLOAD_ITEM_NOTIF_NAME object:nil];

}
//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openDownloadWindow
//------------------------------------
- (IBAction)openDownloadWindow:(id)sender
{
	[downloadWindow makeKeyAndOrderFront:self];
}

//------------------------------------
// restartDownloadItem
//------------------------------------
- (IBAction)restartDownloadItem:(id)sender
{
	[indProc startAnimation:self];

	// init log window
	[logStatusController setLogString:@""];

	if([self restartDownload] == NO){
//		NSLog(@"Can not restart download");
		[indProc stopAnimation:self];
		int result = [self displayMessage:@"alert"
						messageText:@"Can not restart download"
						infoText:@"Please check error log"
						btnList:@"Cancel,Log"
				];

		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController setTitle:@"Error Log"];
			[logStatusController openLogWindow:nil];
		}else{
			[logStatusController setTitle:@""];
			[logStatusController setLogString:@""];
		}
		return;
	}

	[indProc stopAnimation:self];
}

//------------------------------------
// cancelDownloadItem
//------------------------------------
- (IBAction)cancelDownloadItem:(id)sender
{

	if ([tbvDownloadlist numberOfSelectedRows] <= 0){
		return;
	}
	int index = [[tbvDownloadlist selectedRowIndexes] firstIndex];

	id record = [[self downloadList] objectAtIndex:index];
	DownloadItem *downloadItem = [record objectForKey:@"downloadItem"]; 

	[downloadItem cancelDownload];

}
//------------------------------------
// cancelAllDownloadItem
//------------------------------------
- (IBAction)cancelAllDownloadItem:(id)sender
{
	int i;

	for(i = 0; i < [[self downloadList] count]; i++){
		id record = [[self downloadList] objectAtIndex:i];
		DownloadItem *downloadItem = [record objectForKey:@"downloadItem"]; 
		[downloadItem cancelDownload];
	}

}

//------------------------------------
// searchDownloadItem
//------------------------------------
- (IBAction)searchDownloadItem:(id)sender
{

	if ([tbvDownloadlist numberOfSelectedRows] <= 0){
		return;
	}
	int index = [[tbvDownloadlist selectedRowIndexes] firstIndex];

	id record = [[self downloadList] objectAtIndex:index];
	DownloadItem *downloadItem = [record objectForKey:@"downloadItem"]; 

	NSString *filePath = [downloadItem filePath];
//	NSLog(@"filePath=%@", filePath);

	if([[NSFileManager defaultManager] fileExistsAtPath:filePath] == YES){
		[[NSWorkspace sharedWorkspace] selectFile:filePath inFileViewerRootedAtPath:nil];
	}else{
//		NSLog(@"downloaded file is not found");
		[self displayMessage:@"alert"
						messageText:@"Downloaded file is not found"
						infoText:@""
						btnList:@"Cancel"
		];
	}

}
//------------------------------------
// copyItemToPasteboard
//------------------------------------
- (IBAction)copyItemToPasteboard:(id)sender
{
	if([tbvDownloadlist numberOfSelectedRows] <= 0){
		return;
	}
	int index = [[tbvDownloadlist selectedRowIndexes] firstIndex];

	id record = [[self downloadList] objectAtIndex:index];
	DownloadItem *downloadItem = [record objectForKey:@"downloadItem"]; 

	NSString *url = [downloadItem targetURL];
	[self copyStringToPasteboard:url];
}
//------------------------------------
// clearFinishedItem
//------------------------------------
- (IBAction)clearFinishedItem:(id)sender
{

	NSMutableArray *array = [self downloadList];

	id record;

	int index = [array count] - 1;

	while (index >= 0){
		record = [array objectAtIndex:index];
		DownloadItem *downloadItem = [record objectForKey:@"downloadItem"];

		int status = [downloadItem status];
		if( status == DOWNLOAD_COMPLETED ||
			status == DOWNLOAD_CANCELED ||
			status == DOWNLOAD_FAILED
		){
			[array removeObject:record];
		}
		index--;
	}
	[tbvDownloadlist reloadData];

	// set status
	[self setDownloadStatus];

}
//------------------------------------
// downloadItems
//------------------------------------
- (void)downloadItems:(NSDictionary*)object
{

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return;
	}

	NSArray *items = [object objectForKey:@"items"];

	// no count
	if([items count] <= 0){
		return;
	}
	// over count
	if([items count] > 1){
		[self displayMessage:@"alert"
					messageText:@"Can not start downloading over 1 files at the same time."
					infoText:@"Please download each files."
					btnList:@"Cancel"
		];
		return;
	}

	int i;
	id record;
	NSString *videoId;
	NSString *title;

	// open window
//	[self openDownloadWindow:nil];

	for(i = 0; i < [items count]; i++){
		record = [items objectAtIndex:i];

		// create parameter
		videoId = [record valueForKey:@"videoId"];
		title = [record valueForKey:@"title"];
		
		// start download
		[self startDownloadItem:videoId
								fileName:title
		];
	}

}
//------------------------------------
// startDownloadItem
//------------------------------------
- (BOOL)startDownloadItem:(NSString*)videoId
				fileName:(NSString*)fileName
{
	BOOL ret = YES;

	[indProc startAnimation:self];

	// init log window
	[logStatusController setLogString:@""];

	// start download
	if([self startDownload:videoId
					fileName:fileName
		] == NO){

		[indProc stopAnimation:self];

		int result = [self displayMessage:@"alert"
						messageText:@"Can not start download"
						infoText:@"Please check error log"
						btnList:@"Cancel,Log"
				];

		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController setTitle:@"Error Log"];
			[logStatusController openLogWindow:nil];
		}else{
			[logStatusController setTitle:@""];
			[logStatusController setLogString:@""];
		}
		ret = NO;
	}

	[indProc stopAnimation:self];

	return ret;
}
//------------------------------------
// startDownload
//------------------------------------
- (BOOL)startDownload:(NSString*)videoId
				fileName:(NSString*)fileName
{

	NSString *message = @"";
	//
	// create parsed url
	//
	NSString *videoFileURL = [self getVideoFileURL:videoId message:&message];
//	NSLog(@"videoFileURL=%@", videoFileURL);

	if([videoFileURL isEqualToString:@""]){
		[logStatusController appendLogString:message];
//		NSLog(@"Can not create parsed url");
		return NO;
	}

	//
	// start download
	//
	DownloadItem *downloadItem = [[[DownloadItem alloc] initWithURLString:videoFileURL
														videoId:videoId
														fileName:fileName
									] autorelease];

	if(downloadItem == nil){
		[logStatusController appendLogString:@"Can not create download"];
//		NSLog(@"Can not create download");
		return NO;
	}

	[downloadList_ addObject:
		[NSMutableDictionary dictionaryWithObjectsAndKeys:
			downloadItem, @"downloadItem" ,
			nil
		]
	];

	[tbvDownloadlist reloadData];

	return YES;
}
//------------------------------------
// restartDownload
//------------------------------------
- (BOOL)restartDownload
{

	// no select
	if ([tbvDownloadlist numberOfSelectedRows] <= 0){
		return NO;
	}
	int index = [[tbvDownloadlist selectedRowIndexes] firstIndex];

	//
	// get player url
	//
	id record = [[self downloadList] objectAtIndex:index];
	DownloadItem *downloadItem = [record objectForKey:@"downloadItem"];
	NSString *videoId = [downloadItem videoId];
	NSString *fileName = [downloadItem fileName];
	NSString *message = @"";

	//
	// create parsed url
	//
	NSString *videoFileURL = [self getVideoFileURL:videoId message:&message];
//	NSLog(@"videoFileURL=%@", videoFileURL);

	if([videoFileURL isEqualToString:@""]){
		[logStatusController appendLogString:message];
//		NSLog(@"Can not create parsed url");
		return NO;
	}

	// cancel download
	[downloadItem cancelDownload];
	
	if([downloadItem startDownload:videoFileURL
						videoId:videoId
						fileName:fileName
		] == NO){
//		NSLog(@"Can not restart download");
		[logStatusController appendLogString:@"Can not start download"];
		return NO;
	}

	[tbvDownloadlist reloadData];

	return YES;
}
//------------------------------------
// setDownloadStatus
//------------------------------------
- (void)setDownloadStatus
{

	int totalCount = [downloadList_ count];
	int activeCount = 0;
	int i;
	int status;
	DownloadItem *downloadItem;

	for(i = 0; i < totalCount; i++){

		downloadItem = [[downloadList_ objectAtIndex:i] objectForKey:@"downloadItem"]; 
		if(downloadItem != nil){
			status = [downloadItem status];
			if(status == DOWNLOAD_INIT || status == DOWNLOAD_STARTED){
				activeCount++;
			}
		}
	}

	// status
	[txtStatus setStringValue:[NSString stringWithFormat:@"%d of %d", activeCount, totalCount]];

	// dock icon
	[self setDockIconWithValue:activeCount];

}
//------------------------------------
// setDockIconWithValue
//------------------------------------
- (void)setDockIconWithValue:(int)value
{

	NSImage *appIcon = [NSImage imageNamed:@"NSApplicationIcon"];
	NSImage *image = [[NSImage alloc] initWithSize:[appIcon size]];
//	NSImage *image = [[NSImage alloc] initWithSize:NSMakeSize(128,128)];

	NSRect imageBounds;
	imageBounds.origin.x = 0;
	imageBounds.origin.y = 0;
	imageBounds.size = [image size];

	[image lockFocus];

	[appIcon compositeToPoint:NSZeroPoint operation:NSCompositeDestinationOver];

	if(value > 0){

		NSString *valueString = @"";
		if(value < 10){
			valueString = [NSString stringWithFormat:@" %d ", value];
		}else{
			valueString = [NSString stringWithFormat:@"%d", value];
		}

		[self drawLabelAndString:imageBounds
						withString:valueString
						fontSize:32.0
						fontColor:[NSColor whiteColor]
						labelColor:[NSColor redColor]
						align:CELL_ALIGN_RIGHT
						valign:CELL_VALIGN_BOTTOM
						hPadding:12
						vPadding:8
						radius:15
		];

	}

	[image unlockFocus];

	[NSApp setApplicationIconImage:image];
}

//------------------------------------
// handleDownloadsChange
//------------------------------------
-(void)handleDownloadsChange:(NSNotification *)notification
{
//	NSLog(@"handleDownloadsChange");
	DownloadItem *downloadItem = [notification object];

	[tbvDownloadlist reloadData];

	// faile error
	if([downloadItem status] == DOWNLOAD_FAILED){
		// init log window
		if(![[downloadItem errorLog] isEqualToString:@""]){
			[logStatusController setLogString:[downloadItem errorLog]];
			int result = [self displayMessage:@"alert"
							messageText:@"Can not start download"
							infoText:@"Please check error log"
							btnList:@"Cancel,Log"
					];

			// show error log
			if(result == NSAlertSecondButtonReturn){
				[logStatusController setTitle:@"Error Log"];
				[logStatusController openLogWindow:nil];
			}else{
				[logStatusController setTitle:@""];
				[logStatusController setLogString:@""];
			}
		}
	}

	// set status
	[self setDownloadStatus];

}
//------------------------------------
// downloadList
//------------------------------------
- (NSMutableArray*)downloadList
{
	return downloadList_;
}
//------------------------------------
// isSelectItem
//------------------------------------
- (BOOL)isSelectItem
{
	if([tbvDownloadlist numberOfSelectedRows] > 0){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// anyItemIsDownloading
//------------------------------------
- (BOOL)anyItemIsDownloading
{
	BOOL ret = NO;
	int i;

	for(i = 0; i < [[self downloadList] count]; i++){
		id record = [[self downloadList] objectAtIndex:i];
		DownloadItem *downloadItem = [record objectForKey:@"downloadItem"]; 
		if([downloadItem status] <= DOWNLOAD_STARTED){
			ret = YES;
			break;
		}
	}

	return ret;

}
//------------------------------------
// mainWindowWillClose
//------------------------------------
-(void)downloadWindowWillClose:(NSNotification *)notification
{

	// save state from user default
	// window rect
	[self saveWindowRect:downloadWindow key:@"rectWindowDownload"];

}

//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[downloadList_ release];
	[super dealloc];
}
@end
