#import "FullScreenWindow.h"
#import "WebPlayerView.h"
#import "UserDefaultsExtension.h"

static const int WINDOW_CONTROLLER_HEIGHT = 27;

@implementation FullScreenWindow
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setIsFullScreen:NO];

	oldRect_ = [self frame];

	// title bar height
	titleBarHeight_ = [self frame].size.height
					-([[self contentView] frame].size.height + [[self contentView] frame].origin.y);

	[self setShowPlayerController:YES];
	[self setShowWindowController:YES];
}

//------------------------------------
// changeFullScreen
//------------------------------------
- (IBAction)changeFullScreen:(id)sender
{
	BOOL isFullScreen;

	// full screen
	if([self isFullScreen] == YES){
		isFullScreen = NO;
	}else{
		isFullScreen = YES;
		[self setShowPlayerController:YES];
		[self setShowWindowController:YES];
	}

	[self setFullScreen:isFullScreen animate:YES];

}
//------------------------------------
// changeFullScreenWithHide
//------------------------------------
- (IBAction)changeFullScreenWithHide:(id)sender
{
	// full screen
	if([self isFullScreen] == YES){

		// toggle full screen
		[self toggleFullScreen:nil];

	}else{

		// enter full screen & hide all
		[self setShowPlayerController:NO];
		[self setShowWindowController:NO];
		[self setFullScreen:YES animate:YES];

	}


}
//------------------------------------
// toggleFullScreen
//------------------------------------
- (IBAction)toggleFullScreen:(id)sender
{

	// full screen
	if([self isFullScreen] == YES){

		int playerType = [webPlayerView playerType];

		// nico player / watch video
		if(playerType == PLAYER_TYPE_NICO_PLAYER ||
			playerType == PLAYER_TYPE_NICO_PLAYER_Q ||
			playerType == PLAYER_TYPE_WATCH_VIDEO
		){
			// show player controller 
			if([self showPlayerController] == NO && [self showWindowController] == NO){
				[self setShowPlayerController:YES];
			}
			// show window controller 
			else if([self showPlayerController] == YES && [self showWindowController] == NO){
				[self setShowWindowController:YES];
			}
			// hide all
			else{
				[self setShowPlayerController:NO];
				[self setShowWindowController:NO];
			}
		}
		// ipad player
		else if(playerType == PLAYER_TYPE_IPAD_PLAYER){

			// show / hide window controller
			[self setShowWindowController:![self showWindowController]];
			// hide player controller
			[self setShowPlayerController:NO];

		}
		// ext player
		else if(playerType == PLAYER_TYPE_EXT_PLAYER){

			// show / hide window controller
			[self setShowWindowController:![self showWindowController]];
			// hide player controller
			[self setShowPlayerController:NO];

		}

		[self setFrame:[[NSScreen mainScreen] frame] display:YES animate:YES];

	}
}
//------------------------------------
// setFullScreen
//------------------------------------
- (void)setFullScreen:(BOOL)isFullScreen  animate:(BOOL)animate
{

	// same style
	if([self isFullScreen] == isFullScreen){
		return;
	}

	// save oldrect
	if(isFullScreen == YES){
		if([self isFullScreen] == NO){
			oldRect_ = [self frame];
		}
	}

	[self setIsFullScreen:isFullScreen];

	if(isFullScreen == YES){
		[NSMenu setMenuBarVisible:NO];
		[self setShowsResizeIndicator:NO];
		[self setFrame:[[NSScreen mainScreen] frame] display:YES animate:animate];
	}else{
		[NSMenu setMenuBarVisible:YES];
		[self setShowsResizeIndicator:YES];
		[self setFrame:oldRect_ display:YES animate:animate];
	}

	[self setLevel:NSNormalWindowLevel];

	// post notification
	[self postWindowFullScreenChangedNotification:isFullScreen];

}
//------------------------------------
// postWindowFullScreenChangedNotification
//------------------------------------
- (void)postWindowFullScreenChangedNotification:(BOOL)isFullScreen
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_FULLSCREEN_DID_CHANGED object:[NSNumber numberWithBool:isFullScreen]];
}
//------------------------------------
// close
//------------------------------------
- (void)close
{
	if([self isFullScreen] == YES){
		[self setFullScreen:NO animate:YES];
	}else{
		[super close];
	}
	[self setLevel:NSNormalWindowLevel];
}

//------------------------------------
// constrainFrameRect
//------------------------------------
-(NSRect)constrainFrameRect:(NSRect)frameRect toScreen:(NSScreen *)aScreen
{
	if([self isFullScreen] == YES){
		NSRect screenRect = [[NSScreen mainScreen] frame];
		screenRect.size.height += titleBarHeight_;

		// adjust control bar height
		if([self showWindowController] == NO){
			screenRect.size.height += WINDOW_CONTROLLER_HEIGHT;
			screenRect.origin.y -= WINDOW_CONTROLLER_HEIGHT;
		}
		if([self showPlayerController] == NO){
			int controllerHeight = 0;
			int playerType = [webPlayerView playerType];
			if(playerType == PLAYER_TYPE_NICO_PLAYER ||
				playerType == PLAYER_TYPE_NICO_PLAYER_Q ||
				playerType == PLAYER_TYPE_WATCH_VIDEO
			){
				controllerHeight = VIDEO_CONTROLLER_HEIGHT;
			}
			else if(playerType == PLAYER_TYPE_IPAD_PLAYER){
				controllerHeight = VIDEO_IPAD_CONTROLLER_HEIGHT;
			}
			else{
				controllerHeight = VIDEO_EXT_CONTROLLER_HEIGHT;
			}
			screenRect.size.height += controllerHeight;
			screenRect.origin.y -= controllerHeight;
		}

		return screenRect;
	}else{
		return [super constrainFrameRect:frameRect toScreen:aScreen];
	}
}
//------------------------------------
// canBecomeKeyWindow
//------------------------------------
- (BOOL)canBecomeKeyWindow
{
    return YES;
}
//------------------------------------
// isFullScreen
//------------------------------------
- (void)setIsFullScreen:(BOOL)isFullScreen
{
	isFullScreen_ = isFullScreen;
}
- (BOOL)isFullScreen
{
    return isFullScreen_;
}
//------------------------------------
// showPlayerController
//------------------------------------
- (void)setShowPlayerController:(BOOL)showPlayerController
{
	showPlayerController_ = showPlayerController;
}
- (BOOL)showPlayerController
{
    return showPlayerController_;
}
//------------------------------------
// showWindowController
//------------------------------------
- (void)setShowWindowController:(BOOL)showWindowController
{
	showWindowController_ = showWindowController;
}
- (BOOL)showWindowController
{
    return showWindowController_;
}

//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{    
	BOOL isAction = NO;

	// escape key
	if([theEvent keyCode] == 53){
		if([self isFullScreen] == YES){
			[self setFullScreen:NO animate:YES];
			isAction = YES;
		}
	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}
}

@end
