#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation NSObject(helperExtension_)
//------------------------------------
// openVideoWatchURL
//------------------------------------
- (void)openVideoWatchURL:(NSString*)videoId
{

	// create url
	NSString *url = [self convertToWatchURL:videoId];

	// open url
	if([self openURLWithBrowser:url] == NO){
		[self displayMessageAlertOpenURLWithBrowser:url];
	}
}
//------------------------------------
// openDictionaryURL
//------------------------------------
- (void)openDictionaryURL:(NSString*)string
{

	// create url
	NSString *url = [self convertToDictionaryURL:string];

	// open url
	if([self openURLWithBrowser:url] == NO){
		[self displayMessageAlertOpenURLWithBrowser:url];
	}
}
//------------------------------------
// openURLWithBrowser
//------------------------------------
- (BOOL)openURLWithBrowser:(NSString*)url
{

	if(!url){
		return NO;
	}

	// decode
	url = [self decodeToPercentEscapesString:url];
//	NSLog(@"url=%@", url);

	// open browser
	return [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:url]];

}
//------------------------------------
// openLoginURLWithBrowser
//------------------------------------
- (BOOL)openLoginURLWithBrowser
{
	NSString *url = @"";
//	NSString *account = [self defaultStringValue:@"optUserAccount"];
//	NSString *password = [self defaultStringValue:@"optUserPassword"];

//	if( (account && ![account isEqualToString:@""]) &&
//		(password && ![password isEqualToString:@""]))
//	{
//		url = [self convertToLoginURL:account password:password];
//	}else{
		url = [self convertToBaseURL];
//	}

	return [self openURLWithBrowser:url];
}
//------------------------------------
// openLoginURLWithPlayer
//------------------------------------
- (void)openLoginURLWithPlayer
{
	// post notification
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:APP_NOTIF_LOGIN_PAGE_OPEN object:nil];

}
//------------------------------------
// copyStringToPasteboard
//------------------------------------
- (void)copyStringToPasteboard:(NSString*)string
{
	NSPasteboard *pasteboard = [NSPasteboard generalPasteboard];
	[pasteboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
	[pasteboard setString:string forType:NSStringPboardType];
}
//------------------------------------
// getHTMLData
//------------------------------------
- (NSData*)getHTMLData:(NSString*)urlString referer:(NSString*)referer error:(NSError**)error
{
	// decode
	urlString = [self decodeToPercentEscapesString:urlString];
//	NSLog(@"urlString=%@", urlString);

	// get html
	NSURL *url = [NSURL URLWithString:urlString];
	if(url == nil){
		return nil;
	}

	NSMutableURLRequest* request = [NSMutableURLRequest requestWithURL:url
															cachePolicy:NSURLRequestUseProtocolCachePolicy
															timeoutInterval:[self defaultRequestTimeOut]
									];
	if(request == nil){
		return nil;
	}

	// set referer
	if(![referer isEqualToString:@""]){
		[request setValue:referer forHTTPHeaderField:@"Referer"];
	}

	[request setHTTPMethod:@"GET"];
	NSURLResponse *response;
	NSData *data = [NSURLConnection sendSynchronousRequest:request
										   returningResponse:&response
													   error:error];

/*
	// redirect url
	NSString *responseURLString = [[response URL] absoluteString];
	if((responseURLString && ![responseURLString isEqualToString:@""]) &&
		![urlString isEqualToString:responseURLString]){
//		NSLog(@"urlString=%@", urlString);
//		NSLog(@"response=%@", responseURLString);
		return [self getHTMLData:responseURLString referer:referer error:error];
	}else{
//		NSLog(@"ok");
	}
*/
	return data;
}
//------------------------------------
// getVideoFileURL
//------------------------------------
- (NSString*)getVideoFileURL:(NSString*)videoId message:(NSString**)message
{

	NSString *videoFileURL = @"";
	NSString *errorLog = @"";

	//
	// get watch html
	//
	NSString *watchURL = [self convertToWatchURL:videoId];
//	NSLog(@"watchURL=%@", watchURL);

	// access watch page
	NSError *error = nil;
	NSData *htmlData = [self getHTMLData:watchURL referer:@"" error:&error];
	if(error != nil || htmlData == nil){
		errorLog = [errorLog stringByAppendingString:[NSString stringWithFormat:@"Can not get data from url=%@\n", watchURL]];
		errorLog = [errorLog stringByAppendingString:[NSString stringWithFormat:@"error=[%@]\n", error]];
		*message = errorLog;
		return videoFileURL;
	}
//	NSString *html = [[[NSString alloc] initWithData:htmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"html=%@", html);

	//
	// get target html
	//
	NSString *getFileURL= [self convertToGetFileURL:videoId];
//	NSLog(@"getFileURL url=%@", getFileURL);

	// get target html data
	NSData *targetData = [self getHTMLData:getFileURL referer:@"" error:&error];
	if(error != nil || targetData == nil){
		errorLog = [errorLog stringByAppendingString:[NSString stringWithFormat:@"Can not get data from url=%@\n", getFileURL]];
		errorLog = [errorLog stringByAppendingString:[NSString stringWithFormat:@"error=[%@]\n", error]];
		*message = errorLog;
		return videoFileURL;
	}

//	NSString *html = [[[NSString alloc] initWithData:targetData encoding:NSUTF8StringEncoding] autorelease];
	NSString *html = [self encodeHTMLFromData:targetData];
//	NSLog(@"html=%@", html);

	//
	// parse target html to url(&url=xxxxxxx&)
	//
	videoFileURL = [self getVideoFileURLFromHTML:html];
//	NSLog(@"videoFileURL=%@", videoFileURL);

	if([videoFileURL isEqualToString:@""]){
		errorLog = [errorLog stringByAppendingString:[NSString stringWithFormat:@"Can not parse url=%@\n", videoFileURL]];
		errorLog = [errorLog stringByAppendingString:@"You are not login?\n"];
		errorLog = [errorLog stringByAppendingString:[NSString stringWithFormat:@"html=[%@]\n", html]];
		*message = errorLog;
		return videoFileURL;
	}

	return videoFileURL;
}
//------------------------------------
// getVideoFileURLFromHTML
//------------------------------------
- (NSString*)getVideoFileURLFromHTML:(NSString*)html
{

	NSString *urlString = [self getEnclosedString:html
								beginString:@"&url="
								endString:@"&"
								withEndSet:NO
							];

	// convert uri
	if(urlString && ![urlString isEqualToString:@""]){
		urlString = [self convertToURIEncodedString:urlString];
	}

	return urlString;

}
//------------------------------------
// encodeHTMLFromData
//------------------------------------
- (NSString*)encodeHTMLFromData:(NSData*)data
{
	NSStringEncoding encoding = [self getStringEncoding:data];
	NSString *html = [[[NSString alloc] initWithData:data encoding:encoding] autorelease];
	return html;
}

//------------------------------------
// getVideoCountStatusString
//------------------------------------
- (void)getVideoCountStatusString:(NSString*)string
				viewCountStr:(NSString**)viewCountStr
				resCountStr:(NSString**)resCountStr
				mylistCountStr:(NSString**)mylistCountStr
				beginString:(NSString*)beginString
				endString:(NSString*)endString
{

	// ĐF<strong>245,342</strong>@RgF<strong>52,803</strong>@}CXgF<a href="openlist/sm4691189"><strong>13,478</strong></a></p>

	NSScanner *scan = [NSScanner scannerWithString:string];

	// viewCountStr
	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:viewCountStr];

	// resCountStr
	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:resCountStr];

	// mylistCountStr
	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:mylistCountStr];

}
//------------------------------------
// getTagCountStatusString
//------------------------------------
- (void)getTagCountStatusString:(NSString*)string
				countStr:(NSString**)countStr
				beginString:(NSString*)beginString
				endString:(NSString*)endString
{

	// <a href="tag/....">{nq̐[w</a> ( 9 )</td>

	NSScanner *scan = [NSScanner scannerWithString:string];

	// tagname
	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:nil];

	// countStr
	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:countStr];


}
//------------------------------------
// getCategoryRankingPageString
//------------------------------------
- (void)getCategoryRankingPageString:(NSString*)string
				pageStartStr:(NSString**)pageStartStr
				pageEndStr:(NSString**)pageEndStr
				beginString:(NSString*)beginString
				endString:(NSString*)endString
{

	// <a href="..?page=3"><< 201-300</a> | <a href=".?page=5">401-500 >></a></p>
	
	NSScanner *scan = [NSScanner scannerWithString:string];

	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:pageStartStr];

	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:pageEndStr];

}
//------------------------------------
// getKeyAndValueString
//------------------------------------
- (NSDictionary*)getKeyAndValueString:(NSString*)string
				beginString:(NSString*)beginString
				endString:(NSString*)endString
{
	NSString *key = @"";
	NSString *value = @"";

	// 	so.addVariable("v",  "sm1417622");	
	NSScanner *scan = [NSScanner scannerWithString:string];

	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:&key];

	[scan scanUpToString:beginString intoString:nil];
	[scan scanString:beginString intoString:nil];
	[scan scanUpToString:endString intoString:&value];

	if(key == nil){
		key = @"";
	}
	if(value == nil){
		value = @"";
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
											key, @"key" ,
											value, @"value" ,
											nil
			];
}
//------------------------------------
// getEnclosedString
//------------------------------------
- (NSString*)getEnclosedString:(NSString*)string
						beginString:(NSString*)beginString
						endString:(NSString*)endString
						withEndSet:(BOOL)withEndSet
{

	NSString *scanValue = @"";
	NSString *enclosedString = @"";

	if(!string){
		return enclosedString;
	}

	NSScanner *scan = [NSScanner scannerWithString:string];

	if(![beginString isEqualToString:@""]){
		[scan scanUpToString:beginString intoString:nil];
		[scan scanString:beginString intoString:nil];
	}

	// go to endSet
	if(withEndSet == YES){
		NSCharacterSet *endSet = [NSCharacterSet characterSetWithCharactersInString:endString];
		[scan scanUpToCharactersFromSet:endSet intoString:&scanValue];
	}
	// go to endString
	else{
		[scan scanUpToString:endString intoString:&scanValue];
	}

	if(scanValue != nil){
		enclosedString = scanValue;
	}

	return enclosedString;

}
//------------------------------------
// getStringEncoding
//------------------------------------
- (NSStringEncoding)getStringEncoding:(NSData*)textData
{

	NSStringEncoding encodings[] = {
			NSUTF8StringEncoding,
			NSNonLossyASCIIStringEncoding,
			NSShiftJISStringEncoding, 
			NSJapaneseEUCStringEncoding,
			NSMacOSRomanStringEncoding,
			NSWindowsCP1251StringEncoding,
			NSWindowsCP1252StringEncoding,
			NSWindowsCP1253StringEncoding,
			NSWindowsCP1254StringEncoding,
			NSWindowsCP1250StringEncoding,
			NSISOLatin1StringEncoding,
			NSUnicodeStringEncoding, 
			0
	};

	int i = 0;
	NSString *string;
	NSStringEncoding encoding;

	// may be ISO-2022-JP
	if (memchr([textData bytes], 0x1b, [textData length]) != NULL) {
		string = [[[NSString alloc] initWithData:textData 
		encoding:NSISO2022JPStringEncoding] autorelease];
		if (string != nil) 
			return NSISO2022JPStringEncoding;
	}
	// try to encoding
	while(encodings[i] != 0){
		string = [[[NSString alloc] initWithData:textData encoding:encodings[i]] autorelease];
		if (string != nil) {
			encoding = encodings[i];
			break;
		}
		i++;
	}
	return encoding;
}
//------------------------------------
// getChildNode
//------------------------------------
- (NSXMLNode*)getChildNode:(NSXMLNode*)node name:(NSString*)name
{
	NSEnumerator *e = [[node children] objectEnumerator];

	NSXMLNode *childNode;
	while (childNode = [e nextObject]){
		if ([[childNode name] isEqualToString:name]){
			return childNode;
		}
	}

	return nil;
}
//------------------------------------
// getChildNodes
//------------------------------------
- (NSArray*)getChildNodes:(NSXMLNode*)node
{
	NSMutableArray *nodes = [[NSMutableArray arrayWithCapacity:[[node children] count]] retain];
	NSEnumerator *e = [[node children] objectEnumerator];

	NSXMLNode *childNode;
	while (childNode = [e nextObject]){
		[nodes addObject:[childNode stringValue]];
	}
	return [nodes autorelease];
}
//------------------------------------
// checkIsWatchURL
//------------------------------------
- (BOOL)checkIsWatchURL:(NSString*)urlString
{
	return [self checkIsTargetURL:urlString targetString:@"watch/"];
}
//------------------------------------
// checkIsMylistURL
//------------------------------------
- (BOOL)checkIsMylistURL:(NSString*)urlString
{
	return [self checkIsTargetURL:urlString targetString:@"mylist/"];
}
//------------------------------------
// checkIsChannelURL
//------------------------------------
- (BOOL)checkIsChannelURL:(NSString*)urlString
{
	return [self checkIsTargetURL:urlString targetString:@"channel/"];
}

//------------------------------------
// checkIsTargetURL
//------------------------------------
- (BOOL)checkIsTargetURL:(NSString*)urlString targetString:(NSString*)targetString
{
	BOOL ret = NO;

	NSRange rng = [urlString rangeOfString:targetString];
	if(rng.length > 0){
		ret = YES;
	}

	return ret;
}
//------------------------------------
// checkIsPreviewVideoId
//------------------------------------
- (BOOL)checkIsPreviewVideoId:(NSString*)videoId
{
	BOOL ret = YES;

	NSString *prefix = [videoId substringToIndex:2];

	if(
		[prefix isEqualToString:@"nm"] ||
		[prefix isEqualToString:@"so"]
	){
		ret = NO;
	}

	return ret;
}

//------------------------------------
// checkAccessErrorStatus
//------------------------------------
- (BOOL)checkAccessErrorStatus:(NSString*)headHTML bodyHTML:(NSString*)bodyHTML
{
	BOOL ret = NO;

	// check access error
	if([bodyHTML isEqualToString:@""]){
		BOOL isDenied;
		BOOL isBusy;
		BOOL isNotLogin;

		[self checkAccessErrorHTML:headHTML isBusy:&isBusy
											isDenied:&isDenied
											isNotLogin:&isNotLogin
		];

		// access crowded
		if(isBusy == YES){
			[self displayMessageAlertAccessBusy];
			ret = YES;
		}
		// access denied
		else if(isDenied == YES){
			[self displayMessageAlertAccessDenied];
			ret = YES;
		}
		// not login
		else if(isNotLogin == YES){
			[self displayMessageAlertNotLogin];
			ret = YES;
		}
		// other not found
		else{
//			[self displayMessageAlertAccessFailed];
//			ret = YES;
		}
	}
	return ret;
}
//------------------------------------
// checkAccessErrorHTML
//------------------------------------
- (void)checkAccessErrorHTML:(NSString*)html isBusy:(BOOL*)isBusy
											isDenied:(BOOL*)isDenied
											isNotLogin:(BOOL*)isNotLogin
{

	NSString *keyDenied = [self convertToShiftJISString:"ZԂł̘AANZX͂"];
	NSString *keyBusy = [self convertToShiftJISString:"Gł"];
//	NSString *keyNotLogin = [self convertToShiftJISString:"OCĂ܂"];
//	NSString *keyNotLogin = [self convertToShiftJISString:"OCĂ"];
	NSString *keyNotLogin = @"https://secure.nicovideo.jp/secure/login";

	BOOL getDenied = NO;
	BOOL getBusy = NO;
	BOOL getNotLogin = NO;
	NSRange rng;

	// denied
	rng = [html rangeOfString:keyDenied];
	if(rng.length > 0){
		getDenied = YES;
	}
	// busy
	rng = [html rangeOfString:keyBusy];
	if(rng.length > 0){
		getBusy = YES;
	}
	// not login
	rng = [html rangeOfString:keyNotLogin];
	if(rng.length > 0){
		getNotLogin = YES;
	}

	// access denied
	*isDenied = getDenied;

	// access crowded
	*isBusy = getBusy;

	// access crowded
	*isNotLogin = getNotLogin;

}
//------------------------------------
// checkNotFoundHTML
//------------------------------------
- (BOOL)checkNotFoundHTML:(NSString*)html
{
	BOOL ret = NO;
	NSString *keyNotFound = [self convertToShiftJISString:"y[W܂"];
	NSString *keyNotAccess = [self convertToShiftJISString:"y[WɃANZXł܂"];

	// not found
	NSRange rng = [html rangeOfString:keyNotFound];
	if(rng.length > 0){
		ret = YES;
	}
	rng = [html rangeOfString:keyNotAccess];
	if(rng.length > 0){
		ret = YES;
	}

	return ret;
}
//------------------------------------
// checkEnterAdultCategory
//------------------------------------
- (BOOL)checkEnterAdultCategory
{
	BOOL adult = NO;

	if([self defaultBoolValue:@"optAdultContent"] == YES){
		adult = YES;
	}else{
		int result = [self displayMessage:@"alert"
						messageText:@"This category includes adult contents."
						infoText:@"Are you older than 18 years old?"
						btnList:@"Yes,No"
					];

		// set defaults
		if(result == NSAlertFirstButtonReturn){
			[self setDefaultBoolValue:YES key:@"optAdultContent"];
			adult = YES;
		}
	}

	return adult;
}
//------------------------------------
// checkUserSession
//------------------------------------
- (BOOL)checkUserSession
{
	BOOL userSession = NO;

	NSHTTPCookieStorage *cookieStrage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
	NSArray *cookies = [cookieStrage cookies];
//	NSLog(@"cookies=%@", [cookies description]);

	NSHTTPCookie *cookie;
	int i;
	for(i = 0; i < [cookies count]; i++){
		cookie = [cookies objectAtIndex:i];
		if([[cookie domain] isEqualToString:@".nicovideo.jp"]){
			if([[cookie name] isEqualToString:@"user_session"]){
//				NSLog(@"cookie=%@", [cookie description]);
				NSDate *expiresDate = [cookie expiresDate];
				NSDate *today = [NSDate date];
				// today < expiresDate
				if([today earlierDate:expiresDate] == today){
					userSession = YES;
				}
				break;
			}
		}
	}
	return userSession;
}

@end