#import "MenuItemPlayHistory.h"
#import "ViewPlayer.h"
#import "ViewRelatedWindow.h"
#import "TBArrayController.h"

@implementation MenuItemPlayHistory

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"PlayHistoryMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate: self ];
}

//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	NSString *videoId = [sender representedObject]; 

	[viewPlayer setPlayerViewWithVideoId:videoId arrayNo:BIND_ARRAY_NONE];

}
//------------------------------------
// showAllItems
//------------------------------------
- (IBAction)showAllItems:(id)sender
{
	[viewRelatedWindow openPlayHistory:nil];
}
//------------------------------------
// removeAllItems
//------------------------------------
- (IBAction)removeAllItems:(id)sender
{
	[tbArrayController removeAllObjects:@"playhistory"];
}

//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{
	NSString *videoId;
	NSString *title;
	BOOL isPlayed;
	NSColor *itemColor;
	id record;

	// remove menuItem
	NSEnumerator *enumArray = [[aMenu itemArray] objectEnumerator];
	while (record = [enumArray nextObject]) {
		[aMenu removeItem:record];
	}

	// get item
	NSArray *fetchedArray = [tbArrayController getAllObjects:@"playhistory"];
	NSMenuItem *menuItem;

	// Set sort descriptors
	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"playDate" ascending:NO];
	fetchedArray = [fetchedArray sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	[sortDescriptor release];

	int i;
	// create menu item
	for(i = 0; i < [fetchedArray count] && i < 10; i++){

		record = [fetchedArray objectAtIndex:i];

		videoId = [record valueForKey:@"videoId"];
		title = [record valueForKey:@"title"];

		isPlayed = YES;
		if([record valueForKey:@"isPlayed"]){
			isPlayed = [[record valueForKey:@"isPlayed"] boolValue];
		}

		// item color
		itemColor = [NSColor blackColor];
		if(isPlayed == NO){
 			itemColor = [NSColor grayColor];
		}
		NSDictionary *attr = [NSDictionary dictionaryWithObjectsAndKeys:
										itemColor, NSForegroundColorAttributeName,
										nil
									];

//		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:@selector(playItem:) keyEquivalent:@""] autorelease];
		menuItem = [[[NSMenuItem alloc] initWithTitle:@"" action:@selector(playItem:) keyEquivalent:@""] autorelease];
		[menuItem setAttributedTitle:[[[NSMutableAttributedString alloc] initWithString:title attributes:attr] autorelease]];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:videoId];

		[aMenu addItem:menuItem];

	}

	// add separator
	if([fetchedArray count] > 0){
		[aMenu addItem:[NSMenuItem separatorItem]];
	}

	// show all menu
	// clear menu
	menuItem = [[[NSMenuItem alloc] initWithTitle:@"Show All Items" action:@selector(showAllItems:) keyEquivalent:@""] autorelease];
	[menuItem setTarget:self];
	[menuItem setRepresentedObject:nil];
	[aMenu addItem:menuItem];

/*
	// add separator
	[aMenu addItem:[NSMenuItem separatorItem]];

	// clear menu
	menuItem = [[[NSMenuItem alloc] initWithTitle:@"Clear" action:@selector(removeAllItems:) keyEquivalent:@""] autorelease];
	[menuItem setTarget:self];
	[menuItem setRepresentedObject:nil];
	[aMenu addItem:menuItem];
*/

	[self setSubmenu:aMenu];

}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self createSubMenu:menu];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
    [super dealloc];
}


@end
