#import "PopUpButtonMylistOrder.h"
#import "ViewMainSearch.h"
#import "UserDefaultsExtension.h"

@implementation PopUpButtonMylistOrder

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"MylistOrderMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate: self ];

}
//------------------------------------
// changeMylistOrder
//------------------------------------
- (IBAction)changeMylistOrder:(id)sender
{
	// separate keyword
	NSString *orderString = [sender representedObject];

	// set default
	[self setDefaultStringValue:orderString key:@"optMylistOrderString"];

	// sort search
	[viewMainSearch sortSearchResult:nil];

}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	int i;
	id record;
	NSString *orderString;
	NSString *title;
	BOOL enabled;

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuItems = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		// separate keyword, tag
		orderString = [record valueForKey:@"orderString"];

		title = [record valueForKey:@"title"];
		enabled = [[record valueForKey:@"enabled"] boolValue];

		// disabled
		if(enabled == NO){
			continue;
		}

		// add item separator
		if([orderString isEqualToString:@"-"]){
			[aMenu addItem:[NSMenuItem separatorItem]];
			continue;
		}

		// set action
		SEL sel;
		sel = NSSelectorFromString(@"changeMylistOrder:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:sel keyEquivalent:@""] autorelease];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:orderString];

		// set state
		if([orderString isEqualToString:[self defaultStringValue:@"optMylistOrderString"]]){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}

		[aMenu addItem:menuItem];
	}

	[self setMenu:aMenu];

	[menuItems release];

}

//------------------------------------
// updateMenuItem
//------------------------------------
- (void)updateMenuItem:(NSMenu*)aMenu
{

	id record;
	NSString *orderString;
	int i;

	NSArray *menuItems = [[aMenu itemArray] retain];

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		orderString = [record representedObject];

		// set state
		if([orderString isEqualToString:[self defaultStringValue:@"optMylistOrderString"]]){
			[record setState:1];
		}else{
			[record setState:0];
		}
	}

	[menuItems release];
}

//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// value, title
	NSString *keyEnabled = @"enabled";
	NSString *keyOrder = @"orderString";	// keyword
	NSString *keyTitle = @"title";

	return [[NSArray alloc] initWithObjects: 
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"at_d", keyOrder,
			[NSString stringWithCString:"ǉV" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"at_a", keyOrder,
			[NSString stringWithCString:"ǉÂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"tt_a", keyOrder,
			[NSString stringWithCString:"^Cg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"tt_d", keyOrder,
			[NSString stringWithCString:"^Cg~" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"pt_d", keyOrder,
			[NSString stringWithCString:"eV" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"pt_a", keyOrder,
			[NSString stringWithCString:"eÂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"vc_d", keyOrder,
			[NSString stringWithCString:"Đ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:NO], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"vc_a", keyOrder,
			[NSString stringWithCString:"ĐȂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:NO], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"cc_d", keyOrder,
			[NSString stringWithCString:"Rg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:NO], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"cc_a", keyOrder,
			[NSString stringWithCString:"RgȂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:NO], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"mc_d", keyOrder,
			[NSString stringWithCString:"}CXg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:NO], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"mc_a", keyOrder,
			[NSString stringWithCString:"}CXgȂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:NO], keyEnabled,
			nil
		],
		nil
	];
}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateMenuItem:menu];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
	[super dealloc];
}

@end
