#import "PopUpButtonRankingPeriod.h"
#import "ViewMainSearch.h"
#import "UserDefaultsExtension.h"
#import "NicoVideoExtension.h"

@implementation PopUpButtonRankingPeriod

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"RankingPeriodMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate: self ];

}
//------------------------------------
// changeRankingPeriod
//------------------------------------
- (IBAction)changeRankingPeriod:(id)sender
{
	// set default
	[self setDefaultStringValue:[sender representedObject] key:@"optRankingPeriodString"];

	// reload search
	[viewMainSearch reloadSearchPage:nil];

}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	int i;
	id record;
	NSString *timePeriodName;
	NSString *title;
	BOOL enabled;

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuItems = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];
		timePeriodName = [record valueForKey:@"timePeriodName"];
		enabled = [[record valueForKey:@"enabled"] boolValue];

		// disabled
		if(enabled == NO){
			continue;
		}

		// add item separator
		if([timePeriodName isEqualToString:@"-"]){
			[aMenu addItem:[NSMenuItem separatorItem]];
			continue;
		}

		// set title
		title = [self getTimePeriodTitleFromName:timePeriodName];

		// set action
		SEL sel;
		sel = NSSelectorFromString(@"changeRankingPeriod:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:sel keyEquivalent:@""] autorelease];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:timePeriodName];

		// set image
//		NSImage *image = [self getCountryIconImage:timePeriodName];
//		[menuItem setImage:image];

		// set state
		if([timePeriodName isEqualToString:[self defaultStringValue:@"optRankingPeriodString"]]){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}

		[aMenu addItem:menuItem];
	}

	[self setMenu:aMenu];

	[menuItems release];

}

//------------------------------------
// updateMenuItem
//------------------------------------
- (void)updateMenuItem:(NSMenu*)aMenu
{

	id record;
	NSString *timePeriodName;
	int i;
	int selectionIndex = 0;

	NSString *categoryName = [viewMainSearch categoryName];
	SEL sel;
	sel = NSSelectorFromString(@"changeRankingPeriod:");

	NSArray *menuItems = [[aMenu itemArray] retain];

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		timePeriodName = [record representedObject];

		// hourly && not all-> nil action
		if([timePeriodName isEqualToString:TIME_PERIOD_NAME_HOURLY]){
			if(![categoryName isEqualToString:CATEGORY_NAME_ALL]){
				[record setAction:nil];
			}else{
				[record setAction:sel];
			}
		}

		// set state
		if([timePeriodName isEqualToString:[self defaultStringValue:@"optRankingPeriodString"]]){
			[record setState:1];
			selectionIndex = i;
		}else{
			[record setState:0];
		}
	}

	[self selectItemAtIndex:selectionIndex];

	[menuItems release];
}

//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// value, title
	NSString *keyEnabled = @"enabled";
	NSString *keyOrder = @"timePeriodName";

	return [[NSArray alloc] initWithObjects: 
		[NSDictionary dictionaryWithObjectsAndKeys:
			TIME_PERIOD_NAME_HOURLY, keyOrder,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			TIME_PERIOD_NAME_DAILY, keyOrder,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			TIME_PERIOD_NAME_WEEKLY, keyOrder,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			TIME_PERIOD_NAME_MONTHLY, keyOrder,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			TIME_PERIOD_NAME_TOTAL, keyOrder,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		nil
	];
}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateMenuItem:menu];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
	[super dealloc];
}

@end
