#import "PopUpButtonRankingTarget.h"
#import "ViewMainSearch.h"
#import "UserDefaultsExtension.h"

@implementation PopUpButtonRankingTarget

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"RankingTargetMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate: self ];

}
//------------------------------------
// changeRankingTarget
//------------------------------------
- (IBAction)changeRankingTarget:(id)sender
{
	// set default
	[self setDefaultStringValue:[sender representedObject] key:@"optRankingTargetString"];

	// reload search
	[viewMainSearch reloadSearchPage:nil];

}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	int i;
	id record;
	NSString *targetString;
	NSString *title;
	BOOL enabled;

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuItems = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];
		targetString = [record valueForKey:@"targetString"];
		title = [record valueForKey:@"title"];
		enabled = [[record valueForKey:@"enabled"] boolValue];

		// disabled
		if(enabled == NO){
			continue;
		}

		// add item separator
		if([targetString isEqualToString:@"-"]){
			[aMenu addItem:[NSMenuItem separatorItem]];
			continue;
		}

		// set action
		SEL sel;
		sel = NSSelectorFromString(@"changeRankingTarget:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:sel keyEquivalent:@""] autorelease];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:targetString];

		// set image
//		NSImage *image = [self getCountryIconImage:targetString];
//		[menuItem setImage:image];

		// set state
		if([targetString isEqualToString:[self defaultStringValue:@"optRankingTargetString"]]){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}

		[aMenu addItem:menuItem];
	}

	[self setMenu:aMenu];

	[menuItems release];

}

//------------------------------------
// updateMenuItem
//------------------------------------
- (void)updateMenuItem:(NSMenu*)aMenu
{

	id record;
	NSString *targetString;
	int i;

	NSArray *menuItems = [[aMenu itemArray] retain];

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		targetString = [record representedObject];

		// set state
		if([targetString isEqualToString:[self defaultStringValue:@"optRankingTargetString"]]){
			[record setState:1];
		}else{
			[record setState:0];
		}
	}

	[menuItems release];
}

//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// value, title
	NSString *keyEnabled = @"enabled";
	NSString *keyOrder = @"targetString";
	NSString *keyTitle = @"title";

	return [[NSArray alloc] initWithObjects: 
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"view", keyOrder,
			[NSString stringWithCString:"Đ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"res", keyOrder,
			[NSString stringWithCString:"Rg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"mylist", keyOrder,
			[NSString stringWithCString:"}CXg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"fav", keyOrder,
			[NSString stringWithCString:"" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		nil
	];
}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateMenuItem:menu];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
	[super dealloc];
}

@end
