#import "PopUpButtonRelatedOrder.h"
#import "ViewRelatedSearch.h"
#import "UserDefaultsExtension.h"

@implementation PopUpButtonRelatedOrder

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"RelatedOrderMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate: self ];

}
//------------------------------------
// changeSearchOrder
//------------------------------------
- (IBAction)changeSearchOrder:(id)sender
{
	// separate keyword, tag
	NSString *orderString = [sender representedObject];

	// set default
	[self setDefaultStringValue:orderString key:@"optSearchRelatedOrderString"];

	// reload search
	[viewRelatedSearch reloadSearchPage:nil];

}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	int i;
	id record;
	NSString *orderString;
	NSString *title;
	BOOL enabled;

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuItems = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		orderString = [record valueForKey:@"orderString"];
		title = [record valueForKey:@"title"];
		enabled = [[record valueForKey:@"enabled"] boolValue];

		// disabled
		if(enabled == NO){
			continue;
		}

		// add item separator
		if([orderString isEqualToString:@"-"]){
			[aMenu addItem:[NSMenuItem separatorItem]];
			continue;
		}

		// set action
		SEL sel;
		sel = NSSelectorFromString(@"changeSearchOrder:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:sel keyEquivalent:@""] autorelease];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:orderString];

		// set image
//		NSImage *image = [self getCountryIconImage:orderString];
//		[menuItem setImage:image];

		// set state
		if([orderString isEqualToString:[self defaultStringValue:@"optSearchRelatedOrderString"]]){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}

		[aMenu addItem:menuItem];
	}

	[self setMenu:aMenu];

	[menuItems release];

}

//------------------------------------
// updateMenuItem
//------------------------------------
- (void)updateMenuItem:(NSMenu*)aMenu
{

	id record;
	NSString *orderString;
	int i;

	NSArray *menuItems = [[aMenu itemArray] retain];

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		orderString = [record representedObject];

		// set state
		if([orderString isEqualToString:[self defaultStringValue:@"optSearchRelatedOrderString"]]){
			[record setState:1];
		}else{
			[record setState:0];
		}
	}

	[menuItems release];
}

//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// value, title
	NSString *keyEnabled = @"enabled";
	NSString *keyOrder = @"orderString";
	NSString *keyTitle = @"title";

	return [[NSArray alloc] initWithObjects: 
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=p", keyOrder,
			[NSString stringWithCString:"IXXx" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=r", keyOrder,
			[NSString stringWithCString:"Rg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=v", keyOrder,
			[NSString stringWithCString:"Đ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=m", keyOrder,
			[NSString stringWithCString:"}CXg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=f", keyOrder,
			[NSString stringWithCString:"e" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		nil
	];
}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateMenuItem:menu];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
	[super dealloc];
}

@end
