#import "PopUpButtonSearchKeywordType.h"
#import "ViewMainSearch.h"
#import "UserDefaultsExtension.h"

@implementation PopUpButtonSearchKeywordType

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"SearchKeywordTypeMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate: self ];

}
//------------------------------------
// changeSearchKeywordType
//------------------------------------
- (IBAction)changeSearchKeywordType:(id)sender
{

	int searchKeywordType = [[sender representedObject] intValue];

	// set default
	[self setDefaultIntValue:searchKeywordType key:@"optSearchKeywordType"];

	// reload search
	[viewMainSearch reloadSearchPage:nil];

}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	int i;
	id record;
	int searchKeywordType;
	NSString *title;
	BOOL enabled;

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuItems = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];
		searchKeywordType = [[record valueForKey:@"keywordType"] intValue];
		title = [record valueForKey:@"title"];
		enabled = [[record valueForKey:@"enabled"] boolValue];

		// disabled
		if(enabled == NO){
			continue;
		}

		// add item separator
		if([title isEqualToString:@"-"]){
			[aMenu addItem:[NSMenuItem separatorItem]];
			continue;
		}

		// set action
		SEL sel;
		sel = NSSelectorFromString(@"changeSearchKeywordType:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:sel keyEquivalent:@""] autorelease];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:[NSNumber numberWithInt:searchKeywordType]];

		// set state
		if(searchKeywordType == [self defaultIntValue:@"optSearchKeywordType"]){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}

		[aMenu addItem:menuItem];
	}

	[self setMenu:aMenu];

	[menuItems release];

}

//------------------------------------
// updateMenuItem
//------------------------------------
- (void)updateMenuItem:(NSMenu*)aMenu
{

	id record;
	int searchKeywordType;
	int i;


	NSArray *menuItems = [[aMenu itemArray] retain];

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		searchKeywordType = [[record representedObject] intValue];

		// set state
		if(searchKeywordType == [self defaultIntValue:@"optSearchKeywordType"]){
			[record setState:1];
		}else{
			[record setState:0];
		}
	}

	[menuItems release];
}

//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// value, title
	NSString *keyEnabled = @"enabled";
	NSString *keyValue = @"keywordType";
	NSString *keyTitle = @"title";

	return [[NSArray alloc] initWithObjects: 
		[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithInt:SEARCH_WITH_STRING_KEYWORD], keyValue,
			[NSString stringWithCString:"L[[h" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithInt:SEARCH_WITH_STRING_TAG], keyValue,
			[NSString stringWithCString:"^O" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		nil
	];
}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateMenuItem:menu];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
	[super dealloc];
}

@end
