#import "PopUpButtonSearchOrder.h"
#import "ViewMainSearch.h"
#import "UserDefaultsExtension.h"

@implementation PopUpButtonSearchOrder

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"SearchOrderMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate: self ];

}
//------------------------------------
// changeSearchOrder
//------------------------------------
- (IBAction)changeSearchOrder:(id)sender
{
	// separate keyword, tag
	NSString *searchOrderString = [sender representedObject];
	NSArray *array = [searchOrderString componentsSeparatedByString:@","];
	NSString *keywordOrderString = [array objectAtIndex:0];
	NSString *tagOrderString = [array objectAtIndex:1];

	// set default
	[self setDefaultStringValue:keywordOrderString key:@"optSearchOrderString"];
	[self setDefaultStringValue:tagOrderString key:@"optSearchTagOrderString"];

	// reload search
	[viewMainSearch reloadSearchPage:nil];

}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	int i;
	id record;
	NSString *searchOrderString;
	NSString *keywordOrderString;
	NSString *tagOrderString;
	NSArray *array;
	NSString *title;
	BOOL enabled;
	int searchKeywordType = [self defaultIntValue:@"optSearchKeywordType"];

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuItems = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		// separate keyword, tag
		searchOrderString = [record valueForKey:@"orderString"];
		array = [searchOrderString componentsSeparatedByString:@","];
		keywordOrderString = [array objectAtIndex:0];
		tagOrderString = [array objectAtIndex:1];

		title = [record valueForKey:@"title"];
		enabled = [[record valueForKey:@"enabled"] boolValue];

		// disabled
		if(enabled == NO){
			continue;
		}

		// add item separator
		if([searchOrderString isEqualToString:@"-"]){
			[aMenu addItem:[NSMenuItem separatorItem]];
			continue;
		}

		// set action
		SEL sel;
		sel = NSSelectorFromString(@"changeSearchOrder:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:sel keyEquivalent:@""] autorelease];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:searchOrderString];

		// set state
		// keyword
		if(searchKeywordType == SEARCH_WITH_STRING_KEYWORD){
			if([keywordOrderString isEqualToString:[self defaultStringValue:@"optSearchOrderString"]]){
				[menuItem setState:1];
			}else{
				[menuItem setState:0];
			}
		}
		// tag
		else if(searchKeywordType == SEARCH_WITH_STRING_TAG){
			if([tagOrderString isEqualToString:[self defaultStringValue:@"optSearchTagOrderString"]]){
				[menuItem setState:1];
			}else{
				[menuItem setState:0];
			}
		}

		[aMenu addItem:menuItem];
	}

	[self setMenu:aMenu];

	[menuItems release];

}

//------------------------------------
// updateMenuItem
//------------------------------------
- (void)updateMenuItem:(NSMenu*)aMenu
{

	id record;
	NSString *searchOrderString;
	int i;

	NSArray *menuItems = [[aMenu itemArray] retain];

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		searchOrderString = [record representedObject];

		// set state
		if([searchOrderString isEqualToString:[self defaultStringValue:@"optSearchOrderString"]]){
			[record setState:1];
		}else{
			[record setState:0];
		}
	}

	[menuItems release];
}

//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// value, title
	NSString *keyEnabled = @"enabled";
	NSString *keyOrder = @"orderString";	// keyword, tag
	NSString *keyTitle = @"title";

	return [[NSArray alloc] initWithObjects: 
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=f&order=d,sort=f&order=d", keyOrder,
			[NSString stringWithCString:"eV" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=f&order=a,sort=f&order=a", keyOrder,
			[NSString stringWithCString:"eÂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=v&order=d,sort=v&order=d", keyOrder,
			[NSString stringWithCString:"Đ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=v&order=a,sort=v&order=a", keyOrder,
			[NSString stringWithCString:"ĐȂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=n&order=d,sort=n&order=d", keyOrder,
			[NSString stringWithCString:"RgV" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=n&order=a,sort=n&order=a", keyOrder,
			[NSString stringWithCString:"RgÂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=r&order=d,sort=r&order=d", keyOrder,
			[NSString stringWithCString:"Rg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=r&order=a,sort=r&order=a", keyOrder,
			[NSString stringWithCString:"RgȂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=m&order=d,sort=m&order=d", keyOrder,
			[NSString stringWithCString:"}CXg" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=m&order=a,sort=m&order=a", keyOrder,
			[NSString stringWithCString:"}CXgȂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=l&order=d,sort=l&order=d", keyOrder,
			[NSString stringWithCString:"ĐԂ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		[NSDictionary dictionaryWithObjectsAndKeys:
			@"sort=l&order=a,sort=l&order=a", keyOrder,
			[NSString stringWithCString:"ĐԂZ" encoding:NSShiftJISStringEncoding], keyTitle,
			[NSNumber numberWithBool:YES], keyEnabled,
			nil
		],
		nil
	];
}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateMenuItem:menu];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
	[super dealloc];
}

@end
