#import "PreviewImageView.h"
#import "ViewMainSearch.h"
#import "CellAttributeExtension.h"
#import "NicoVideoExtension.h"

@implementation PreviewImageView

//------------------------------------
// init
//------------------------------------
- (id)init
{
    if (self = [super init])
	{
		[self setVideoType:@""];
    }
    return self;
}
//------------------------------------
// drawRect
//------------------------------------
- (void)drawRect:(NSRect)rect
{

	// draw background
	[[NSColor blackColor] set];
	NSRectFill([self bounds]);

    NSPoint	imagePoint;
    imagePoint.x = [self bounds].origin.x;
    imagePoint.y = [self bounds].origin.y;

    // draw image
    if([self image]){

		NSImage *image = [[self image] copy];
 
		// set image size
		[image setSize:[self setCellImageSize:[image size]
									frameSize:[self bounds].size
									paddingSize:NSMakeSize(2,2)
						]
		];
		[image setScalesWhenResized:YES];

		// set image rect
		NSRect imageRect = [self setCellImageRect:[image size]
										frameRect:[self bounds]
							];

		imagePoint.x += imageRect.origin.x;
		imagePoint.y += imageRect.origin.y;

		// set image round rect

		NSRect imageRoundRect = [self setCellImageRoundRect:[image size]
													frameRect:[self bounds]
								];

		// draw image rect
		NSBezierPath *imagePath = [self setRoundPathForRect:imageRoundRect radius:5];
		[NSGraphicsContext saveGraphicsState];

		[imagePath addClip];

		// draw image
		[image compositeToPoint:imagePoint operation:NSCompositeSourceOver];

		// draw video type icon
		if(![[self videoType] isEqualToString:@""]){
			NSPoint iconPoint;
			NSImage *iconImage = [self getVideoTypeIcon:[self videoType]];
			NSSize iconSize = [self setSmallImageSize:[iconImage size]
											frameSize:NSMakeSize(imageRect.size.width * 0.8, imageRect.size.height * 0.3)
											paddingSize:NSMakeSize(0,0)
							];
			[iconImage setScalesWhenResized:YES];
			[iconImage setSize:iconSize];

			// right bottom
			iconPoint.x = imagePoint.x + imageRect.size.width - [iconImage size].width - 1;
			iconPoint.y = imagePoint.y - 1;
			// left top
//			iconPoint.x = imagePoint.x + 1;
//			iconPoint.y = imagePoint.y - imageRect.size.height + [iconImage size].height + 1;
			[iconImage compositeToPoint:iconPoint operation:NSCompositeSourceOver];
			[iconImage release];
		}

		[NSGraphicsContext restoreGraphicsState];

		[image release];
    }

}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)event
{
	// double clicked
	if([event type] == NSLeftMouseDown){
		if([event clickCount] == 2){
			[viewMainSearch playItem:nil];
		}
	}
}
//------------------------------------
// scrollWheel
//------------------------------------
- (void)scrollWheel:(NSEvent *)theEvent
{
	int wheelDelta;

	wheelDelta = [theEvent deltaY];

	if(wheelDelta <= 0){
		[searchlistArrayController selectNext:nil];
	}else{
		[searchlistArrayController selectPrevious:nil];
	}
}
//------------------------------------
// videoType
//------------------------------------
- (void)setVideoType:(NSString*)videoType
{
	[videoType retain];
	[videoType_ release];
	videoType_ = videoType;
}
- (NSString*)videoType
{
	return videoType_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[videoType_ release];
    [super dealloc];
}

@end
