#import "RelatedlistTableView.h"
#import "ViewRelatedSearch.h"
#import "RoundImageCell.h"
#import "ItemDescTextCell.h"
#import "TableColumnController.h"
#import "TableColumnExtension.h"
#import "ConvertExtension.h"
#import "CellAttributeExtension.h"
#import "UserDefaultsExtension.h"

@implementation RelatedlistTableView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];
	[super setDataSource:self];

	// set row height
	float rowHeight = [self defaultFloatValue:@"optRelatedRowHeight"];
	[sliderRowHeight setFloatValue:rowHeight];
	[self setRowHeight:rowHeight];

	// image column
	NSTableColumn *imageColumn = [self tableColumnWithIdentifier:@"image"];
	RoundImageCell *imageCell = [[[RoundImageCell alloc] init] autorelease];
	[imageColumn setDataCell:imageCell];

	// description column
	NSTableColumn *descColumn = [self tableColumnWithIdentifier:@"description"];
	ItemDescTextCell *descTextCell = [[[ItemDescTextCell alloc] init] autorelease];
	[descColumn setDataCell:descTextCell];

    // set click action
	[self setTarget:viewRelatedSearch];
	[self setDoubleAction:@selector(playItem:)];

	// set table column state
	[self setTableColumnState:self key:@"stateTableColumnRelatedlist"];

	// init column controller
	if (tcc_ == nil) {
		tcc_ = [[TableColumnController alloc] initWithTable:self];
	}

	// showTableColumns
	// set default hide columns
	NSArray *defaultHideCols = [NSArray arrayWithObjects:
									nil
								];
	[self showTableColumns:tcc_ aTableView:self defaultHideCols:defaultHideCols key:@"stateTableColumnRelatedlist"];

	// set sort descriptor
	[self setArrayControllerSortDescriptor:relatedlistArrayController key:@"sortDescColumnRelatedlist"];

	// set header menu
	[[self headerView] setMenu:cmRelatedlistHeader];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[self window]];
	[nc addObserver:self selector:@selector(handleImageItemLoad:) name:IMAGE_ITEM_NOTIF_NAME_RELATED object:nil];

	// set registerForDraggedTypes
	[self registerForDraggedTypes:[NSArray arrayWithObjects:ItemlistPboardType, NSStringPboardType, nil]];

	// set drag operation
	[self setDraggingSourceOperationMask:NSDragOperationLink forLocal:NO];

}
//------------------------------------
// changeRowHeight
//------------------------------------
- (IBAction)changeRowHeight:(id)sender
{
	[self setRowHeight:[sender floatValue]];

	if([self numberOfSelectedRows] > 0){
		[self scrollRowToVisible:[self selectedRow]];
	}

}
//------------------------------------
// change table column state
//------------------------------------
- (IBAction)changeColumnState:(id)sender
{
	[self setColumnState:tcc_ sender:sender];
}
//------------------------------------
// isShowColumn
//------------------------------------
- (BOOL)isShowColumn:(NSString*)identifier
{
	return [self isShowColumnState:tcc_ identifier:identifier];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{

	// table column state
	[self saveTableColumnState:self key:@"stateTableColumnRelatedlist"];
	// save sort descriptor
	[self saveArrayControllerSortDescriptor:relatedlistArrayController key:@"sortDescColumnRelatedlist"];

}
//=======================================================================
// bind to tableview column
//=======================================================================
//------------------------------------
// objectValueForTableColumn
//------------------------------------
- (id)tableView:(NSTableView *)aTableView
        objectValueForTableColumn:(NSTableColumn *)aTableColumn
        row:(int)rowIndex
{

	// get record from relatedlistArrayController
	id record = [[relatedlistArrayController arrangedObjects] objectAtIndex:rowIndex];

	// image
	if([[aTableColumn identifier] isEqualToString:@"image"]){
		ImageItem *imageItem = [record valueForKey:@"imageItem"];

		// image column
		RoundImageCell *imageCell = [aTableColumn dataCell];
		[imageCell setImage:[imageItem image]];
	}
	// description
	else if([[aTableColumn identifier] isEqualToString:@"description"]){

		NSString *title = [record valueForKey:@"title"];
		NSString *length = [record valueForKey:@"length"];
		NSString *postedDate = [record valueForKey:@"postedDate"];
		int viewCount = [[record valueForKey:@"viewCount"] intValue];
		int resCount = [[record valueForKey:@"resCount"] intValue];
		int mylistCount = [[record valueForKey:@"mylistCount"] intValue];
		BOOL hasCountInfo = [[record valueForKey:@"hasCountInfo"] boolValue];
		int itemStatus = [[record valueForKey:@"itemStatus"] intValue];

		NSString *desc1 = @"";
		NSString *desc2 = @"";
		if(itemStatus == VIDEO_ITEM_SUCCESS){
			desc1 = [NSString stringWithFormat:@"Time: %@  Date: %@", length, postedDate];
			if(hasCountInfo == YES){
				desc2 = [NSString stringWithFormat:@"Views: %@ Res: %@ Mylist: %@",
													[self convertToComma:viewCount],
													[self convertToComma:resCount],
													[self convertToComma:mylistCount]
						];
			}
		}
		else if(itemStatus == VIDEO_ITEM_STATUS_FAILED){
			NSString *errorDescription = [record valueForKey:@"errorDescription"];
			desc1 = errorDescription;
		}

		ItemDescTextCell *descTextCell = [aTableColumn dataCell];
		[descTextCell setStringValue:[NSString stringWithFormat:@"%@\t%@\t%@", title, desc1, desc2]];
		[descTextCell setItemStatus:itemStatus];

		return descTextCell;

	}

    return nil;
}

//------------------------------------
// numberOfRowsInTableView
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[relatedlistArrayController arrangedObjects] count];
}
//------------------------------------
// toolTipForCell
//------------------------------------
- (NSString *)tableView:(NSTableView *)aTableView toolTipForCell:(NSCell *)aCell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *)aTableColumn row:(int)row mouseLocation:(NSPoint)mouseLocation
{
    if ([[aTableColumn identifier] isEqualToString:@"description"]) {
		id record = [[relatedlistArrayController arrangedObjects] objectAtIndex:row];
		return [record valueForKey:@"description"];
    }
	return nil;

}
//------------------------------------
// tableViewSelectionDidChange
//------------------------------------
-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([self numberOfSelectedRows] == 1){
		[self scrollRowToVisible:[[self selectedRowIndexes] firstIndex]];
	}
}
//------------------------------------
// handleImageItemLoad
//------------------------------------
-(void)handleImageItemLoad:(NSNotification *)notification
{
	[self reloadData];
}
//=======================================================================
// drag & drop method
//=======================================================================
//------------------------------------
// writeRowsWithIndexes
//------------------------------------
- (BOOL)tableView:(NSTableView *)tv writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard*)pboard {

    BOOL success = NO;

	NSArray *objects = [[relatedlistArrayController arrangedObjects] objectsAtIndexes:rowIndexes];

	// create array for pasteboard
 	NSMutableArray *items = [NSMutableArray array];
 	NSString *videoURL = @"";
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		[items addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[item valueForKey:@"videoId"], @"videoId" ,
				[item valueForKey:@"title"], @"title" ,
				nil
			]
		];
	}
	// videoURL for other app
	if([objects count] > 0){
		videoURL = [self convertToWatchURL:[[objects objectAtIndex:0] valueForKey:@"videoId"]];
		if([self defaultEcoMode] == YES){
			videoURL = [self appendEcoModeStringToURL:videoURL];
		}
	}

	[pboard declareTypes:[NSArray arrayWithObjects:ItemlistPboardType, NSStringPboardType, nil] owner:nil];
	[pboard addTypes:[NSArray arrayWithObjects:ItemlistPboardType, NSStringPboardType, nil] owner:nil];
	[pboard setPropertyList:items forType:ItemlistPboardType];        
	if(![videoURL isEqualToString:@""]){     
		[pboard setString:videoURL forType:NSStringPboardType];        
	}

	if([items count] > 0){
		success = YES;
	}

	return success;

}
//=======================================================================
// drag & drop method
//=======================================================================
//------------------------------------
// validateDrop
//------------------------------------
- (NSDragOperation)tableView:(NSTableView*)tableView 
        validateDrop:(id <NSDraggingInfo>)info 
        proposedRow:(int)row 
        proposedDropOperation:(NSTableViewDropOperation)operation
{
	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard) {
		return NSDragOperationNone;
	}

	// Check pboard type
	// ignore self
	if (draggingSource == self) {
		return NSDragOperationNone;
	}
	if ([[pboard types] containsObject:ItemlistPboardType]) {
		return NSDragOperationCopy;
	}

	return NSDragOperationNone;

}

//------------------------------------
// acceptDrop
//------------------------------------
- (BOOL)tableView:(NSTableView*)tableView 
        acceptDrop:(id <NSDraggingInfo>)info 
        row:(int)row 
        dropOperation:(NSTableViewDropOperation)operation
{

	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard) {
		return NO;
	}

	// Check pboard type
	if (![[pboard types] containsObject:ItemlistPboardType]) {
		return NO;
	}
	// ignore self
	if (draggingSource == self) {
		return NO;
	}

	//
	// from itemlist
	//
	// get array for pasteboard
	NSArray *items = [pboard propertyListForType:ItemlistPboardType];

	// items
	if([items count] > 0){
		// search items after delay
		SEL sel = NSSelectorFromString(@"searchRelatedVideoWithItems:");
		// with command key
		if([[[NSApplication sharedApplication] currentEvent] modifierFlags] & NSCommandKeyMask){
			sel = NSSelectorFromString(@"searchPostUserWithItems:");
		}
		[viewRelatedSearch performSelector:sel withObject:items afterDelay:0.1];
		return YES;
	}

	return NO;
}
//------------------------------------
// dragImageForRowsWithIndexes
//------------------------------------
- (NSImage *)dragImageForRowsWithIndexes:(NSIndexSet*)dragRows
							tableColumns:(NSArray*)tableColumns
							event:(NSEvent*)dragEvent
							offset:(NSPointPointer)dragImageOffset
{
	// one item
/*
	if ([dragRows count] == 1){
		return [super dragImageForRowsWithIndexes:dragRows tableColumns:tableColumns event:dragEvent offset:dragImageOffset];
	}
	// many items
	else{
*/
		// create drag image
		NSImage	*dragImage = [[NSImage alloc] init];
		[dragImage setSize:NSMakeSize(65.0, 40.0)];

		// set count image bounds
		NSString *countStr = [NSString stringWithFormat:@"%i", [dragRows count]];
		NSRect imageBounds;
		imageBounds.origin = NSMakePoint(0, 0);
		imageBounds.size = [dragImage size];

		// create source image
		NSImage *sourceImage = [[[NSImage alloc] initByReferencingFile: [[NSBundle mainBundle] pathForResource:@"icon_video" ofType:@"png"]] autorelease];

		[sourceImage setScalesWhenResized:YES];
		[sourceImage setSize:NSMakeSize(24, 24)];

		NSPoint sourcePoint;
		sourcePoint.x = imageBounds.origin.x + ((imageBounds.size.width - [sourceImage size].width) / 2);
		sourcePoint.y = imageBounds.origin.y + ((imageBounds.size.height - [sourceImage size].height) / 2);

		[dragImage lockFocus];

		// draw label
		[self drawLabelAndString:imageBounds
						withString:countStr
						fontSize:LABEL_DRAG_FSIZE
						fontColor:[NSColor whiteColor]
						labelColor:[NSColor redColor]
						align:CELL_ALIGN_RIGHT
						valign:CELL_VALIGN_TOP
						hPadding:LABEL_DRAG_HPADDING
						vPadding:LABEL_DRAG_VPADDING
						radius:LABEL_DRAG_RADIUS
		];
		// draw source image
		[sourceImage compositeToPoint:sourcePoint operation:NSCompositeDestinationOver fraction:0.8];

		[dragImage unlockFocus];

		return [dragImage autorelease];
//	}
}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	
	BOOL isAction = NO;

	// control click
	if([theEvent modifierFlags] & NSControlKeyMask){
		[self rightMouseDown:theEvent];
		isAction = YES;
	}

	if(isAction == NO){
		[super mouseDown:theEvent];
	}
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)theEvent
{	

	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	BOOL isAction = NO;
	
	int i = [self rowAtPoint:p];
	
	if (i < [self numberOfRows]){
		// single selected
		if ([self numberOfSelectedRows] <= 1){
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
		}
		// multiple selected
		else{
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:YES];
		}
		[NSMenu popUpContextMenu:cmRelatedlist withEvent:theEvent forView:self];
		isAction = YES;
	}

	if(isAction == NO){
		[super rightMouseDown:theEvent];
	}

}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
