#import "ResizeSplitView.h"
#import "NotificationExtension.h"

@implementation ResizeSplitView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
//	[super setDelegate:self];
	[self setIsAnimate:NO];
	[self setOldSize:NSZeroSize];
	if([self isVertical] == YES){
		[self setControlIndex:RESIZE_VIEW_VERTICAL_CONTROL_INDEX];
	}else{
		[self setControlIndex:RESIZE_VIEW_HORIZONTAL_CONTROL_INDEX];
	}

}
//=======================================================================
// action
//=======================================================================

//=======================================================================
// method
//=======================================================================
//------------------------------------
// maximizeView
//------------------------------------
- (void)maximizeView:(BOOL)animate
{

	int index = [self controlIndex];

	NSPoint newPos = NSZeroPoint;

	id subView = [[self subviews] objectAtIndex:index];
	NSRect frame = [self frame];
	NSRect subFrame = [subView frame];
	float dividerThickness = [self dividerThickness];

	[self setOldSize:NSMakeSize(subFrame.size.width, subFrame.size.height)];

	// vertical
	if([self isVertical]){
		if(index == 0){
			newPos.x = frame.size.width - dividerThickness;
		}
	}
	// horizontal
	else{
		if(index == 1){
			newPos.y = frame.size.height - dividerThickness;
		}
	}

	[self setSplitterPosition:newPos animate:animate];
	[self splitViewDidResize];
}
//------------------------------------
// closeView
//------------------------------------
- (void)closeView:(BOOL)animate
{

	int index = [self controlIndex];

	NSPoint newPos = NSZeroPoint;

	id subView = [[self subviews] objectAtIndex:index];
	NSRect frame = [self frame];
	NSRect subFrame = [subView frame];
	float dividerThickness = [self dividerThickness];

	[self setOldSize:NSMakeSize(subFrame.size.width, subFrame.size.height)];

	// vertical
	if([self isVertical]){
		if(index == 1){
			newPos.x = frame.size.width - dividerThickness;
		}
	}
	// horizontal
	else{
		if(index == 0){
			newPos.y = frame.size.height - dividerThickness;
		}
	}

	[self setSplitterPosition:newPos animate:animate];
	[self splitViewDidResize];

}
//------------------------------------
// restoreView
//------------------------------------
- (void)restoreView:(BOOL)animate
{
	int index = [self controlIndex];

	NSPoint newPos = NSZeroPoint;
	NSSize oldSize = [self oldSize];

	NSRect frame = [self frame];
	float dividerThickness = [self dividerThickness];

	// vertical
	if([self isVertical]){
		if(oldSize.width <= 0){
			oldSize.width = 200;
		}
		if(index == 0){
			newPos.x = oldSize.width;
		}
		else{
			newPos.x = frame.size.width - dividerThickness - oldSize.width;
		}
	}
	// horizontal
	else{
		if(oldSize.height <= 0){
			oldSize.height = 150;
		}
		if(index == 0){
			newPos.y = frame.size.height - dividerThickness - oldSize.height;
		}
		else{
			newPos.y = oldSize.height;
		}
	}

	[self setSplitterPosition:newPos animate:animate];
	[self splitViewDidResize];

}
//
// set divider size
//
/*
- (void)setDividerSize:(float)dividerSize
{
	dividerSize_ = dividerSize;
}

//
// divider size
//
- (float)dividerThickness
{
    return dividerSize_;
}
*/
/*
- (void)drawDividerInRect:(NSRect)rect
{
    [[NSColor grayColor] set];
    NSRectFill(rect);
}
*/
//------------------------------------
// setSplitterPosition
//------------------------------------
- (void)setSplitterPosition:(NSPoint)newPos animate:(BOOL)animate
{

	if([[self subviews] count] < 2){
		return;
	}

	NSRect frame = [self frame];
	NSView *subview0 = [[self subviews] objectAtIndex:0];
	NSView *subview1 = [[self subviews] objectAtIndex:1];

	NSRect subview0EndFrame = [subview0 frame];
	NSRect subview1EndFrame = [subview1 frame];
	float dividerThickness = [self dividerThickness];

	if([self isVertical]){
		subview0EndFrame.size.width = newPos.x;
		subview1EndFrame.origin.x = newPos.x + dividerThickness;
		subview1EndFrame.size.width = frame.size.width - dividerThickness - newPos.x;
	}else{
		subview0EndFrame.origin.y = newPos.y + dividerThickness;
		subview0EndFrame.size.height = frame.size.height - dividerThickness - newPos.y;
		subview1EndFrame.size.height = newPos.y;
	}

//	NSLog(@"subview0EndFrame=%@", NSStringFromRect(subview0EndFrame));
//	NSLog(@"subview1EndFrame=%@", NSStringFromRect(subview1EndFrame));

	// same rect
	if( NSEqualRects([subview0 frame], subview0EndFrame) && 
		NSEqualRects([subview1 frame], subview1EndFrame)){
		return;
	}

	// Be sure the subview isn't hidden from a previous animation.
//	[subview0 setHidden:NO];
//	[subview1 setHidden:NO];

	// Update subviewEndFrame.origin so that the frame is positioned
	if(animate){

		NSDictionary *subview0Animation = [NSDictionary dictionaryWithObjectsAndKeys:
											subview0, NSViewAnimationTargetKey,
											[NSValue valueWithRect:subview0EndFrame], NSViewAnimationEndFrameKey,
											nil
										];
		NSDictionary *subview1Animation = [NSDictionary dictionaryWithObjectsAndKeys:
											subview1, NSViewAnimationTargetKey,
											[NSValue valueWithRect:subview1EndFrame], NSViewAnimationEndFrameKey,
											nil
										];

		NSViewAnimation *animation = [[NSViewAnimation alloc]
										initWithViewAnimations:[NSArray arrayWithObjects:
																	subview0Animation,
																	subview1Animation,
																	nil
																]
									];

		// NSAnimationBlocking
		// NSAnimationNonblocking
		// NSAnimationNonblockingThreaded
		[animation setAnimationBlockingMode:NSAnimationBlocking];

		[animation setDuration:0.5];

		[self setIsAnimate:YES];
		[animation startAnimation];
		[self setIsAnimate:NO];

		[animation release];

	}else{
		[subview0 setFrame:subview0EndFrame];
		[subview1 setFrame:subview1EndFrame];
	}

	// adjust rect
	if( !NSEqualRects([subview0 frame], subview0EndFrame) ||
		!NSEqualRects([subview1 frame], subview1EndFrame)){
		[subview0 setFrame:subview0EndFrame];
		[subview1 setFrame:subview1EndFrame];
	}

	[subview0 setHidden:NO];
	[subview1 setHidden:NO];

	[self adjustSubviews];

}
//------------------------------------
// resizeSubviewsWithOldSize
//------------------------------------
- (void)splitView:(ResizeSplitView *)sender resizeSubviewsWithOldSize:(NSSize)oldSize
{
	// Don't interfere with animation.
	if ([sender isAnimate]){
		// I got infinite recursion when I used plain old -display.
		[sender setNeedsDisplay:YES];
		return;
	}

	// Do whatever else you want to do in this delegate.
}
//------------------------------------
// viewDidEndLiveResize
//------------------------------------
- (void)viewDidEndLiveResize
{
//	NSLog(@"viewDidEndLiveResize");
	[self splitViewDidResize];
}
//------------------------------------
// splitViewDidResize
//------------------------------------
- (void)splitViewDidResize
{
//	NSLog(@"splitViewDidResize");
	[self postPreviewSizeDidChangedNotification:[self isVertical] isMaximized:[self isMaximized]];
}
//------------------------------------
// controlIndex
//------------------------------------
- (void)setControlIndex:(int)controlIndex
{
	controlIndex_ = controlIndex;
}
- (int)controlIndex
{
	return controlIndex_;
}
//------------------------------------
// isAnimate
//------------------------------------
- (void)setIsAnimate:(BOOL)isAnimate
{
	isAnimate_ = isAnimate;
}
- (BOOL)isAnimate
{
	return isAnimate_;
}
//------------------------------------
// oldSize
//------------------------------------
- (void)setOldSize:(NSSize)oldSize
{
	oldSize_ = oldSize;
}
- (NSSize)oldSize
{
	return oldSize_;
}
//------------------------------------
// isMaximized
//------------------------------------
- (BOOL)isMaximized
{

	int index = [self controlIndex];
	BOOL ret = NO;

	id subView = [[self subviews] objectAtIndex:index];
	float dividerThickness = [self dividerThickness];
	NSRect frame = [self frame];
	NSRect subFrame = [subView frame];
	if([self isVertical]){
		if(subFrame.size.width >= frame.size.width - dividerThickness){
			ret = YES;
		}
	}else{
		if(subFrame.size.height >= frame.size.height - dividerThickness){
			ret = YES;
		}
	}

	return ret;

}
//------------------------------------
// isClosed
//------------------------------------
- (BOOL)isClosed
{

	int index = [self controlIndex];
	BOOL ret = NO;

	id subView = [[self subviews] objectAtIndex:index];
	NSRect subFrame = [subView frame];
	if([self isVertical]){
		if(subFrame.size.width <= 0){
			ret = YES;
		}
	}else{
		if(subFrame.size.height <= 0){
			ret = YES;
		}
	}

	return ret;

}

@end
