#import "RoundImageCell.h"
#import "CellAttributeExtension.h"
#import "NicoVideoExtension.h"
#import "UserDefaultsExtension.h"

@implementation RoundImageCell
//------------------------------------
// init
//------------------------------------
- (id)init
{
    if (self = [super init])
	{
		[self setIsPlayed:NO];
		[self setVideoType:@""];
    }
    return self;
}
//------------------------------------
// drawInteriorWithFrame
//------------------------------------
- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView*)controlView
{

    NSPoint	imagePoint;

    imagePoint.x = cellFrame.origin.x;
    imagePoint.y = cellFrame.origin.y;

//	BOOL isPlayed = [self isPlayed];

    // draw image
    if([self image]){

		NSImage *image = [[self image] copy];

		// set image rect
		int padding = 2;
		int imageBGType = [self defaultIntValue:@"optImageBGType"];
		if(imageBGType == 1){
			padding = 4;
		}
		else if(imageBGType == 2){
			padding = 5;
		}
		// set image size
		[image setSize:[self setCellImageSize:[image size]
									frameSize:cellFrame.size
									paddingSize:NSMakeSize(padding,padding)
						]
		];
		[image setScalesWhenResized:YES];

		NSRect imageRect = [self setCellImageRect:[image size]
										frameRect:cellFrame
							];

		imagePoint.x += imageRect.origin.x;
		imagePoint.y += imageRect.origin.y;

        if([controlView isFlipped]) {
            imagePoint.y += [image size].height;
        }
		
		// set image round rect
		NSRect imageRoundRect = [self setCellImageRoundRect:[image size]
													frameRect:cellFrame
								];

		// draw background rect
		if(imageBGType == 1){
			NSRect bgRect = imageRoundRect;
			bgRect.origin.x -= 3;
			bgRect.origin.y -= 3;
			bgRect.size.width += 6;
			bgRect.size.height += 6;
			NSBezierPath *bgPath = [self setRoundPathForRect:bgRect radius:5];
			[[NSColor blackColor] set];
			[bgPath fill];
		}
		// draw shadow rect
		else if(imageBGType == 2){
			NSRect shadowRect = imageRoundRect;
			shadowRect.origin.x += 4;
			shadowRect.origin.y += 4;
			NSBezierPath *shadowPath = [self setRoundPathForRect:shadowRect radius:5];
			[[NSColor darkGrayColor] set];
			[shadowPath fill];
		}

		// draw image rect
		NSBezierPath *imagePath = [self setRoundPathForRect:imageRoundRect radius:5];
		[NSGraphicsContext saveGraphicsState];

		[imagePath addClip];

		// draw image
		[image compositeToPoint:imagePoint operation:NSCompositeSourceOver];

/*
		// draw iconImage
		if(isPlayed == YES){
			NSPoint iconPoint;
			NSImage *iconImage = [[NSImage imageNamed:@"icon_video"] copy];
			[iconImage setScalesWhenResized:YES];
			iconPoint.x = imagePoint.x + imageRect.size.width - [iconImage size].width - 3;
			iconPoint.y = imagePoint.y - 2;
			[iconImage compositeToPoint:iconPoint operation:NSCompositeSourceOver];
			[iconImage release];
		}
*/
		// draw video type icon
		if(![[self videoType] isEqualToString:@""]){
			NSPoint iconPoint;
			NSImage *iconImage = [self getVideoTypeIcon:[self videoType]];
			NSSize iconSize = [self setSmallImageSize:[iconImage size]
											frameSize:NSMakeSize(imageRect.size.width * 0.8, imageRect.size.height * 0.3)
											paddingSize:NSMakeSize(0,0)
							];
			[iconImage setScalesWhenResized:YES];
			[iconImage setSize:iconSize];
												
			// right bottom
			iconPoint.x = imagePoint.x + imageRect.size.width - [iconImage size].width - 1;
			iconPoint.y = imagePoint.y - 1;
			// left top
//			iconPoint.x = imagePoint.x + 1;
//			iconPoint.y = imagePoint.y - imageRect.size.height + [iconImage size].height + 1;
			[iconImage compositeToPoint:iconPoint operation:NSCompositeSourceOver];
			[iconImage release];
		}

		[NSGraphicsContext restoreGraphicsState];

		[image release];

    }

}
//------------------------------------
// isPlayed
//------------------------------------
- (void)setIsPlayed:(BOOL)isPlayed
{
	isPlayed_ = isPlayed;
}
- (BOOL)isPlayed
{
	return isPlayed_;
}
//------------------------------------
// videoType
//------------------------------------
- (void)setVideoType:(NSString*)videoType
{
	[videoType retain];
	[videoType_ release];
	videoType_ = videoType;
}
- (NSString*)videoType
{
	return videoType_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[videoType_ release];
    [super dealloc];
}

@end
