#import "TaglistTableView.h"
#import "ViewTagSearch.h"
#import "TagDescTextCell.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"

@implementation TaglistTableView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];
	[super setDataSource:self];

	// description column
	NSTableColumn *descColumn = [self tableColumnWithIdentifier:@"tagName"];
	TagDescTextCell *descTextCell = [[[TagDescTextCell alloc] init] autorelease];
	[descColumn setDataCell:descTextCell];

    // set click action
	[self setTarget:viewTagSearch];
	[self setDoubleAction:@selector(searchItem:)];

	// set table column state
	[self setTableColumnState:self key:@"stateTableColumnTaglist"];

	// set sort descriptor
	[self setArrayControllerSortDescriptor:taglistArrayController key:@"sortDescColumnTaglist"];

	// set observer
	[taglistArrayController addObserver:self forKeyPath:@"arrangedObjects" options:0 context:nil];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[self window]];

	// set registerForDraggedTypes
	[self registerForDraggedTypes:[NSArray arrayWithObjects:ItemlistPboardType, NSStringPboardType, nil]];

	// set drag operation
	[self setDraggingSourceOperationMask:NSDragOperationLink forLocal:NO];

}
//=======================================================================
// bind to tableview column
//=======================================================================
//------------------------------------
// objectValueForTableColumn
//------------------------------------
- (id)tableView:(NSTableView *)aTableView
        objectValueForTableColumn:(NSTableColumn *)aTableColumn
        row:(int)rowIndex
{

	// get record from taglistArrayController
	id record = [[taglistArrayController arrangedObjects] objectAtIndex:rowIndex];

	// tagName
	if([[aTableColumn identifier] isEqualToString:@"tagName"]){

		NSString *tagName = [record valueForKey:@"tagName"];
		int fontSize = [[record valueForKey:@"fontSize"] intValue];
		BOOL changeFontSize = [[record valueForKey:@"changeFontSize"] boolValue];

		TagDescTextCell *descTextCell = [aTableColumn dataCell];
		[descTextCell setStringValue:tagName];
		[descTextCell setFontSize:fontSize];
		[descTextCell setChangeFontSize:changeFontSize];

		return descTextCell;

	}
	// tagCount
	else if([[aTableColumn identifier] isEqualToString:@"tagCount"]){
		NSString *tagCountStr = @"-";
		BOOL showTagCount = [[record valueForKey:@"showTagCount"] boolValue];
		int tagCount = [[record valueForKey:@"tagCount"] intValue];
		if(showTagCount == YES && tagCount > 0){
			tagCountStr = [self convertToComma:tagCount];
		}
		return tagCountStr;
	}

    return nil;

}

//------------------------------------
// numberOfRowsInTableView
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[taglistArrayController arrangedObjects] count];
}
//------------------------------------
// heightOfRow
//------------------------------------
- (float)tableView:(NSTableView *)tableView heightOfRow:(int)rowIndex
{
	int rowHeight = 17;

	// get record from taglistArrayController
	id record = [[taglistArrayController arrangedObjects] objectAtIndex:rowIndex];
	int fontSize = [[record valueForKey:@"fontSize"] intValue];
	BOOL changeFontSize = [[record valueForKey:@"changeFontSize"] boolValue];
	if(changeFontSize == YES){
		rowHeight = fontSize + 5;
	}

	return rowHeight;
}

/*
//------------------------------------
// willDisplayCell
//------------------------------------
- (void)tableView:(NSTableView *)aTableView
      willDisplayCell:(id)aCell
      forTableColumn:(NSTableColumn *)aTableColumn
      row:(int)rowIndex
{
	int rowHeight = 17;

	// get record from taglistArrayController
	id record = [[taglistArrayController arrangedObjects] objectAtIndex:rowIndex];
	int fontSize = [[record valueForKey:@"fontSize"] intValue];
	BOOL changeFontSize = [[record valueForKey:@"changeFontSize"] boolValue];
	if(changeFontSize == YES){
		rowHeight = fontSize + 5;
	}
	[aCell setRowHeight:rowHeight];
}
*/
//------------------------------------
// tableViewSelectionDidChange
//------------------------------------
-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([self numberOfSelectedRows] == 1){
		[self scrollRowToVisible:[[self selectedRowIndexes] firstIndex]];
	}
}
//=======================================================================
// drag & drop method
//=======================================================================
//------------------------------------
// writeRowsWithIndexes
//------------------------------------
- (BOOL)tableView:(NSTableView *)tv writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard*)pboard {

    BOOL success = NO;

	NSArray *objects = [[taglistArrayController arrangedObjects] objectsAtIndexes:rowIndexes];

	// url for other app
	if([objects count] > 0){
		NSString *tagName = [[objects objectAtIndex:0] valueForKey:@"tagName"];
		NSString *urlString = [self convertToDictionaryURL:tagName];
		
		[pboard declareTypes:[NSArray arrayWithObjects:NSStringPboardType, nil] owner:nil];
		[pboard addTypes:[NSArray arrayWithObjects:NSStringPboardType, nil] owner:nil];
		[pboard setString:urlString forType:NSStringPboardType];        
		success = YES;
	}

	return success;

}
//------------------------------------
// validateDrop
//------------------------------------
- (NSDragOperation)tableView:(NSTableView*)tableView 
        validateDrop:(id <NSDraggingInfo>)info 
        proposedRow:(int)row 
        proposedDropOperation:(NSTableViewDropOperation)operation
{
	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	if (!pboard) {
		return NSDragOperationNone;
	}

	// Check pboard type
	if ([[pboard types] containsObject:ItemlistPboardType]) {
		return NSDragOperationCopy;
	}

	return NSDragOperationNone;

}

//------------------------------------
// acceptDrop
//------------------------------------
- (BOOL)tableView:(NSTableView*)tableView 
        acceptDrop:(id <NSDraggingInfo>)info 
        row:(int)row 
        dropOperation:(NSTableViewDropOperation)operation
{

	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	if (!pboard) {
		return NO;
	}

	// Check pboard type
	if (![[pboard types] containsObject:ItemlistPboardType]) {
		return NO;
	}

	//
	// from itemlist
	//
	// get array for pasteboard
	NSArray *items = [pboard propertyListForType:ItemlistPboardType];

	// items
	if([items count] > 0){
		// search items after delay
		SEL sel = NSSelectorFromString(@"searchWithItems:");
		[viewTagSearch performSelector:sel withObject:items afterDelay:0.1];
		return YES;
	}

	return NO;
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{

	// table column state
	[self saveTableColumnState:self key:@"stateTableColumnTaglist"];
	// save sort descriptor
	[self saveArrayControllerSortDescriptor:taglistArrayController key:@"sortDescColumnTaglist"];

}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// change review image
	if([keyPath isEqualToString:@"arrangedObjects"]){
		[self reloadData];
	}
}


//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	
	BOOL isAction = NO;

	// control click
	if([theEvent modifierFlags] & NSControlKeyMask){
		[self rightMouseDown:theEvent];
		isAction = YES;
	}

	if(isAction == NO){
		[super mouseDown:theEvent];
	}
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)theEvent
{	

	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	BOOL isAction = NO;
	
	int i = [self rowAtPoint:p];
	
	if (i < [self numberOfRows]){
		// single selected
		if ([self numberOfSelectedRows] <= 1){
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
		}
		// multiple selected
		else{
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:YES];
		}
		[NSMenu popUpContextMenu:cmTaglist withEvent:theEvent forView:self];
		isAction = YES;
	}

	if(isAction == NO){
		[super rightMouseDown:theEvent];
	}

}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[taglistArrayController removeObserver:self forKeyPath:@"arrangedObjects"];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
