#import "VideoInfoItem.h"
#import "ConvertExtension.h"
#import "HelperExtension.h"

@implementation VideoInfoItem
//------------------------------------
// init
//------------------------------------
- (id)initWithVideo:(NSString*)videoId
				title:(NSString*)title
				target:(id)target
				notifType:(int)notifType
				hasImage:(BOOL)hasImage
				params:(NSDictionary*)params
{
    if (self = [super init])
	{
		[self setVideoId:videoId];
		[self setTitle:title];
		[self setDescription:@""];
		[self setParams:params];
		[self setImageURL:@""];
		[self setLength:@""];
		[self setPostedDate:@""];
		[self setPostUserId:@""];
		[self setPublishedDate:@""];
		[self setLastResBody:@""];
		[self setTagList:nil];

		viewCount_ = 0;
		resCount_ = 0;
		mylistCount_ = 0;
		[self setErrorDescription:@""];

		hasImage_ = hasImage;
		notifType_ = notifType;
		itemStatus_ = VIDEO_ITEM_INIT;
		target_ = target;

		NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
		if(notifType_ == VIDEO_ITEM_NOTIF_LIST){
			[nc addObserver:target selector:@selector(handleItemObjectChange:) name:VIDEO_ITEM_NOTIF_NAME_LIST object:self];
		}
		else if(notifType_ == VIDEO_ITEM_NOTIF_ITEM){
			[nc addObserver:target selector:@selector(handleItemObjectChange:) name:VIDEO_ITEM_NOTIF_NAME_ITEM object:self];
		}
		else if(notifType_ == VIDEO_ITEM_NOTIF_INFO){
			[nc addObserver:target selector:@selector(handleItemObjectInfo:) name:VIDEO_ITEM_NOTIF_NAME_INFO object:self];
		}

		if([self createConnection:[self convertToThumbInfoURL:videoId]] == NO){
			itemStatus_ = VIDEO_ITEM_FAILED;
			[self notifyVideoInfoItemDidLoad:self];
		}

   }
    return self;
}
//------------------------------------
// createConnection
//------------------------------------
- (BOOL)createConnection:(NSString*)urlString
{

//	NSLog(@"urlString = %@", urlString);

	// get xml
	NSURL *url = [NSURL URLWithString:urlString];
	if(url == nil){
		return NO;
	}

	NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url
															cachePolicy:NSURLRequestUseProtocolCachePolicy
															timeoutInterval:10
									];
	if(request == nil){
		return NO;
	}

	[request setHTTPMethod:@"GET"];

	[self setConnection:[[[NSURLConnection alloc] initWithRequest:request delegate:self] autorelease]];
	if([self connection] != nil){
		return YES;
	}else{
		return NO;
	}

}

//------------------------------------
// parseXMLWithString
//------------------------------------
- (int)parseXMLWithString:(NSString*)xml
{

	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xml options:nil error:&error] autorelease];

	if(error != nil){
		return VIDEO_ITEM_NOT_FOUND;
	}

	// parse xml document
	NSXMLNode *rootNode = [xmlDoc rootElement];
//	NSLog(@"rootNode=%@", [rootNode name]);
//	NSLog(@"chileNodes=%@", [[self getChildNodes:rootNode] description]);

	if(rootNode == nil){
		return VIDEO_ITEM_NOT_FOUND;
	}

	// get status
	NSString *status = [[(NSXMLElement*)rootNode attributeForName:@"status"] stringValue];
	// failed
	if([status isEqualToString:@"fail"]){
		NSXMLNode *errorNode = [self getChildNode:rootNode name:@"error"];
		NSString *errorDescription = @"";
		if(errorNode != nil){
			NSXMLNode *descNode = [self getChildNode:errorNode name:@"description"];
			if(descNode != nil){
				errorDescription = [descNode stringValue];
			}
/*
			NSXMLNode *childNode;
			NSEnumerator *e = [[errorNode children] objectEnumerator];
			while (childNode = [e nextObject]){
				errorDescription = [errorDescription stringByAppendingFormat:@"%@ ", [childNode stringValue]];
			}
*/
		}
		[self setErrorDescription:[NSString stringWithFormat:@"Status : %@", errorDescription]];
//		NSLog(@"errorDescription=%@", errorDescription);
		return VIDEO_ITEM_STATUS_FAILED;
	}


	NSXMLNode *detailsNode = [self getChildNode:rootNode name:@"thumb"];
//	NSLog(@"detailsNode=%@", [detailsNode name]);
	if(detailsNode == nil){
		return VIDEO_ITEM_NOT_FOUND;
	}

	// set value
//	videoId_ = [[self getChildNode:detailsNode name:@"video_id"] stringValue];
	NSString *title = [[self getChildNode:detailsNode name:@"title"] stringValue];
	NSString *description = [[self getChildNode:detailsNode name:@"description"] stringValue];
	NSString *lastResBody = [[self getChildNode:detailsNode name:@"last_res_body"] stringValue];
	NSString *postUserId = [[self getChildNode:detailsNode name:@"user_id"] stringValue];

	if(!title){title = @"";}
	if(!description){description = @"";}
	if(!lastResBody){lastResBody = @"";}
	if(!postUserId){postUserId = @"";}

//	NSLog(@"postUserId=%@", postUserId);

	title = [self replaceWebCharacters:title];
	description = [self replaceWebCharacters:description];
	lastResBody = [self replaceWebCharacters:lastResBody];

	[self setTitle:title];
	[self setDescription:description];
	[self setImageURL:[[self getChildNode:detailsNode name:@"thumbnail_url"] stringValue]];
	[self setLength:[[self getChildNode:detailsNode name:@"length"] stringValue]];
	[self setLastResBody:lastResBody];

	NSString *dateStr = [[self getChildNode:detailsNode name:@"first_retrieve"] stringValue];
	[self setPostedDate:[self convertFormatedDateString:dateStr]];
	[self setPublishedDate:[self convertFormatedDateString:dateStr]];
	[self setPostUserId:postUserId];

/*
	NSString *year = [dateStr substringWithRange:NSMakeRange(0,4)];
	NSString *month = [dateStr substringWithRange:NSMakeRange(5,2)];
	NSString *date = [dateStr substringWithRange:NSMakeRange(8,2)];
	NSString *time = [dateStr substringWithRange:NSMakeRange(11,5)];
	[self setPostedDate:[NSString stringWithFormat:@"%@/%@/%@ %@", year, month, date, time]];
*/

	NSString *viewCountStr = [[self getChildNode:detailsNode name:@"view_counter"] stringValue];
	NSString *resCountStr = [[self getChildNode:detailsNode name:@"comment_num"] stringValue];
	NSString *mylistCountStr = [[self getChildNode:detailsNode name:@"mylist_counter"] stringValue];
	viewCount_ = [viewCountStr intValue];
	resCount_ = [resCountStr intValue];
	mylistCount_ = [mylistCountStr intValue];

	//
	// get tag list
	//
	NSXMLNode *childNode;
	NSXMLNode *tagsNode;
	NSString *domain;
	NSString *tagName;
	NSMutableArray *tagList = [[[NSMutableArray alloc] init] autorelease];

	NSEnumerator *e = [[detailsNode children] objectEnumerator];
	while (childNode = [e nextObject]){

		// tags
		if([[childNode name] isEqualToString:@"tags"]){

			// domain
			domain = [[(NSXMLElement*)childNode attributeForName:@"domain"] stringValue];
//			NSLog(@"domain=%@", domain);

			if([domain isEqualToString:@"jp"]){

				NSEnumerator *e2 = [[childNode children] objectEnumerator];
				while (tagsNode = [e2 nextObject]){

//					NSLog(@"tagsNode=%@", [tagsNode name]);
					// tag
					if(![[tagsNode name] isEqualToString:@"tag"]){
						continue;
					}
					tagName = [tagsNode stringValue];
					if([tagName isEqualToString:@""]){
						continue;
					}

					tagName = [self replaceWebCharacters:tagName];
					[tagList addObject:tagName];
				}
				break;
			}
			break;
		}
	}

	[self setTagList:tagList];

	return VIDEO_ITEM_SUCCESS;

}
//------------------------------------
// didReceiveResponse
//------------------------------------
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	// none
}
//------------------------------------
// redirectResponse
//------------------------------------
-(NSURLRequest *)connection:(NSURLConnection *)connection
            willSendRequest:(NSURLRequest *)request
           redirectResponse:(NSURLResponse *)redirectResponse
{
//	NSLog(@"redirectResponse");
//	NSLog(@"response=%@", [[redirectResponse URL] absoluteString]);
	return request;
}
//------------------------------------
// didReceiveData
//------------------------------------
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	if(!xmlData_) xmlData_ = [[NSMutableData data] retain];
	[xmlData_ appendData:data];
}
//------------------------------------
// didFailWithError
//------------------------------------
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	itemStatus_ = VIDEO_ITEM_FAILED;
	[self notifyVideoInfoItemDidLoad:self];
}
//------------------------------------
// connectionDidFinishLoading
//------------------------------------
- (void)connectionDidFinishLoading:(NSURLConnection*)connection
{
	NSString *xml = [[[NSString alloc] initWithData:xmlData_ encoding:NSUTF8StringEncoding] autorelease];

	// parse xml
	itemStatus_ = [self parseXMLWithString:xml];

	[self notifyVideoInfoItemDidLoad:self];

}
//------------------------------------
// notifyDownloadItemChange
//------------------------------------
- (void)notifyVideoInfoItemDidLoad:(VideoInfoItem*)item
{
//	NSLog(@"notifyDownloadItemChange");
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	if(notifType_ == VIDEO_ITEM_NOTIF_LIST){
		[nc postNotificationName:VIDEO_ITEM_NOTIF_NAME_LIST object:item];
	}
	else if(notifType_ == VIDEO_ITEM_NOTIF_ITEM){
		[nc postNotificationName:VIDEO_ITEM_NOTIF_NAME_ITEM object:item];
	}
	else if(notifType_ == VIDEO_ITEM_NOTIF_INFO){
		[nc postNotificationName:VIDEO_ITEM_NOTIF_NAME_INFO object:item];
	}
	// remove observer
	[self removeItemObserver:item];
}
//------------------------------------
// cancelConnection
//------------------------------------
- (void)cancelConnection
{
	if(connection_ != nil){
		[connection_ cancel];
	}
	itemStatus_ = VIDEO_ITEM_NOT_FOUND;

	// remove observer
	[self removeItemObserver:self];

}
//------------------------------------
// removeItemObserver
//------------------------------------
- (void)removeItemObserver:(VideoInfoItem*)item
{
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	if(notifType_ == VIDEO_ITEM_NOTIF_LIST){
		[nc removeObserver:target_ name:VIDEO_ITEM_NOTIF_NAME_LIST object:item];
	}
	else if(notifType_ == VIDEO_ITEM_NOTIF_ITEM){
		[nc removeObserver:target_ name:VIDEO_ITEM_NOTIF_NAME_ITEM object:item];
	}
	else if(notifType_ == VIDEO_ITEM_NOTIF_INFO){
		[nc removeObserver:target_ name:VIDEO_ITEM_NOTIF_NAME_INFO object:item];
	}
}
//------------------------------------
// connection
//------------------------------------
- (void)setConnection:(NSURLConnection*)connection
{
	[connection retain];
	[connection_ release];
	connection_ = connection;
}
- (NSURLConnection*)connection
{
	return connection_;
}

//------------------------------------
// videoId
//------------------------------------
- (void)setVideoId:(NSString*)videoId
{
	[videoId retain];
	[videoId_ release];
	videoId_ = videoId;
}
- (NSString*)videoId
{
	return videoId_;
}
//------------------------------------
// postUserId
//------------------------------------
- (void)setPostUserId:(NSString*)postUserId
{
	[postUserId retain];
	[postUserId_ release];
	postUserId_ = postUserId;
}
- (NSString*)postUserId
{
	return postUserId_;
}
//------------------------------------
// title
//------------------------------------
- (void)setTitle:(NSString*)title
{
	[title retain];
	[title_ release];
	title_ = title;
}
- (NSString*)title;
{
	return title_;
}
//------------------------------------
// description
//------------------------------------
- (void)setDescription:(NSString*)description
{
	[description retain];
	[description_ release];
	description_ = description;
}
- (NSString*)description;
{
	return description_;
}
//------------------------------------
// errorDescription
//------------------------------------
- (void)setErrorDescription:(NSString*)errorDescription
{
	[errorDescription retain];
	[errorDescription_ release];
	errorDescription_ = errorDescription;
}
- (NSString*)errorDescription;
{
	return errorDescription_;
}
//------------------------------------
// imageURL
//------------------------------------
- (void)setImageURL:(NSString*)imageURL
{
	[imageURL retain];
	[imageURL_ release];
	imageURL_ = imageURL;
}
- (NSString*)imageURL;
{
	return imageURL_;
}
//------------------------------------
// length
//------------------------------------
- (void)setLength:(NSString*)length
{
	[length retain];
	[length_ release];
	length_ = length;
}
- (NSString*)length;
{
	return length_;
}
//------------------------------------
// postedDate
//------------------------------------
- (void)setPostedDate:(NSString*)postedDate
{
	[postedDate retain];
	[postedDate_ release];
	postedDate_ = postedDate;
}
- (NSString*)postedDate;
{
	return postedDate_;
}
//------------------------------------
// publishedDate
//------------------------------------
- (void)setPublishedDate:(NSString*)publishedDate
{
	[publishedDate retain];
	[publishedDate_ release];
	publishedDate_ = publishedDate;
}
- (NSString*)publishedDate;
{
	return publishedDate_;
}
//------------------------------------
// lastResBody
//------------------------------------
- (void)setLastResBody:(NSString*)lastResBody
{
	[lastResBody retain];
	[lastResBody_ release];
	lastResBody_ = lastResBody;
}
- (NSString*)lastResBody
{
	return lastResBody_;
}

//------------------------------------
// tagList
//------------------------------------
- (void)setTagList:(NSArray*)tagList
{
	[tagList retain];
	[tagList_ release];
	tagList_ = tagList;
}
- (NSArray*)tagList
{
	return tagList_;
}
//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
	[params_ release];
	params_ = params;
}
- (NSDictionary*)params
{
	return params_;
}

//------------------------------------
// viewCount
//------------------------------------
- (int)viewCount;
{
	return viewCount_;
}
//------------------------------------
// resCount
//------------------------------------
- (int)resCount;
{
	return resCount_;
}
//------------------------------------
// mylistCount
//------------------------------------
- (int)mylistCount;
{
	return mylistCount_;
}
//------------------------------------
// itemStatus
//------------------------------------
- (int)itemStatus;
{
	return itemStatus_;
}
//------------------------------------
// notifType
//------------------------------------
- (int)notifType;
{
	return notifType_;
}
//------------------------------------
// hasImage
//------------------------------------
- (BOOL)hasImage;
{
	return hasImage_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{

//	NSLog(@"dealloc=%@", videoId_);

	[self removeItemObserver:self];

	[videoId_ release];
	[title_ release];
	[description_ release];
	[errorDescription_ release];
	[imageURL_ release];
	[length_ release];
	[postedDate_ release];
	[postUserId_ release];
	[publishedDate_ release];
	[lastResBody_ release];
	[tagList_ release];
	[params_ release];

	[xmlData_ release];
	[connection_ release];

    [super dealloc];
}

@end
