#import "ViewChannelSetting.h"
#import "ViewPlaylistSetting.h"
#import "ViewMainSearch.h"
#import "ViewPlaylist.h"
#import "TBArrayController.h"
#import "UserDefaultsExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"

@implementation ViewChannelSetting

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setPlistId:@""];
	[self setIsNewList:NO];
	[txtChannelTitle setStringValue:@""];
	[txtChannelId setStringValue:@""];
	[self changeTakeButtonEnable];
}
//------------------------------------
// updateChannelSetting
//------------------------------------
- (IBAction)updateChannelSetting:(id)sender
{

	NSString *title = [txtChannelTitle stringValue];
	NSString *channelId = [txtChannelId stringValue];

	if([title isEqualToString:@""]){
		[self displayMessage:@"alert"
							messageText:@"Title is not entered."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}
	if([channelId isEqualToString:@""]){
		[self displayMessage:@"alert"
							messageText:@"Channel ID is not entered."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}

	// chXXX
	if([self checkIsChannelString:channelId] == NO){
		[self displayMessage:@"alert"
							messageText:@"Channel ID must be \"ch\" + number."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}

	// new
	if([self isNewList] == YES){
		int itemType = ITEM_CHANNEL;
		int itemSubType = ITEM_SEARCH_STRING;
		int itemSubTypeAttribute = 0;

		// add item
		[viewPlaylist addItem:itemType
					itemSubType:itemSubType
					itemSubTypeAttribute:itemSubTypeAttribute
					title:title
					keyword:channelId
					isFolder:NO
					isSelect:YES
					isEdit:NO
		];

		[viewPlaylistSetting closeWindow:nil];
	}
	// edit
	else{

		NSString *plistId = [self plistId];

		if([plistId isEqualToString:@""]){
			[self displayMessage:@"alert"
								messageText:@"Channel is not found"
								infoText:@""
								btnList:@"Cancel"
			];
			return;
		}

		// update playlist
		if([self updatePlaylist:plistId title:title keyword:channelId] == YES){
			[viewPlaylistSetting closeWindow:nil];
		}
	}

}
//------------------------------------
// takeChannelValuesFromMainWindow:
//------------------------------------
- (IBAction)takeChannelValuesFromMainWindow:(id)sender
{

	NSString *title = [viewMainSearch channelTitle];
	NSString *channelId = [viewMainSearch channelId];

	[txtChannelTitle setStringValue:title];
	[txtChannelId setStringValue:channelId];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createChannelSettingWindow
//------------------------------------
- (BOOL)createChannelSettingWindow:(BOOL)isNewList
{

	NSString *plistId = @"";
	NSString *title = @"";
	NSString *channelId = @"";

	[self setIsNewList:isNewList];

	// new
	if(isNewList == YES){
		// none
	}
	// edit
	else{
		NSArray *fetchedArray = [tbArrayController getSelectedObjects:@"playlist"];

		// not result
		if([fetchedArray count] <= 0){
			[self displayMessage:@"alert"
								messageText:@"Channel is not found"
								infoText:@""
								btnList:@"Cancel"
			];
			return NO;
		}

		// get value
		id record = [fetchedArray objectAtIndex:0];
		plistId = [record valueForKey:@"plistId"];
		title = [record valueForKey:@"title"];
		channelId = [record valueForKey:@"keyword"];

		int itemType = [[record valueForKey:@"itemType"] intValue];

		// not channel
		if(itemType != ITEM_CHANNEL){
			[self displayMessage:@"alert"
								messageText:@"Channel is not selected"
								infoText:@""
								btnList:@"Cancel"
			];
			return NO;
		}
	}

	[self setPlistId:plistId];

	[txtChannelTitle setStringValue:title];
	[txtChannelId setStringValue:channelId];
	[self changeTakeButtonEnable];

	return YES;

}
//------------------------------------
// updatePlaylist
//------------------------------------
- (BOOL)updatePlaylist:(NSString*)plistId title:(NSString*)title keyword:(NSString*)keyword
{

	// get playllist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"plistId == %@", plistId];
	NSArray *fetchedArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not result
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Channel is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// get value
	id record = [fetchedArray objectAtIndex:0];

	int itemType = [[record valueForKey:@"itemType"] intValue];

	// not keyword
	if(itemType != ITEM_CHANNEL){
		[self displayMessage:@"alert"
							messageText:@"Channel is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// update value
	[record setValue:title forKey:@"title"];
	[record setValue:keyword forKey:@"keyword"];

	return YES;

}
//------------------------------------
// changeTakeButtonEnable
//------------------------------------
- (void)changeTakeButtonEnable
{
	BOOL enable = NO;

	if(![[viewMainSearch channelId] isEqualToString:@""]){
		enable = YES;
	}
	[btnTakeValue setEnabled:enable];
}
//------------------------------------
// setPlistId
//------------------------------------
- (void)setPlistId:(NSString*)plistId
{
	[plistId retain];
	[plistId_ release];
	plistId_ = plistId;
}
//------------------------------------
// plistId
//------------------------------------
- (NSString*)plistId
{
	return plistId_;
}
//------------------------------------
// isNewList
//------------------------------------
- (void)setIsNewList:(BOOL)isNewList
{
	isNewList_ = isNewList;
}
- (BOOL)isNewList;
{
	return isNewList_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[plistId_ release];
	[super dealloc];
}
@end
