#import "ViewKeywordSetting.h"
#import "ViewPlaylistSetting.h"
#import "TBArrayController.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation ViewKeywordSetting

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setPlistId:@""];
	[txtTitle setStringValue:@""];
}
//------------------------------------
// changeKeywordType
//------------------------------------
- (IBAction)changeKeywordType:(id)sender
{

	int keywordType = [[btnKeywordType selectedItem] tag];
	[self setKeywordType:keywordType];

}
//------------------------------------
// changeKeywordSetting
//------------------------------------
- (IBAction)changeKeywordSetting:(id)sender
{

	int keywordType = [[btnKeywordType selectedItem] tag];

	NSString *plistId = [self plistId];
	if([plistId isEqualToString:@""]){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}

	// update playlist
	if([self updatePlaylist:plistId itemSubTypeAttribute:keywordType] == YES){
		[self setKeywordType:keywordType];
		[viewPlaylistSetting closeWindow:nil];
	}

}

//=======================================================================
// Common subroutines
//=======================================================================
- (BOOL)createKeywordSettingWindow
{

	NSArray *fetchedArray = [tbArrayController getSelectedObjects:@"playlist"];

	// not result
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// get value
	id record = [fetchedArray objectAtIndex:0];
	NSString *plistId = [record valueForKey:@"plistId"];
	NSString *title = [record valueForKey:@"title"];

	int itemType = [[record valueForKey:@"itemType"] intValue];
	int itemSubType = [[record valueForKey:@"itemSubType"] intValue];
	int itemSubTypeAttribute = ITEM_SEARCH_STRING_DEFAULT;

	if([record valueForKey:@"itemSubTypeAttribute"]){
		itemSubTypeAttribute = [[record valueForKey:@"itemSubTypeAttribute"] intValue];
	}

	// not keyword
	if(itemType != ITEM_SEARCH || itemSubType != ITEM_SEARCH_STRING){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}


	[self setPlistId:plistId];
	[self setKeywordType:itemSubTypeAttribute];

	[txtTitle setStringValue:title];
	[btnKeywordType selectItemWithTag:itemSubTypeAttribute];

	return YES;

}

//------------------------------------
// updatePlaylist
//------------------------------------
- (BOOL)updatePlaylist:(NSString*)plistId itemSubTypeAttribute:(int)itemSubTypeAttribute
{

	// get playllist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"plistId == %@", plistId];
	NSArray *fetchedArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not result
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// get value
	id record = [fetchedArray objectAtIndex:0];

	int itemType = [[record valueForKey:@"itemType"] intValue];
	int itemSubType = [[record valueForKey:@"itemSubType"] intValue];

	// not keyword
	if(itemType != ITEM_SEARCH || itemSubType != ITEM_SEARCH_STRING){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// update value
	[record setValue:[NSNumber numberWithInt:itemSubTypeAttribute] forKey:@"itemSubTypeAttribute"];

	return YES;

}

//------------------------------------
// setPlistId
//------------------------------------
- (void)setPlistId:(NSString*)plistId
{
	[plistId retain];
	[plistId_ release];
	plistId_ = plistId;
}
//------------------------------------
// plistId
//------------------------------------
- (NSString*)plistId
{
	return plistId_;
}
//------------------------------------
// keywordType
//------------------------------------
- (void)setKeywordType:(int)keywordType
{
	keywordType_ = keywordType;
}
- (int)keywordType
{
	return keywordType_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[plistId_ release];
	[super dealloc];
}
@end
