/* ViewMainSearch */

#import <Cocoa/Cocoa.h>
#import "SearchTypes.h"
#import "SearchTagTypes.h"
#import "PreviewImageView.h"
#import "ResizeSplitView.h"
#import "ImageItem.h"
#import "VideoInfoItem.h"
#import "PopUpButtonRankingPeriod.h"
#import "PlaylistItemTypes.h"
#import "TimePeriodTypes.h"
#import "ConnectionDataItem.h"
#import "BindObjectTypes.h"
#import "ViewInfoStatus.h"

@interface ViewMainSearch : NSObject
{

	IBOutlet id viewRelatedWindow;
	IBOutlet id viewRelatedSearch;
	IBOutlet id viewTagSearch;
	IBOutlet id viewPlayer;
	IBOutlet id viewPreviewPlayer;
	IBOutlet id viewVideoContent;
	IBOutlet id viewPlaylistSetting;
	
	IBOutlet id tbArrayController;
	IBOutlet id downloadManager;
	IBOutlet id logStatusController;
	IBOutlet NSArrayController *searchlistArrayController;

	IBOutlet NSWindow *mainWindow;
	
	IBOutlet NSSearchField *txtSearchField;
	IBOutlet NSTextField *txtSearchResults;
	IBOutlet NSButton *btnPagePrev;
	IBOutlet NSButton *btnPageNext;
	IBOutlet NSButton *btnPlay;
	IBOutlet NSButton *btnInfo;

	IBOutlet NSTabView *tabViewSearchHead;
	IBOutlet NSPopUpButton *pbtnSearchOrder;
	IBOutlet NSPopUpButton *pbtnSearchKeywordType;
	IBOutlet NSPopUpButton *pbtnMylistOrder;
	IBOutlet NSButton *btnAddMylist;
	IBOutlet NSPopUpButton *pbtnPostUserOrder;
	IBOutlet NSButton *btnAddPostUser;
	IBOutlet NSPopUpButton *pbtnPlaylistOrder;
	IBOutlet NSPopUpButton *pbtnRankingTarget;
	IBOutlet PopUpButtonRankingPeriod *pbtnRankingPeriod;
	IBOutlet NSButton *btnRankingEnable;
	IBOutlet NSButton *btnSearchCancel;

	IBOutlet NSButton *btnTabViewSearchTable;
	IBOutlet NSButton *btnTabViewSearchGrid;
	IBOutlet NSTabView *tabViewSearchResult;
	IBOutlet NSTabView *tabViewSearchSlider;

	IBOutlet NSTabView *tabViewPreview;
	IBOutlet NSButton *btnPreviewImage;
	IBOutlet NSButton *btnPreviewMovie;

	IBOutlet PreviewImageView *imgPreview;
	IBOutlet NSProgressIndicator *indProc;

	IBOutlet ResizeSplitView *spvMain;
	IBOutlet ResizeSplitView *spvNavi;
	IBOutlet NSSplitView *spvHead;
	IBOutlet NSButton *btnFrameMainChange;

	ConnectionDataItem *connectionDataItem_;
	NSMutableArray *itemList_;
	NSMutableArray *searchList_;
 
	NSString *urlString_;
	NSString *searchString_;
	NSString *searchURL_;
	NSString *searchTitle_;
	NSString *categoryName_;
	NSString *contentsName_;
	NSString *plistId_;

	NSString *mylistId_;
	NSString *mylistTitle_;
	NSString *mylistAuthor_;

	NSString *postUserId_;
	NSString *postUserTitle_;

	NSString *channelId_;
	NSString *channelTitle_;

	NSString *logString_;

	int startIndex_;
	int lastIndex_;
	int totalResults_;
	int maxResults_;

	int pageIndex_;
	int pageMaxIndex_;

	int searchType_;
	int dataType_;
	BOOL isKeywordSearch_;

}
- (IBAction)openMainWindow:(id)sender;
- (IBAction)searchWithKeyword:(id)sender;
- (IBAction)changeSearchPage:(id)sender;
- (IBAction)moveSearchPage:(id)sender;
- (IBAction)reloadSearchPage:(id)sender;
- (IBAction)sortSearchResult:(id)sender;
- (IBAction)cancelSearch:(id)sender;
- (IBAction)changeCategoryRanking:(id)sender;

- (IBAction)playItem:(id)sender;
- (IBAction)previewItem:(id)sender;
- (IBAction)downloadItem:(id)sender;
- (IBAction)removeItem:(id)sender;
- (IBAction)removeItemFromPlaylist:(id)sender;
- (IBAction)removeItemFromPlayHistory:(id)sender;

- (IBAction)openVideoPageWithBrowser:(id)sender;
- (IBAction)copyItemToPasteboard:(id)sender;
- (IBAction)showItemDescription:(id)sender;
- (IBAction)showVideoContent:(id)sender;
- (IBAction)addItemToPlaylist:(id)sender;
- (IBAction)addMylist:(id)sender;
- (IBAction)addPostUser:(id)sender;

- (IBAction)searchRelatedVideo:(id)sender;
- (IBAction)searchPostUserVideo:(id)sender;
- (IBAction)searchRelatedTag:(id)sender;

- (IBAction)changePreviewView:(id)sender;
- (IBAction)changePreviewImage:(id)sender;
- (IBAction)changeTabViewSearchResult:(id)sender;
- (IBAction)changeSplitViewMainFrame:(id)sender;
- (IBAction)changeSplitViewPreviewFrame:(id)sender;
- (IBAction)maximizePreviewFrame:(id)sender;
- (IBAction)restorePreviewFrame:(id)sender;

- (void)openSearchWithMylist:(NSString*)mylistId;
- (void)openSearchWithPostUser:(NSString*)postUserId;
- (void)openSearchWithChannel:(NSString*)channelId;
- (void)reloadWithPageIndex:(int)pageIndex;
- (void)searchWithString:(NSString*)searchString pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithURL:(NSString*)searchURL searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithContentsName:(NSString*)contentsName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithCategoryName:(NSString*)categoryName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithCategoryRanking:(NSString*)categoryName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithMylist:(NSString*)mylistId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithPostUser:(NSString*)postUserId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithChannel:(NSString*)channelId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithPlaylist:(NSString*)plistId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithPlaylistItems:(NSArray*)items pageIndex:(int)pageIndex searchType:(int)searchType dataType:(int)dataType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)searchWithResultItems:(NSMutableArray*)items pageIndex:(int)pageIndex searchType:(int)searchType dataType:(int)dataType;
- (void)sortSearchList:(int)searchType sortString:(NSString*)sortString;
- (NSDictionary*)getMylistSearchParams:(NSString*)sortString;
- (NSDictionary*)getPlaylistSearchParams:(NSString*)sortString;

- (void)parseResultHTML:(NSString*)html params:(NSDictionary*)params;
- (void)parseSearchHTML:(NSString*)html maxResults:(int)maxResults breakType:(int)breakType dataType:(int)dataType;
- (void)parsePageStatusHTML:(NSString*)html;
- (NSDictionary*)splitResultHTML:(NSString*)html splitType:(int)splitType;

- (void)parseResultRss:(NSString*)html params:(NSDictionary*)params;
- (BOOL)parseResultXML:(NSString*)xml
			maxResults:(int)maxResults
			searchType:(int)searchType
			dataType:(int)dataType
			parseRssType:(int)parseRssType;

- (void)changeTabViewSearchHeadIndex;
- (void)changePageButtonEnable;
- (void)changeOrderButtonEnable;
- (void)changeRankingButtonState;

- (void)changeTabViewSearchResultIndex:(int)index;
- (void)changeTabViewSearchSliderIndex:(int)index;
- (void)changeTabViewSearchButtonEnable:(int)index;
- (void)changeTabViewPreview:(int)index;
- (void)changeFrameMainChangeButton;

- (void)handleDataItemStatus:(NSNotification *)notification;
- (void)handleDataItemDidLoad:(NSNotification *)notification;
- (void)clearConnection;

- (NSDictionary*)createSearchItem:(int)index
						videoId:(NSString*)videoId
						imageURL:(NSString*)imageURL
						title:(NSString*)title;
- (void)addItemList:(NSDictionary*)object dataType:(int)dataType;
- (void)removeItemList;
- (void)removeSearchList;

- (void)setSearchStatus:(NSDictionary*)params;
- (void)setWindowStatus:(NSDictionary*)params;
- (void)setWindowTitle:(NSString*)searchTitle searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch;
- (void)setLoadingStatus:(BOOL)start tag:(int)tag;

- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem;
- (ConnectionDataItem*)connectionDataItem;

- (void)setUrlString:(NSString*)urlString;
- (NSString*)urlString;
- (void)setSearchString:(NSString*)searchString;
- (NSString*)searchString;
- (void)setSearchURL:(NSString*)searchURL;
- (NSString*)searchURL;
- (void)setSearchTitle:(NSString*)searchTitle;
- (NSString*)searchTitle;
- (void)setCategoryName:(NSString*)categoryName;
- (NSString*)categoryName;
- (void)setContentsName:(NSString*)contentsName;
- (NSString*)contentsName;
- (void)setPlistId:(NSString*)plistId;
- (NSString*)plistId;
- (void)setMylistId:(NSString*)mylistId;
- (NSString*)mylistId;
- (void)setMylistTitle:(NSString*)mylistTitle;
- (NSString*)mylistTitle;
- (void)setMylistAuthor:(NSString*)mylistAuthor;
- (NSString*)mylistAuthor;
- (void)setPostUserId:(NSString*)postUserId;
- (NSString*)postUserId;
- (void)setPostUserTitle:(NSString*)postUserTitle;
- (NSString*)postUserTitle;
- (void)setChannelId:(NSString*)channelId;
- (NSString*)channelId;
- (void)setChannelTitle:(NSString*)channelTitle;
- (NSString*)channelTitle;

- (void)setLogString:(NSString *)logString;
- (NSString *)logString;
- (void)appendLogString:(NSString *)logString;

- (void)setSearchType:(int)searchType;
- (int)searchType;
- (void)setDataType:(int)dataType;
- (int)dataType;
- (void)setIsKeywordSearch:(BOOL)isKeywordSearch;
- (BOOL)isKeywordSearch;

- (void)setStartIndex:(int)startIndex;
- (int)startIndex;
- (void)setLastIndex:(int)lastIndex;
- (int)lastIndex;
- (void)setTotalResults:(int)totalResults;
- (int)totalResults;
- (void)setMaxResults:(int)maxResults;
- (int)maxResults;
- (void)setPageIndex:(int)pageIndex;
- (int)pageIndex;
- (void)setPageMaxIndex:(int)pageMaxIndex;
- (int)pageMaxIndex;

- (BOOL)canRemoveItem;

- (BOOL)isKeyMainWindow;
- (BOOL)isSearchWithPlaylist;
- (BOOL)isSearchWithPlayHistory;
- (BOOL)isMaximizedPreviewFrame;
- (BOOL)isPreviewFrame;

@end
