#import "ViewMainSearch.h"
#import "ViewRelatedWindow.h"
#import "ViewRelatedSearch.h"
#import "ViewTagSearch.h"
#import "ViewPlayer.h"
#import "ViewPreviewPlayer.h"
#import "ViewVideoContent.h"
#import "ViewPlaylistSetting.h"
#import "DownloadManager.h"
#import "LogStatusController.h"
#import "TBArrayController.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "NicoVideoExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"
#import "NotificationExtension.h"

@implementation ViewMainSearch

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// window rect
	[self setWindowRect:mainWindow key:@"rectWindowMain"];
	// split view rect
	[self setSplitViewRect:spvMain key:@"rectSplitViewMain"];
	[self setSplitViewRect:spvNavi key:@"rectSplitViewNavi"];
	[self setSplitViewRect:spvHead key:@"rectSplitViewHead"];

	// recent searches
	[self setSearchFieldRecentSearches:txtSearchField key:@"arrayRecentSearches"];

	// alloc searchList and set content
	searchList_ = [[NSMutableArray alloc] init];

	// alloc itemList and set content
	itemList_ = [[NSMutableArray alloc] init];
	[searchlistArrayController setContent:itemList_];

	[self setConnectionDataItem:nil];

	[self setUrlString:@""];
	[self setSearchTitle:@""];
	[self setSearchString:@""];
	[self setSearchURL:@""];
	[self setCategoryName:@""];
	[self setContentsName:@""];
	[self setSearchType:SEARCH_WITH_STRING];
	[self setDataType:DATA_WITH_SEARCH_RESULT];
	[self setIsKeywordSearch:NO];

	[self setStartIndex:0];
	[self setLastIndex:0];
	[self setTotalResults:0];
	[self setMaxResults:[self defaultIntValue:@"optMaxResults"]];

	[self setPageIndex:1];
	[self setPageMaxIndex:1];
	[self setPlistId:@""];
	[self setMylistId:@""];
	[self setMylistTitle:@""];
	[self setMylistAuthor:@""];
	[self setChannelId:@""];
	[self setChannelTitle:@""];
	[self setPostUserId:@""];
	[self setPostUserTitle:@""];

	[self setLogString:@""];

	[self changePageButtonEnable];
	[self changeOrderButtonEnable];
	[self changeRankingButtonState];

	int index = [self defaultIntValue:@"optTabViewSearchIndex"];

	[self changeTabViewSearchResultIndex:index];
	[self changeTabViewSearchSliderIndex:index];
	[self changeTabViewSearchButtonEnable:index];
	[self changeFrameMainChangeButton];
	[self changeTabViewPreview:PREVIEW_TAB_INDEX_IMAGE];
	[self setLoadingStatus:NO tag:0];

	[txtSearchResults setStringValue:@""];

	// set first responder
	[mainWindow makeFirstResponder:txtSearchField];

	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];
	[[btnInfo cell] setKeyEquivalent:@" "];
	[[btnInfo cell] setKeyEquivalentModifierMask:NSAlternateKeyMask];

	// delete focus ring
//	[btnTabViewSearchTable setFocusRingType:NSFocusRingTypeNone];
//	[btnTabViewSearchGrid setFocusRingType:NSFocusRingTypeNone];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(mainWindowWillClose:) name:NSWindowWillCloseNotification object:mainWindow];
	[nc addObserver:self selector:@selector(splitViewMainDidResize:) name:NSSplitViewDidResizeSubviewsNotification object:spvMain];

	// set observer
	[searchlistArrayController addObserver:self forKeyPath:@"selection" options:0 context:nil];

}
//=======================================================================
// Actions
//=======================================================================
//------------------------------------
// openMainWindow
//------------------------------------
- (IBAction)openMainWindow:(id)sender
{

	[mainWindow makeKeyAndOrderFront:self];

}
//------------------------------------
// searchWithKeyword
//------------------------------------
- (IBAction)searchWithKeyword:(id)sender
{
	NSString *keyword = [txtSearchField stringValue];

	int searchTypeWithNumber = [self defaultIntValue:@"optSearchTypeWithNumber"];

	// number only
	if([self checkIsNumberString:keyword] == YES){
		// keyword
		if(searchTypeWithNumber == 0){
			[self searchWithString:keyword pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:YES];
		}
		// mylist
		else if(searchTypeWithNumber == 1){
			[self searchWithMylist:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_MYLIST isKeywordSearch:YES];
		}
		// postuser
		else if(searchTypeWithNumber == 2){
			[self searchWithPostUser:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_POST_USER isKeywordSearch:YES];
		}
	}
	else{
		// channel
		if( [self checkIsChannelString:keyword] == YES &&
			[self defaultBoolValue:@"optSearchWithChannel"] == YES
		){
			[self searchWithChannel:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_CHANNEL isKeywordSearch:YES];
		}
		// keyword
		else{
			[self searchWithString:keyword pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:YES];
		}
	}

}
/*
//------------------------------------
// searchWithKeyword
//------------------------------------
- (IBAction)searchWithKeyword:(id)sender
{

	NSString *keyword = [txtSearchField stringValue];

	BOOL isSearched = NO;

	// mylist
	if(isSearched == NO && [self defaultBoolValue:@"optSearchWithMylist"] == YES){
		if([self checkIsMylistNoString:keyword] == YES){
			[self searchWithMylist:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_MYLIST isKeywordSearch:YES];
			isSearched = YES;
		}
	}

	// postuser
	if(isSearched == NO && [self defaultBoolValue:@"optSearchWithPostUser"] == YES){
		if([self checkIsPostUserIdString:keyword] == YES){
			[self searchWithPostUser:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_POST_USER isKeywordSearch:YES];
			isSearched = YES;
		}
	}

	// channel
	if(isSearched == NO && [self defaultBoolValue:@"optSearchWithChannel"] == YES){
		if([self checkIsChannelString:keyword] == YES){
			[self searchWithChannel:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_CHANNEL isKeywordSearch:YES];
			isSearched = YES;
		}
	}

	// keyword
	if(isSearched == NO){
		[self searchWithString:keyword pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:YES];
		isSearched = YES;
	}

}
*/
//------------------------------------
// changeSearchPage
//------------------------------------
- (IBAction)changeSearchPage:(id)sender
{
	int tag = [sender tag];

	int pageIndex = [self pageIndex];
	int pageMaxIndex = [self pageMaxIndex];
	int dataType = [self dataType];

	// prev
	if(tag == 0){
		if(pageIndex > 1){
			pageIndex--;
		}else{
			pageIndex = 1;
		}
	}
	// next
	else{
		if(pageIndex < pageMaxIndex){
			pageIndex++;
		}
	}

	// reload
	if(dataType == DATA_WITH_SEARCH_RESULT){
		[self reloadWithPageIndex:pageIndex];
	}
	else if(dataType == DATA_WITH_SEARCH_ITEMS){
		[self searchWithResultItems:searchList_ pageIndex:pageIndex searchType:[self searchType] dataType:DATA_WITH_SEARCH_RESULT];
	}


}
//------------------------------------
// moveSearchPage
//------------------------------------
- (IBAction)moveSearchPage:(id)sender
{
	int pageIndex = [[sender representedObject] intValue];
	int dataType = [self dataType];

	// reload
	if(dataType == DATA_WITH_SEARCH_RESULT){
		[self reloadWithPageIndex:pageIndex];
	}
	else if(dataType == DATA_WITH_SEARCH_ITEMS){
		[self searchWithResultItems:searchList_ pageIndex:pageIndex searchType:[self searchType] dataType:DATA_WITH_SEARCH_RESULT];
	}

}

//------------------------------------
// reloadSearchPage
//------------------------------------
- (IBAction)reloadSearchPage:(id)sender
{

	// reload
	[self reloadWithPageIndex:1];

}

//------------------------------------
// sortSearchResult
//------------------------------------
- (IBAction)sortSearchResult:(id)sender
{

	int searchType = [self searchType];

	// sort searchList
	[self sortSearchList:searchType sortString:[self defaultStringValue:@"optMylistOrderString"]];
	[self searchWithResultItems:searchList_ pageIndex:1 searchType:searchType dataType:DATA_WITH_SEARCH_RESULT];

}

//------------------------------------
// cancelSearch
//------------------------------------
- (IBAction)cancelSearch:(id)sender
{
	int tag = [sender tag];

	if(tag == 0){
		[self clearConnection];
	}else{
		[viewPreviewPlayer clearConnection];
	}

}

//------------------------------------
// changeCategoryRanking
//------------------------------------
- (IBAction)changeCategoryRanking:(id)sender
{
//	if([self defaultBoolValue:@"optRankingEnabled"] == YES){
//		[self setDefaultBoolValue:NO key:@"optRankingEnabled"];
//		[self setSearchType:SEARCH_WITH_CATEGORY];
//	}else{
		[self setDefaultBoolValue:YES key:@"optRankingEnabled"];
		[self setSearchType:SEARCH_WITH_RANKING];
//	}

	[self changeRankingButtonState];
	[self reloadSearchPage:nil];

}

//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];

	// set params
	NSDictionary *params = record;

	if([viewPlayer setPlayerView:videoId title:title params:params arrayNo:BIND_ARRAY_SEARCH] == YES){
		[viewPlayer openPlayerWindow:nil];
		int actionType = [self defaultPreviewActionType];
		if(actionType == 1){
			[viewPreviewPlayer setMoviePlay:NO];
		}
		else if(actionType == 2){
			[viewPreviewPlayer closeMovie:nil];
		}
	}

}
//------------------------------------
// previewItem
//------------------------------------
- (IBAction)previewItem:(id)sender
{
	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];

	// set params
	NSDictionary *params = record;

	// open preview
	if([viewPreviewPlayer setPreviewView:videoId title:title params:params arrayNo:BIND_ARRAY_SEARCH] == YES){
		[self changeTabViewPreview:PREVIEW_TAB_INDEX_PREVIEW];
	}

}
//------------------------------------
// downloadItem
//------------------------------------
- (IBAction)downloadItem:(id)sender
{

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return;
	}

	NSArray *array = [searchlistArrayController selectedObjects];
	// no select
	if([array count] <= 0){
		return;
	}
	// over count
	if([array count] > 1){
		[self displayMessage:@"alert"
					messageText:@"Can not start downloading over 1 files at the same time."
					infoText:@"Please download each files."
					btnList:@"Cancel"
		];
		return;
	}

	int i;
	id record;
	NSString *videoId;
	NSString *title;

	// open window
	[downloadManager openDownloadWindow:nil];

	for(i = 0; i < [array count]; i++){
		record = [array objectAtIndex:i];
		videoId = [record valueForKey:@"videoId"];
		title = [record valueForKey:@"title"];

		// start download
		[downloadManager startDownloadItem:videoId
									fileName:title
		];
	}

}
//------------------------------------
// removeItem
//------------------------------------
- (IBAction)removeItem:(id)sender
{
	// remove from playlist
	if([self searchType] == SEARCH_WITH_PLAYLIST){
		[self removeItemFromPlaylist:nil];
	}
	// remove from play history
	else if([self searchType] == SEARCH_WITH_PLAYHISTORY){
		[self removeItemFromPlayHistory:nil];
	}
}
//------------------------------------
// removeItemFromPlaylist
//------------------------------------
- (IBAction)removeItemFromPlaylist:(id)sender
{
	NSArray *items = [[searchlistArrayController selectedObjects] valueForKey:@"videoId"];

	// no select
	if([items count] <= 0){
		return;
	}

	// not result from playlist
	if([self isSearchWithPlaylist] == NO){
		return;
	}

	// get playllist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"plistId == %@", [self plistId]];
	NSArray *fetchedArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not result of playllist
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Playlist is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}

	NSString *title = [[fetchedArray objectAtIndex:0] valueForKey:@"title"];

	int result = [self displayMessage:@"alert"
							messageText:[NSString stringWithFormat:@"Delete item from \"%@\" ?", title]
							infoText:@""
							btnList:@"Delete,Cancel"
				];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove  itemlist
	[tbArrayController removeItemlistFromPlaylist:[self plistId] items:items];

	// remove searchlist
	[[searchlistArrayController content] removeObjectsInArray:[searchlistArrayController selectedObjects]];
	[searchlistArrayController removeObjects:[searchlistArrayController selectedObjects]];

	// reload searchlist
	[searchlistArrayController rearrangeObjects];

}
//------------------------------------
// removeItemFromPlayHistory
//------------------------------------
- (IBAction)removeItemFromPlayHistory:(id)sender
{
	NSArray *items = [[searchlistArrayController selectedObjects] valueForKey:@"videoId"];

	// no select
	if([items count] <= 0){
		return;
	}

	// not resunt of playllist
	if([self isSearchWithPlayHistory] == NO){
		return;
	}

	int result = [self displayMessage:@"alert"
							messageText:@"Delete item from play history?"
							infoText:@""
							btnList:@"Delete,Cancel"
				];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove  itemlist
	[tbArrayController removePlayHistory:items];

	// remove searchlist
	[[searchlistArrayController content] removeObjectsInArray:[searchlistArrayController selectedObjects]];
	[searchlistArrayController removeObjects:[searchlistArrayController selectedObjects]];

	// reload searchlist
	[searchlistArrayController rearrangeObjects];

}
//------------------------------------
// openVideoPageWithBrowser
//------------------------------------
- (IBAction)openVideoPageWithBrowser:(id)sender
{

	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];

	// open watch url
	[self openVideoWatchURL:videoId];

}
//------------------------------------
// copyItemToPasteboard
//------------------------------------
- (IBAction)copyItemToPasteboard:(id)sender
{
	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];

	NSString *string = @"";
	NSString *url = [self convertToWatchURL:[record valueForKey:@"videoId"]];
	NSString *title = [record valueForKey:@"title"];

	// url
	if([sender tag] == 0){
		string = url;
	}
	// title
	else if([sender tag] == 1){
		string = title;
	}

	[self copyStringToPasteboard:string];

}
//------------------------------------
// showItemDescription
//------------------------------------
- (IBAction)showItemDescription:(id)sender
{
	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];

	NSString *title = [record valueForKey:@"title"];
	NSString *description = [record valueForKey:@"description"];

	[logStatusController setTitle:title];
	[logStatusController setLogString:description];
	[logStatusController openLogWindow:nil];

}
//------------------------------------
// showVideoContent
//------------------------------------
- (IBAction)showVideoContent:(id)sender
{
	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];

	[viewVideoContent createVideoContentWindow:(NSDictionary*)record arrayNo:BIND_ARRAY_SEARCH refViewNo:REF_VIEW_MAIN open:YES];

}
//------------------------------------
// addItemToPlaylist
//------------------------------------
- (IBAction)addItemToPlaylist:(id)sender
{

	NSManagedObject *targetItem = [sender representedObject];

	NSArray *objects = [searchlistArrayController selectedObjects];

	// no select
	if([objects count] <= 0){
		return;
	}

	// create array for itemlist
 	NSMutableArray *items = [NSMutableArray array];
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		[items addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[item valueForKey:@"videoId"], @"videoId" ,
				[item valueForKey:@"title"], @"title" ,
				nil
			]
		];
	}

	[tbArrayController createItemlist:[targetItem valueForKey:@"plistId"] items:items];

}
//------------------------------------
// addMylist
//------------------------------------
- (IBAction)addMylist:(id)sender
{

	NSString *mylistId = [self mylistId];
	NSString *title = [self convertToMylistHeadString:[self mylistTitle] author:[self mylistAuthor]];

	if(![mylistId isEqualToString:@""]){
		[viewPlaylistSetting openMylistNewSettingWindowWithValue:mylistId title:title];
	}

}
//------------------------------------
// addPostUser
//------------------------------------
- (IBAction)addPostUser:(id)sender
{

	NSString *postUserId = [self postUserId];
	NSString *title = [self postUserTitle];

	if(![postUserId isEqualToString:@""]){
		[viewPlaylistSetting openPostUserNewSettingWindowWithValue:postUserId title:title];
	}

}
//------------------------------------
// searchRelatedVideo
//------------------------------------
- (IBAction)searchRelatedVideo:(id)sender
{

	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record objectForKey:@"videoId"];
	NSString *title = [record objectForKey:@"title"];

	if(![videoId isEqualToString:@""]){
		[viewRelatedWindow openRelatedSearch:nil];
		[viewRelatedSearch searchWithRelatedVideo:videoId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_VIDEO];
	}
}
//------------------------------------
// searchPostUserVideo
//------------------------------------
- (IBAction)searchPostUserVideo:(id)sender
{

	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record objectForKey:@"videoId"];
	NSString *postUserId = [record objectForKey:@"postUserId"];
	NSString *title = [record objectForKey:@"title"];

	if(!postUserId){postUserId =@"";}

	[viewRelatedWindow openRelatedSearch:nil];

	[viewRelatedSearch searchPostUser:videoId postUserId:postUserId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_POST_USER];

}

//------------------------------------
// searchRelatedTag
//------------------------------------
- (IBAction)searchRelatedTag:(id)sender
{

	// no select
	if([[searchlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record objectForKey:@"videoId"];
	NSString *title = [record objectForKey:@"title"];

	if(![videoId isEqualToString:@""]){
		[viewRelatedWindow openTagSearch:nil];
		[viewTagSearch searchWithRelatedTag:videoId searchTitle:title pageIndex:1 searchType:SEARCH_TAG_WITH_VIDEO];
	}

}
//------------------------------------
// changePreviewView
//------------------------------------
- (IBAction)changePreviewView:(id)sender
{
	int index = [sender tag];
	[self changeTabViewPreview:index];

}
//------------------------------------
// changePreviewImage
//------------------------------------
- (IBAction)changePreviewImage:(id)sender
{
	NSImage *image = nil;
	NSString *videoType = @"";

	if([[searchlistArrayController selectedObjects] count] > 0){
		id record = [[searchlistArrayController selectedObjects] objectAtIndex:0];
		image = [[record valueForKey:@"imageItem"] image];
		videoType = [record valueForKey:@"videoType"];
	}

	[imgPreview setImage:image];
	[imgPreview setVideoType:videoType];
}
//------------------------------------
// changeTabViewSearchResult
//------------------------------------
- (IBAction)changeTabViewSearchResult:(id)sender
{
	int index = [sender tag];
	[self changeTabViewSearchResultIndex:index];
	[self changeTabViewSearchSliderIndex:index];
	[self changeTabViewSearchButtonEnable:index];

	[self setDefaultIntValue:index key:@"optTabViewSearchIndex"];
}
//------------------------------------
// changeSplitViewMainFrame
//------------------------------------
- (IBAction)changeSplitViewMainFrame:(id)sender
{

	BOOL animate = [self defaultBoolValue:@"optFrameAnimate"];

	if(animate == YES){
		[self postFrameAnimNotification:YES];
	}

	if([spvMain isClosed] == NO){
		[spvMain closeView:animate];
	}else{
		[spvMain restoreView:animate];
	}

	if(animate == YES){
		[self postFrameAnimNotification:NO];
	}

	[self changeFrameMainChangeButton];

}
//------------------------------------
// changeSplitViewPreviewFrame
//------------------------------------
- (IBAction)changeSplitViewPreviewFrame:(id)sender
{
	if([self isMaximizedPreviewFrame] == NO){
		[self maximizePreviewFrame:sender];
	}else{
		[self restorePreviewFrame:sender];
	}

}
//------------------------------------
// maximizePreviewFrame
//------------------------------------
- (IBAction)maximizePreviewFrame:(id)sender
{

	BOOL animate = [self defaultBoolValue:@"optFrameAnimate"];

	if(animate == YES){
		[self postFrameAnimNotification:YES];
	}

	[spvMain maximizeView:animate];
	[spvNavi maximizeView:animate];

	if(animate == YES){
		[self postFrameAnimNotification:NO];
	}

	[self changeFrameMainChangeButton];

}
//------------------------------------
// restorePreviewFrame
//------------------------------------
- (IBAction)restorePreviewFrame:(id)sender
{

	BOOL animate = [self defaultBoolValue:@"optFrameAnimate"];

	if(animate == YES){
		[self postFrameAnimNotification:YES];
	}

	[spvNavi restoreView:animate];
	[spvMain restoreView:animate];

	if(animate == YES){
		[self postFrameAnimNotification:NO];
	}

	[self changeFrameMainChangeButton];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// openSearchWithMylist
//------------------------------------
- (void)openSearchWithMylist:(NSString*)mylistId
{

	[txtSearchField setStringValue:mylistId];

	BOOL isNumber = [self checkIsNumberString:mylistId];
	if(isNumber == YES){
		[self searchWithMylist:mylistId searchTitle:mylistId pageIndex:1 searchType:SEARCH_WITH_MYLIST isKeywordSearch:YES];
	}else{
		[self searchWithString:mylistId pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:YES];
	}
}
//------------------------------------
// openSearchWithPostUser
//------------------------------------
- (void)openSearchWithPostUser:(NSString*)postUserId
{

	[txtSearchField setStringValue:postUserId];

	BOOL isNumber = [self checkIsNumberString:postUserId];
	if(isNumber == YES){
		[self searchWithPostUser:postUserId searchTitle:postUserId pageIndex:1 searchType:SEARCH_RELATED_WITH_POST_USER isKeywordSearch:YES];
	}else{
		[self searchWithString:postUserId pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:YES];
	}
}
//------------------------------------
// openSearchWithChannel
//------------------------------------
- (void)openSearchWithChannel:(NSString*)channelId
{

	[txtSearchField setStringValue:channelId];

	BOOL isChannel = [self checkIsChannelString:channelId];
	if(isChannel == YES){
		[self searchWithChannel:channelId searchTitle:channelId pageIndex:1 searchType:SEARCH_WITH_CHANNEL isKeywordSearch:YES];
	}else{
		[self searchWithString:channelId pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:YES];
	}
}

//------------------------------------
// reloadWithPageIndex
//------------------------------------
- (void)reloadWithPageIndex:(int)pageIndex
{

	if([self searchType] == SEARCH_WITH_STRING){
		[self searchWithString:[self searchString] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_URL){
		[self searchWithURL:[self searchURL] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_CATEGORY){
		[self searchWithCategoryName:[self categoryName] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_RANKING){
		[self searchWithCategoryRanking:[self categoryName] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_CONTENTS){
		[self searchWithContentsName:[self contentsName] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_MYLIST){
		[self searchWithMylist:[self mylistId] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_POST_USER){
		[self searchWithPostUser:[self postUserId] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_CHANNEL){
		[self searchWithChannel:[self channelId] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType] isKeywordSearch:[self isKeywordSearch]];
	}
	else if([self searchType] == SEARCH_WITH_PLAYLIST){
		[self searchWithPlaylist:[self plistId] searchTitle:[self searchTitle] pageIndex:pageIndex isKeywordSearch:[self isKeywordSearch]];
	}

}
//------------------------------------
// searchWithString
//------------------------------------
- (void)searchWithString:(NSString*)searchString pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((searchString != nil) && ([searchString length] > 0))
	{

		// clear connection
		[self clearConnection];

		// check user session
		if([self checkUserSession] == NO){
			int result = [self displayMessageAlertNeedsLogin];
			// open login page
			if(result == NSAlertFirstButtonReturn){
//				[self openLoginURLWithBrowser];
				[self openLoginURLWithPlayer];
			}
			return;
		}

		BOOL isAdded = NO;

		// search url
		NSString *urlString = @"";
		NSString *searchOrderString = @"";
		NSString *subName = @"";
		int searchKeywordType = [self defaultIntValue:@"optSearchKeywordType"];
		int searchItemType = SEARCH_ITEM_TYPE_GET_INFO;

		BOOL withRss = NO;
		int parseRssType = PARSE_RSS_TYPE_NONE;
		int splitType = PARSE_SPLIT_NONE;
		int breakType = PARSE_BRAKE_NONE;
		BOOL isParsePageStatus = NO;

		// keyword
		if(searchKeywordType == SEARCH_WITH_STRING_KEYWORD){
			urlString = [self convertToSearchKeywordURL:searchString];
			searchOrderString = [self defaultStringValue:@"optSearchOrderString"];
			subName = SEARCH_WITH_STRING_NAME_KEYWORD;
			searchItemType = SEARCH_ITEM_TYPE_GET_INFO;
			withRss = NO;
			parseRssType = PARSE_RSS_TYPE_NONE;
			splitType = PARSE_SPLIT_PAGEBODY;
			breakType = PARSE_BRAKE_THUMB_VINFO;
			isParsePageStatus = YES;
		}
		// tag rss
		else if(searchKeywordType == SEARCH_WITH_STRING_TAG){
			urlString = [self convertToSearchTagURL:searchString];
			searchOrderString = [self defaultStringValue:@"optSearchTagOrderString"];
			subName = SEARCH_WITH_STRING_NAME_TAG;
			searchItemType = SEARCH_ITEM_TYPE_PARSE_RSS;
			withRss = YES;
			parseRssType = PARSE_RSS_TYPE_2_0;
			splitType = PARSE_SPLIT_NONE;
			breakType = PARSE_BRAKE_NONE;
			isParsePageStatus = YES;
		}
		// unknown
		else{
			urlString = [self convertToSearchKeywordURL:searchString];
		}

		// order string
		if(![searchOrderString isEqualToString:@""]){
			urlString = [self appendStringToURL:urlString string:searchOrderString isAdded:&isAdded];
		}

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];
		}

		// rss
		if(withRss == YES){
			urlString = [self convertToRssURL:urlString rssType:parseRssType];
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchString, @"searchTitle" ,
									searchString, @"searchString" ,
									[self searchURL], @"searchURL" ,
									[self contentsName], @"contentsName" ,
									[self categoryName], @"categoryName" ,
									[self mylistId], @"mylistId" ,
									[self postUserId], @"postUserId" ,
									[self channelId], @"channelId" ,
									subName, @"subName" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_RESULT], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:searchItemType], @"searchItemType" ,
									[NSNumber numberWithInt:parseRssType], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:splitType], @"splitType" ,
									[NSNumber numberWithInt:breakType], @"breakType" ,
									[NSNumber numberWithBool:isParsePageStatus], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];
	}

}
//------------------------------------
// searchWithURL
//------------------------------------
- (void)searchWithURL:(NSString*)searchURL searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((searchURL != nil) && ([searchURL length] > 0))
	{

		// clear connection
		[self clearConnection];

		// search url
		NSString *urlString = [self convertToSearchURL:searchURL];

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchTitle, @"searchTitle" ,
									[self searchString], @"searchString" ,
									searchURL, @"searchURL" ,
									[self contentsName], @"contentsName" ,
									[self categoryName], @"categoryName" ,
									[self mylistId], @"mylistId" ,
									[self postUserId], @"postUserId" ,
									[self channelId], @"channelId" ,
									@"", @"subName" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_RESULT], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:SEARCH_ITEM_TYPE_GET_INFO], @"searchItemType" ,
									[NSNumber numberWithInt:PARSE_RSS_TYPE_NONE], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:PARSE_SPLIT_NONE], @"splitType" ,
									[NSNumber numberWithInt:PARSE_BRAKE_TD], @"breakType" ,
									[NSNumber numberWithBool:NO], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithContentsName
//------------------------------------
- (void)searchWithContentsName:(NSString*)contentsName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((contentsName != nil) && ([contentsName length] > 0))
	{

		// clear connection
		[self clearConnection];

		BOOL isAdded = NO;

		// params
		int searchItemType = SEARCH_ITEM_TYPE_GET_INFO;
		int parseRssType = PARSE_RSS_TYPE_NONE;
		int splitType = PARSE_SPLIT_PAGEBODY;
		int breakType = PARSE_BRAKE_THUMB_VINFO;
		BOOL isParsePageStatus = YES;
		BOOL withRss = NO;

		// newarrival / recent RSS
		if( [contentsName isEqualToString:CONTENTS_NAME_NEWARRIVAL] ||
			[contentsName isEqualToString:CONTENTS_NAME_RECENT]){
			searchItemType = SEARCH_ITEM_TYPE_PARSE_RSS;
			parseRssType = PARSE_RSS_TYPE_ATOM;
			if([contentsName isEqualToString:CONTENTS_NAME_RECENT]){
				parseRssType = PARSE_RSS_TYPE_2_0;
			}
			breakType = PARSE_BRAKE_NONE;
			isParsePageStatus = YES;
			withRss = YES;
		}

		// hotlist
//		if([contentsName isEqualToString:CONTENTS_NAME_HOTLIST]){
//			splitType = PARSE_SPLIT_PAGEBODY;
//		}

		if(withRss == YES){
			isAdded = YES;
		}

		// search url
		NSString *urlString = [self convertToContentsURL:contentsName withRss:withRss rssType:parseRssType];

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchTitle, @"searchTitle" ,
									[self searchString], @"searchString" ,
									[self searchURL], @"searchURL" ,
									contentsName, @"contentsName" ,
									[self categoryName], @"categoryName" ,
									contentsName, @"subName" ,
									[self mylistId], @"mylistId" ,
									[self postUserId], @"postUserId" ,
									[self channelId], @"channelId" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_RESULT], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:searchItemType], @"searchItemType" ,
									[NSNumber numberWithInt:parseRssType], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:splitType], @"splitType" ,
									[NSNumber numberWithInt:breakType], @"breakType" ,
									[NSNumber numberWithBool:isParsePageStatus], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithCategoryName
//------------------------------------
- (void)searchWithCategoryName:(NSString*)categoryName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((categoryName != nil) && ([categoryName length] > 0))
	{

		// clear connection
		[self clearConnection];

		// check adult content
		BOOL adult = NO;
		if([categoryName isEqualToString:CATEGORY_NAME_R18] ||
			[categoryName isEqualToString:CATEGORY_NAME_G_R18]){
			if([self checkEnterAdultCategory] == YES){
				adult = YES;
			}else{
				return;
			}
		}

		// search url
		NSString *urlString = [self convertToCategoryURL:categoryName adult:adult];

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchTitle, @"searchTitle" ,
									[self searchString], @"searchString" ,
									[self searchURL], @"searchURL" ,
									[self contentsName], @"contentsName" ,
									categoryName, @"categoryName" ,
									categoryName, @"subName" ,
									[self mylistId], @"mylistId" ,
									[self postUserId], @"postUserId" ,
									[self channelId], @"channelId" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_RESULT], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:SEARCH_ITEM_TYPE_GET_INFO], @"searchItemType" ,
									[NSNumber numberWithInt:PARSE_RSS_TYPE_NONE], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:PARSE_SPLIT_PAGEBODY], @"splitType" ,
									[NSNumber numberWithInt:PARSE_BRAKE_THUMB_FRM], @"breakType" ,
									[NSNumber numberWithBool:NO], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithCategoryRanking
//------------------------------------
- (void)searchWithCategoryRanking:(NSString*)categoryName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((categoryName != nil) && ([categoryName length] > 0))
	{
		// clear connection
		[self clearConnection];

		// check adult content
		if([categoryName isEqualToString:CATEGORY_NAME_R18] ||
			[categoryName isEqualToString:CATEGORY_NAME_G_R18]){
			if([self checkEnterAdultCategory] == YES){
				// continue
			}else{
				return;
			}
		}

		// search url
		NSString *rankingTarget = [self defaultStringValue:@"optRankingTargetString"];
		NSString *rankingPeriod = [self defaultStringValue:@"optRankingPeriodString"];

		// not all && hourly -> daily
		if(![categoryName isEqualToString:CATEGORY_NAME_ALL] &&
			[rankingPeriod isEqualToString:TIME_PERIOD_NAME_HOURLY]){
			rankingPeriod = TIME_PERIOD_NAME_DAILY;
			// set default
			[self setDefaultStringValue:rankingPeriod key:@"optRankingPeriodString"];
			[pbtnRankingPeriod updateMenuItem:[pbtnRankingPeriod menu]];
		}

		BOOL isAdded = NO;
		BOOL withRss = YES;
		int parseRssType = PARSE_RSS_TYPE_ATOM;

		if(withRss == YES){
			isAdded = YES;
		}

		NSString *urlString = [self convertToCategoryRankingURL:categoryName target:rankingTarget period:rankingPeriod withRss:withRss rssType:parseRssType];

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchTitle, @"searchTitle" ,
									[self searchString], @"searchString" ,
									[self searchURL], @"searchURL" ,
									[self contentsName], @"contentsName" ,
									categoryName, @"categoryName" ,
									categoryName, @"subName" ,
									[self mylistId], @"mylistId" ,
									[self postUserId], @"postUserId" ,
									[self channelId], @"channelId" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_RESULT], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:SEARCH_ITEM_TYPE_PARSE_RSS], @"searchItemType" ,
									[NSNumber numberWithInt:parseRssType], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:PARSE_SPLIT_NONE], @"splitType" ,
									[NSNumber numberWithInt:PARSE_BRAKE_NONE], @"breakType" ,
									[NSNumber numberWithBool:YES], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithMylist
//------------------------------------
- (void)searchWithMylist:(NSString*)mylistId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((mylistId != nil) && ([mylistId length] > 0))
	{

		// clear connection
		[self clearConnection];

		// search url
		int parseRssType = PARSE_RSS_TYPE_ATOM;

		NSString *urlString = [self convertToMylistURL:mylistId withRss:YES rssType:parseRssType];

//		NSLog(@"mylist urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchTitle, @"searchTitle" ,
									[self searchString], @"searchString" ,
									[self searchURL], @"searchURL" ,
									[self contentsName], @"contentsName" ,
									[self categoryName], @"categoryName" ,
									mylistId, @"mylistId" ,
									[self postUserId], @"postUserId" ,
									[self channelId], @"channelId" ,
									@"", @"subName" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_ITEMS], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:SEARCH_ITEM_TYPE_PARSE_RSS], @"searchItemType" ,
									[NSNumber numberWithInt:parseRssType], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:PARSE_SPLIT_NONE], @"splitType" ,
									[NSNumber numberWithInt:PARSE_BRAKE_NONE], @"breakType" ,
									[NSNumber numberWithBool:NO], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithPostUser
//------------------------------------
- (void)searchWithPostUser:(NSString*)postUserId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((postUserId != nil) && ([postUserId length] > 0))
	{

		// clear connection
		[self clearConnection];

		// search url
		BOOL isAdded = YES;
		int parseRssType = PARSE_RSS_TYPE_ATOM;

		NSString *urlString = [self convertToPostUserURL:postUserId withRss:YES rssType:parseRssType];

		// order string
		NSString *orderString = [self defaultStringValue:@"optPostUserOrderStringMain"];

		if(![orderString isEqualToString:@""]){
			urlString = [self appendStringToURL:urlString string:orderString isAdded:&isAdded];
		}

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];
		}

//		NSLog(@"posuser urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchTitle, @"searchTitle" ,
									[self searchString], @"searchString" ,
									[self searchURL], @"searchURL" ,
									[self contentsName], @"contentsName" ,
									[self categoryName], @"categoryName" ,
									[self mylistId], @"mylistId" ,
									postUserId, @"postUserId" ,
									[self channelId], @"channelId" ,
									@"", @"subName" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_RESULT], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:SEARCH_ITEM_TYPE_PARSE_RSS], @"searchItemType" ,
									[NSNumber numberWithInt:parseRssType], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:PARSE_SPLIT_NONE], @"splitType" ,
									[NSNumber numberWithInt:PARSE_BRAKE_NONE], @"breakType" ,
									[NSNumber numberWithBool:YES], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithChannel
//------------------------------------
- (void)searchWithChannel:(NSString*)channelId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	if ((channelId != nil) && ([channelId length] > 0))
	{

		// clear connection
		[self clearConnection];

		// search url
		int parseRssType = PARSE_RSS_TYPE_2_0;

		NSString *urlString = [self convertToChannelURL:channelId withRss:YES rssType:parseRssType];

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									urlString, @"urlString" ,
									searchTitle, @"searchTitle" ,
									[self searchString], @"searchString" ,
									[self searchURL], @"searchURL" ,
									[self contentsName], @"contentsName" ,
									[self categoryName], @"categoryName" ,
									[self mylistId], @"mylistId" ,
									[self postUserId], @"postUserId" ,
									channelId, @"channelId" ,
									@"", @"subName" ,
									[NSNumber numberWithInt:DATA_WITH_SEARCH_RESULT], @"dataType" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									[NSNumber numberWithBool:isKeywordSearch], @"isKeywordSearch" ,
									[NSNumber numberWithInt:SEARCH_ITEM_TYPE_PARSE_RSS], @"searchItemType" ,
									[NSNumber numberWithInt:parseRssType], @"parseRssType" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:PARSE_SPLIT_NONE], @"splitType" ,
									[NSNumber numberWithInt:PARSE_BRAKE_NONE], @"breakType" ,
									[NSNumber numberWithBool:NO], @"isParsePageStatus" ,
									nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithPlaylist
//------------------------------------
- (void)searchWithPlaylist:(NSString*)plistId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex isKeywordSearch:(BOOL)isKeywordSearch
{
	// get item
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"plistId == %@", plistId];
	NSArray *items = [tbArrayController getObjectsWithPred:@"itemlist" pred:pred];

	// Set sort descriptors
	NSString *sortString = [self defaultStringValue:@"optPlaylistOrderString"];
	NSDictionary *params = [self getPlaylistSearchParams:sortString];
	NSString *key = [params objectForKey:@"key"];
	BOOL isAsc = [[params objectForKey:@"isAsc"] boolValue];
	BOOL isStr = [[params objectForKey:@"isStr"] boolValue];
//	NSLog(@"key=%@ isAsc=%d isStr=%d", key, isAsc, isStr);

	if([key isEqualToString:@""]){
		key = @"index";
	}

	NSSortDescriptor *sortDescriptor = nil;
	if(isStr == YES){
		sortDescriptor = [[NSSortDescriptor alloc] initWithKey:key ascending:isAsc selector:@selector(caseInsensitiveCompare:)];
	}else{
		sortDescriptor = [[NSSortDescriptor alloc] initWithKey:key ascending:isAsc];
	}
	NSArray *sortedItems = [items sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	[sortDescriptor release];

	// search
	[self setPlistId:plistId];
	[self setSearchTitle:searchTitle];
	[self setWindowTitle:searchTitle searchType:SEARCH_WITH_PLAYLIST isKeywordSearch:isKeywordSearch];
	[self searchWithPlaylistItems:sortedItems pageIndex:pageIndex searchType:SEARCH_WITH_PLAYLIST dataType:DATA_WITH_SEARCH_RESULT isKeywordSearch:isKeywordSearch];

	[self changeOrderButtonEnable];
	[self changeTabViewSearchHeadIndex];

}
//------------------------------------
// searchWithPlaylistItems
//------------------------------------
- (void)searchWithPlaylistItems:(NSArray*)items pageIndex:(int)pageIndex searchType:(int)searchType dataType:(int)dataType isKeywordSearch:(BOOL)isKeywordSearch
{

	// remove all objects
	[self removeItemList];
	[self removeSearchList];

	int maxResults = [self getMaxResults:searchType subName:@""];

	int index = (pageIndex - 1) * maxResults;

	int startIndex = index + 1;
	int lastIndex = startIndex + maxResults - 1;
	int totalResults = [items count];
	if(lastIndex > totalResults){
		lastIndex = totalResults;
	}

//	int pageMaxIndex = ceil(totalResults / maxResults);
	int pageMaxIndex = totalResults / maxResults;
	if(totalResults % maxResults != 0){
		pageMaxIndex++;
	}

	[self setSearchType:searchType];
	[self setIsKeywordSearch:isKeywordSearch];
	[self setDataType:dataType];

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:totalResults];
	[self setMaxResults:maxResults];

	[self setPageIndex:pageIndex];
	[self setPageMaxIndex:pageMaxIndex];

	[self changePageButtonEnable];

	// create itemList
	id record;
	int i;

	for (i = startIndex - 1; i < lastIndex; i++){

		if(i < 0 || i > [items count] - 1){
			continue;
		}

		index++;
		record = [items objectAtIndex:i];

		// add object
		[self addItemList:[NSDictionary dictionaryWithObjectsAndKeys:
									[NSNumber numberWithInt:index], @"index" ,
									[record valueForKey:@"videoId"], @"videoId" ,
									[record valueForKey:@"title"], @"title" ,
									@"", @"postUserId" ,
									@"", @"postedDate" ,
									@"", @"publishedDate" ,
									@"", @"imageURL" ,
									@"", @"description" ,
									@"", @"errorDescription" ,
									@"", @"lastResBody" ,
									@"", @"length" ,
									@"", @"videoType" ,
									[NSNumber numberWithInt:0], @"viewCount" ,
									[NSNumber numberWithInt:0], @"resCount" ,
									[NSNumber numberWithInt:0], @"mylistCount" ,
									[NSNumber numberWithBool:NO], @"hasCountInfo" ,
									[NSNumber numberWithBool:NO], @"isPlayed" ,
									[NSNumber numberWithBool:NO], @"hasImage" ,
									[NSNumber numberWithBool:YES], @"getVideoInfo" ,
									[NSNumber numberWithInt:VIDEO_ITEM_INIT], @"itemStatus" ,
									nil
							]
			 dataType:dataType
		];

	}

	// result string
	NSString *resultString = [self convertToResultString:startIndex
												lastIndex:lastIndex
												totalResults:totalResults
												totalString:@""
												showTotal:1
							];
	[txtSearchResults setStringValue:resultString];

	// reload
	[searchlistArrayController rearrangeObjects];

}
//------------------------------------
// searchWithResultItems
//------------------------------------
- (void)searchWithResultItems:(NSMutableArray*)items pageIndex:(int)pageIndex searchType:(int)searchType dataType:(int)dataType
{

	// remove all objects
	[self removeItemList];

	int maxResults = [self getMaxResults:searchType subName:@""];

	int index = (pageIndex - 1) * maxResults;

	int startIndex = index + 1;
	int lastIndex = startIndex + maxResults - 1;
	int totalResults = [items count];
	if(lastIndex > totalResults){
		lastIndex = totalResults;
	}

	int pageMaxIndex = totalResults / maxResults;
	if(totalResults % maxResults != 0){
		pageMaxIndex++;
	}

	[self setSearchType:searchType];

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:totalResults];
	[self setMaxResults:maxResults];

	[self setPageIndex:pageIndex];
	[self setPageMaxIndex:pageMaxIndex];

	[self changePageButtonEnable];

	// create itemList
	id record;
	int i;

	for (i = startIndex - 1; i < lastIndex; i++){

		if(i < 0 || i > [items count] - 1){
			continue;
		}

		index++;
		record = [items objectAtIndex:i];

		[record setValue:[NSNumber numberWithInt:index] forKey:@"index"];
//		[record setValue:[NSNumber numberWithBool:NO] forKey:@"hasImage"];
//		[record setValue:[NSNumber numberWithBool:YES] forKey:@"getVideoInfo"];
//		[record setValue:[NSNumber numberWithInt:VIDEO_ITEM_INIT] forKey:@"itemStatus"];

		// add object
		[self addItemList:record dataType:dataType];

	}

	// result string
	NSString *resultString = [self convertToResultString:startIndex
												lastIndex:lastIndex
												totalResults:totalResults
												totalString:@""
												showTotal:1
							];
	[txtSearchResults setStringValue:resultString];

	// reload
	[searchlistArrayController rearrangeObjects];

}
//------------------------------------
// sortSearchList
//------------------------------------
- (void)sortSearchList:(int)searchType sortString:(NSString*)sortString
{

	// get sort params
	NSDictionary *params;
	if(searchType == SEARCH_WITH_MYLIST){
		params = [self getMylistSearchParams:sortString];
	}
	else{
		return;
	}

	NSString *key = [params objectForKey:@"key"];
	BOOL isAsc = [[params objectForKey:@"isAsc"] boolValue];
	BOOL isStr = [[params objectForKey:@"isStr"] boolValue];
//	NSLog(@"key=%@ isAsc=%d isStr=%d", key, isAsc, isStr);

	// set sort descriptors
	if(![key isEqualToString:@""]){
		NSSortDescriptor *sortDescriptor = nil;
		if(isStr == YES){
			sortDescriptor = [[NSSortDescriptor alloc] initWithKey:key ascending:isAsc selector:@selector(caseInsensitiveCompare:)];
		}else{
			sortDescriptor = [[NSSortDescriptor alloc] initWithKey:key ascending:isAsc];
		}

		[searchList_ sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];

		// this is bug?
//		NSMutableArray *searchList = (NSMutableArray*)[searchList_ sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
		// replace searchList_
//		[searchList retain];
//		[searchList_ release];
//		searchList_ = searchList;

		[sortDescriptor release];
	}

}

//------------------------------------
// getMylistSearchParams
//------------------------------------
- (NSDictionary*)getMylistSearchParams:(NSString*)sortString
{
	NSString *key = @"";
	NSString *sort = @"at";
	NSString *order = @"d";

	BOOL isAsc = YES;
	BOOL isStr = NO;

	// separate sortString
	if(![sortString isEqualToString:@""]){
		NSArray *vars = [sortString componentsSeparatedByString:@"_"];
		sort = [vars objectAtIndex:0];
		if([vars count] > 1){
			order = [vars objectAtIndex:1];
		}
	}

//	NSLog(@"sort=%@ order=%@", sort, order);

	// sort
	if([sort isEqualToString:@"at"]){
		key = @"publishedDate";
		isStr = YES;
	}
	else if([sort isEqualToString:@"tt"]){
		key = @"title";
		isStr = YES;
	}
	else if([sort isEqualToString:@"pt"]){
		key = @"postedDate";
		isStr = YES;
	}
	else if([sort isEqualToString:@"vc"]){
		key = @"viewCount";
	}
	else if([sort isEqualToString:@"cc"]){
		key = @"resCount";
	}
	else if([sort isEqualToString:@"mc"]){
		key = @"mylistCount";
	}

	// order
	if([order isEqualToString:@"a"]){
		isAsc = YES;
	}
	else if([order isEqualToString:@"d"]){
		isAsc = NO;
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				key, @"key" ,
				[NSNumber numberWithBool:isAsc], @"isAsc" ,
				[NSNumber numberWithBool:isStr], @"isStr" ,
				nil
			];

}
//------------------------------------
// getPlaylistSearchParams
//------------------------------------
- (NSDictionary*)getPlaylistSearchParams:(NSString*)sortString
{
	NSString *key = @"";
	NSString *sort = @"at";
	NSString *order = @"d";

	BOOL isAsc = YES;
	BOOL isStr = NO;

	// separate sortString
	if(![sortString isEqualToString:@""]){
		NSArray *vars = [sortString componentsSeparatedByString:@"_"];
		sort = [vars objectAtIndex:0];
		if([vars count] > 1){
			order = [vars objectAtIndex:1];
		}
	}

//	NSLog(@"sort=%@ order=%@", sort, order);

	// sort
	if([sort isEqualToString:@"at"]){
		key = @"index";
	}
	else if([sort isEqualToString:@"tt"]){
		key = @"title";
		isStr = YES;
	}

	// order
	if([order isEqualToString:@"a"]){
		isAsc = YES;
	}
	else if([order isEqualToString:@"d"]){
		isAsc = NO;
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				key, @"key" ,
				[NSNumber numberWithBool:isAsc], @"isAsc" ,
				[NSNumber numberWithBool:isStr], @"isStr" ,
				nil
			];

}
//------------------------------------
// handleDataItemStatus
//------------------------------------
- (void)handleDataItemStatus:(NSNotification *)notification
{

//	NSLog(@"handleDataItemStatus");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	//  indicator
	int status = [connectionDataItem status];

	if(status == CONNECTION_INIT || status == CONNECTION_STARTED){
		[self setLoadingStatus:YES tag:0];
	}
	else{
		[self setLoadingStatus:NO tag:0];
	}
}
//------------------------------------
// handleDataItemDidLoad
//------------------------------------
- (void)handleDataItemDidLoad:(NSNotification *)notification
{

//	NSLog(@"handleDataItemDidLoad");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	NSData *htmlData = [connectionDataItem data];
	NSError *error = [connectionDataItem error];
	int result;

	// link params
	NSDictionary *params = [connectionDataItem params];
	NSString *urlString = [params objectForKey:@"urlString"];

	if(error != nil){
		result = [self displayMessageAlertGetData];
		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController openLogWindowWithError:urlString error:error];
		}
		return;
	}

	NSStringEncoding encoding = [self getStringEncoding:htmlData];
	NSString *html = [[[NSString alloc] initWithData:htmlData encoding:encoding] autorelease];
//	NSString *html = [[[NSString alloc] initWithData:htmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"html=%@", html);
	if(html == nil){
		[self displayMessage:@"alert"
				messageText:[NSString stringWithFormat:@"Can not init html from = %@", urlString]
				infoText:@"May be encoding error."
				btnList:@"Cancel"
		];
		return;
	}
	if([html length] <= 0){
		[self displayMessage:@"alert"
				messageText:@"Can not get html"
				infoText:@"May be server is busy."
				btnList:@"Cancel"
		];
		return;
	}

	// parse html
	int searchType = [[params objectForKey:@"searchType"] intValue];
	int searchItemType = [[params objectForKey:@"searchItemType"] intValue];
	int dataType = [[params objectForKey:@"dataType"] intValue];

	if(searchItemType == SEARCH_ITEM_TYPE_PARSE_RSS){
		[self parseResultRss:html params:params];
	}else{
		[self parseResultHTML:html params:params];
	}

	// searchList -> itemList
	if(dataType == DATA_WITH_SEARCH_ITEMS){
		// sort searchList
		[self sortSearchList:searchType sortString:[self defaultStringValue:@"optMylistOrderString"]];
		[self searchWithResultItems:searchList_ pageIndex:1 searchType:searchType dataType:DATA_WITH_SEARCH_RESULT];
	}
}
//------------------------------------
// parseResultHTML
//------------------------------------
- (void)parseResultHTML:(NSString*)html params:(NSDictionary*)params
{

	// params
	int searchType = [[params objectForKey:@"searchType"] intValue];
	int dataType = [[params objectForKey:@"dataType"] intValue];
	int splitType = [[params objectForKey:@"splitType"] intValue];
	int breakType = [[params objectForKey:@"breakType"] intValue];
	BOOL isParsePageStatus = [[params objectForKey:@"isParsePageStatus"] boolValue];

	// split each html
	NSDictionary *htmlObject = [self splitResultHTML:html splitType:splitType];
	NSString *headHTML = [htmlObject objectForKey:@"headHTML"];
	NSString *bodyHTML = [htmlObject objectForKey:@"bodyHTML"];

//	NSLog(@"headHTML=%@", headHTML);
//	NSLog(@"bodyHTML=%@", bodyHTML);

	// check access error
	if([self checkAccessErrorStatus:headHTML bodyHTML:bodyHTML] == YES){
		return;
	}

	// set status
	[self setSearchStatus:params];

	// remove
	[self removeItemList];
	[self removeSearchList];

	// parse html & add itemList
//	[self parseSearchHTML:bodyHTML maxResults:[self maxResults] breakType:breakType dataType:dataType];
	[self parseSearchHTML:html maxResults:[self maxResults] breakType:breakType dataType:dataType];

	// parse page & total status
	if(isParsePageStatus == YES){
//		[self parsePageStatusHTML:headHTML];
		[self parsePageStatusHTML:html];
		// string totalResults -> pageMaxIndex

		int pageMaxIndex = 0;
		int maxResults = [self maxResults];
		int totalResults = [self totalResults];

		if(searchType == SEARCH_WITH_STRING){
			if(maxResults > 0){
				pageMaxIndex = [self calcPageMaxIndex:totalResults maxResults:maxResults];
			}
		}
		[self setPageMaxIndex:pageMaxIndex];
		[self setTotalResults:totalResults];
	}

	// set status
	[self setWindowStatus:params];

	// itemList
	if(dataType == DATA_WITH_SEARCH_RESULT){
		// result string
		NSString *resultString = [self convertToResultString:[self startIndex]
													lastIndex:[self lastIndex]
													totalResults:[self totalResults]
													totalString:@""
													showTotal:1
								];
		[txtSearchResults setStringValue:resultString];

		// page button
		[self changePageButtonEnable];

		// reload
		[searchlistArrayController rearrangeObjects];

	}

}
//------------------------------------
// parseResultRss
//------------------------------------
- (void)parseResultRss:(NSString*)html params:(NSDictionary*)params
{

//	NSLog(@"html=%@", html);

	// params
	NSString *urlString = [params objectForKey:@"urlString"];
	int searchType = [[params objectForKey:@"searchType"] intValue];
	int parseRssType = [[params objectForKey:@"parseRssType"] intValue];
	int dataType = [[params objectForKey:@"dataType"] intValue];
	BOOL isParsePageStatus = [[params objectForKey:@"isParsePageStatus"] boolValue];
	int showTotal = 1;
	NSString *totalString = @"";

	// set status
	[self setSearchStatus:params];

	int maxResults = [self maxResults];

	// remove
	[self removeItemList];
	[self removeSearchList];

	// clear log
	[self setLogString:@""];

	// parse xml & add itemList
//	if(html != nil && [html length] > 0){
		if([self parseResultXML:html
					maxResults:maxResults
					searchType:searchType
					dataType:dataType
					parseRssType:parseRssType
			] == NO){

			// check not found
			if([self checkNotFoundHTML:html] == YES){
				int result = [self displayMessageAlertNotFound];
				if(result == NSAlertSecondButtonReturn){
					[self openURLWithBrowser:urlString];
				}
				return;
			}

			// check log
			if([self checkIsExistString:[self logString] searchString:@"empty"] == NO){
				int result = [self displayMessage:@"alert"
									messageText:@"Can not parse result XML."
									infoText:@""
									btnList:@"Cancel,Log"
							];
				// show error log
				if(result == NSAlertSecondButtonReturn){
					[logStatusController setTitle:@"Error Log"];
					[logStatusController setLogString:[self logString]];
					[logStatusController openLogWindow:nil];
				}
			}
			return;
		}
//	}

	// parse page & total status
	if(isParsePageStatus == YES){
		int pageIndex = 0;
		int pageMaxIndex = 0;
		int totalResults = 0;
		// ranking pageMaxIndex -> totalResults
		if(searchType == SEARCH_WITH_RANKING){
			pageMaxIndex = [self getRankingMaxPageIndex:[params objectForKey:@"categoryName"]
												rankingTarget:[self defaultStringValue:@"optRankingTargetString"]
												rankingPeriod:[self defaultStringValue:@"optRankingPeriodString"]
								];
			totalResults = maxResults * pageMaxIndex;
		}
		// contents pageMaxIndex -> totalResults
		else if(searchType == SEARCH_WITH_CONTENTS){
			pageMaxIndex = [self getContentsMaxPageIndex:[params objectForKey:@"contentsName"]];
			totalResults = maxResults * pageMaxIndex;
		}
		// string totalResults -> pageMaxIndex
		else if(searchType == SEARCH_WITH_STRING){
			totalResults = [self totalResults];
			if(maxResults > 0){
				pageMaxIndex = [self calcPageMaxIndex:totalResults maxResults:maxResults];
			}
		}
		// postuser totalResults -> pageMaxIndex
		else if(searchType == SEARCH_WITH_POST_USER){
			totalResults = [self totalResults];
			pageIndex = [self pageIndex];
			if(totalResults >= maxResults){
				pageMaxIndex = pageIndex + 1;
				showTotal = 2;
				totalString = @"...";
			}else{
				showTotal = 0;
			}
		}
		[self setPageMaxIndex:pageMaxIndex];
		[self setTotalResults:totalResults];
	}

	// set status
	[self setWindowStatus:params];

	// itemList
	if(dataType == DATA_WITH_SEARCH_RESULT){

		// result string
		NSString *resultString = [self convertToResultString:[self startIndex]
													lastIndex:[self lastIndex]
													totalResults:[self totalResults]
													totalString:totalString
													showTotal:showTotal
								];
		[txtSearchResults setStringValue:resultString];

		// page button
		[self changePageButtonEnable];

		// reload
		[searchlistArrayController rearrangeObjects];

	}

}
//------------------------------------
// parseResultXML
//------------------------------------
- (BOOL)parseResultXML:(NSString*)xml
			maxResults:(int)maxResults
			searchType:(int)searchType
			dataType:(int)dataType
			parseRssType:(int)parseRssType
{

//	NSLog(@"xml=%@", xml);

	// replace & -> &amp;
	xml = [self replaceCharacter:xml str1:@"&" str2:@"&amp;"];

	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xml options:nil error:&error] autorelease];
	if(error != nil){
		[self setLogString:@"Can not parse result XML\n"];
		[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
		[self appendLogString:[NSString stringWithFormat: @"error=[%@]\n", error]];
		return NO;
	}
	// parse xml document
	NSXMLNode *rootNode = [xmlDoc rootElement];
	if(rootNode == nil){
		[self setLogString:@"Can not parse result XML\n"];
		[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
		[self appendLogString:@"error=[rootNode is null]\n"];
		return NO;
	}
	// get parentNode
	NSXMLNode *parentNode = rootNode;
	if(parseRssType == PARSE_RSS_TYPE_2_0){
		parentNode = [self getChildNode:rootNode name:@"channel"];
		if(parentNode == nil){
			[self setLogString:@"Can not parse result XML\n"];
			[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
			[self appendLogString:@"error=[parentNode is null]\n"];
			return NO;
		}
	}


	// node name
	NSString *childNodeName = @"entry";
	NSString *linkNodeName = @"link";
	NSString *publishedNodeName = @"published";
	NSString *contentNodeName = @"content";
	if(parseRssType == PARSE_RSS_TYPE_2_0){
		childNodeName = @"item";
		linkNodeName = @"link";
		publishedNodeName = @"pubDate";
		contentNodeName = @"description";
	}

	NSString *videoId = @"";
	NSString *imageURL = @"";
	NSString *length = @"-";
	NSString *viewCountStr = @"0";
	NSString *resCountStr = @"0";
	NSString *mylistCountStr = @"0";
	NSString *postedDate = @"";
	NSString *publishedDate = @"";
	NSString *title = @"";
	NSString *description = @"";
	NSString *videoType = @"";
	NSString *resultTitle = @"";
	NSString *resultAuthor = @"";
	NSString *resultDescription = @"";

	NSString *keyImage = @"class=\"nico-thumbnail\"";
	NSString *keyInfo = @"class=\"nico-info\"";
	NSString *keyLength = @"class=\"nico-info-length\">";
	NSString *keyDate = @"class=\"nico-info-date\">";
	NSString *keyViewCount = @"class=\"nico-info-total-view\">";
	NSString *keyResCount = @"class=\"nico-info-total-res\">";
	NSString *keyMylistCount = @"class=\"nico-info-total-mylist\">";
	NSString *keyDescription = @"class=\"nico-description\"";

	int index = ([self pageIndex] - 1) * maxResults;
	int startIndex = index + 1;
	int lastIndex = startIndex;
	int totalResults = 0;
	int itemCount = 0;

	int viewCount;
	int resCount;
	int mylistCount;
	BOOL hasCountInfo;
	BOOL getVideoInfo;

	NSXMLNode *childNode;
	NSXMLElement *elemLink;
	NSXMLElement *elemContent;
	NSString *videoURL;
	NSString *content;
	NSRange rng;

	NSArray *lines;
	NSString *line;
	int i;

	NSEnumerator *e = [[parentNode children] objectEnumerator];
	while (childNode = [e nextObject]){

//		NSLog(@"name=%@", [childNode name]);

		// mylist
		if(searchType == SEARCH_WITH_MYLIST){
			// resultTitle
			if([[childNode name] isEqualToString:@"title"]){
				resultTitle = [self getEnclosedString:[childNode stringValue]
											beginString:[self convertToShiftJISString:"}CXg "]
											endString:[self convertToShiftJISString:"]jRjR"]
											withEndSet:NO
							];
			}
			// resultAuthor
			if([[childNode name] isEqualToString:@"author"]){
				resultAuthor = [[self getChildNode:childNode name:@"name"] stringValue];
			}
		}
		// user
		else if(searchType == SEARCH_WITH_POST_USER){
			// resultTitle
			if([[childNode name] isEqualToString:@"title"]){
				resultTitle = [self replaceCharacter:[childNode stringValue] str1:[self convertToShiftJISString:"]jRjR"] str2:@""];
			}
		}
		// channel
		else if(searchType == SEARCH_WITH_CHANNEL){
			// channelTitle
			if([[childNode name] isEqualToString:@"title"]){
				resultTitle = [self getEnclosedString:[childNode stringValue]
											beginString:@""
											endString:[self convertToShiftJISString:"]jRjR"]
											withEndSet:NO
							];
			}
		}
		// string total results
		else if(searchType == SEARCH_WITH_STRING){
			// channelDescription get total results
			if([[childNode name] isEqualToString:@"description"]){
				resultDescription = [childNode stringValue];
//				NSLog(@"resultDescription=%@", resultDescription);
				if(resultDescription && ![resultDescription isEqualToString:@""]){
					NSArray *cols = [resultDescription componentsSeparatedByString:@"("];
					if([cols count] > 1){
						NSString *totalResultsStr = [cols objectAtIndex:[cols count] - 1];
						totalResultsStr = [self replaceCharacter:totalResultsStr str1:[self convertToShiftJISString:"S"] str2:@""];
						totalResultsStr = [self replaceCharacter:totalResultsStr str1:[self convertToShiftJISString:""] str2:@""];
						totalResultsStr = [self replaceCharacter:totalResultsStr str1:@")" str2:@""];
						totalResultsStr = [self replaceCharacter:totalResultsStr str1:@" " str2:@""];
						totalResultsStr = [self replaceCharacter:totalResultsStr str1:@"," str2:@""];
//						NSLog(@"totalResultsStr=%@", totalResultsStr);
						totalResults = [self convertStringToIntValue:totalResultsStr];
					}
				}
			}
		}

		// entry
		if([[childNode name] isEqualToString:childNodeName]){

			// link
			elemLink = (NSXMLElement*)[self getChildNode:childNode name:linkNodeName];
			if(elemLink == nil){
				continue;
			}
			// videoURL
			if(parseRssType == PARSE_RSS_TYPE_ATOM){
				videoURL = [[elemLink attributeForName:@"href"] stringValue];
			}
			else if(parseRssType == PARSE_RSS_TYPE_2_0){
				videoURL = [elemLink stringValue];
			}
			else{
				continue;
			}
//			NSLog(@"videoURL=%@", videoURL);
			if(videoURL == nil || [videoURL length] <= 0){
				continue;
			}
			// videoId
			videoId = [self getVideoIdFromURL:videoURL setDefault:NO];
			if(videoId == nil || [videoId length] <= 0){
				continue;
			}
			// title / include rank num at ranking
//			title = [[self getChildNode:childNode name:@"title"] stringValue];

			// publishedDate
			publishedDate = [[self getChildNode:childNode name:publishedNodeName] stringValue];

			// content
			hasCountInfo = NO;
			elemContent = (NSXMLElement*)[self getChildNode:childNode name:contentNodeName];
			if(elemContent != nil){

				content = [elemContent stringValue];
//				NSLog(@"content=%@", content);

				// line breaked array
				lines = [content componentsSeparatedByString:@"\n"];

				for(i = 0; i < [lines count]; i++){

					line = [lines objectAtIndex:i];
//					NSLog(@"line=%@", line);

					// title & imageURL
					rng = [line rangeOfString:keyImage];
					if(rng.length > 0){
						title = [self getEnclosedString:line
													beginString:@"alt=\""
													endString:@"\""
													withEndSet:NO
									];
						imageURL = [self getEnclosedString:line
													beginString:@"src=\""
													endString:@"\""
													withEndSet:NO
									];
//						continue;
					}

					// description
					rng = [line rangeOfString:keyDescription];
					if(rng.length > 0){
						description = [self getEnclosedString:line
													beginString:@">"
													endString:@"</p>"
													withEndSet:NO
									];
//						continue;
					}

					// keyInfo
					rng = [line rangeOfString:keyInfo];
					if(rng.length > 0){
						length = [self getEnclosedString:line
													beginString:keyLength
													endString:@"<"
													withEndSet:NO
								];
						postedDate = [self getEnclosedString:line
													beginString:keyDate
													endString:@"<"
													withEndSet:NO
								];

						viewCountStr = [self getEnclosedString:line
													beginString:keyViewCount
													endString:@"<"
													withEndSet:NO
								];
						resCountStr = [self getEnclosedString:line
													beginString:keyResCount
													endString:@"<"
													withEndSet:NO
								];
						mylistCountStr = [self getEnclosedString:line
													beginString:keyMylistCount
													endString:@"<"
													withEndSet:NO
								];
//						continue;
					}

				}

			}

			index++;
			itemCount++;

			if( ![viewCountStr isEqualToString:@""] &&
				![resCountStr isEqualToString:@""] &&
				![mylistCountStr isEqualToString:@""]
			){
				hasCountInfo = YES;
			}
			viewCount = [self convertStringToIntValue:viewCountStr];
			resCount = [self convertStringToIntValue:resCountStr];
			mylistCount = [self convertStringToIntValue:mylistCountStr];

			// replace character
			title = [self replaceWebCharacters:title];
			description = [self replaceWebCharacters:description];
			if(parseRssType == PARSE_RSS_TYPE_ATOM){
				publishedDate = [self convertFormatedDateString:publishedDate];
			}else if(parseRssType == PARSE_RSS_TYPE_2_0){
				publishedDate = [self convertCalenderDateString:publishedDate];
			}

			postedDate = [self replaceDateCharacter:postedDate];

			// reget video info
			if( searchType == SEARCH_WITH_MYLIST ||
				searchType == SEARCH_WITH_POST_USER ||
				searchType == SEARCH_WITH_CHANNEL ||
				searchType == SEARCH_WITH_CONTENTS ||
				searchType == SEARCH_WITH_STRING
			){
				getVideoInfo = YES;
			}else{
				getVideoInfo = NO;
//				getVideoInfo = YES;
			}

/*
			NSLog(@"videoId=%@", videoId);
			NSLog(@"imageURL=%@", imageURL);
			NSLog(@"title=%@", title);
			NSLog(@"postedDate=%@", postedDate);
			NSLog(@"publishedDate=%@", publishedDate);
			NSLog(@"description=%@", description);
			NSLog(@"length=%@", length);
			NSLog(@"viewCount=%d resCount=%d mylistCount=%d", viewCount, resCount, mylistCount);
*/
			// add object
			[self addItemList:[NSDictionary dictionaryWithObjectsAndKeys:
									[NSNumber numberWithInt:index], @"index" ,
									videoId, @"videoId" ,
									imageURL, @"imageURL" ,
									title, @"title" ,
									@"", @"postUserId" ,
									postedDate, @"postedDate" ,
									publishedDate, @"publishedDate" ,
									description, @"description" ,
									@"", @"errorDescription" ,
									@"", @"lastResBody" ,
									length, @"length" ,
									videoType, @"videoType" ,
									[NSNumber numberWithInt:viewCount], @"viewCount" ,
									[NSNumber numberWithInt:resCount], @"resCount" ,
									[NSNumber numberWithInt:mylistCount], @"mylistCount" ,
									[NSNumber numberWithBool:hasCountInfo], @"hasCountInfo" ,
									[NSNumber numberWithBool:NO], @"isPlayed" ,
									[NSNumber numberWithBool:YES], @"hasImage" ,
									[NSNumber numberWithBool:getVideoInfo], @"getVideoInfo" ,
									[NSNumber numberWithInt:VIDEO_ITEM_SUCCESS], @"itemStatus" ,
									nil
							]
				dataType:dataType
			];

			// init column
			videoId = @"";
			imageURL = @"";
			length = @"-";
			viewCountStr = @"0";
			resCountStr = @"0";
			mylistCountStr = @"0";
			postedDate = @"";
			publishedDate = @"";
			title = @"";
			description = @"";
			videoType = @"";

			// break
			if(maxResults > 1 && itemCount >= maxResults){
				break;
			}

		}
	}

	// set index
	lastIndex = index;
	if(totalResults <= 0){
		totalResults = itemCount;
	}

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:totalResults];

//	NSLog(@"resultTitle=%@", resultTitle);
	// mylist
	if(searchType == SEARCH_WITH_MYLIST){
		[self setMylistTitle:resultTitle];
		[self setMylistAuthor:resultAuthor];
	}
	// user
	if(searchType == SEARCH_WITH_POST_USER){
		[self setPostUserTitle:resultTitle];
	}
	else if(searchType == SEARCH_WITH_CHANNEL){
		[self setChannelTitle:resultTitle];
	}

	return YES;
}
//------------------------------------
// parseSearchHTML
//------------------------------------
- (void)parseSearchHTML:(NSString*)html maxResults:(int)maxResults breakType:(int)breakType dataType:(int)dataType
{

	NSString *keyVideoId = @"href=\"watch/";
	NSString *keyImage = @"smilevideo.jp/smile?i";
//	NSString *keyTitle = @"class=\"vinfo_title";
	NSString *keyTitle = @"class=\"watch";
	NSString *videoId = @"";
	NSString *videoIdOld = @"";
	NSString *imageURL = @"";
	NSString *title = @"";

	BOOL getImageURL = NO;
	BOOL getTitle = NO;

	int index = ([self pageIndex] - 1) * maxResults;

	int startIndex = index + 1;
	int lastIndex = startIndex;
	int itemCount = 0;

	NSArray *lines;
	NSString *line;
	NSRange rng1;
	int i;

	// remove unused tags
	html = [self removeTagCharacters:html];
	// break lines "><" and "\n"
	lines = [self createLineBreakArray:html];

	//
	// parse item list
	//
	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];

		// imageURL
		rng1 = [line rangeOfString:keyImage];
		if(rng1.length > 0 && getImageURL == NO){
			imageURL = [self getEnclosedString:line
										beginString:@"src=\""
										endString:@"\""
										withEndSet:NO
						];
			getImageURL = YES;
//			continue;
		}
		// title
		rng1 = [line rangeOfString:keyTitle];
		if(rng1.length > 0 && getTitle == NO){
			title = [self getEnclosedString:line
										beginString:@"title=\""
										endString:@"\""
										withEndSet:NO
						];
			// for category
			if(!title || [title isEqualToString:@""]){
				title = [self getEnclosedString:line
											beginString:@">"
											endString:@"<"
											withEndSet:NO
							];
			}
			getTitle = YES;
//			continue;
		}

		// videoId
		rng1 = [line rangeOfString:keyVideoId];
		if(rng1.length > 0){

			// videoId
			videoId = [self getEnclosedString:line
										beginString:keyVideoId
										endString:@"\"?&"
										withEndSet:YES
					];

			// not videoId
			if(!videoId || [self checkIsAlphabetOrNumberString:videoId] == NO){
				continue;
			}

			// set first videoId
			if([videoIdOld isEqualToString:@""]){
				videoIdOld = videoId;
			}

			// change videoId
			if(![videoId isEqualToString:videoIdOld]){


				index++;
				itemCount++;

//				NSLog(@"%d videoIdOld=%@ %@", itemCount, videoIdOld, imageURL);

				// replace character
				title = [self replaceWebCharacters:title];

				// add object
				[self addItemList:
					[self createSearchItem:index
									videoId:videoIdOld
									imageURL:imageURL
									title:title
					]
					dataType:dataType
				];

				// set old
				videoIdOld = videoId;

				// clear
				videoId = @"";
				imageURL = @"";
				title = @"";

				getImageURL = NO;
				getTitle = NO;

				// break
				if(maxResults > 1 && itemCount >= maxResults){
					break;
				}

				continue;
			}
		}

	}

//	NSLog(@"%d videoIdOld=%@ %@", itemCount, videoIdOld, imageURL);

	// add last item
	if(maxResults <= 0 || itemCount < maxResults){
		if(![videoIdOld isEqualToString:@""]){

			index++;
			itemCount++;

			// replace character
			title = [self replaceWebCharacters:title];

			// add object
			[self addItemList:
				[self createSearchItem:index
								videoId:videoIdOld
								imageURL:imageURL
								title:title
				]
				dataType:dataType
			];
		}
	}

	// set index
	lastIndex = index;

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:itemCount];

}
//------------------------------------
// parsePageStatusHTML
//------------------------------------
- (void)parsePageStatusHTML:(NSString*)html
{
	NSString *keyResults = @"class=\"search_total\">";
	NSString *keyPageStart = @"class=\"pager\"";
	NSString *keyPageEnd = @"</p>";
	NSString *keyPageCurrent = @"<span>";
	NSString *keyPageLink = @"<a href=\"";

	NSString *totalResultsStr = @"";
	NSString *pageIndexStr = @"";

	int totalResults = [self totalResults];
//	int maxResults = [self maxResults];

	int pageIndexCurrent = 1;
	int pageIndexLink = 1;
	int pageMaxIndex = 1;

	BOOL getPageLink;
	BOOL getPageEnd;
	BOOL getResults;

	NSMutableString *line;
	NSString *lowLine;
	NSRange rng;
	int i;

	//
	// parse status
	//
	html = [self replaceCharacter:html str1:@"><" str2:@">\n<"];

	NSArray *lines = [html componentsSeparatedByString:@"\n"];

	getResults = NO;
	getPageLink = NO;
	getPageEnd = NO;

	for(i = 0; i < [lines count]; i++){

		line = [NSMutableString stringWithString:[lines objectAtIndex:i]];

//		NSLog(@"line=%@", line);

		lowLine = [line lowercaseString];

		// results
		rng = [lowLine rangeOfString:[keyResults lowercaseString]];
		if(rng.length > 0 && getResults == NO){
			totalResultsStr = [self getEnclosedString:line
										beginString:keyResults
										endString:@"<"
										withEndSet:NO
							];
			getResults = YES;
			continue;
		}

		// pageStart
		rng = [lowLine rangeOfString:[keyPageStart lowercaseString]];
		if(rng.length > 0){
			getPageLink = YES;
			continue;
		}
		// pageEnd
		if(getPageLink == YES){
			rng = [lowLine rangeOfString:[keyPageEnd lowercaseString]];
			if(rng.length > 0){
				getPageEnd = YES;
				continue;
			}
		}
		// pageIndex
		if(getPageLink == YES && getPageEnd == NO){
			// keyPageCurrent
			rng = [lowLine rangeOfString:[keyPageCurrent lowercaseString]];
			if(rng.length > 0){
				pageIndexStr = [self getEnclosedString:line
											beginString:@">"
											endString:@"<"
											withEndSet:NO
							];
				if(pageIndexStr && [self checkIsNumberString:pageIndexStr] == YES){
					pageIndexCurrent = [pageIndexStr intValue];
					if(pageIndexCurrent > pageMaxIndex){
						pageMaxIndex = pageIndexCurrent;
					}
				}
				continue;
			}
			// keyPageLink
			rng = [lowLine rangeOfString:[keyPageLink lowercaseString]];
			if(rng.length > 0){
				pageIndexStr = [self getEnclosedString:line
											beginString:@">"
											endString:@"<"
											withEndSet:NO
							];
				if(pageIndexStr && [self checkIsNumberString:pageIndexStr] == YES){
					pageIndexLink = [pageIndexStr intValue];
					if(pageIndexLink > pageMaxIndex){
						pageMaxIndex = pageIndexLink;
					}
				}
				continue;
			}

		}

	}

	// set total results
	if(![totalResultsStr isEqualToString:@""]){
		// replace
		totalResultsStr = [self replaceCharacter:totalResultsStr str1:[self convertToShiftJISString:""] str2:@""];
		totalResults = [self convertStringToIntValue:totalResultsStr];
		[self setTotalResults:totalResults];
	}

}
//------------------------------------
// splitResultHTML
//------------------------------------
- (NSDictionary*)splitResultHTML:(NSString*)html splitType:(int)splitType
{

	NSString *headHTML = @"";
	NSString *bodyHTML = @"";
	NSString *splitString = [self getParseSplitString:splitType];

	// no split
	if([splitString isEqualToString:@""]){
		headHTML = html;
		bodyHTML = html;
	}
	// split
	else{
		NSArray *lines = [html componentsSeparatedByString:splitString];
		headHTML = [lines objectAtIndex:0];
		if([lines count] > 1){
			bodyHTML = [lines objectAtIndex:1];
		}
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				headHTML, @"headHTML" ,
				bodyHTML, @"bodyHTML" ,
				nil
			];

}
//------------------------------------
// changeTabViewSearchHeadIndex
//------------------------------------
- (void)changeTabViewSearchHeadIndex
{
	int index = -1;
	int searchType = [self searchType];

	if(searchType == SEARCH_WITH_STRING){
		index = 0;
	}
	else if(searchType == SEARCH_WITH_CATEGORY){
		index = 1;
	}
	else if(searchType == SEARCH_WITH_RANKING){
		index = 1;
	}
	else if(searchType == SEARCH_WITH_CONTENTS){
		// none
	}
	else if(searchType == SEARCH_WITH_MYLIST){
		index = 2;
	}
	else if(searchType == SEARCH_WITH_POST_USER){
		index = 3;
	}
	else if(searchType == SEARCH_WITH_PLAYLIST){
		index = 4;
	}

	if(index >= 0){
		[tabViewSearchHead selectTabViewItemAtIndex:index];
	}

}
//------------------------------------
// changePageButtonEnable
//------------------------------------
- (void)changePageButtonEnable
{

	int pageIndex = [self pageIndex];
	int pageMaxIndex = [self pageMaxIndex];

	// page button
/*
	if( [self searchType] == SEARCH_WITH_PLAYLIST ||
		[self searchType] == SEARCH_WITH_PLAYHISTORY ||
		[self searchType] == SEARCH_WITH_ITEMS
		){
		[btnPagePrev setEnabled:NO];
		[btnPageNext setEnabled:NO];
	}
	else{
*/		// prev
		if(pageIndex > 1){
			[btnPagePrev setEnabled:YES];
		}else{
			[btnPagePrev setEnabled:NO];
		}
		// next
		if(pageIndex < pageMaxIndex){
			[btnPageNext setEnabled:YES];
		}else{
			[btnPageNext setEnabled:NO];
		}
//	}

}
//------------------------------------
// changeOrderButtonEnable
//------------------------------------
- (void)changeOrderButtonEnable
{

	int searchType = [self searchType];
	int totalResults = [self totalResults];

	if(searchType == SEARCH_WITH_STRING){
		[pbtnSearchOrder setEnabled:YES];
		[pbtnSearchKeywordType setEnabled:YES];
		[pbtnRankingTarget setEnabled:NO];
		[pbtnRankingPeriod setEnabled:NO];
		[btnRankingEnable setEnabled:NO];
		[pbtnMylistOrder setEnabled:NO];
		[btnAddMylist setEnabled:NO];
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
		[pbtnPlaylistOrder setEnabled:NO];
	}
	else if(searchType == SEARCH_WITH_MYLIST){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchKeywordType setEnabled:NO];
		[pbtnRankingTarget setEnabled:NO];
		[pbtnRankingPeriod setEnabled:NO];
		[btnRankingEnable setEnabled:NO];
		[pbtnMylistOrder setEnabled:YES];
		[btnAddMylist setEnabled:YES];
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
		[pbtnPlaylistOrder setEnabled:NO];
	}
	else if(searchType == SEARCH_WITH_POST_USER){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchKeywordType setEnabled:NO];
		[pbtnRankingTarget setEnabled:NO];
		[pbtnRankingPeriod setEnabled:NO];
		[btnRankingEnable setEnabled:NO];
		[pbtnMylistOrder setEnabled:NO];
		[btnAddMylist setEnabled:NO];
		if(totalResults > 0){
			[pbtnPostUserOrder setEnabled:YES];
			[btnAddPostUser setEnabled:YES];
		}else{
			[pbtnPostUserOrder setEnabled:NO];
			[btnAddPostUser setEnabled:NO];
		}
		[pbtnPlaylistOrder setEnabled:NO];
	}
	else if(searchType == SEARCH_WITH_CHANNEL){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchKeywordType setEnabled:NO];
		[pbtnRankingTarget setEnabled:NO];
		[pbtnRankingPeriod setEnabled:NO];
		[btnRankingEnable setEnabled:NO];
		[pbtnMylistOrder setEnabled:NO];
		[btnAddMylist setEnabled:NO];
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
		[pbtnPlaylistOrder setEnabled:NO];
	}
	else if(searchType == SEARCH_WITH_CATEGORY){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchKeywordType setEnabled:NO];
		[pbtnRankingTarget setEnabled:NO];
		[pbtnRankingPeriod setEnabled:NO];
//		[btnRankingEnable setEnabled:YES];
//		if([[self categoryName] isEqualToString:CATEGORY_NAME_ALL]){
			[btnRankingEnable setEnabled:NO];
//		}else{
//			[btnRankingEnable setEnabled:YES];
//		}
		[pbtnMylistOrder setEnabled:NO];
		[btnAddMylist setEnabled:NO];
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
		[pbtnPlaylistOrder setEnabled:NO];
	}
	else if(searchType == SEARCH_WITH_RANKING){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchKeywordType setEnabled:NO];
		[pbtnRankingTarget setEnabled:YES];
		[pbtnRankingPeriod setEnabled:YES];
//		[btnRankingEnable setEnabled:YES];
//		if([[self categoryName] isEqualToString:CATEGORY_NAME_ALL]){
			[btnRankingEnable setEnabled:NO];
//		}else{
//			[btnRankingEnable setEnabled:YES];
//		}
		[pbtnMylistOrder setEnabled:NO];
		[btnAddMylist setEnabled:NO];
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
		[pbtnPlaylistOrder setEnabled:NO];
	}
	else if(searchType == SEARCH_WITH_CONTENTS){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchKeywordType setEnabled:NO];
		[pbtnRankingTarget setEnabled:NO];
		[pbtnRankingPeriod setEnabled:NO];
		[btnRankingEnable setEnabled:NO];
		[pbtnMylistOrder setEnabled:NO];
		[btnAddMylist setEnabled:NO];
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
		[pbtnPlaylistOrder setEnabled:NO];
	}
	else if(searchType == SEARCH_WITH_PLAYLIST){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchKeywordType setEnabled:NO];
		[pbtnRankingTarget setEnabled:NO];
		[pbtnRankingPeriod setEnabled:NO];
		[btnRankingEnable setEnabled:NO];
		[pbtnMylistOrder setEnabled:NO];
		[btnAddMylist setEnabled:NO];
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
		[pbtnPlaylistOrder setEnabled:YES];
	}

}
//------------------------------------
// changeRankingButtonState
//------------------------------------
- (void)changeRankingButtonState
{
//	if([self defaultBoolValue:@"optRankingEnabled"] == YES){
		[btnRankingEnable setImage:[NSImage imageNamed:@"btn_ranking_on"]];
//	}else{
//		[btnRankingEnable setImage:[NSImage imageNamed:@"btn_ranking_off"]];
//	}
}
//------------------------------------
// changeTabViewSearchResultIndex
//------------------------------------
- (void)changeTabViewSearchResultIndex:(int)index
{
	[tabViewSearchResult selectTabViewItemAtIndex:index];
}
//------------------------------------
// changeTabViewSearchSliderIndex
//------------------------------------
- (void)changeTabViewSearchSliderIndex:(int)index
{
	[tabViewSearchSlider selectTabViewItemAtIndex:index];
}
//------------------------------------
// changeTabViewSearchButtonEnable
//------------------------------------
- (void)changeTabViewSearchButtonEnable:(int)index
{

	NSString *btnTableName = @"btn_search_table_off";
	NSString *btnGrigName = @"btn_search_grid_off";
	BOOL tableEnabled = NO;
	BOOL gridEnabled = NO;

	if(index == 0){
		gridEnabled = YES;
		btnTableName = @"btn_search_table_on";
	}
	else if(index == 1){
		tableEnabled = YES;
		btnGrigName = @"btn_search_grid_on";
	}

	[btnTabViewSearchTable setImage:[NSImage imageNamed:btnTableName]];
	[btnTabViewSearchGrid setImage:[NSImage imageNamed:btnGrigName]];

//	[btnTabViewSearchTable setEnabled:tableEnabled];
//	[btnTabViewSearchGrid setEnabled:gridEnabled];

}
//------------------------------------
// changeTabViewPreview
//------------------------------------
- (void)changeTabViewPreview:(int)index
{
	[tabViewPreview selectTabViewItemAtIndex:index];

	BOOL enableImage = NO;
	BOOL enableMovie = NO;

	if(index == PREVIEW_TAB_INDEX_IMAGE){
		enableMovie = YES;
	}else{
		enableImage = YES;
	}
	[btnPreviewImage setEnabled:enableImage];
	[btnPreviewMovie setEnabled:enableMovie];
	[self postPreviewIndexDidChangedNotification:index];

}
//------------------------------------
// changeFrameMainChangeButton
//------------------------------------
- (void)changeFrameMainChangeButton
{
	NSString *btnName = @"btn_frame_horizontal_close";

	// close
	if([spvMain isClosed] == NO){
		btnName = @"btn_frame_horizontal_close";
	}
	// open
	else{
		btnName = @"btn_frame_horizontal_open";
	}

	[btnFrameMainChange setImage:[NSImage imageNamed:btnName]];
}

//------------------------------------
// splitViewMainDidResize
//------------------------------------
-(void)splitViewMainDidResize:(NSNotification *)notification
{
	[self changeFrameMainChangeButton];
}
//------------------------------------
//------------------------------------
// createSearchItem
//------------------------------------
- (NSDictionary*)createSearchItem:(int)index
						videoId:(NSString*)videoId
						imageURL:(NSString*)imageURL
						title:(NSString*)title
{
	BOOL hasImage = NO;
	if(![imageURL isEqualToString:@""]){
		hasImage = YES;
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
					[NSNumber numberWithInt:index], @"index" ,
					videoId, @"videoId" ,
					imageURL, @"imageURL" ,
					title, @"title" ,
					@"", @"postUserId" ,
					@"", @"postedDate" ,
					@"", @"publishedDate" ,
					@"", @"description" ,
					@"", @"errorDescription" ,
					@"", @"lastResBody" ,
					@"", @"length" ,
					@"", @"videoType" ,
					[NSNumber numberWithInt:0], @"viewCount" ,
					[NSNumber numberWithInt:0], @"resCount" ,
					[NSNumber numberWithInt:0], @"mylistCount" ,
					[NSNumber numberWithBool:NO], @"hasCountInfo" ,
					[NSNumber numberWithBool:NO], @"isPlayed" ,
					[NSNumber numberWithBool:hasImage], @"hasImage" ,
					[NSNumber numberWithBool:YES], @"getVideoInfo" ,
					[NSNumber numberWithInt:VIDEO_ITEM_INIT], @"itemStatus" ,
					nil
	];
}
//------------------------------------
// addItemList
//------------------------------------
- (void)addItemList:(NSDictionary*)object dataType:(int)dataType
{

//	NSLog(@"object=%@", [object description]);
	NSMutableDictionary *record = [[[NSMutableDictionary alloc] init] autorelease];

	BOOL hasImage = [[object objectForKey:@"hasImage"] boolValue];
	BOOL getVideoInfo = [[object objectForKey:@"getVideoInfo"] boolValue];
	NSString *imageURL = [object objectForKey:@"imageURL"];

	[record setValue:[object objectForKey:@"index"] forKey:@"index"];
	[record setValue:[object objectForKey:@"videoId"] forKey:@"videoId"];
	[record setValue:[object objectForKey:@"title"] forKey:@"title"];
	[record setValue:[object objectForKey:@"imageURL"] forKey:@"imageURL"];

	[record setValue:[object objectForKey:@"postUserId"] forKey:@"postUserId"];
	[record setValue:[object objectForKey:@"postedDate"] forKey:@"postedDate"];
	[record setValue:[object objectForKey:@"publishedDate"] forKey:@"publishedDate"];
	[record setValue:[object objectForKey:@"description"] forKey:@"description"];
	[record setValue:[object objectForKey:@"errorDescription"] forKey:@"errorDescription"];
	[record setValue:[object objectForKey:@"lastResBody"] forKey:@"lastResBody"];

	[record setValue:[object objectForKey:@"length"] forKey:@"length"];
	[record setValue:[object objectForKey:@"videoType"] forKey:@"videoType"];
	[record setValue:[object objectForKey:@"viewCount"] forKey:@"viewCount"];
	[record setValue:[object objectForKey:@"resCount"] forKey:@"resCount"];
	[record setValue:[object objectForKey:@"mylistCount"] forKey:@"mylistCount"];
	[record setValue:[object objectForKey:@"hasCountInfo"] forKey:@"hasCountInfo"];
	[record setValue:[object objectForKey:@"itemStatus"] forKey:@"itemStatus"];

	[record setValue:[object objectForKey:@"hasImage"] forKey:@"hasImage"];
	[record setValue:[object objectForKey:@"getVideoInfo"] forKey:@"getVideoInfo"];
	[record setValue:[NSNumber numberWithBool:NO] forKey:@"isPlayed"];

	// itemList_
	if(dataType == DATA_WITH_SEARCH_RESULT){
		// get video item
		if(getVideoInfo == YES){
			[record setValue:[[[VideoInfoItem alloc] initWithVideo:
										[object objectForKey:@"videoId"]
										title:[object objectForKey:@"title"]
										target:self
										notifType:VIDEO_ITEM_NOTIF_LIST
										hasImage:hasImage
										params:nil
							] autorelease]
					forKey:@"videoInfoItem"
			];
		}

		// get image
		if( hasImage == YES &&
			(imageURL && ![imageURL isEqualToString:@""])
		){
			[record setValue:[[[ImageItem alloc] initWithImageURL:imageURL notifType:IMAGE_ITEM_NOTIF_SEARCH] autorelease] forKey:@"imageItem"];
		}

		[itemList_ addObject:record];
	}
	// searchList
	else if(dataType == DATA_WITH_SEARCH_ITEMS){
		[searchList_ addObject:record];
	}

}

//------------------------------------
// removeItemList
//------------------------------------
- (void)removeItemList
{

	int i;
	// cancel all connection
	for(i = 0; i < [itemList_ count]; i++){
		id record = [itemList_ objectAtIndex:i];
		ImageItem *imageItem = [record objectForKey:@"imageItem"];
		if(imageItem){
			[imageItem cancelConnection];
		}
		BOOL getVideoInfo = [[record valueForKey:@"getVideoInfo"] boolValue];
		VideoInfoItem *videoInfoItem = [record valueForKey:@"videoInfoItem"];
		if(getVideoInfo == YES && videoInfoItem){
			[videoInfoItem cancelConnection];
		}
	}

	// remove all
	[itemList_ removeAllObjects];

	// reload
	[searchlistArrayController rearrangeObjects];

}
//------------------------------------
// removeSearchList
//------------------------------------
- (void)removeSearchList
{
	// remove all
	if([searchList_ count] > 0){
		[searchList_ removeAllObjects];
	}
}
//------------------------------------
// handleItemObjectChange
//------------------------------------
- (void)handleItemObjectChange:(NSNotification *)notification
{

//	NSLog(@"handleItemObjectChange");
	VideoInfoItem *videoInfoItem = [notification object];

	if(videoInfoItem == nil){
		return;
	}

	if(![videoInfoItem videoId]){
//		NSLog(@"videoId is invalid");
		return;
	}

	NSString *videoId = [videoInfoItem videoId];
	NSString *title = [videoInfoItem title];
	NSString *imageURL = [videoInfoItem imageURL];
	NSString *postUserId = [videoInfoItem postUserId];
	NSString *postedDate = [videoInfoItem postedDate];
	NSString *publishedDate = [videoInfoItem publishedDate];
	NSString *description = [videoInfoItem description];
	NSString *errorDescription = [videoInfoItem errorDescription];
	NSString *length = [videoInfoItem length];
	NSString *lastResBody = [videoInfoItem lastResBody];
	NSArray *tagList = [videoInfoItem tagList];
	BOOL hasImage = [videoInfoItem hasImage];
	int viewCount = [videoInfoItem viewCount];
	int resCount = [videoInfoItem resCount];
	int mylistCount = [videoInfoItem mylistCount];
	int itemStatus = [videoInfoItem itemStatus];

//	NSLog(@"videoId=%@", videoId);

	// fetch itemList_ with itemid
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"videoId == %@", videoId];
	NSArray *fetchedArray = [itemList_ filteredArrayUsingPredicate:pred];
//	NSLog(@"pred=%@", [pred description]);

	// if exists, set image item
	if([fetchedArray count] > 0){

		id record = [fetchedArray objectAtIndex:0];

		if(itemStatus == VIDEO_ITEM_SUCCESS){
			[record setValue:title forKey:@"title"];
			[record setValue:postUserId forKey:@"postUserId"];
			[record setValue:postedDate forKey:@"postedDate"];
			[record setValue:publishedDate forKey:@"publishedDate"];
			[record setValue:description forKey:@"description"];
			[record setValue:length forKey:@"length"];
			[record setValue:lastResBody forKey:@"lastResBody"];
			[record setValue:tagList forKey:@"tagList"];
			[record setValue:[NSNumber numberWithInt:viewCount] forKey:@"viewCount"];
			[record setValue:[NSNumber numberWithInt:resCount] forKey:@"resCount"];
			[record setValue:[NSNumber numberWithInt:mylistCount] forKey:@"mylistCount"];
			[record setValue:[NSNumber numberWithBool:YES] forKey:@"hasCountInfo"];

			// set image object
			if( hasImage == NO &&
				imageURL && ![imageURL isEqualToString:@""]
			){
				[record setValue:[[[ImageItem alloc] initWithImageURL:imageURL notifType:IMAGE_ITEM_NOTIF_SEARCH] autorelease] forKey:@"imageItem"];
			}
		}

		[record setValue:[NSNumber numberWithInt:itemStatus] forKey:@"itemStatus"];
		[record setValue:errorDescription forKey:@"errorDescription"];

	}

	// reload
	[searchlistArrayController rearrangeObjects];

}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// change review image
	if([keyPath isEqualToString:@"selection"]){
		[self changePreviewImage:nil];
	}
}
//------------------------------------
// mainWindowWillClose
//------------------------------------
-(void)mainWindowWillClose:(NSNotification *)notification
{

	// save state from user default
	// window rect
	[self saveWindowRect:mainWindow key:@"rectWindowMain"];

	// split view rect
	[self saveSplitViewRect:spvMain key:@"rectSplitViewMain"];
	[self saveSplitViewRect:spvNavi key:@"rectSplitViewNavi"];
	[self saveSplitViewRect:spvHead key:@"rectSplitViewHead"];

	// recent searches
	[self saveSearchFieldRecentSearches:txtSearchField key:@"arrayRecentSearches"];

}
//------------------------------------
// setSearchStatus
//------------------------------------
- (void)setSearchStatus:(NSDictionary*)params
{

	// params
	NSString *searchTitle = [params objectForKey:@"searchTitle"];
	NSString *subName = [params objectForKey:@"subName"];
	int searchType = [[params objectForKey:@"searchType"] intValue];
	int dataType = [[params objectForKey:@"dataType"] intValue];
	int maxResults = [self getMaxResults:searchType subName:subName];

	// change all gets
	if(dataType == DATA_WITH_SEARCH_ITEMS){
		maxResults = 1;
	}

	// set status
	[self setUrlString:[params objectForKey:@"urlString"]];
	[self setSearchTitle:searchTitle];
	[self setSearchString:[params objectForKey:@"searchString"]];
	[self setSearchURL:[params objectForKey:@"searchURL"]];
	[self setCategoryName:[params objectForKey:@"categoryName"]];
	[self setContentsName:[params objectForKey:@"contentsName"]];
	[self setMylistId:[params objectForKey:@"mylistId"]];
	[self setPostUserId:[params objectForKey:@"postUserId"]];
	[self setChannelId:[params objectForKey:@"channelId"]];
	[self setSearchType:[[params objectForKey:@"searchType"] intValue]];
	[self setIsKeywordSearch:[[params objectForKey:@"isKeywordSearch"] boolValue]];
	[self setDataType:[[params objectForKey:@"dataType"] intValue]];
	[self setPageIndex:[[params objectForKey:@"pageIndex"] intValue]];
	[self setPageMaxIndex:[[params objectForKey:@"pageIndex"] intValue]];
	[self setMaxResults:maxResults];

}
//------------------------------------
// setWindowStatus
//------------------------------------
- (void)setWindowStatus:(NSDictionary*)params
{
	NSString *searchTitle = [params objectForKey:@"searchTitle"];
	int searchType = [[params objectForKey:@"searchType"] intValue];
	BOOL isKeywordSearch = [[params objectForKey:@"isKeywordSearch"] boolValue];

	// window title
	[self setWindowTitle:searchTitle searchType:searchType isKeywordSearch:isKeywordSearch];

	[self changeOrderButtonEnable];
	[self changeTabViewSearchHeadIndex];

}
//------------------------------------
// setWindowTitle
//------------------------------------
- (void)setWindowTitle:(NSString*)searchTitle searchType:(int)searchType isKeywordSearch:(BOOL)isKeywordSearch
{

	NSString *string = @"";
	NSString *appTitle = @"MacNicoTunes";

	if([searchTitle isEqualToString:@""]){
		string = appTitle;
	}else{
//		string = [NSString stringWithFormat:@"%@ ( %@ )", appTitle, searchTitle];
		string = searchTitle;
		// ranking
		if(searchType == SEARCH_WITH_RANKING){
			string = [string stringByAppendingString:[self convertToShiftJISString:" - LO"]];
		}
		// category
		else if(searchType == SEARCH_WITH_CATEGORY){
			string = [string stringByAppendingString:[self convertToShiftJISString:" - VRg/Ve"]];
		}
		// string
		else if(searchType == SEARCH_WITH_STRING){

			int searchKeywordType = [self defaultIntValue:@"optSearchKeywordType"];

			// keyword
			if(searchKeywordType == SEARCH_WITH_STRING_KEYWORD){
				string = [string stringByAppendingString:[self convertToShiftJISString:" - L[[h"]];
			}
			// tag
			else if(searchKeywordType == SEARCH_WITH_STRING_TAG){
				string = [string stringByAppendingString:[self convertToShiftJISString:" - ^O"]];
			}
		}
		// mylist
		else if(searchType == SEARCH_WITH_MYLIST){

			NSString *headTitle = @"";

			if(isKeywordSearch == YES){
				headTitle = [self convertToMylistHeadString:[self mylistTitle] author:[self mylistAuthor]];
			}else{
				headTitle = searchTitle;
			}

			if(![headTitle isEqualToString:@""]){
				string = [NSString stringWithFormat:@"%@ / %@", headTitle, [self mylistId]];
			}
			string = [string stringByAppendingString:[self convertToShiftJISString:" - }CXg"]];
		}
		// user
		else if(searchType == SEARCH_WITH_POST_USER){

			NSString *headTitle = @"";

			if(isKeywordSearch == YES){
				headTitle = [self postUserTitle];
			}else{
				headTitle = searchTitle;
			}

			if(![headTitle isEqualToString:@""]){
				string = [NSString stringWithFormat:@"%@ / %@", headTitle, [self postUserId]];
			}
//			string = [string stringByAppendingString:[self convertToShiftJISString:" - e"]];
		}
		// channel
		else if(searchType == SEARCH_WITH_CHANNEL){

			NSString *headTitle = @"";

			if(isKeywordSearch == YES){
				headTitle = [self channelTitle];
			}else{
				headTitle = searchTitle;
			}

			if(![headTitle isEqualToString:@""]){
				string = [NSString stringWithFormat:@"%@ / %@", headTitle, [self channelId]];
			}
			string = [string stringByAppendingString:[self convertToShiftJISString:" - `l"]];
		}

	}

	[mainWindow setTitle:string];

}
//------------------------------------
// setLoadingStatus
//------------------------------------
- (void)setLoadingStatus:(BOOL)start tag:(int)tag
{
	if(start == YES){
		[indProc startAnimation:nil];
		[btnSearchCancel setEnabled:YES];
		[btnSearchCancel setTag:tag];
	}
	else{
		[indProc stopAnimation:nil];
		[btnSearchCancel setEnabled:NO];
		[btnSearchCancel setTag:tag];
	}
}
//------------------------------------
// connectionDataItem
//------------------------------------
- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem
{
	[connectionDataItem retain];
	[connectionDataItem_ release];
	connectionDataItem_ = connectionDataItem;
}
- (ConnectionDataItem*)connectionDataItem
{
	return connectionDataItem_;
}
//------------------------------------
// urlString
//------------------------------------
- (void)setUrlString:(NSString*)urlString
{
	[urlString retain];
	[urlString_ release];
	urlString_ = urlString;
}
- (NSString*)urlString
{
	return urlString_;
}
//------------------------------------
// searchString
//------------------------------------
- (void)setSearchString:(NSString*)searchString
{
	[searchString retain];
	[searchString_ release];
	searchString_ = searchString;
}
- (NSString*)searchString
{
	return searchString_;
}
//------------------------------------
// searchURL
//------------------------------------
- (void)setSearchURL:(NSString*)searchURL
{
	[searchURL retain];
	[searchURL_ release];
	searchURL_ = searchURL;
}
- (NSString*)searchURL
{
	return searchURL_;
}
//------------------------------------
// searchTitle
//------------------------------------
- (void)setSearchTitle:(NSString*)searchTitle
{
	[searchTitle retain];
	[searchTitle_ release];
	searchTitle_ = searchTitle;
}
- (NSString*)searchTitle
{
	return searchTitle_;
}
//------------------------------------
// categoryName
//------------------------------------
- (void)setCategoryName:(NSString*)categoryName
{
	[categoryName retain];
	[categoryName_ release];
	categoryName_ = categoryName;
}
- (NSString*)categoryName
{
	return categoryName_;
}
//------------------------------------
// contentsName
//------------------------------------
- (void)setContentsName:(NSString*)contentsName
{
	[contentsName retain];
	[contentsName_ release];
	contentsName_ = contentsName;
}
- (NSString*)contentsName
{
	return contentsName_;
}
//------------------------------------
// plistId
//------------------------------------
- (void)setPlistId:(NSString*)plistId
{
	[plistId retain];
	[plistId_ release];
	plistId_ = plistId;
}
- (NSString*)plistId
{
	return plistId_;
}
//------------------------------------
// mylistId
//------------------------------------
- (void)setMylistId:(NSString*)mylistId
{
	[mylistId retain];
	[mylistId_ release];
	mylistId_ = mylistId;
}
- (NSString*)mylistId
{
	return mylistId_;
}
//------------------------------------
// mylistTitle
//------------------------------------
- (void)setMylistTitle:(NSString*)mylistTitle
{
	[mylistTitle retain];
	[mylistTitle_ release];
	mylistTitle_ = mylistTitle;
}
- (NSString*)mylistTitle
{
	return mylistTitle_;
}
//------------------------------------
// mylistAuthor
//------------------------------------
- (void)setMylistAuthor:(NSString*)mylistAuthor
{
	[mylistAuthor retain];
	[mylistAuthor_ release];
	mylistAuthor_ = mylistAuthor;
}
- (NSString*)mylistAuthor
{
	return mylistAuthor_;
}
//------------------------------------
// postUserId
//------------------------------------
- (void)setPostUserId:(NSString*)postUserId
{
	[postUserId retain];
	[postUserId_ release];
	postUserId_ = postUserId;
}
- (NSString*)postUserId
{
	return postUserId_;
}
//------------------------------------
// postUserTitle
//------------------------------------
- (void)setPostUserTitle:(NSString*)postUserTitle
{
	[postUserTitle retain];
	[postUserTitle_ release];
	postUserTitle_ = postUserTitle;
}
- (NSString*)postUserTitle
{
	return postUserTitle_;
}

//------------------------------------
// channelId
//------------------------------------
- (void)setChannelId:(NSString*)channelId
{
	[channelId retain];
	[channelId_ release];
	channelId_ = channelId;
}
- (NSString*)channelId
{
	return channelId_;
}
//------------------------------------
// channelTitle
//------------------------------------
- (void)setChannelTitle:(NSString*)channelTitle
{
	[channelTitle retain];
	[channelTitle_ release];
	channelTitle_ = channelTitle;
}
- (NSString*)channelTitle
{
	return channelTitle_;
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}

//------------------------------------
// searchType
//------------------------------------
- (void)setSearchType:(int)searchType
{
	searchType_ = searchType;
}
- (int)searchType
{
	return searchType_;
}
//------------------------------------
// dataType
//------------------------------------
- (void)setDataType:(int)dataType
{
	dataType_ = dataType;
}
- (int)dataType
{
	return dataType_;
}
//------------------------------------
// isKeywordSearch
//------------------------------------
- (void)setIsKeywordSearch:(BOOL)isKeywordSearch
{
	isKeywordSearch_ = isKeywordSearch;
}
- (BOOL)isKeywordSearch
{
	return isKeywordSearch_;
}

//------------------------------------
// startIndex
//------------------------------------
- (void)setStartIndex:(int)startIndex
{
	startIndex_ = startIndex;
}
- (int)startIndex
{
	return startIndex_;
}
//------------------------------------
// lastIndex
//------------------------------------
- (void)setLastIndex:(int)lastIndex
{
	lastIndex_ = lastIndex;
}
- (int)lastIndex
{
	return lastIndex_;
}
//------------------------------------
// totalResults
//------------------------------------
- (void)setTotalResults:(int)totalResults
{
	totalResults_ = totalResults;
}
- (int)totalResults
{
	return totalResults_;
}
//------------------------------------
// maxResults
//------------------------------------
- (void)setMaxResults:(int)maxResults
{
	maxResults_ = maxResults;
}
- (int)maxResults
{
	return maxResults_;
}
//------------------------------------
// pageIndex
//------------------------------------
- (void)setPageIndex:(int)pageIndex
{
	pageIndex_ = pageIndex;
}
- (int)pageIndex
{
	return pageIndex_;
}
//------------------------------------
// pageMaxIndex
//------------------------------------
- (void)setPageMaxIndex:(int)pageMaxIndex
{
	pageMaxIndex_ = pageMaxIndex;
}
- (int)pageMaxIndex
{
	return pageMaxIndex_;
}
//------------------------------------
// clearConnection
//------------------------------------
- (void)clearConnection
{

	ConnectionDataItem *connectionDataItem = [self connectionDataItem];
	// cancel connection
	if(connectionDataItem != nil){
		if([connectionDataItem connection] != nil){
//			NSLog(@"cancelConnection");
			[connectionDataItem cancelConnection];
		}
		[self setConnectionDataItem:nil];
	}

}
//------------------------------------
// canRemoveItem
//------------------------------------
- (BOOL)canRemoveItem
{
	if([self isKeyMainWindow] == YES &&
		([self isSearchWithPlaylist] == YES ||
		 [self isSearchWithPlayHistory] == YES)){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// isKeyMainWindow
//------------------------------------
- (BOOL)isKeyMainWindow
{
	return [mainWindow isKeyWindow];
}
//------------------------------------
// isSearchWithPlaylist
//------------------------------------
- (BOOL)isSearchWithPlaylist
{
	if( [self searchType] == SEARCH_WITH_PLAYLIST &&
		![[self plistId] isEqualToString:@""]){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// isSearchWithPlayHistory
//------------------------------------
- (BOOL)isSearchWithPlayHistory
{
	if([self searchType] == SEARCH_WITH_PLAYHISTORY){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// isMaximizedPreviewFrame
//------------------------------------
- (BOOL)isMaximizedPreviewFrame
{
	BOOL ret = NO;
	if([spvMain isMaximized] == YES && [spvNavi isMaximized] == YES){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// isPreviewFrame
//------------------------------------
- (BOOL)isPreviewFrame
{
	BOOL ret = NO;
	NSString *identifier = [[tabViewPreview selectedTabViewItem] identifier];
	
	if([identifier isEqualToString:PREVIEW_TAB_IDENTIFIER_PREVIEW]){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[searchlistArrayController removeObserver:self forKeyPath:@"selection"];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[urlString_ release];
	[searchString_ release];
	[searchURL_ release];
	[searchTitle_ release];
	[categoryName_ release];
	[contentsName_ release];
	[plistId_ release];
	[mylistId_ release];
	[mylistTitle_ release];
	[mylistAuthor_ release];
	[postUserId_ release];
	[postUserTitle_ release];
	[channelId_ release];
	[channelTitle_ release];
	[logString_ release];

	[itemList_ release];
	[searchList_ release];
	[connectionDataItem_ release];
	[super dealloc];
}

@end

