#import "ViewMylistSetting.h"
#import "ViewPlaylistSetting.h"
#import "ViewMainSearch.h"
#import "ViewPlaylist.h"
#import "TBArrayController.h"
#import "UserDefaultsExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"

@implementation ViewMylistSetting

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setPlistId:@""];
	[self setIsNewList:NO];
	[txtMylistTitle setStringValue:@""];
	[txtMylistId setStringValue:@""];
	[self changeTakeButtonEnable:NO];
}
//------------------------------------
// updateMylistSetting
//------------------------------------
- (IBAction)updateMylistSetting:(id)sender
{

	NSString *title = [txtMylistTitle stringValue];
	NSString *mylistId = [txtMylistId stringValue];

	if([title isEqualToString:@""]){
		[self displayMessage:@"alert"
							messageText:@"Title is not entered."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}
	if([mylistId isEqualToString:@""]){
		[self displayMessage:@"alert"
							messageText:@"Mylist ID is not entered."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}

	if([self checkIsNumberString:mylistId] == NO){
		[self displayMessage:@"alert"
							messageText:@"Mylist ID is not a number."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}

	// new
	if([self isNewList] == YES){
		int itemType = ITEM_MYLIST;
		int itemSubType = ITEM_SEARCH_STRING;
		int itemSubTypeAttribute = 0;

		// add item
		[viewPlaylist addItem:itemType
					itemSubType:itemSubType
					itemSubTypeAttribute:itemSubTypeAttribute
					title:title
					keyword:mylistId
					isFolder:NO
					isSelect:YES
					isEdit:NO
		];

		[viewPlaylistSetting closeWindow:nil];
	}
	// edit
	else{

		NSString *plistId = [self plistId];

		if([plistId isEqualToString:@""]){
			[self displayMessage:@"alert"
								messageText:@"Mylist is not found"
								infoText:@""
								btnList:@"Cancel"
			];
			return;
		}

		// update playlist
		if([self updatePlaylist:plistId title:title keyword:mylistId] == YES){
			[viewPlaylistSetting closeWindow:nil];
		}
	}

}
//------------------------------------
// takeMylistValuesFromMainWindow:
//------------------------------------
- (IBAction)takeMylistValuesFromMainWindow:(id)sender
{

	NSString *title = [self convertToMylistHeadString:[viewMainSearch mylistTitle] author:[viewMainSearch mylistAuthor]];
	NSString *mylistId = [viewMainSearch mylistId];

	[txtMylistTitle setStringValue:title];
	[txtMylistId setStringValue:mylistId];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createMylistSettingWindow
//------------------------------------
- (BOOL)createMylistSettingWindow:(BOOL)isNewList
{

	NSString *plistId = @"";
	NSString *title = @"";
	NSString *mylistId = @"";

	[self setIsNewList:isNewList];

	// new
	if(isNewList == YES){
		// none
	}
	// edit
	else{
		NSArray *fetchedArray = [tbArrayController getSelectedObjects:@"playlist"];

		// not result
		if([fetchedArray count] <= 0){
			[self displayMessage:@"alert"
								messageText:@"Mylist is not found"
								infoText:@""
								btnList:@"Cancel"
			];
			return NO;
		}

		// get value
		id record = [fetchedArray objectAtIndex:0];
		plistId = [record valueForKey:@"plistId"];
		title = [record valueForKey:@"title"];
		mylistId = [record valueForKey:@"keyword"];

		int itemType = [[record valueForKey:@"itemType"] intValue];

		// not mylist
		if(itemType != ITEM_MYLIST){
			[self displayMessage:@"alert"
								messageText:@"Mylist is not selected"
								infoText:@""
								btnList:@"Cancel"
			];
			return NO;
		}
	}

	[self setPlistId:plistId];

	[txtMylistTitle setStringValue:title];
	[txtMylistId setStringValue:mylistId];
	[self changeTakeButtonEnable:[self canTakeValue]];

	return YES;

}
//------------------------------------
// createMylistSettingWindowWithValue
//------------------------------------
- (BOOL)createMylistSettingWindowWithValue:(NSString*)mylistId title:(NSString*)title
{

	if([self createMylistSettingWindow:YES] == YES){
		[self changeTakeButtonEnable:NO];
		[self setSettingValue:mylistId title:title];
	}else{
		return NO;
	}

	return YES;

}
//------------------------------------
// setSettingValue
//------------------------------------
- (void)setSettingValue:(NSString*)mylistId title:(NSString*)title
{
	[txtMylistId setStringValue:mylistId];
	[txtMylistTitle setStringValue:title];
}
//------------------------------------
// updatePlaylist
//------------------------------------
- (BOOL)updatePlaylist:(NSString*)plistId title:(NSString*)title keyword:(NSString*)keyword
{

	// get playllist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"plistId == %@", plistId];
	NSArray *fetchedArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not result
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Mylist is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// get value
	id record = [fetchedArray objectAtIndex:0];

	int itemType = [[record valueForKey:@"itemType"] intValue];

	// not keyword
	if(itemType != ITEM_MYLIST){
		[self displayMessage:@"alert"
							messageText:@"Mylist is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// update value
	[record setValue:title forKey:@"title"];
	[record setValue:keyword forKey:@"keyword"];

	return YES;

}
//------------------------------------
// changeTakeButtonEnable
//------------------------------------
- (void)changeTakeButtonEnable:(BOOL)enable
{
	[btnTakeValue setEnabled:enable];
}
//------------------------------------
// canTakeValue
//------------------------------------
- (BOOL)canTakeValue
{
	BOOL ret = NO;
	if(![[viewMainSearch mylistId] isEqualToString:@""]){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// setPlistId
//------------------------------------
- (void)setPlistId:(NSString*)plistId
{
	[plistId retain];
	[plistId_ release];
	plistId_ = plistId;
}
//------------------------------------
// plistId
//------------------------------------
- (NSString*)plistId
{
	return plistId_;
}
//------------------------------------
// isNewList
//------------------------------------
- (void)setIsNewList:(BOOL)isNewList
{
	isNewList_ = isNewList;
}
- (BOOL)isNewList;
{
	return isNewList_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[plistId_ release];
	[super dealloc];
}
@end
