#import "ViewPlayHistory.h"
#import "ViewPlayer.h"
#import "TBArrayController.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewPlayHistory

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];

	[self setShowIsNotPlayed:NO];
	[self setIsPlayedButtonImage];

}

//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openPlayHistory
//------------------------------------
- (IBAction)openPlayHistory:(id)sender
{
	//	none
}
//------------------------------------
// showIsNotPlayedItem
//------------------------------------
- (IBAction)showIsNotPlayedItem:(id)sender
{
	BOOL showIsNotPlayed = ![self showIsNotPlayed];
	NSPredicate *pred = nil;

	if(showIsNotPlayed == YES){
		pred = [NSPredicate predicateWithFormat:@"isPlayed == NO"];
	}

	[playhistoryArrayController setFilterPredicate:pred];

	[self setShowIsNotPlayed:showIsNotPlayed];
	[self setIsPlayedButtonImage];
}
//------------------------------------
// changeItemStatusIsPlayed
//------------------------------------
- (IBAction)changeItemStatusIsPlayed:(id)sender
{
	int tag = [sender tag];

	NSArray *items = [playhistoryArrayController selectedObjects];
	// no select
	if([items count] <= 0){
		return;
	}

	BOOL isPlayed;
	if(tag == 0){
		isPlayed = NO;
	}else{
		isPlayed = YES;
	}

	// update objects
	NSEnumerator *enumItems = [items objectEnumerator];
	id record;
	// update objects
	while (record = [enumItems nextObject]) {
		[record setValue:[NSNumber numberWithBool:isPlayed] forKey:@"isPlayed"];
	}

	[playhistoryArrayController rearrangeObjects];

}
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	// no select
	if([[playhistoryArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[playhistoryArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"]; 

	[viewPlayer setPlayerViewWithVideoId:videoId arrayNo:BIND_ARRAY_NONE];
	[viewPlayer openPlayerWindow:nil];

}
//------------------------------------
// addItemToPlaylist
//------------------------------------
- (IBAction)addItemToPlaylist:(id)sender
{

	NSManagedObject *targetItem = [sender representedObject];

	NSArray *objects = [playhistoryArrayController selectedObjects];
 
	// no select
	if([objects count] <= 0){
		return;
	}

	// create array for itemlist
 	NSMutableArray *items = [NSMutableArray array];
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		[items addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[item valueForKey:@"videoId"], @"videoId" ,
				[item valueForKey:@"title"], @"title" ,
				nil
			]
		];
	}

	[tbArrayController createItemlist:[targetItem valueForKey:@"plistId"] items:items];

}
//------------------------------------
// removeItem
//------------------------------------
- (IBAction)removeItem:(id)sender
{
	NSArray *items = [[playhistoryArrayController selectedObjects] valueForKey:@"videoId"];

	// no select
	if([items count] <= 0){
		return;
	}

	int result = [self displayMessage:@"alert"
							messageText:@"Delete selected items from history?"
							infoText:@""
							btnList:@"Delete,Cancel"
				];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove  itemlist
	[tbArrayController removePlayHistory:items];

}
//------------------------------------
// removeAllItems
//------------------------------------
- (IBAction)removeAllItems:(id)sender
{

	NSArray *items = [[playhistoryArrayController arrangedObjects] valueForKey:@"videoId"];
	// no select
	if([items count] <= 0){
		return;
	}

	int result = [self displayMessage:@"alert"
							messageText:@"Clear all displayed items from history?"
							infoText:@""
							btnList:@"Delete,Cancel"
				];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove  itemlist
	[tbArrayController removePlayHistory:items];
//	[tbArrayController removeAllObjects:@"playhistory"];
}
//------------------------------------
// setIsPlayedButtonImage
//------------------------------------
- (void)setIsPlayedButtonImage
{

	BOOL showIsNotPlayed = [self showIsNotPlayed];
	NSString *imageName = @"btn_video_blue_off";

	if(showIsNotPlayed == YES){
		imageName =  @"btn_video_blue_on";
	}else{
		imageName =  @"btn_video_blue_off";
	}

	[btnIsPlayed setImage:[NSImage imageNamed:imageName]];
}

//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose
{
	// none
}
//------------------------------------
// showIsNotPlayed
//------------------------------------
- (void)setShowIsNotPlayed:(BOOL)showIsNotPlayed
{
    showIsNotPlayed_ = showIsNotPlayed;
}
- (BOOL)showIsNotPlayed
{
    return showIsNotPlayed_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
    [super dealloc];
}

@end
