#import "ViewPlayer.h"
#import "ViewRelatedWindow.h"
#import "ViewRelatedSearch.h"
#import "ViewTagSearch.h"
#import "ViewVideoContent.h"
#import "DownloadManager.h"
#import "TBArrayController.h"
#import "WebPlayerView.h"
#import "ConvertExtension.h"
#import "HelperExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewPlayer

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rect
	[self setWindowRect:playerWindow key:@"rectWindowPlayer"];

	[self setParams:nil];
	[self setArrayNo:BIND_ARRAY_NONE];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClosePlayer:) name:NSWindowWillCloseNotification object:playerWindow];
	[nc addObserver:self selector:@selector(handleLoginPageOpen:) name:APP_NOTIF_LOGIN_PAGE_OPEN object:nil];


}
//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openPlayerWindow
//------------------------------------
- (IBAction)openPlayerWindow:(id)sender
{
	[playerWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// orderFrontPlayerWindow
//------------------------------------
- (IBAction)orderFrontPlayerWindow:(id)sender
{
	[playerWindow orderFront:self];
}
//------------------------------------
// changeWindowScale
//------------------------------------
- (IBAction)changeWindowScale:(id)sender
{
	int scaleType = [sender tag];

	float scale = 1.0;

	if(scaleType == 0){
		scale = 0.5;
	}
	else if(scaleType == 1){
		scale = 1.0;
	}
	else if(scaleType == 2){
		scale = 2.0;
	}
	else if(scaleType == 3){
		scale = 3.0;
	}

	[self changePlayerWindowWithScale:scale];

}
//------------------------------------
// openEnterURLWindow
//------------------------------------
- (IBAction)openEnterURLWindow:(id)sender
{
	[playerWindow makeKeyAndOrderFront:self];
	[self openModalSheet:enterURLWindow parentWindow:playerWindow];
}

//------------------------------------
// closeEnterURLWindow
//------------------------------------
- (IBAction)closeEnterURLWindow:(id)sender
{

	[[NSApplication sharedApplication] endSheet:enterURLWindow];

}
//------------------------------------
// openPlayerViewWithVideoId
//------------------------------------
- (IBAction)openPlayerViewWithVideoId:(id)sender
{

	[self closeEnterURLWindow:nil];

	NSString *url = [txtURL stringValue];
	url = [url stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];

	// null error
	if(!url || [url isEqualToString:@""]){
		return;
	}

	// get videoId (watch/smXXXXXX -> smXXXXXX)
	NSString *videoId = [self getVideoIdFromURL:url setDefault:YES];
	if([videoId isEqualToString:@""]){
		return;
	}

	[self setPlayerViewWithVideoId:videoId arrayNo:BIND_ARRAY_NONE];

}
//------------------------------------
// openNicoVideoBaseURL
//------------------------------------
- (IBAction)openNicoVideoBaseURL:(id)sender
{
	NSString *urlString = [self convertToSecureLoginURL];

	[self openPlayerWindow:nil];
	[webPlayerView setWebPlayerWithURL:urlString];
//	[webPlayerView setWebPlayerWithURL:urlString videoId:@""];

}
//------------------------------------
// downloadItem
//------------------------------------
- (IBAction)downloadItem:(id)sender
{

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return;
	}

	NSString *videoId = [webPlayerView videoId];
	NSString *title = [webPlayerView title];

	if(!videoId || [videoId isEqualToString:@""]){
		return;
	}

	// start download
	[downloadManager openDownloadWindow:nil];
	[downloadManager startDownloadItem:videoId
								fileName:title
	];

}
//------------------------------------
// openVideoPageWithBrowser
//------------------------------------
- (IBAction)openVideoPageWithBrowser:(id)sender
{

	NSString *videoId = [webPlayerView videoId];

	// no videoId
	if([videoId isEqualToString:@""]){
		return;
	}

	// open watch url
	[self openVideoWatchURL:videoId];

}
//------------------------------------
// copyItemToPasteboard
//------------------------------------
- (IBAction)copyItemToPasteboard:(id)sender
{

	NSString *videoId = [webPlayerView videoId];
	NSString *title = [webPlayerView title];
	NSString *string = @"";

	// no videoId
	if([videoId isEqualToString:@""]){
		return;
	}

	// url
	if([sender tag] == 0){
		string = [self convertToWatchURL:videoId];
	}
	// title
	else if([sender tag] == 1){
		string = title;
	}

	[self copyStringToPasteboard:string];

}
//------------------------------------
// showVideoContent
//------------------------------------
- (IBAction)showVideoContent:(id)sender
{
	// close
	if([viewVideoContent windowIsVisible] == YES){
		[viewVideoContent closeVideoContentWindow:nil];
	}else{
		NSDictionary *params = [self params];
		if(params == nil){
			return;
		}
		[viewVideoContent createVideoContentWindow:params arrayNo:[self arrayNo] refViewNo:REF_VIEW_PLAYER open:YES];
	}
}
//------------------------------------
// addItemToPlaylist
//------------------------------------
- (IBAction)addItemToPlaylist:(id)sender
{

	NSManagedObject *targetItem = [sender representedObject];

	// from webPlayerView
	NSString *videoId = [webPlayerView videoId];
	NSString *title = [webPlayerView title];

//	NSLog(@"videoId=%@", videoId);
//	NSLog(@"title=%@", title);

	// error
	if( [videoId isEqualToString:@""] ||
		[title isEqualToString:@""]
	){
		[self displayMessage:@"alert"
					messageText:@"Can not add video to playlist."
					infoText:@""
					btnList:@"Cancel"
		];
		return;
	}

	// create array for itemlist
 	NSMutableArray *items = [NSMutableArray array];
	[items addObject:
		[NSMutableDictionary dictionaryWithObjectsAndKeys:
			videoId, @"videoId" ,
			title, @"title" ,
			nil
		]
	];

	[tbArrayController createItemlist:[targetItem valueForKey:@"plistId"] items:items];

}
//------------------------------------
// searchRelatedVideo
//------------------------------------
- (IBAction)searchRelatedVideo:(id)sender
{

	NSString *videoId = [webPlayerView videoId];
	NSString *title = [webPlayerView title];

	if(![videoId isEqualToString:@""]){
		[viewRelatedWindow openRelatedSearch:nil];
		[viewRelatedSearch searchWithRelatedVideo:videoId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_VIDEO];
	}
}
//------------------------------------
// searchPostUserVideo
//------------------------------------
- (IBAction)searchPostUserVideo:(id)sender
{

	NSDictionary *params = [self params];
	if(params == nil){
		return;
	}
	NSString *videoId = [params valueForKey:@"videoId"];
	NSString *title = [params valueForKey:@"title"];
	NSString *postUserId =[params valueForKey:@"postUserId"];

	if(!videoId){videoId = @"";}
	if(!title){title = @"";}
	if(!postUserId){postUserId = @"";}

//	NSLog(@"videoId=%@", videoId);
//	NSLog(@"title=%@", title);
//	NSLog(@"postUserId=%@", postUserId);

	if(![videoId isEqualToString:@""]){
		[viewRelatedWindow openRelatedSearch:nil];
		[viewRelatedSearch searchPostUser:videoId postUserId:postUserId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_POST_USER];
	}

}

//------------------------------------
// searchRelatedTag
//------------------------------------
- (IBAction)searchRelatedTag:(id)sender
{

	NSString *videoId = [webPlayerView videoId];
	NSString *title = [webPlayerView title];

	if(![videoId isEqualToString:@""]){
		[viewRelatedWindow openTagSearch:nil];
		[viewTagSearch searchWithRelatedTag:videoId searchTitle:title pageIndex:1 searchType:SEARCH_TAG_WITH_VIDEO];
	}
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// changePlayerWindowWithScale
//------------------------------------
- (void)changePlayerWindowWithScale:(float)scale
{

	// default bar height
	float windowControllerHeight = [webPlayerView frame].origin.y;
	float titleBarHeight = [playerWindow frame].size.height - ([webPlayerView frame].size.height + windowControllerHeight);
	float totalBarHeight = titleBarHeight + windowControllerHeight;

	NSSize minSize = [playerWindow minSize];
	NSSize scaledSize = NSZeroSize;
	float baseWidth = 0;
	float baseHeight = 0;
	float controllerHeight = 0;

	int playerType = [webPlayerView playerType];

	// fix player size
	if( playerType == PLAYER_TYPE_NICO_PLAYER &&
		[webPlayerView isMaximizedPlayer] == NO &&
		scale == 1.0
	){
		scaledSize.width = 1000;
		scaledSize.height = 545;
	}
	// scale player size
	else{
		// new player / watch video
		if(playerType == PLAYER_TYPE_NICO_PLAYER ||
			playerType == PLAYER_TYPE_NICO_PLAYER_Q ||
			playerType == PLAYER_TYPE_WATCH_VIDEO
		){
			baseWidth = VIDEO_FRAME_HD_WIDTH;
			baseHeight = VIDEO_FRAME_HD_HEIGHT;
			controllerHeight = VIDEO_CONTROLLER_HEIGHT;
			minSize.width = VIDEO_FRAME_HD_WIDTH;	// for controller width
		}
		else if(playerType == PLAYER_TYPE_IPAD_PLAYER){
			baseWidth = VIDEO_IPAD_FRAME_WIDTH;
			baseHeight = VIDEO_IPAD_FRAME_HEIGHT;
			controllerHeight = VIDEO_IPAD_CONTROLLER_HEIGHT;
		}
		else{
			baseWidth = VIDEO_EXT_FRAME_WIDTH;
			baseHeight = VIDEO_EXT_FRAME_HEIGHT;
			controllerHeight = VIDEO_EXT_CONTROLLER_HEIGHT;
		}

		// base video size
		NSSize baseSize = NSMakeSize(baseWidth, baseHeight);

		// scaled video size
		scaledSize.width = baseSize.width * scale;
		scaledSize.height = baseSize.height * scale;
		// add controller height
		scaledSize.height += controllerHeight;
	}

	// adjust max size from screen size
	NSRect screenRect = [[NSScreen mainScreen] visibleFrame];
	if(scaledSize.width > screenRect.size.width){
		scale = screenRect.size.width / scaledSize.width;
		scaledSize.width *= scale;
		scaledSize.height *= scale;
	}
	if(scaledSize.height > (screenRect.size.height - totalBarHeight)){
		scale = (screenRect.size.height - totalBarHeight) / scaledSize.height;
		scaledSize.width *= scale;
		scaledSize.height *= scale;
	}

	// adjust min size from window min size
	if(scaledSize.width < minSize.width){
		scaledSize.width = minSize.width;
	}
	if(scaledSize.height < (minSize.height - totalBarHeight)){
		scaledSize.height = (minSize.height - totalBarHeight);
	}

	// change window frame
	[self changePlayerWindowFrame:scaledSize marginHeight:windowControllerHeight];

}
//------------------------------------
// changePlayerWindowFrame
//------------------------------------
- (void)changePlayerWindowFrame:(NSSize)newSize marginHeight:(float)marginHeight
{

	// get current window size
	NSRect oldRect = [NSWindow contentRectForFrameRect:[playerWindow frame] styleMask:[playerWindow styleMask]];

	// set point
	NSPoint newPoint;
	
	// xy / center
	newPoint.x = NSMinX(oldRect) - ((newSize.width / 2) -  (oldRect.size.width / 2));
//	newPoint.y = NSMinY(oldRect) - (((newSize.height + marginHeight) / 2) -  (oldRect.size.height / 2));

	// y / bottom
	newPoint.y = NSMaxY(oldRect) - (newSize.height + marginHeight);


	// set new window size
	NSRect newRect = [NSWindow frameRectForContentRect:
														NSMakeRect( 
																newPoint.x,
																newPoint.y,
																newSize.width,
																newSize.height + marginHeight
														)
											styleMask:[playerWindow styleMask]
					];

	[playerWindow setFrame:newRect display:YES animate:[playerWindow isVisible]];

}

//------------------------------------
// setPlayerView
//------------------------------------
- (BOOL)setPlayerView:(NSString*)videoId title:(NSString*)title params:(NSDictionary*)params arrayNo:(int)arrayNo
{

	// set params
	[self setParams:params];

	// set binding array no
	if(arrayNo != BIND_ARRAY_NONE){
		[self setArrayNo:arrayNo];
	}


	// set player view
	if([webPlayerView setPlayerView:videoId title:title reload:NO] == YES){
		// update history
		int playerType = [self defaultPlayerType];
		BOOL isPlayed = NO;
		if(playerType == PLAYER_TYPE_WATCH_VIDEO || playerType == PLAYER_TYPE_NICO_PLAYER_Q){
			isPlayed = YES;
		}
		[tbArrayController createPlayHistory:videoId title:title isPlayed:isPlayed];
		// set video content
		[viewVideoContent createVideoContentWindow:params arrayNo:arrayNo refViewNo:REF_VIEW_PLAYER open:NO];
		return YES;
	}
	// error
	else{
		if([self checkUserSession] == YES){
			[self displayMessage:@"alert"
							messageText:@"Can not open video."
							infoText:@""
							btnList:@"Cancel"
			];
		}
		return NO;
	}

//	return [webPlayerView setPlayerView:videoId title:title];

}
//------------------------------------
// setPlayerInfo
//------------------------------------
- (BOOL)setPlayerInfo:(NSString*)videoId title:(NSString*)title params:(NSDictionary*)params
{

	// set params
	[self setParams:params];

	// set player view
	if([webPlayerView setPlayerInfo:videoId title:title] == YES){
		// update history
		int playerType = [webPlayerView playerType];
		BOOL isPlayed = NO;
		if(playerType == PLAYER_TYPE_WATCH_VIDEO || playerType == PLAYER_TYPE_NICO_PLAYER_Q){
			isPlayed = YES;
		}
		[tbArrayController createPlayHistory:videoId title:title isPlayed:isPlayed];
		// set video content
		[viewVideoContent createVideoContentWindow:params arrayNo:[self arrayNo] refViewNo:REF_VIEW_PLAYER open:NO];
		return YES;
	}
	// error
	else{
/*
		[self displayMessage:@"alert"
						messageText:@"Can not open video."
						infoText:@""
						btnList:@"Cancel"
		];
*/
		return NO;
	}

}
//------------------------------------
// setPlayerViewWithURL
//------------------------------------
- (BOOL)setPlayerViewWithURL:(NSString*)urlString
{

	// null error
	if(!urlString || [urlString isEqualToString:@""]){
		return NO;
	}

	// get videoId (watch/smXXXXXX -> smXXXXXX)
	NSString *videoId = [self getVideoIdFromURL:urlString setDefault:YES];
	if([videoId isEqualToString:@""]){
		return NO;
	}

	[self setPlayerViewWithVideoId:videoId arrayNo:BIND_ARRAY_NONE];
	return YES;

}
//------------------------------------
// setPlayerViewWithVideoId
//------------------------------------
- (void)setPlayerViewWithVideoId:(NSString*)videoId arrayNo:(int)arrayNo;
{

	// set binding array no
	if(arrayNo != BIND_ARRAY_NONE){
		[self setArrayNo:arrayNo];
	}

	// alloc video info object
	[[[VideoInfoItem alloc] initWithVideo:videoId
									title:@""
									target:self
									notifType:VIDEO_ITEM_NOTIF_ITEM
									hasImage:YES
									params:nil
	] autorelease];

}
//------------------------------------
// setPlayerInfoWithVideoId
//------------------------------------
- (void)setPlayerInfoWithVideoId:(NSString*)videoId
{

	// alloc video info object
	[[[VideoInfoItem alloc] initWithVideo:videoId
									title:@""
									target:self
									notifType:VIDEO_ITEM_NOTIF_INFO
									hasImage:YES
									params:nil
	] autorelease];

}
//------------------------------------
// setPlayerIsPlayed
//------------------------------------
- (void)setPlayerIsPlayed:(NSString*)videoId isPlayed:(BOOL)isPlayed
{
	// update history
	[tbArrayController updatePlayHistoryIsPlayed:videoId isPlayed:isPlayed];
}
//------------------------------------
// handleItemObjectChange
//------------------------------------
-(void)handleItemObjectChange:(NSNotification *)notification
{

//	NSLog(@"handleItemObjectChange");
	VideoInfoItem *videoInfoItem = [notification object];

	if(videoInfoItem == nil){
		[self displayMessage:@"alert"
						messageText:@"Can not open video."
						infoText:@""
						btnList:@"Cancel"
						];
		return;
	}

//	NSLog(@"status=%d", [videoInfoItem status]);

	// success
	if([videoInfoItem itemStatus] == VIDEO_ITEM_SUCCESS){

		NSString *videoId = [videoInfoItem videoId];
		NSString *title = [videoInfoItem title];

		if(!videoId){
			return;
		}
		if(!title){
			title = videoId;
		}

		// create params
		NSDictionary *params = [self createParamsFromVideoInfoItem:videoInfoItem];

		// set player view
		[self setPlayerView:videoId title:title params:params arrayNo:BIND_ARRAY_NONE];

	}
	//failed
	else{
		int result;
		if([videoInfoItem itemStatus] == VIDEO_ITEM_STATUS_FAILED){
			result = [self displayMessageAlertOpenVideoWithError:[videoInfoItem errorDescription]];
		}else{
			result = [self displayMessageAlertOpenVideo];
		}
		if(result == NSAlertSecondButtonReturn){
			// open watch url
			[self openVideoWatchURL:[videoInfoItem videoId]];
			return;
		}
	}

}
//------------------------------------
// handleItemObjectInfo
//------------------------------------
-(void)handleItemObjectInfo:(NSNotification *)notification
{

//	NSLog(@"handleItemObjectInfo");
	VideoInfoItem *videoInfoItem = [notification object];

	if(videoInfoItem == nil){
		[self displayMessage:@"alert"
						messageText:@"Can not get video info."
						infoText:@"Can not change player info."
						btnList:@"Cancel"
						];
		return;
	}

//	NSLog(@"status=%d", [videoInfoItem status]);

	// success
	if([videoInfoItem itemStatus] == VIDEO_ITEM_SUCCESS){

		NSString *videoId = [videoInfoItem videoId];
		NSString *title = [videoInfoItem title];

		if(!videoId){
			return;
		}
		if(!title){
			title = videoId;
		}

		// create params
		NSDictionary *params = [self createParamsFromVideoInfoItem:videoInfoItem];

		// set player info
		[self setPlayerInfo:videoId title:title params:params];

	}
	//failed
	else{
		NSString *messageText = @"";
		if([videoInfoItem itemStatus] == VIDEO_ITEM_STATUS_FAILED){
			messageText = [NSString stringWithFormat:@"Failed getting video info.\n%@", [videoInfoItem errorDescription]];
		}else{
			messageText = @"Failed getting video info.";
		}
		[self displayMessage:@"alert"
				messageText:messageText
				infoText:@"Can not change player info."
				btnList:@"Cancel"
		];
		return;
	}

}
//------------------------------------
// createParamsFromVideoInfoItem
//------------------------------------
- (NSDictionary*)createParamsFromVideoInfoItem:(VideoInfoItem*)videoInfoItem
{
	return [NSDictionary dictionaryWithObjectsAndKeys:
					[videoInfoItem videoId], @"videoId" ,
					[videoInfoItem title], @"title" ,
					[videoInfoItem length], @"length" ,
					[videoInfoItem postUserId], @"postUserId" ,
					[videoInfoItem postedDate], @"postedDate" ,
					[videoInfoItem publishedDate], @"publishedDate" ,
					[videoInfoItem description], @"description" ,
					[videoInfoItem errorDescription], @"errorDescription" ,
					[videoInfoItem lastResBody], @"lastResBody" ,
					[videoInfoItem tagList], @"tagList" ,
					[NSNumber numberWithInt:[videoInfoItem viewCount]], @"viewCount" ,
					[NSNumber numberWithInt:[videoInfoItem resCount]], @"resCount" ,
					[NSNumber numberWithInt:[videoInfoItem mylistCount]], @"mylistCount" ,
					[NSNumber numberWithBool:YES], @"hasCountInfo" ,
					[NSNumber numberWithInt:[videoInfoItem itemStatus]], @"itemStatus" ,					
					[NSNumber numberWithBool:YES], @"getVideoInfo" ,
					[[[ImageItem alloc] initWithImageURL:[videoInfoItem imageURL] notifType:IMAGE_ITEM_NOTIF_ITEM] autorelease], @"imageItem" ,
					nil
			];
}
//------------------------------------
// openModalSheet
//------------------------------------
- (void)openModalSheet:(NSWindow*)childindow parentWindow:(NSWindow*)parentWindow
{

	[[NSApplication sharedApplication] beginSheet:childindow
									modalForWindow:parentWindow
									modalDelegate:self
									didEndSelector:@selector(didEndSheet:returnCode:contextInfo:)
									contextInfo: nil
	];

}
//------------------------------------
// showCustomSheet
//------------------------------------
- (void)didEndSheet:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{

	[sheet orderOut:self];

}
//------------------------------------
// handleLoginPageOpen
//------------------------------------
-(void)handleLoginPageOpen:(NSNotification *)notification
{
	[self openNicoVideoBaseURL:nil];
}

//------------------------------------
// windowWillClosePlayer
//------------------------------------
-(void)windowWillClosePlayer:(NSNotification *)notification
{
	[webPlayerView deletePlayerView];

	// window rect
	[self saveWindowRect:playerWindow key:@"rectWindowPlayer"];

}
//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
	[params_ release];
	params_ = params;
}
- (NSDictionary*)params
{
	return params_;
}
//------------------------------------
// setArrayNo
//------------------------------------
- (void)setArrayNo:(int)arrayNo
{
	arrayNo_ = arrayNo;
}
- (int)arrayNo
{
	return arrayNo_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[params_ release];
    [super dealloc];
}

@end
