#import "ViewPlaylist.h"
#import "ViewMainSearch.h"
#import "TBArrayController.h"
#import "ImpExpManager.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"
#import "NicoVideoExtension.h"

@implementation ViewPlaylist

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set display setting
	[self setControlButtonEnable];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(frameDidResize:) name:NSViewFrameDidChangeNotification object:[olvPlaylist enclosingScrollView]];

}
//=======================================================================
// action methods
//=======================================================================
//------------------------------------
// addPlaylist
//------------------------------------
- (IBAction)addPlaylist:(id)sender
{
	int itemType = ITEM_PLAYLIST;
	int itemSubType = ITEM_SEARCH_STRING;
	NSString *title = @"New Playlist";

	[self addItem:itemType
				itemSubType:itemSubType
				itemSubTypeAttribute:0
				title:title
				keyword:@""
				isFolder:NO
				isSelect:YES
				isEdit:YES
	];

}
//------------------------------------
// addSearchlist
//------------------------------------
- (IBAction)addSearchlist:(id)sender
{
	int itemType = ITEM_SEARCH;
	int itemSubType = ITEM_SEARCH_STRING;
	int itemSubTypeAttribute = 0;

	if([sender tag] == 0){
		itemSubType = ITEM_SEARCH_STRING;
	}
	else if([sender tag] == 1){
		itemSubType = ITEM_SEARCH_AUTHOR;
	}

	NSString *title = @"";
	if(itemSubType == ITEM_SEARCH_STRING){
		title = @"Keyword";
	}
	else if(itemSubType == ITEM_SEARCH_AUTHOR){
		title = @"Author";
	}

	[self addItem:itemType
				itemSubType:itemSubType
				itemSubTypeAttribute:itemSubTypeAttribute
				title:title
				keyword:@""
				isFolder:NO
				isSelect:YES
				isEdit:YES
	];

}
//------------------------------------
// addMylist
//------------------------------------
- (IBAction)addMylist:(id)sender
{

	int itemType = ITEM_MYLIST;
	int itemSubType = ITEM_SEARCH_STRING;
	int itemSubTypeAttribute = 0;

	NSString *title = @"xxxxx";

	[self addItem:itemType
				itemSubType:itemSubType
				itemSubTypeAttribute:itemSubTypeAttribute
				title:title
				keyword:@""
				isFolder:NO
				isSelect:YES
				isEdit:YES
	];

}
//------------------------------------
// addPostUserlist
//------------------------------------
- (IBAction)addPostUserlist:(id)sender
{

	int itemType = ITEM_POST_USER;
	int itemSubType = ITEM_SEARCH_STRING;
	int itemSubTypeAttribute = 0;

	NSString *title = @"xxxxx";

	[self addItem:itemType
				itemSubType:itemSubType
				itemSubTypeAttribute:itemSubTypeAttribute
				title:title
				keyword:@""
				isFolder:NO
				isSelect:YES
				isEdit:YES
	];

}
//------------------------------------
// addCategorylist
//------------------------------------
- (IBAction)addCategorylist:(id)sender
{
	int itemType = ITEM_CATEGORY;
	NSString *categoryName = [sender representedObject];

	NSString *title = [self getCategoryTitle:categoryName];

	[self addItem:itemType
				itemSubType:0
				itemSubTypeAttribute:0
				title:title
				keyword:categoryName
				isFolder:NO
				isSelect:YES
				isEdit:YES
	];

}
//------------------------------------
// addContentslist
//------------------------------------
- (IBAction)addContentslist:(id)sender
{
	int itemType = ITEM_CONTENTS;
	int itemSubType = [sender tag];

	NSString *title = [self getContentsTitle:itemSubType];

	[self addItem:itemType
				itemSubType:itemSubType
				itemSubTypeAttribute:0
				title:title
				keyword:@""
				isFolder:NO
				isSelect:YES
				isEdit:YES
	];

}
//------------------------------------
// addFolder
//------------------------------------	  
- (IBAction)addFolder:(id)sender
{

	// add folder item
	[self addItem:ITEM_FOLDER
			itemSubType:0
			itemSubTypeAttribute:0
			title:@"New Folder"
			keyword:@""
			isFolder:YES
			isSelect:YES
			isEdit:YES
	];

}
//------------------------------------
// searchItem
//------------------------------------
- (IBAction)searchItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];
	// no select
	if(!selectedItem){
		return;
	}

	NSString *plistId = [selectedItem valueForKey:@"plistId"];
	NSString *searchString = [selectedItem valueForKey:@"title"];
	NSString *title = [selectedItem valueForKey:@"title"];
	NSString *keyword = [selectedItem valueForKey:@"keyword"];
	int itemType = [[selectedItem valueForKey:@"itemType"] intValue];
	int itemSubType = [[selectedItem valueForKey:@"itemSubType"] intValue];
	int itemSubTypeAttribute = 0;

	if([selectedItem valueForKey:@"itemSubTypeAttribute"]){
		itemSubTypeAttribute = [[selectedItem valueForKey:@"itemSubTypeAttribute"] intValue];
	}

//	NSLog(@"searchString=%@", searchString);
//	NSLog(@"itemSubTypeAttribute=%d", itemSubTypeAttribute);
	// action
	// search with keyword
	if(itemType == ITEM_SEARCH){
		// keyword search
		if(itemSubType == ITEM_SEARCH_STRING){
			// change default
			if(itemSubTypeAttribute == ITEM_SEARCH_STRING_KEYWORD){
				[self setDefaultIntValue:SEARCH_WITH_STRING_KEYWORD key:@"optSearchKeywordType"];
			}
			else if(itemSubTypeAttribute == ITEM_SEARCH_STRING_TAG){
				[self setDefaultIntValue:SEARCH_WITH_STRING_TAG key:@"optSearchKeywordType"];
			}
			[viewMainSearch searchWithString:searchString pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:NO];
		}
	}
	// search with mylist
	else if(itemType == ITEM_MYLIST){
		[viewMainSearch searchWithMylist:keyword searchTitle:title pageIndex:1 searchType:SEARCH_WITH_MYLIST isKeywordSearch:NO];
	}
	// search with postuser
	else if(itemType == ITEM_POST_USER){
		[viewMainSearch searchWithPostUser:keyword searchTitle:title pageIndex:1 searchType:SEARCH_WITH_POST_USER isKeywordSearch:NO];
	}
	// search with channel
	else if(itemType == ITEM_CHANNEL){
		[viewMainSearch searchWithChannel:keyword searchTitle:title pageIndex:1 searchType:SEARCH_WITH_CHANNEL isKeywordSearch:NO];
	}
	// search with playlist
	else if(itemType == ITEM_PLAYLIST){
		[viewMainSearch searchWithPlaylist:plistId searchTitle:title pageIndex:1 isKeywordSearch:NO];
	}
	// search with category
	else if(itemType == ITEM_CATEGORY){

		// get categoryName (for older version)
		if(!keyword || [keyword isEqualToString:@""]){
			keyword = [self getCategoryName:itemSubType];
		}

		// check enable category
		NSString *message = @"";
		if([self checkCategoryEnable:keyword message:&message] == NO){
			[self displayMessageWithIcon:@"icon_alert_nico"
						messageText:message
						infoText:@""
						btnList:@"Cancel"
			];
			return;
		}

		// force change to ranking
		[self setDefaultBoolValue:YES key:@"optRankingEnabled"];
/*
		if([keyword isEqualToString:CATEGORY_NAME_ALL]){
			if([self defaultBoolValue:@"optRankingEnabled"] == NO){
				[self setDefaultBoolValue:YES key:@"optRankingEnabled"];
				[viewMainSearch changeRankingButtonState];
			}
		}
*/
		// ranking
//		if([self defaultBoolValue:@"optRankingEnabled"] == YES){
			[viewMainSearch searchWithCategoryRanking:keyword searchTitle:title pageIndex:1 searchType:SEARCH_WITH_RANKING isKeywordSearch:NO];
//		}
		// category
//		else{
//			[viewMainSearch searchWithCategoryName:keyword searchTitle:title pageIndex:1 searchType:SEARCH_WITH_CATEGORY isKeywordSearch:NO];
//		}
	}
	// search with contents
	else if(itemType == ITEM_CONTENTS){
//		if(itemSubType == ITEM_CONTENTS_TOPPAGE){
//			[viewMainSearch searchWithURL:@"" searchTitle:title pageIndex:1 searchType:SEARCH_WITH_URL];
//		}else{
			[viewMainSearch searchWithContentsName:[self getContentsName:itemSubType] searchTitle:title pageIndex:1 searchType:SEARCH_WITH_CONTENTS isKeywordSearch:NO];
//		}
	}

}

//------------------------------------
// removeItem
//------------------------------------	  
- (IBAction)removeItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];
	// no select
	if(!selectedItem){
		return;
	}

	BOOL isFolder = [[selectedItem valueForKey:@"isFolder"] boolValue];
	int itemType = [[selectedItem valueForKey:@"itemType"] intValue];
	int itemSubType = [[selectedItem valueForKey:@"itemSubType"] intValue];
	int result;

	// Alert before remove
	// folder
	if(isFolder == YES){
		result = [self displayMessageWithIcon:@"icon_alert_folder"
											messageText:@"Delete Folder?"
											infoText:@"All items in folder are deleted"
											btnList:@"Delete,Cancel"
											];
	}
	else{
		// playlist
		if(itemType == ITEM_PLAYLIST){
			result = [self displayMessageWithIcon:@"icon_alert_playlist"
												messageText:@"Delete Playlist?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		// searchlist
		else if(itemType == ITEM_SEARCH){
			// keyword
			if(itemSubType == ITEM_SEARCH_STRING){
				result = [self displayMessageWithIcon:@"icon_alert_search"
													messageText:@"Delete keyword list?"
													infoText:@""
													btnList:@"Delete,Cancel"
													];
			}
/*
			// author
			else if(itemSubType == ITEM_SEARCH_AUTHOR){
				result = [self displayMessageWithIcon:@"icon_alert_author"
													messageText:@"Delete author list?"
													infoText:@""
													btnList:@"Delete,Cancel"
													];
			}
*/
			else{
				return;
			}
		}
		// mylist
		else if(itemType == ITEM_MYLIST){
			result = [self displayMessageWithIcon:@"icon_alert_mylist"
												messageText:@"Delete Mylist?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		// postuser
		else if(itemType == ITEM_POST_USER){
			result = [self displayMessageWithIcon:@"icon_alert_post_user"
												messageText:@"Delete Post User?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		// channel
		else if(itemType == ITEM_CHANNEL){
			result = [self displayMessageWithIcon:@"icon_alert_channel"
												messageText:@"Delete Channel?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		// categorylist
		else if(itemType == ITEM_CATEGORY){
			result = [self displayMessageWithIcon:@"icon_alert_category"
												messageText:@"Delete category list?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		// contentslist
		else if(itemType == ITEM_CONTENTS){
			result = [self displayMessageWithIcon:@"icon_alert_contents"
												messageText:@"Delete contents list?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		else{
			return;
		}
	}

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove tree of selectedItem
	[tbArrayController removeTreeItems:selectedItem];

	// reload
	[playlistTreeController rearrangeObjects];

}
//------------------------------------
// exportItem
//------------------------------------
- (IBAction)exportItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];

	// no select
	if(!selectedItem){
		return;
	}

	NSString *title = [selectedItem valueForKey:@"title"];

	[impExpManager exportPlaylistWithItem:selectedItem title:title rootFolder:NO];
}
//------------------------------------
// editItem
//------------------------------------
- (IBAction)editItem:(id)sender
{

	[olvPlaylist editSelectedItem];

}
//------------------------------------
// deselectItem
//------------------------------------
- (IBAction)deselectItem:(id)sender
{
	[olvPlaylist deselectItem];
}
//------------------------------------
// nullAction
//------------------------------------
- (IBAction)nullAction:(id)sender
{
	// null action
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// addItem
//------------------------------------
- (void)addItem:(int)itemType
		itemSubType:(int)itemSubType
		itemSubTypeAttribute:(int)itemSubTypeAttribute
		title:(NSString*)title
		keyword:(NSString*)keyword
		isFolder:(BOOL)isFolder
		isSelect:(BOOL)isSelect
		isEdit:(BOOL)isEdit
{


	// get selected item
	id selectedRowItem = [olvPlaylist selectedRowObject];
	NSManagedObject *selectedItem = nil;
	if(selectedRowItem){
		selectedItem = [olvPlaylist selectedObservedObject];
	}

	// create item
	id item = [tbArrayController insertPlaylist:itemType
									itemSubType:itemSubType
									itemSubTypeAttribute:itemSubTypeAttribute
									index:0
									title:title
									keyword:keyword
									parentItem:nil
									isFolder:isFolder
				];

	// add item to patent of selected item
	[tbArrayController addItemToParent:selectedItem item:item maxNum:YES];			

	// reload
	[playlistTreeController rearrangeObjects];

	// expand selectedRowItem
	if(selectedItem){
		if([[selectedItem valueForKey:@"isFolder"] boolValue] ==YES){
			[olvPlaylist expandItem:selectedRowItem expandChildren:YES];
		}
	}

	// select add item
	if(isSelect == YES){
		[olvPlaylist selectItem:item];
	}

	// edit selected row
	if(isEdit == YES){
		[olvPlaylist editSelectedItem];
	}

}
//------------------------------------
// setControlButtonEnable
//------------------------------------
- (void)setControlButtonEnable
{
	BOOL canAdd = [playlistTreeController canAdd];
	int selectedRow = [olvPlaylist selectedRow];
	BOOL isDisplay = [olvPlaylist isDisplay];

	if(canAdd == YES && isDisplay == YES){
		[btnAdd setEnabled:YES];
	}else{
		[btnAdd setEnabled:NO];
	}

	if(selectedRow >= 0 && isDisplay == YES){
		[btnRemove setEnabled:YES];
	}else{
		[btnRemove setEnabled:NO];
	}

}
//------------------------------------
// frameDidResize
//------------------------------------
-(void)frameDidResize:(NSNotification *)notification
{
	[self setControlButtonEnable];
}
//--------------------------------------
// isSelectItem
//--------------------------------------
- (BOOL)isSelectItem
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if([[item valueForKey:@"isFolder"] boolValue] == NO){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectFolder
//--------------------------------------
- (BOOL)isSelectFolder
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if([[item valueForKey:@"isFolder"] boolValue] == YES){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectPlaylist
//--------------------------------------
- (BOOL)isSelectPlaylist
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if( [[item valueForKey:@"isFolder"] boolValue] == NO &&
			[[item valueForKey:@"itemType"] intValue] == ITEM_PLAYLIST
			){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectSearchKeyword
//--------------------------------------
- (BOOL)isSelectSearchKeyword
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if( [[item valueForKey:@"isFolder"] boolValue] == NO &&
			[[item valueForKey:@"itemType"] intValue] == ITEM_SEARCH &&
			[[item valueForKey:@"itemSubType"] intValue] == ITEM_SEARCH_STRING
			){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectSearchAuthor
//--------------------------------------
- (BOOL)isSelectSearchAuthor
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if( [[item valueForKey:@"isFolder"] boolValue] == NO &&
			[[item valueForKey:@"itemType"] intValue] == ITEM_SEARCH &&
			[[item valueForKey:@"itemSubType"] intValue] == ITEM_SEARCH_AUTHOR
			){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectMylist
//--------------------------------------
- (BOOL)isSelectMylist
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if( [[item valueForKey:@"isFolder"] boolValue] == NO &&
			[[item valueForKey:@"itemType"] intValue] == ITEM_MYLIST
			){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectPostUser
//--------------------------------------
- (BOOL)isSelectPostUser
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if( [[item valueForKey:@"isFolder"] boolValue] == NO &&
			[[item valueForKey:@"itemType"] intValue] == ITEM_POST_USER
			){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectChannel
//--------------------------------------
- (BOOL)isSelectChannel
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if( [[item valueForKey:@"isFolder"] boolValue] == NO &&
			[[item valueForKey:@"itemType"] intValue] == ITEM_CHANNEL
			){
			ret = YES;
		}
	}

	return ret;
}
//------------------------------------
// isKeyMainWindow
//------------------------------------
- (BOOL)isKeyMainWindow
{
	return [mainWindow isKeyWindow];
}
//------------------------------------
// canAddPlaylist
//------------------------------------
- (BOOL)canAddPlaylist
{
	if( [self isKeyMainWindow] == YES &&
		[self isDisplayPlaylist] == YES){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// canEditPlaylistSetting
//------------------------------------
- (BOOL)canEditPlaylistSetting
{
	BOOL ret = NO;

	if( [self isSelectSearchKeyword] == YES ||
		[self isSelectMylist] == YES ||
		[self isSelectPostUser] == YES ||
		[self isSelectChannel] == YES
	){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// isDisplayPlaylist
//------------------------------------
- (BOOL)isDisplayPlaylist
{
	return [olvPlaylist isDisplay];
}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[super dealloc];
}
@end
