#import "ViewPlaylistSetting.h"
#import "ViewKeywordSetting.h"
#import "ViewMylistSetting.h"
#import "ViewPostUserSetting.h"
#import "ViewChannelSetting.h"
#import "TBArrayController.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

static int TAB_INDEX_KEYWORD = 0;
static int TAB_INDEX_MYLIST = 1;
static int TAB_INDEX_POST_USER = 2;
static int TAB_INDEX_CHANNEL = 3;
static NSString *TITLE_KEYWORD = @"Keyword Setting";
static NSString *TITLE_MYLIST = @"Mylist Setting";
static NSString *TITLE_POST_USER = @"Post User Setting";
static NSString *TITLE_CHANNEL = @"Channel Setting";

@implementation ViewPlaylistSetting

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window position
	[self setWindowRect:playlistSettingWindow key:@"rectWindowPlaylistSetting"];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:playlistSettingWindow];
}
//------------------------------------
// openPlaylistSettingWindow
//------------------------------------
- (IBAction)openPlaylistSettingWindow:(id)sender
{
	[playlistSettingWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// openPlaylistSetting
//------------------------------------
- (IBAction)openPlaylistSetting:(id)sender
{

	NSArray *fetchedArray = [tbArrayController getSelectedObjects:@"playlist"];

	// not result
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Playlist is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}

	// get value
	id record = [fetchedArray objectAtIndex:0];
	int itemType = [[record valueForKey:@"itemType"] intValue];

	// keyword
	if(itemType == ITEM_SEARCH){
		[self openKeywordSettingWindow:nil];
	}
	// mylist / edit
	else if(itemType == ITEM_MYLIST){
		[self openMylistEditSettingWindow:nil];
	}
	// postuser / edit
	else if(itemType == ITEM_POST_USER){
		[self openPostUserEditSettingWindow:nil];
	}
	// channel / edit
	else if(itemType == ITEM_CHANNEL){
		[self openChannelEditSettingWindow:nil];
	}
}

//------------------------------------
// openKeywordSettingWindow
//------------------------------------
- (IBAction)openKeywordSettingWindow:(id)sender
{
	// keyword / edit
	if([viewKeywordSetting createKeywordSettingWindow] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_KEYWORD];
		[self setWindowTitle:TITLE_KEYWORD];
		[self openPlaylistSettingWindow:nil];
	}
}
//------------------------------------
// openMylistNewSettingWindow
//------------------------------------
- (IBAction)openMylistNewSettingWindow:(id)sender
{
	// mylist / new
	if([viewMylistSetting createMylistSettingWindow:YES] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_MYLIST];
		[self setWindowTitle:TITLE_MYLIST];
		[self openPlaylistSettingWindow:nil];
	}
}
//------------------------------------
// openPostUserNewSettingWindow
//------------------------------------
- (IBAction)openPostUserNewSettingWindow:(id)sender
{
	// postuser / new
	if([viewPostUserSetting createPostUserSettingWindow:YES] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_POST_USER];
		[self setWindowTitle:TITLE_POST_USER];
		[self openPlaylistSettingWindow:nil];
	}
}
//------------------------------------
// openPostUserEditSettingWindow
//------------------------------------
- (IBAction)openPostUserEditSettingWindow:(id)sender
{
	// postuser / edit
	if([viewPostUserSetting createPostUserSettingWindow:NO] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_POST_USER];
		[self setWindowTitle:TITLE_POST_USER];
		[self openPlaylistSettingWindow:nil];
	}
}
//------------------------------------
// openMylistEditSettingWindow
//------------------------------------
- (IBAction)openMylistEditSettingWindow:(id)sender
{
	// mylist / edit
	if([viewMylistSetting createMylistSettingWindow:NO] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_MYLIST];
		[self setWindowTitle:TITLE_MYLIST];
		[self openPlaylistSettingWindow:nil];
	}
}
//------------------------------------
// openChannelNewSettingWindow
//------------------------------------
- (IBAction)openChannelNewSettingWindow:(id)sender
{
	// mylist / new
	if([viewChannelSetting createChannelSettingWindow:YES] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_CHANNEL];
		[self setWindowTitle:TITLE_CHANNEL];
		[self openPlaylistSettingWindow:nil];
	}
}
//------------------------------------
// openChannelEditSettingWindow
//------------------------------------
- (IBAction)openChannelEditSettingWindow:(id)sender
{
	// mylist / edit
	if([viewChannelSetting createChannelSettingWindow:NO] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_CHANNEL];
		[self setWindowTitle:TITLE_CHANNEL];
		[self openPlaylistSettingWindow:nil];
	}
}
//------------------------------------
// closeWindow
//------------------------------------
- (IBAction)closeWindow:(id)sender
{
	[playlistSettingWindow performClose:nil];
}

//------------------------------------
// openMylistNewSettingWindowWithValue
//------------------------------------
- (BOOL)openMylistNewSettingWindowWithValue:(NSString*)mylistId title:(NSString*)title
{

	if([viewMylistSetting createMylistSettingWindowWithValue:mylistId title:title] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_MYLIST];
		[self setWindowTitle:TITLE_MYLIST];
		[self openPlaylistSettingWindow:nil];
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// openPostUserNewSettingWindowWithValue
//------------------------------------
- (BOOL)openPostUserNewSettingWindowWithValue:(NSString*)postUserId title:(NSString*)title
{

	if([viewPostUserSetting createPostUserSettingWindowWithValue:postUserId title:title] == YES){
		[tabViewSetting selectTabViewItemAtIndex:TAB_INDEX_POST_USER];
		[self setWindowTitle:TITLE_POST_USER];
		[self openPlaylistSettingWindow:nil];
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// setWindowTitle
//------------------------------------
- (void)setWindowTitle:(NSString*)title
{
	[playlistSettingWindow setTitle:title];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	[self saveWindowRect:playlistSettingWindow key:@"rectWindowPlaylistSetting"];

}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
