#import "ViewPostUserSetting.h"
#import "ViewPlaylistSetting.h"
#import "ViewMainSearch.h"
#import "ViewPlaylist.h"
#import "TBArrayController.h"
#import "UserDefaultsExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"

@implementation ViewPostUserSetting

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setPlistId:@""];
	[self setIsNewList:NO];
	[txtPostUserTitle setStringValue:@""];
	[txtPostUserId setStringValue:@""];
	[self changeTakeButtonEnable:NO];
}
//------------------------------------
// updatePostUserSetting
//------------------------------------
- (IBAction)updatePostUserSetting:(id)sender
{

	NSString *title = [txtPostUserTitle stringValue];
	NSString *postUserId= [txtPostUserId stringValue];

	if([title isEqualToString:@""]){
		[self displayMessage:@"alert"
							messageText:@"Title is not entered."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}
	if([postUserId isEqualToString:@""]){
		[self displayMessage:@"alert"
							messageText:@"Post User ID is not entered."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}

	if([self checkIsNumberString:postUserId] == NO){
		[self displayMessage:@"alert"
							messageText:@"Post User ID is not a number."
							infoText:@""
							btnList:@"Cancel"
		];
		return;	
	}

	// new
	if([self isNewList] == YES){
		int itemType = ITEM_POST_USER;
		int itemSubType = ITEM_SEARCH_STRING;
		int itemSubTypeAttribute = 0;

		// add item
		[viewPlaylist addItem:itemType
					itemSubType:itemSubType
					itemSubTypeAttribute:itemSubTypeAttribute
					title:title
					keyword:postUserId
					isFolder:NO
					isSelect:YES
					isEdit:NO
		];

		[viewPlaylistSetting closeWindow:nil];
	}
	// edit
	else{

		NSString *plistId = [self plistId];

		if([plistId isEqualToString:@""]){
			[self displayMessage:@"alert"
								messageText:@"Post User is not found"
								infoText:@""
								btnList:@"Cancel"
			];
			return;
		}

		// update playlist
		if([self updatePlaylist:plistId title:title keyword:postUserId] == YES){
			[viewPlaylistSetting closeWindow:nil];
		}
	}

}
//------------------------------------
// takePostUserValuesFromMainWindow:
//------------------------------------
- (IBAction)takePostUserValuesFromMainWindow:(id)sender
{

	NSString *title = [viewMainSearch postUserTitle];
	NSString *postUserId = [viewMainSearch postUserId];

	[txtPostUserTitle setStringValue:title];
	[txtPostUserId setStringValue:postUserId];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createPostUserSettingWindow
//------------------------------------
- (BOOL)createPostUserSettingWindow:(BOOL)isNewList
{

	NSString *plistId = @"";
	NSString *title = @"";
	NSString *postUserId = @"";

	[self setIsNewList:isNewList];

	// new
	if(isNewList == YES){
		// none
	}
	// edit
	else{
		NSArray *fetchedArray = [tbArrayController getSelectedObjects:@"playlist"];

		// not result
		if([fetchedArray count] <= 0){
			[self displayMessage:@"alert"
								messageText:@"Post User is not found"
								infoText:@""
								btnList:@"Cancel"
			];
			return NO;
		}

		// get value
		id record = [fetchedArray objectAtIndex:0];
		plistId = [record valueForKey:@"plistId"];
		title = [record valueForKey:@"title"];
		postUserId = [record valueForKey:@"keyword"];

		int itemType = [[record valueForKey:@"itemType"] intValue];

		// not mylist
		if(itemType != ITEM_POST_USER){
			[self displayMessage:@"alert"
								messageText:@"Post User is not selected"
								infoText:@""
								btnList:@"Cancel"
			];
			return NO;
		}

	}

	[self setSettingValue:postUserId title:title];
	[self setPlistId:plistId];
	[self changeTakeButtonEnable:[self canTakeValue]];

	return YES;

}
//------------------------------------
// createPostUserSettingWindow
//------------------------------------
- (BOOL)createPostUserSettingWindowWithValue:(NSString*)postUserId title:(NSString*)title
{

	if([self createPostUserSettingWindow:YES] == YES){
		[self changeTakeButtonEnable:NO];
		[self setSettingValue:postUserId title:title];
	}else{
		return NO;
	}

	return YES;

}

//------------------------------------
// setSettingValue
//------------------------------------
- (void)setSettingValue:(NSString*)postUserId title:(NSString*)title
{
	[txtPostUserId setStringValue:postUserId];
	[txtPostUserTitle setStringValue:title];
}
//------------------------------------
// updatePlaylist
//------------------------------------
- (BOOL)updatePlaylist:(NSString*)plistId title:(NSString*)title keyword:(NSString*)keyword
{

	// get playllist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"plistId == %@", plistId];
	NSArray *fetchedArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not result
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Post User is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// get value
	id record = [fetchedArray objectAtIndex:0];

	int itemType = [[record valueForKey:@"itemType"] intValue];

	// not keyword
	if(itemType != ITEM_POST_USER){
		[self displayMessage:@"alert"
							messageText:@"Post User is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// update value
	[record setValue:title forKey:@"title"];
	[record setValue:keyword forKey:@"keyword"];

	return YES;

}
//------------------------------------
// changeTakeButtonEnable
//------------------------------------
- (void)changeTakeButtonEnable:(BOOL)enable
{
	[btnTakeValue setEnabled:enable];
}
//------------------------------------
// canTakeValue
//------------------------------------
- (BOOL)canTakeValue
{
	BOOL ret = NO;
	if(![[viewMainSearch postUserId] isEqualToString:@""]){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// setPlistId
//------------------------------------
- (void)setPlistId:(NSString*)plistId
{
	[plistId retain];
	[plistId_ release];
	plistId_ = plistId;
}
//------------------------------------
// plistId
//------------------------------------
- (NSString*)plistId
{
	return plistId_;
}
//------------------------------------
// isNewList
//------------------------------------
- (void)setIsNewList:(BOOL)isNewList
{
	isNewList_ = isNewList;
}
- (BOOL)isNewList;
{
	return isNewList_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[plistId_ release];
	[super dealloc];
}
@end
