#import "ViewPrefs.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewPrefs

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	//
	// set state from user default
	//
	// window position
	[self setWindowPosition:prefsWindow key:@"rectWindowPrefs"];
	[self setWindowPosition:helpWindow key:@"rectWindowHelp"];

	// set status
	[self changePlayerBoxEnable];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:prefsWindow];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:helpWindow];

}

//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// Open Info Window Action
//------------------------------------
- (IBAction)openPrefsWindow:(id)sender
{
	[prefsWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// Open help Window Action
//------------------------------------
- (IBAction)openHelpWindow:(id)sender
{
	[helpWindow makeKeyAndOrderFront:self];
}

//------------------------------------
// changeWindowTheme
//------------------------------------
- (IBAction)changeWindowTheme:(id)sender
{
	[mainWindow changeWindowTheme:nil];
	[relatedWindow changeWindowTheme:nil];
//	[playerWindow changeWindowTheme:nil];
	[downloadWindow changeWindowTheme:nil];
	[logWindow changeWindowTheme:nil];
}
//------------------------------------
// changePlayerType
//------------------------------------
- (IBAction)changePlayerType:(id)sender
{
	[self changePlayerBoxEnable];
}
//------------------------------------
// selectDownloadFolderPath
//------------------------------------
- (IBAction)selectDownloadFolderPath:(id)sender
{

	NSArray *selectedPaths;
	NSString *defaultPath = [self defaultStringValue:@"optDownloadFolderPath"];

	// get path
	selectedPaths = [self selectFilePathWithOpenPanel:NO
												isDir:YES
												isMultiSel:NO
												isPackage:NO
												isAlias:YES
												canCreateDir:YES
												defaultPath:defaultPath
					];

	// set path
	if([selectedPaths count] > 0){
		[self setDefaultStringValue:[selectedPaths objectAtIndex:0] key:@"optDownloadFolderPath"];
	}
}
//------------------------------------
// changePlayerBoxEnable
//------------------------------------
- (void)changePlayerBoxEnable
{

	BOOL enabledNicoPlayerAttr;
	BOOL enabledAutoPlay;
	BOOL enabledResize;
	BOOL enabledEco;

	NSColor *colorNicoPlayerAttr;
	NSColor *colorAutoPlay;
	NSColor *colorResize;
	NSColor *colorEco;

	NSColor *enableColor = [NSColor blackColor];
	NSColor *disableColor = [NSColor lightGrayColor];

	int playerType = [self defaultPlayerType];

	// nico player attributes
	if(playerType == PLAYER_TYPE_NICO_PLAYER
//		playerType == PLAYER_TYPE_NICO_PLAYER_Q
	){
		enabledNicoPlayerAttr = YES;
		colorNicoPlayerAttr = enableColor;
	}else{
		enabledNicoPlayerAttr = NO;
		colorNicoPlayerAttr = disableColor;
	}

	// auto play
	if( playerType == PLAYER_TYPE_NICO_PLAYER
//		playerType == PLAYER_TYPE_NICO_PLAYER_Q ||
//		playerType == PLAYER_TYPE_EXT_PLAYER
	){
		enabledAutoPlay = YES;
		colorAutoPlay = enableColor;
	}else{
		enabledAutoPlay = NO;
		colorAutoPlay = disableColor;
	}

	// live resize
	if( playerType == PLAYER_TYPE_EXT_PLAYER 
	){
		enabledResize = YES;
		colorResize = enableColor;
	}else{
		enabledResize = NO;
		colorResize = disableColor;
	}

	// eco mode
	if( playerType == PLAYER_TYPE_NICO_PLAYER ||
		playerType == PLAYER_TYPE_NICO_PLAYER_Q || 
		playerType == PLAYER_TYPE_WATCH_VIDEO 
	){
		enabledEco = YES;
		colorEco = enableColor;
	}else{
		enabledEco = NO;
		colorEco = disableColor;
	}

	// check box
	[cbxAutoPlay setEnabled:enabledAutoPlay];
	[cbxLiveResize setEnabled:enabledResize];
	[cbxEcoMode setEnabled:enabledEco];

	[cbxAutoMaximize setEnabled:enabledNicoPlayerAttr];
	[cbxNoBanner setEnabled:enabledNicoPlayerAttr];
	[cbxNoMarquee setEnabled:enabledNicoPlayerAttr];
	[cbxNoNicowari setEnabled:enabledNicoPlayerAttr];

	// text color
	[txtAutoPlay setTextColor:colorAutoPlay];
	[txtLiveResize setTextColor:colorResize];
	[txtEcoMode setTextColor:colorEco];

	[txtAutoMaximize setTextColor:colorNicoPlayerAttr];
	[txtNoBanner setTextColor:colorNicoPlayerAttr];
	[txtNoMarquee setTextColor:colorNicoPlayerAttr];
	[txtNoNicowari setTextColor:colorNicoPlayerAttr];

/*
	// debug mode
	if([self defaultBoolValue:@"optNicoPlayerDebugMode"] == YES){

		[cbxNoBanner setHidden:NO];
		[cbxNoMarquee setHidden:NO];
		[cbxNoNicowari setHidden:NO];

		[txtNoBanner setHidden:NO];
		[txtNoMarquee setHidden:NO];
		[txtNoNicowari setHidden:NO];
	}
*/

}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	if([notification object] == prefsWindow){
		[self saveWindowRect:prefsWindow key:@"rectWindowPrefs"];
	}
	else if([notification object] == helpWindow){
		[self saveWindowRect:helpWindow key:@"rectWindowHelp"];
	}

}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
}

@end
