#import "ViewPreviewPlayer.h"
#import "ViewMainSearch.h"
#import "ViewPlayer.h"
#import "LogStatusController.h"
#import "ConvertExtension.h"
#import "HelperExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

static int FETCH_TYPE_WATCH = 0;
static int FETCH_TYPE_VIDEO = 1;
static NSString *defaultKeyPreviewMovieVolume = @"optPreviewMovieVolume";

@implementation ViewPreviewPlayer

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rect
//	[self setWindowRect:previewPlayerWindow key:@"rectWindowPreviewPlayer"];

    [self setMovie:nil];
	[self setParams:nil];
	[self setConnectionDataItem:nil];
	[self setIsContinuePlaying:NO];
	[self setArrayNo:BIND_ARRAY_NONE];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handleFrameAnimDidChanged:) name:FRAME_NOTIF_ANIM_DID_CHANGED object:nil];

	// add observer
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addObserver:self forKeyPath:defaultKeyPreviewMovieVolume options:0 context:nil];

}
//=======================================================================
// Event Actions
//=======================================================================
/*
//------------------------------------
// openPreviewPlayerWindow
//------------------------------------
- (IBAction)openPreviewPlayerWindow:(id)sender
{
	[previewPlayerWindow makeKeyAndOrderFront:self];
}
*/
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	// no select
	NSDictionary *params = [self params];
	if(params == nil){
		return;
	}

	NSString *videoId = [params valueForKey:@"videoId"];
	NSString *title = [params valueForKey:@"title"];

	if([viewPlayer setPlayerView:videoId title:title params:params arrayNo:[self arrayNo]] == YES){
		[viewPlayer openPlayerWindow:nil];
		int actionType = [self defaultPreviewActionType];
		if(actionType == 1){
			[self setMoviePlay:NO];
		}
		else if(actionType == 2){
			[self closeMovie:nil];
		}
	}

}
//------------------------------------
// controlMoviePlay
//------------------------------------
- (IBAction)controlMoviePlay:(id)sender
{

	BOOL isPlaying = [self isPlaying];
	[self setMoviePlay:!isPlaying];

}
//------------------------------------
// closeMovie
//------------------------------------
- (IBAction)closeMovie:(id)sender
{

	[self closeMovieView];
	[viewMainSearch changeTabViewPreview:PREVIEW_TAB_INDEX_IMAGE];
	if([viewMainSearch isMaximizedPreviewFrame] == YES){
		[viewMainSearch restorePreviewFrame:sender];
	}

}
//------------------------------------
// maximizePreviewFrame
//------------------------------------
- (IBAction)maximizePreviewFrame:(id)sender
{
	[viewMainSearch maximizePreviewFrame:sender];
}
//=======================================================================
// methods
//=======================================================================

//------------------------------------
// setPreviewView
//------------------------------------
- (BOOL)setPreviewView:(NSString*)videoId title:(NSString*)title params:(NSDictionary*)params arrayNo:(int)arrayNo
{

	if([self checkIsPreviewVideoId:videoId] == NO){
		[self displayMessage:@"alert"
						messageText:@"Can not play preview video."
						infoText:[NSString stringWithFormat:@"%@ is not supported.", videoId]
						btnList:@"Cancel"
		];
		return NO;
	}

	// clear movie
    [self setMoviePlay:NO];
	[self deleteMovie];
	[self clearConnection];

	//
	// get watch html
	//
	int playerType = [self defaultPlayerType];
	NSString *watchURL;
	if(playerType == PLAYER_TYPE_NICO_PLAYER_Q){
		watchURL = [self convertToWatchQPlayerURL:videoId];
	}else{
		watchURL = [self convertToWatchURL:videoId];
	}

	NSDictionary *linkParams =  [NSDictionary dictionaryWithObjectsAndKeys:
									videoId, @"videoId" ,
									title, @"title" ,
									params, @"params" ,
									[NSNumber numberWithInt:arrayNo], @"arrayNo" ,
									[NSNumber numberWithInt:FETCH_TYPE_WATCH], @"fetchType" ,
									nil
							];
	[self getConnectionData:watchURL params:linkParams];

	return YES;

}
//------------------------------------
// createPreviewMovieView
//------------------------------------
- (BOOL)createPreviewMovieView:(NSString*)videoURL params:(NSDictionary*)params arrayNo:(int)arrayNo
{
	//
	// create movie view
	//
	NSString *message = @"";

	if([self createPreviewMovie:videoURL message:&message] == NO){

		[logStatusController appendLogString:message];

		int result = [self displayMessage:@"alert"
						messageText:@"Can not open video file"
						infoText:@"Please check error log"
						btnList:@"Cancel,Log"
				];

		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController setTitle:@"Error Log"];
			[logStatusController openLogWindow:nil];
		}else{
			[logStatusController setTitle:@""];
			[logStatusController setLogString:@""];
		}
		return NO;
	}

	// set params
	[self setParams:params];
	[self setArrayNo:arrayNo];

	return YES;
}
//------------------------------------
// createPreviewMovie
//------------------------------------
- (BOOL)createPreviewMovie:(NSString*)videoURL message:(NSString**)message
{

	if(!videoURL || [videoURL isEqualToString:@""]){
		*message = [NSString stringWithFormat:@"Can't create URL from %@ ", videoURL];
		return NO;
	}

	NSURL *movieURL = [NSURL URLWithString:videoURL];
	if(!movieURL){
		*message = [NSString stringWithFormat:@"Can't create URL from %@ ", videoURL];
		return NO;
	}

//	if([QTMovie canInitWithURL:movieURL] == NO){
//		NSLog(@"Can't init movie from %@ ", videoURL);
//		return NO;
//	}

	NSError *error = nil;
	QTMovie *movie = [QTMovie movieWithURL:movieURL error:&error];
	if(error != nil){
		*message = [NSString stringWithFormat:@"Can't create movie error= %@ ", error];
		return NO;
	}

	[self setMovie:movie];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handleMovieLoadStateDidChanged:) name:QTMovieLoadStateDidChangeNotification object:movie];
	[nc addObserver:self selector:@selector(handleMovieVolumeDidChanged:) name:QTMovieVolumeDidChangeNotification object:movie];
	[nc addObserver:self selector:@selector(handleMovieRateDidChanged:) name:QTMovieRateDidChangeNotification object:movie];

	return YES;

}
//------------------------------------
// getConnectionData
//------------------------------------
- (void)getConnectionData:(NSString*)urlString params:(NSDictionary*)params
{
//	NSLog(@"urlString=%@", urlString);

	// get video file
	[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																userInfo:nil
																notifyTarget:self
																params:params
								] autorelease]
	];
}

//------------------------------------
// handleDataItemStatus
//------------------------------------
- (void)handleDataItemStatus:(NSNotification *)notification
{

//	NSLog(@"handleDataItemStatus");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	//  indicator
	int status = [connectionDataItem status];

	if(status == CONNECTION_INIT || status == CONNECTION_STARTED){
		[viewMainSearch setLoadingStatus:YES tag:1];
	}
	else{
		[viewMainSearch setLoadingStatus:NO tag:1];
	}
}
//------------------------------------
// handleDataItemDidLoad
//------------------------------------
- (void)handleDataItemDidLoad:(NSNotification *)notification
{

//	NSLog(@"handleDataItemDidLoad");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	NSString *urlString = [connectionDataItem urlString];
	NSData *htmlData = [connectionDataItem data];
	NSError *error = [connectionDataItem error];
	NSDictionary *linkParams = [connectionDataItem params];

	NSString *videoId = [linkParams valueForKey:@"videoId"];
	NSString *title = [linkParams valueForKey:@"title"];
	NSDictionary *params = [linkParams valueForKey:@"params"];
	int arrayNo = [[linkParams valueForKey:@"arrayNo"] intValue];
	int fetchType = [[linkParams valueForKey:@"fetchType"] intValue];

	int result;

	[logStatusController setLogString:@""];

	if(error != nil){
		result = [self displayMessageAlertGetData];
		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController openLogWindowWithError:urlString error:error];
		}
		return;
	}

	//
	// get watch html
	//
	if(fetchType == FETCH_TYPE_WATCH){

		NSDictionary *linkParams =  [NSDictionary dictionaryWithObjectsAndKeys:
										videoId, @"videoId" ,
										title, @"title" ,
										params, @"params" ,
										[NSNumber numberWithInt:arrayNo], @"arrayNo" ,
										[NSNumber numberWithInt:FETCH_TYPE_VIDEO], @"fetchType" ,
										nil
									];
		// get video file
		NSString *getFileURL= [self convertToGetFileURL:videoId];
		[self getConnectionData:getFileURL params:linkParams];

	}

	//
	// get video file
	//
	else if(fetchType == FETCH_TYPE_VIDEO){

		NSString *html = [self encodeHTMLFromData:htmlData];

		//	NSLog(@"html=%@", html);
		if(html == nil){
			[self displayMessage:@"alert"
					messageText:[NSString stringWithFormat:@"Can not init html from = %@", urlString]
					infoText:@"May be encoding error."
					btnList:@"Cancel"
			];
			return;
		}

		NSString *videoFileURL = [self getVideoFileURLFromHTML:html];
//		NSLog(@"videoFileURL=%@", videoFileURL);
		if([videoFileURL isEqualToString:@""]){

			[logStatusController appendLogString:[NSString stringWithFormat:@"Can not get from=%@\n", videoFileURL]];
			[logStatusController appendLogString:@"You are not login?\n"];
			[logStatusController appendLogString:[NSString stringWithFormat:@"html=[%@]\n", html]];

			result = [self displayMessageAlertGetData];
			// show error log
			if(result == NSAlertSecondButtonReturn){
				[logStatusController setTitle:@"Error Log"];
				[logStatusController openLogWindow:nil];
			}else{
				[logStatusController setTitle:@""];
				[logStatusController setLogString:@""];
			}

			return;
		}

		// create movie view
		[self createPreviewMovieView:videoFileURL params:params arrayNo:arrayNo];

	}

}
//------------------------------------
// handleMovieLoadStateDidChanged
//------------------------------------
- (void)handleMovieLoadStateDidChanged:(NSNotification *)notification
{

	QTMovie *movie = [notification object];

	long loadState = [[movie attributeForKey:QTMovieLoadStateAttribute] longValue];

//	NSLog(@"loadState=%d", loadState);

	// kMovieLoadStatePlaythroughOK / 20000
	if (loadState >= kMovieLoadStatePlaythroughOK){

		[self setMovieVolume:[self defaultPreviewMovieVolume]];
		[self setMoviePlay:[self defaultPreviewMoviePlay]];

		// remove notification
		NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
		[nc removeObserver:self name:QTMovieLoadStateDidChangeNotification object:movie];
	}
	// kMovieLoadStatePlayable / 10000
	else if (loadState >= kMovieLoadStatePlayable){

	}
	// error / -1
	else if (loadState == -1){
//		NSLog(@"error loadState=%d", loadState);
	}

}
//------------------------------------
// handleMovieVolumeDidChanged
//------------------------------------
- (void)handleMovieVolumeDidChanged:(NSNotification *)notification
{

	QTMovie *movie = [notification object];

//	NSLog(@"handleMovieVolumeDidChanged");
	[self setDefaultFloatValue:[movie volume] key:defaultKeyPreviewMovieVolume];

}
//------------------------------------
// handleMovieRateDidChanged
//------------------------------------
- (void)handleMovieRateDidChanged:(NSNotification *)notification
{
	if([self isPlaying] == YES){
		[self setMovieVolume:[self defaultPreviewMovieVolume]];
	}
}

//------------------------------------
// handleFrameAnimDidChanged
//------------------------------------
- (void)handleFrameAnimDidChanged:(NSNotification *)notification
{

	BOOL isStarted = [[notification object] boolValue];

	if(isStarted == YES){
		if([self isPlaying] == YES){
			[self setMoviePlay:NO];
			[self setIsContinuePlaying:YES];
		}
	}else{
		if([self isContinuePlaying] == YES){
			[self setMoviePlay:YES];
			[self setIsContinuePlaying:NO];
		}
	}

}
//------------------------------------
// setMovie
//------------------------------------
- (void)setMovie:(QTMovie*)movie
{
    [previewMovieView setMovie:movie];
}
//------------------------------------
// setMoviePlay
//------------------------------------
- (void)setMoviePlay:(BOOL)isPlay
{
	if(isPlay == YES){
		// set volume
//		[self setMovieVolume:[self defaultPlayVolume]];
		[[previewMovieView movie] play];
	}else{
		[[previewMovieView movie] stop];
	}
}
//----------------------------------------
// setMovieVolume
//----------------------------------------
- (void)setMovieVolume:(float)volume
{
	// set volume
	[[previewMovieView movie] setVolume:volume];
}
//------------------------------------
// deleteMovie
//------------------------------------
- (void)deleteMovie
{
    [previewMovieView setMovie:nil];
}
//------------------------------------
// closeMovieView
//------------------------------------
- (void)closeMovieView
{

    [self setMoviePlay:NO];
	[self deleteMovie];
	[self clearConnection];
	[self setParams:nil];

}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqualToString:defaultKeyPreviewMovieVolume]){
		[self setMovieVolume:[self defaultPreviewMovieVolume]];
	}
}

//------------------------------------
// windowWillClosePlayer
//------------------------------------
-(void)windowWillClosePlayer:(NSNotification *)notification
{
	[self deleteMovie];

	// window rect
//	[self saveWindowRect:previewPlayerWindow key:@"rectWindowPreviewPlayer"];

}
//------------------------------------
// connectionDataItem
//------------------------------------
- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem
{
	[connectionDataItem retain];
	[connectionDataItem_ release];
	connectionDataItem_ = connectionDataItem;
}
- (ConnectionDataItem*)connectionDataItem
{
	return connectionDataItem_;
}
//------------------------------------
// clearConnection
//------------------------------------
- (void)clearConnection
{

	ConnectionDataItem *connectionDataItem = [self connectionDataItem];
	// cancel connection
	if(connectionDataItem != nil){
		if([connectionDataItem connection] != nil){
//			NSLog(@"cancelConnection");
			[connectionDataItem cancelConnection];
		}
		[self setConnectionDataItem:nil];
	}

}
//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
	[params_ release];
	params_ = params;
}
- (NSDictionary*)params
{
	return params_;
}
//------------------------------------
// isContinuePlaying
//------------------------------------
- (void)setIsContinuePlaying:(BOOL)isContinuePlaying
{
	isContinuePlaying_ = isContinuePlaying;
}
- (BOOL)isContinuePlaying
{
	return isContinuePlaying_;
}
//------------------------------------
// setArrayNo
//------------------------------------
- (void)setArrayNo:(int)arrayNo
{
	arrayNo_ = arrayNo;
}
- (int)arrayNo
{
	return arrayNo_;
}

//------------------------------------
// hasMovie
//------------------------------------
- (BOOL)hasMovie
{
	BOOL ret = NO;
	if([previewMovieView movie] != nil){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// isPlaying
//------------------------------------
- (BOOL)isPlaying
{
	return ([previewMovieView movie] != nil) && ([[previewMovieView movie] rate] != 0);
}
//------------------------------------
// boolPlaying
//------------------------------------
- (NSNumber*)boolPlaying
{
	return [NSNumber numberWithBool:[self isPlaying]];
}
//------------------------------------
// isMaximizedPreviewFrame
//------------------------------------
- (BOOL)isMaximizedPreviewFrame
{
	return [viewMainSearch isMaximizedPreviewFrame];
}
//------------------------------------
// hasParams
//------------------------------------
- (BOOL)hasParams
{
	BOOL ret = NO;
	if([self params] != nil){
		ret = YES;
	}
	return ret;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults removeObserver:self forKeyPath:defaultKeyPreviewMovieVolume];

	[params_ release];
    [super dealloc];
}

@end
