#import "ViewRelatedSearch.h"
#import "ViewRelatedWindow.h"
#import "ViewMainSearch.h"
#import "ViewTagSearch.h"
#import "ViewPlayer.h"
#import "ViewPreviewPlayer.h"
#import "ViewVideoContent.h"
#import "ViewPlaylistSetting.h"
#import "TBArrayController.h"
#import "DownloadManager.h"
#import "LogStatusController.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "NicoVideoExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewRelatedSearch

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// alloc itemList and set content
	itemList_ = [[NSMutableArray alloc] init];
	[relatedlistArrayController setContent:itemList_];

	[self setConnectionDataItem:nil];

	[self setUrlString:@""];
	[self setVideoId:@""];
	[self setPostUserId:@""];
	[self setSearchTitle:@""];
	[self setResultTitle:RESULT_TITLE_RELATED];
	[self setSearchType:SEARCH_RELATED_WITH_VIDEO];
	[self setLogString:@""];

	[self setStartIndex:0];
	[self setLastIndex:0];
	[self setTotalResults:0];
	[self setMaxResults:[self getMaxResultsRelated:SEARCH_RELATED_WITH_VIDEO]];

	[self setPageIndex:1];
	[self setPageMaxIndex:1];

	[self changePageButtonEnable];
	[self changeOrderButtonEnable];
	[self changeTabViewRelatedHeadIndex];

	[txtSearchResults setStringValue:@""];

	// set first responder
//	[relatedWindow makeFirstResponder:txtSearchField];

	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];
	[[btnInfo cell] setKeyEquivalent:@" "];
	[[btnInfo cell] setKeyEquivalentModifierMask:NSAlternateKeyMask];
	[btnSearchCancel setEnabled:NO];


	// set observer
	[relatedlistArrayController addObserver:self forKeyPath:@"selection" options:0 context:nil];

}
//=======================================================================
// Actions
//=======================================================================
//------------------------------------
// openRelatedSearch
//------------------------------------
- (IBAction)openRelatedSearch:(id)sender
{
	//	none
}
//------------------------------------
// searchRelatedVideo
//------------------------------------
- (IBAction)searchRelatedVideo:(id)sender
{
	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];

	[self searchWithRelatedVideo:videoId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_VIDEO];

}
//------------------------------------
// searchPostUserVideo
//------------------------------------
- (IBAction)searchPostUserVideo:(id)sender
{

	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record objectForKey:@"videoId"];
	NSString *postUserId = [record objectForKey:@"postUserId"];
	NSString *title = [record objectForKey:@"title"];

	if(!postUserId){postUserId =@"";}

	[self searchPostUser:videoId postUserId:postUserId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_POST_USER];

}

//------------------------------------
// searchRelatedTag
//------------------------------------
- (IBAction)searchRelatedTag:(id)sender
{

	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];

	if(![videoId isEqualToString:@""]){
		[viewRelatedWindow openTagSearch:nil];
		[viewTagSearch searchWithRelatedTag:videoId searchTitle:title pageIndex:1 searchType:SEARCH_TAG_WITH_VIDEO];
	}

}
//------------------------------------
// changeSearchPage
//------------------------------------
- (IBAction)changeSearchPage:(id)sender
{
	int tag = [sender tag];

	int pageIndex = [self pageIndex];
	int pageMaxIndex = [self pageMaxIndex];

	// prev
	if(tag == 0){
		if(pageIndex > 1){
			pageIndex--;
		}else{
			pageIndex = 1;
		}
	}
	// next
	else{
		if(pageIndex < pageMaxIndex){
			pageIndex++;
		}
	}

	// reload
	[self reloadWithPageIndex:pageIndex];

}
//------------------------------------
// moveSearchPage
//------------------------------------
- (IBAction)moveSearchPage:(id)sender
{
	int pageIndex = [[sender representedObject] intValue];

	// reload
	[self reloadWithPageIndex:pageIndex];

}

//------------------------------------
// reloadSearchPage
//------------------------------------
- (IBAction)reloadSearchPage:(id)sender
{

	// reload
	[self reloadWithPageIndex:1];

}
//------------------------------------
// cancelSearch
//------------------------------------
- (IBAction)cancelSearch:(id)sender
{
	[self clearConnection];
}

//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];

	// set params
	NSDictionary *params = record;

	if([viewPlayer setPlayerView:videoId title:title params:params arrayNo:BIND_ARRAY_RELATED] == YES){
		[viewPlayer openPlayerWindow:nil];
		int actionType = [self defaultPreviewActionType];
		if(actionType == 1){
			[viewPreviewPlayer setMoviePlay:NO];
		}
		else if(actionType == 2){
			[viewPreviewPlayer closeMovie:nil];
		}
	}
}
//------------------------------------
// previewItem
//------------------------------------
- (IBAction)previewItem:(id)sender
{
	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];

	// set params
	NSDictionary *params = record;

	// open preview
	if([viewPreviewPlayer setPreviewView:videoId title:title params:params arrayNo:BIND_ARRAY_RELATED] == YES){
		[viewMainSearch changeTabViewPreview:PREVIEW_TAB_INDEX_PREVIEW];
	}

}
//------------------------------------
// downloadItem
//------------------------------------
- (IBAction)downloadItem:(id)sender
{

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return;
	}

	NSArray *array = [relatedlistArrayController selectedObjects];
	// no select
	if([array count] <= 0){
		return;
	}
	// over count
	if([array count] > 1){
		[self displayMessage:@"alert"
					messageText:@"Can not start downloading over 1 files at the same time."
					infoText:@"Please download each files."
					btnList:@"Cancel"
		];
		return;
	}

	int i;
	id record;
	NSString *videoId;
	NSString *title;

	// open window
	[downloadManager openDownloadWindow:nil];

	for(i = 0; i < [array count]; i++){
		record = [array objectAtIndex:i];
		videoId = [record valueForKey:@"videoId"];
		title = [record valueForKey:@"title"];

		// start download
		[downloadManager startDownloadItem:videoId
									fileName:title
		];
	}

}

//------------------------------------
// openVideoPageWithBrowser
//------------------------------------
- (IBAction)openVideoPageWithBrowser:(id)sender
{

	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];

	// open watch url
	[self openVideoWatchURL:videoId];

}
//------------------------------------
// copyItemToPasteboard
//------------------------------------
- (IBAction)copyItemToPasteboard:(id)sender
{
	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];

	NSString *string = @"";
	NSString *url = [self convertToWatchURL:[record valueForKey:@"videoId"]];
	NSString *title = [record valueForKey:@"title"];

	// url
	if([sender tag] == 0){
		string = url;
	}
	// title
	else if([sender tag] == 1){
		string = title;
	}

	[self copyStringToPasteboard:string];

}
//------------------------------------
// showItemDescription
//------------------------------------
- (IBAction)showItemDescription:(id)sender
{
	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];

	NSString *title = [record valueForKey:@"title"];
	NSString *description = [record valueForKey:@"description"];

	[logStatusController setTitle:title];
	[logStatusController setLogString:description];
	[logStatusController openLogWindow:nil];

}
//------------------------------------
// showVideoContent
//------------------------------------
- (IBAction)showVideoContent:(id)sender
{
	// no select
	if([[relatedlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[relatedlistArrayController selectedObjects] objectAtIndex:0];

	[viewVideoContent createVideoContentWindow:(NSDictionary*)record arrayNo:BIND_ARRAY_RELATED  refViewNo:REF_VIEW_RELATED open:YES];

}
//------------------------------------
// addItemToPlaylist
//------------------------------------
- (IBAction)addItemToPlaylist:(id)sender
{

	NSManagedObject *targetItem = [sender representedObject];

	NSArray *objects = [relatedlistArrayController selectedObjects];

	// no select
	if([objects count] <= 0){
		return;
	}

	// create array for itemlist
 	NSMutableArray *items = [NSMutableArray array];
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		[items addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[item valueForKey:@"videoId"], @"videoId" ,
				[item valueForKey:@"title"], @"title" ,
				nil
			]
		];
	}

	[tbArrayController createItemlist:[targetItem valueForKey:@"plistId"] items:items];

}
//------------------------------------
// addPostUser
//------------------------------------
- (IBAction)addPostUser:(id)sender;
{

	NSString *postUserId = [self postUserId];
	NSString *title = [self searchTitle];

	if(![postUserId isEqualToString:@""]){
		[viewPlaylistSetting openPostUserNewSettingWindowWithValue:postUserId title:title];
	}

}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// searchPostUser
//------------------------------------
- (void)searchPostUser:(NSString*)videoId postUserId:(NSString*)postUserId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType
{

	// get postUserId
	if([postUserId isEqualToString:@""]){
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
									searchTitle, @"searchTitle" ,
									[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
									[NSNumber numberWithInt:searchType], @"searchType" ,
									nil
								];
		[self getVideoInfoItem:videoId params:params];
	}else{
		[self searchWithPostUser:postUserId searchTitle:searchTitle pageIndex:pageIndex searchType:searchType];
	}

}
//------------------------------------
// reloadWithPageIndex
//------------------------------------
- (void)reloadWithPageIndex:(int)pageIndex
{

	int searchType = [self searchType];

	if(searchType == SEARCH_RELATED_WITH_VIDEO){
		[self searchWithRelatedVideo:[self videoId] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType]];
	}
	else if(searchType == SEARCH_RELATED_WITH_POST_USER){
		[self searchWithPostUser:[self postUserId] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:searchType];
	}

}
//------------------------------------
// searchRelatedVideoWithItems
//------------------------------------
- (void)searchRelatedVideoWithItems:(NSArray*)items
{
	// no item
	if([items count] <= 0){
		return;
	}

	id record = [items objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];

	[viewRelatedWindow openRelatedSearch:nil];
	[self searchWithRelatedVideo:videoId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_VIDEO];

}
//------------------------------------
// searchPostUserWithItems
//------------------------------------
- (void)searchPostUserWithItems:(NSArray*)items
{
	// no item
	if([items count] <= 0){
		return;
	}

	id record = [items objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"];
	NSString *title = [record valueForKey:@"title"];
	NSString *postUserId = [record valueForKey:@"postUserId"];
	if(!postUserId){postUserId =@"";}

	[viewRelatedWindow openRelatedSearch:nil];
	[self searchPostUser:videoId postUserId:postUserId searchTitle:title pageIndex:1 searchType:SEARCH_RELATED_WITH_POST_USER];

}

//------------------------------------
// searchWithRelatedVideo
//------------------------------------
- (void)searchWithRelatedVideo:(NSString*)videoId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType
{

	if ((videoId != nil) && ([videoId length] > 0))
	{

		// clear connection
		[self clearConnection];

		BOOL isAdded = YES;
		int maxResults = [self getMaxResultsRelated:SEARCH_RELATED_WITH_VIDEO];

		// search url
		NSString *urlString = [self convertToRelatedVideoURL:videoId];

		// order string
		NSString *searchOrderString = [self defaultStringValue:@"optSearchRelatedOrderString"];
		if(![searchOrderString isEqualToString:@""]){
			urlString = [self appendStringToURL:urlString string:searchOrderString isAdded:&isAdded];
		}
		// order type
		int searchOrderType = [self defaultIntValue:@"optSearchRelatedOrderType"];
		if(searchOrderType == 1){
			urlString = [self appendStringToURL:urlString string:@"order=a" isAdded:&isAdded];
		}

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
												urlString, @"urlString" ,
												searchTitle, @"searchTitle" ,
												videoId, @"videoId" ,
												[self postUserId], @"postUserId" ,
												[NSNumber numberWithInt:searchType], @"searchType" ,
												[NSNumber numberWithInt:0], @"parseRssType" ,	// unused
												[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
												[NSNumber numberWithInt:maxResults], @"maxResults" ,
												nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithPostUser
//------------------------------------
- (void)searchWithPostUser:(NSString*)postUserId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType
{

	if ((postUserId != nil) && ([postUserId length] > 0))
	{

		// clear connection
		[self clearConnection];

		// search url
		BOOL isAdded = YES;
		int parseRssType = PARSE_RSS_TYPE_ATOM;
		int maxResults = [self getMaxResultsRelated:SEARCH_RELATED_WITH_POST_USER];

		NSString *urlString = [self convertToPostUserURL:postUserId withRss:YES rssType:parseRssType];

		// order string
		NSString *orderString = [self defaultStringValue:@"optPostUserOrderStringRelated"];

		if(![orderString isEqualToString:@""]){
			urlString = [self appendStringToURL:urlString string:orderString isAdded:&isAdded];
		}

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
												urlString, @"urlString" ,
												searchTitle, @"searchTitle" ,
												[self videoId], @"videoId" ,
												postUserId, @"postUserId" ,
												[NSNumber numberWithInt:searchType], @"searchType" ,
												[NSNumber numberWithInt:parseRssType], @"parseRssType" ,
												[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
												[NSNumber numberWithInt:maxResults], @"maxResults" ,
												nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// getVideoInfoItem
//------------------------------------
- (void)getVideoInfoItem:(NSString*)videoId params:(NSDictionary*)params
{

	[indProc startAnimation:nil];

	// alloc video info object
	[[[VideoInfoItem alloc] initWithVideo:videoId
									title:@""
									target:self
									notifType:VIDEO_ITEM_NOTIF_INFO
									hasImage:YES
									params:params
	] autorelease];

}
//------------------------------------
// handleItemObjectInfo
//------------------------------------
- (void)handleItemObjectInfo:(NSNotification *)notification
{

	[indProc stopAnimation:nil];

//	NSLog(@"handleItemObjectInfo");
	VideoInfoItem *videoInfoItem = [notification object];

	if(videoInfoItem == nil){
		[self displayMessage:@"alert"
						messageText:@"Can not get video info."
						infoText:@"Failed getting video info."
						btnList:@"Cancel"
						];
		return;
	}

//	NSLog(@"status=%d", [videoInfoItem status]);

	// success
	if([videoInfoItem itemStatus] == VIDEO_ITEM_SUCCESS){

		NSString *postUserId = [videoInfoItem postUserId];

		NSDictionary *params = [videoInfoItem params];
		NSString *searchTitle = [params valueForKey:@"searchTitle"];
		int pageIndex = [[params valueForKey:@"pageIndex"] intValue];
		int searchType = [[params valueForKey:@"searchType"] intValue];

		if([postUserId isEqualToString:@""]){
			[self displayMessage:@"alert"
							messageText:@"Can not get post user ID."
							infoText:@"Post user ID is empty."
							btnList:@"Cancel"
							];
			return;
		}

		// search
		[self searchWithPostUser:postUserId searchTitle:searchTitle pageIndex:pageIndex searchType:searchType];

	}
	//failed
	else{
		NSString *messageText = @"";
		if([videoInfoItem itemStatus] == VIDEO_ITEM_STATUS_FAILED){
			messageText = [NSString stringWithFormat:@"Failed getting video info.\n%@", [videoInfoItem errorDescription]];
		}else{
			messageText = @"Failed getting video info.";
		}
		[self displayMessage:@"alert"
				messageText:messageText
				infoText:@"Can not get post user ID."
				btnList:@"Cancel"
		];
		return;
	}
}
//------------------------------------
// handleDataItemStatus
//------------------------------------
- (void)handleDataItemStatus:(NSNotification *)notification
{

//	NSLog(@"handleDataItemStatus");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	//  indicator
	int status = [connectionDataItem status];

	if(status == CONNECTION_INIT || status == CONNECTION_STARTED){
		[indProc startAnimation:nil];
		[btnSearchCancel setEnabled:YES];
	}
	else{
		[indProc stopAnimation:nil];
		[btnSearchCancel setEnabled:NO];
	}
}
//------------------------------------
// handleDataItemDidLoad
//------------------------------------
- (void)handleDataItemDidLoad:(NSNotification *)notification
{

//	NSLog(@"handleDataItemDidLoad");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	NSData *xmlData = [connectionDataItem data];
	NSError *error = [connectionDataItem error];
	int result;

	// link params
	NSDictionary *params = [connectionDataItem params];
	NSString *urlString = [params objectForKey:@"urlString"];

	if(error != nil){
		result = [self displayMessageAlertGetData];
		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController openLogWindowWithError:urlString error:error];
		}
		return;
	}

	NSString *xml = [[[NSString alloc] initWithData:xmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"xml=%@", xml);
	if(xml == nil){
		[self displayMessage:@"alert"
				messageText:[NSString stringWithFormat:@"Can not init XML from = %@", urlString]
				infoText:@"May be encoding error."
				btnList:@"Cancel"
		];
		return;
	}
	if([xml length] <= 0){
		[self displayMessage:@"alert"
				messageText:@"Can not get html"
				infoText:@"May be server is busy."
				btnList:@"Cancel"
		];
		return;
	}

	// parse xml
	[self parseResultRss:xml params:params];

}
//------------------------------------
// parseResultRss
//------------------------------------
- (void)parseResultRss:(NSString*)xml params:(NSDictionary*)params
{

	// params
	NSString *urlString = [params objectForKey:@"urlString"];
	NSString *videoId = [params objectForKey:@"videoId"];
	NSString *postUserId = [params objectForKey:@"postUserId"];
	NSString *searchTitle = [params objectForKey:@"searchTitle"];
	int searchType = [[params objectForKey:@"searchType"] intValue];
	int parseRssType = [[params objectForKey:@"parseRssType"] intValue];
	int pageIndex = [[params objectForKey:@"pageIndex"] intValue];
	int maxResults = [[params objectForKey:@"maxResults"] intValue];
	int showTotal = 0;
	NSString *totalString = @"";

	// set status
	[self setUrlString:urlString];
	[self setSearchTitle:searchTitle];
	[self setVideoId:videoId];
	[self setPostUserId:postUserId];

	[self setSearchType:searchType];
	[self setPageIndex:pageIndex];
	[self setPageMaxIndex:pageIndex];
	[self setMaxResults:maxResults];

	// remove
	[self removeItemList];
	// clear log
	[self setLogString:@""];

//	if(xml != nil && [xml length] > 0){
		if(searchType == SEARCH_RELATED_WITH_VIDEO){

			if([self parseRelatedXML:xml maxResults:maxResults] == NO){
				int result = [self displayMessage:@"alert"
									messageText:@"Can not parse related XML."
									infoText:@""
									btnList:@"Cancel,Log"
							];
				// show error log
				if(result == NSAlertSecondButtonReturn){
					[logStatusController setTitle:@"Error Log"];
					[logStatusController setLogString:[self logString]];
					[logStatusController openLogWindow:nil];
				}
				return;
			}
			showTotal = 1;
		}
		else if(searchType == SEARCH_RELATED_WITH_POST_USER){

			if([self parseResultXML:xml
						maxResults:maxResults
						searchType:searchType
						parseRssType:parseRssType
			] == NO){
				// check not found
				if([self checkNotFoundHTML:xml] == YES){
					int result = [self displayMessageAlertNotFound];
					if(result == NSAlertSecondButtonReturn){
						[self openURLWithBrowser:urlString];
					}
					return;
				}
				// check log
				if([self checkIsExistString:[self logString] searchString:@"empty"] == NO){
					int result = [self displayMessage:@"alert"
										messageText:@"Can not parse user XML."
										infoText:@""
										btnList:@"Cancel,Log"
								];
					// show error log
					if(result == NSAlertSecondButtonReturn){
						[logStatusController setTitle:@"Error Log"];
						[logStatusController setLogString:[self logString]];
						[logStatusController openLogWindow:nil];
					}
				}
				return;
			}

			if([self pageMaxIndex] > [self pageIndex]){
				showTotal = 2;
				totalString = @"...";
			}
		}
//	}

	// reload
	[relatedlistArrayController rearrangeObjects];


	// change searched title
	if(searchType == SEARCH_RELATED_WITH_POST_USER){
		searchTitle = [self searchTitle];
	}

	// window title
	[self setWindowTitle:searchTitle searchType:searchType];

	// result string
	NSString *resultString = [self convertToResultString:[self startIndex]
												lastIndex:[self lastIndex]
												totalResults:[self totalResults]
												totalString:totalString
												showTotal:showTotal
							];
	[txtSearchResults setStringValue:resultString];


	// order button
	[self changeOrderButtonEnable];
	[self changeTabViewRelatedHeadIndex];

	// page button
	[self changePageButtonEnable];

}
//------------------------------------
// parseRelatedXML
//------------------------------------
- (BOOL)parseRelatedXML:(NSString*)xml maxResults:(int)maxResults
{

	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xml options:nil error:&error] autorelease];
	if(error != nil){
		[self setLogString:@"Can not parse result XML\n"];
		[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
		[self appendLogString:[NSString stringWithFormat: @"error=[%@]\n", error]];
		return NO;
	}

	// parse xml document
	NSXMLNode *rootNode = [xmlDoc rootElement];
//	NSLog(@"rootNode=%@", [rootNode name]);
	if(rootNode == nil){
		[self setLogString:@"Can not parse result XML\n"];
		[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
		[self appendLogString:@"error=[rootNode is null]\n"];
		return NO;
	}

	int index = ([self pageIndex] - 1) * maxResults;
	int startIndex = index + 1;
	int lastIndex = startIndex;

	NSString *total_count = [[self getChildNode:rootNode name:@"total_count"] stringValue];
	NSString *page_count = [[self getChildNode:rootNode name:@"page_count"] stringValue];
//	NSString *data_count = [[self getChildNode:rootNode name:@"data_count"] stringValue];
//	NSLog(@"total_count=%@", total_count);
//	NSLog(@"page_count=%@", page_count);
//	NSLog(@"data_count=%@", data_count);

	NSString *videoId;
	NSString *title;
	NSString *imageURL;

	NSXMLNode *childNode;

	NSEnumerator *e = [[rootNode children] objectEnumerator];
	while (childNode = [e nextObject]){

//		NSLog(@"name=%@", [childNode name]);

		// video
		if([[childNode name] isEqualToString:@"video"]){

			videoId = [self getVideoIdFromURL:[[self getChildNode:childNode name:@"url"] stringValue] setDefault:NO];
			title = [[self getChildNode:childNode name:@"title"] stringValue];
			imageURL = [[self getChildNode:childNode name:@"thumbnail"] stringValue];

			// no videoId
			if([videoId isEqualToString:@""]){
				continue;
			}

			index++;

			// replace web character
			title = [self replaceWebCharacters:title];

			// add object
			[self addItemList:[NSDictionary dictionaryWithObjectsAndKeys:
										[NSNumber numberWithInt:index], @"index" ,
										videoId, @"videoId" ,
										title, @"title" ,
										imageURL, @"imageURL" ,
										@"", @"postUserId" ,
										@"", @"postedDate" ,
										@"", @"publishedDate" ,
										@"", @"description" ,
										@"", @"errorDescription" ,
										@"", @"lastResBody" ,
										@"", @"length" ,
										@"", @"videoType" ,
										[NSNumber numberWithInt:0], @"viewCount" ,
										[NSNumber numberWithInt:0], @"resCount" ,
										[NSNumber numberWithInt:0], @"mylistCount" ,
										[NSNumber numberWithBool:NO], @"hasCountInfo" ,
										[NSNumber numberWithBool:NO], @"isPlayed" ,
										[NSNumber numberWithBool:YES], @"hasImage" ,
										[NSNumber numberWithBool:YES], @"getVideoInfo" ,
										[NSNumber numberWithInt:VIDEO_ITEM_INIT], @"itemStatus" ,
										nil
								]
			];


		}
	}

	// set index
	lastIndex = index;

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:[total_count intValue]];
	[self setPageMaxIndex:[page_count intValue]];

	return YES;
}
//------------------------------------
// parseResultXML
//------------------------------------
- (BOOL)parseResultXML:(NSString*)xml
			maxResults:(int)maxResults
			searchType:(int)searchType
			parseRssType:(int)parseRssType
{

	// replace & -> &amp;
	xml = [self replaceCharacter:xml str1:@"&" str2:@"&amp;"];

//	NSLog(@"xml=%@", xml);

	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xml options:nil error:&error] autorelease];
	if(error != nil){
		[self setLogString:@"Can not parse result XML\n"];
		[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
		[self appendLogString:[NSString stringWithFormat: @"error=[%@]\n", error]];
		return NO;
	}
	// parse xml document
	NSXMLNode *rootNode = [xmlDoc rootElement];
	if(rootNode == nil){
		[self setLogString:@"Can not parse result XML\n"];
		[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
		[self appendLogString:@"error=[rootNode is null]\n"];
		return NO;
	}
	// get parentNode
	NSXMLNode *parentNode = rootNode;
	if(parseRssType == PARSE_RSS_TYPE_2_0){
		parentNode = [self getChildNode:rootNode name:@"channel"];
		if(parentNode == nil){
			[self setLogString:@"Can not parse result XML\n"];
			[self appendLogString:[NSString stringWithFormat: @"url=[%@]\n", [self urlString]]];
			[self appendLogString:@"error=[parentNode is null]\n"];
			return NO;
		}
	}

	// node name
	NSString *childNodeName = @"entry";
	NSString *linkNodeName = @"link";
	NSString *contentNodeName = @"content";
	if(parseRssType == PARSE_RSS_TYPE_2_0){
		childNodeName = @"item";
		linkNodeName = @"link";
		contentNodeName = @"description";
	}

	NSXMLNode *childNode;
	NSXMLElement *elemLink;
	NSXMLElement *elemContent;

	NSString *videoId = @"";
	NSString *videoURL = @"";
	NSString *imageURL = @"";
	NSString *title = @"";
	NSString *searchTitle = @"";
	NSString *keyImage = @"class=\"nico-thumbnail\"";
	NSString *content;
	NSRange rng;

	NSArray *lines;
	NSString *line;
	int i;

	int pageIndex = [self pageIndex];
	int index = (pageIndex - 1) * maxResults;
	int startIndex = index + 1;
	int lastIndex = startIndex;
	int pageMaxIndex = [self pageMaxIndex];
	int count = 0;


	NSEnumerator *e = [[parentNode children] objectEnumerator];
	while (childNode = [e nextObject]){

		// user
		if(searchType == SEARCH_RELATED_WITH_POST_USER){
			// searchTitle
			if([[childNode name] isEqualToString:@"title"]){
				searchTitle = [self replaceCharacter:[childNode stringValue] str1:[self convertToShiftJISString:"]jRjR"] str2:@""];
			}
		}

		// entry
		if([[childNode name] isEqualToString:childNodeName]){

			// link
			elemLink = (NSXMLElement*)[self getChildNode:childNode name:linkNodeName];
			if(elemLink == nil){
				continue;
			}
			// videoURL
			if(parseRssType == PARSE_RSS_TYPE_ATOM){
				videoURL = [[elemLink attributeForName:@"href"] stringValue];
			}
			else if(parseRssType == PARSE_RSS_TYPE_2_0){
				videoURL = [elemLink stringValue];
			}
			else{
				continue;
			}

//			NSLog(@"videoURL=%@", videoURL);

			if(videoURL == nil || [videoURL length] <= 0){
				continue;
			}
			// videoId
			videoId = [self getVideoIdFromURL:videoURL setDefault:NO];
			if(videoId == nil || [videoId length] <= 0){
				continue;
			}
			// title / include rank num at ranking
			title = [[self getChildNode:childNode name:@"title"] stringValue];

			// content
			elemContent = (NSXMLElement*)[self getChildNode:childNode name:contentNodeName];
			if(elemContent != nil){

				content = [elemContent stringValue];
//				NSLog(@"content=%@", content);

				// line breaked array
				lines = [content componentsSeparatedByString:@"\n"];

				for(i = 0; i < [lines count]; i++){

					line = [lines objectAtIndex:i];
//					NSLog(@"line=%@", line);

					// imageURL
					rng = [line rangeOfString:keyImage];
					if(rng.length > 0){
						imageURL = [self getEnclosedString:line
													beginString:@"src=\""
													endString:@"\""
													withEndSet:NO
									];
						break;
					}

				}
			}

			index++;
			count++;

			// replace web character
			title = [self replaceWebCharacters:title];

			// add object
			[self addItemList:[NSDictionary dictionaryWithObjectsAndKeys:
										[NSNumber numberWithInt:index], @"index" ,
										videoId, @"videoId" ,
										title, @"title" ,
										imageURL, @"imageURL" ,
										@"", @"postUserId" ,
										@"", @"postedDate" ,
										@"", @"publishedDate" ,
										@"", @"description" ,
										@"", @"errorDescription" ,
										@"", @"lastResBody" ,
										@"", @"length" ,
										@"", @"videoType" ,
										[NSNumber numberWithInt:0], @"viewCount" ,
										[NSNumber numberWithInt:0], @"resCount" ,
										[NSNumber numberWithInt:0], @"mylistCount" ,
										[NSNumber numberWithBool:NO], @"hasCountInfo" ,
										[NSNumber numberWithBool:NO], @"isPlayed" ,
										[NSNumber numberWithBool:YES], @"hasImage" ,
										[NSNumber numberWithBool:YES], @"getVideoInfo" ,
										[NSNumber numberWithInt:VIDEO_ITEM_INIT], @"itemStatus" ,
										nil
								]
			];
		}
	}

	// set index
	lastIndex = index;

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:count];

	if(searchType == SEARCH_RELATED_WITH_POST_USER){
		if(count >= maxResults){
			pageMaxIndex = pageIndex + 1;
		}
	}
	[self setPageMaxIndex:pageMaxIndex];

	// user
	if(searchType == SEARCH_RELATED_WITH_POST_USER){
		[self setSearchTitle:searchTitle];
	}

	return YES;

}
//------------------------------------
// changeTabViewRelatedHeadIndex
//------------------------------------
- (void)changeTabViewRelatedHeadIndex
{
	int index = -1;
	int searchType = [self searchType];

	if(searchType == SEARCH_RELATED_WITH_VIDEO){
		index = 0;
	}
	else if(searchType == SEARCH_RELATED_WITH_POST_USER){
		index = 1;
	}

	if(index >= 0){
		[tabViewRelatedHead selectTabViewItemAtIndex:index];
	}

}
//------------------------------------
// changePageButtonEnable
//------------------------------------
- (void)changePageButtonEnable
{

	int pageIndex = [self pageIndex];
	int pageMaxIndex = [self pageMaxIndex];

	// page button
	// prev
	if(pageIndex > 1){
		[btnPagePrev setEnabled:YES];
	}else{
		[btnPagePrev setEnabled:NO];
	}
	// next
	if(pageIndex < pageMaxIndex){
		[btnPageNext setEnabled:YES];
	}else{
		[btnPageNext setEnabled:NO];
	}

}
//------------------------------------
// changeOrderButtonEnable
//------------------------------------
- (void)changeOrderButtonEnable
{

	int searchType = [self searchType];
	int totalResults = [self totalResults];

	if(searchType == SEARCH_RELATED_WITH_VIDEO){
		if(totalResults > 0){
			[pbtnSearchOrder setEnabled:YES];
			[pbtnSearchSort setEnabled:YES];
		}else{
			[pbtnSearchOrder setEnabled:NO];
			[pbtnSearchSort setEnabled:NO];
		}
		[pbtnPostUserOrder setEnabled:NO];
		[btnAddPostUser setEnabled:NO];
	}
	else if(searchType == SEARCH_RELATED_WITH_POST_USER){
		[pbtnSearchOrder setEnabled:NO];
		[pbtnSearchSort setEnabled:NO];
		if(totalResults > 0){
			[pbtnPostUserOrder setEnabled:YES];
			[btnAddPostUser setEnabled:YES];
		}else{
			[pbtnPostUserOrder setEnabled:NO];
			[btnAddPostUser setEnabled:NO];
		}
	}
}

//------------------------------------
// addItemList
//------------------------------------
- (void)addItemList:(NSDictionary*)object
{

//	NSLog(@"object=%@", [object description]);
	NSMutableDictionary *record = [[[NSMutableDictionary alloc] init] autorelease];

	BOOL hasImage = [[object objectForKey:@"hasImage"] boolValue];
	BOOL getVideoInfo = [[object objectForKey:@"getVideoInfo"] boolValue];
	NSString *imageURL = [object objectForKey:@"imageURL"];

	[record setValue:[object objectForKey:@"index"] forKey:@"index"];
	[record setValue:[object objectForKey:@"videoId"] forKey:@"videoId"];
	[record setValue:[object objectForKey:@"title"] forKey:@"title"];
	[record setValue:[object objectForKey:@"imageURL"] forKey:@"imageURL"];

	[record setValue:[object objectForKey:@"postUserId"] forKey:@"postUserId"];
	[record setValue:[object objectForKey:@"postedDate"] forKey:@"postedDate"];
	[record setValue:[object objectForKey:@"publishedDate"] forKey:@"publishedDate"];
	[record setValue:[object objectForKey:@"description"] forKey:@"description"];
	[record setValue:[object objectForKey:@"errorDescription"] forKey:@"errorDescription"];
	[record setValue:[object objectForKey:@"lastResBody"] forKey:@"lastResBody"];
	[record setValue:[object objectForKey:@"length"] forKey:@"length"];
	[record setValue:[object objectForKey:@"videoType"] forKey:@"videoType"];
	[record setValue:[object objectForKey:@"viewCount"] forKey:@"viewCount"];
	[record setValue:[object objectForKey:@"resCount"] forKey:@"resCount"];
	[record setValue:[object objectForKey:@"mylistCount"] forKey:@"mylistCount"];
	[record setValue:[object objectForKey:@"hasCountInfo"] forKey:@"hasCountInfo"];
	[record setValue:[object objectForKey:@"itemStatus"] forKey:@"itemStatus"];

	[record setValue:[object objectForKey:@"hasImage"] forKey:@"hasImage"];
	[record setValue:[object objectForKey:@"getVideoInfo"] forKey:@"getVideoInfo"];
	[record setValue:[NSNumber numberWithBool:NO] forKey:@"isPlayed"];

	// get video item
	if(getVideoInfo == YES){
		[record setValue:[[[VideoInfoItem alloc] initWithVideo:
									[object objectForKey:@"videoId"]
									title:[object objectForKey:@"title"]
									target:self
									notifType:VIDEO_ITEM_NOTIF_LIST
									hasImage:hasImage
									params:nil
						] autorelease]
				forKey:@"videoInfoItem"
		];
	}

	// get image
	if( hasImage == YES &&
		(imageURL && ![imageURL isEqualToString:@""])
	){
		[record setValue:[[[ImageItem alloc] initWithImageURL:imageURL notifType:IMAGE_ITEM_NOTIF_RELATED] autorelease] forKey:@"imageItem"];
	}

	[itemList_ addObject:record];

}

//------------------------------------
// removeItemList
//------------------------------------
- (void)removeItemList
{

	int i;
	// cancel all connection
	for(i = 0; i < [itemList_ count]; i++){
		id record = [itemList_ objectAtIndex:i];
		ImageItem *imageItem = [record objectForKey:@"imageItem"];
		if(imageItem){
			[imageItem cancelConnection];
		}
		BOOL getVideoInfo = [[record valueForKey:@"getVideoInfo"] boolValue];
		VideoInfoItem *videoInfoItem = [record valueForKey:@"videoInfoItem"];
		if(getVideoInfo == YES && videoInfoItem){
			[videoInfoItem cancelConnection];
		}
	}

	// remove all
	[itemList_ removeAllObjects];

	// reload
	[relatedlistArrayController rearrangeObjects];

}
//------------------------------------
// handleItemObjectChange
//------------------------------------
- (void)handleItemObjectChange:(NSNotification *)notification
{

//	NSLog(@"handleItemObjectChange");
	VideoInfoItem *videoInfoItem = [notification object];

	if(videoInfoItem == nil){
		return;
	}

	if(![videoInfoItem videoId]){
//		NSLog(@"videoId is invalid");
		return;
	}

	NSString *videoId = [videoInfoItem videoId];
	NSString *imageURL = [videoInfoItem imageURL];
	NSString *postUserId = [videoInfoItem postUserId];
	NSString *postedDate = [videoInfoItem postedDate];
	NSString *publishedDate = [videoInfoItem publishedDate];
	NSString *description = [videoInfoItem description];
	NSString *errorDescription = [videoInfoItem errorDescription];
	NSString *lastResBody = [videoInfoItem lastResBody];
	NSString *length = [videoInfoItem length];
	NSArray *tagList = [videoInfoItem tagList];
	BOOL hasImage = [videoInfoItem hasImage];
	int viewCount = [videoInfoItem viewCount];
	int resCount = [videoInfoItem resCount];
	int mylistCount = [videoInfoItem mylistCount];
	int itemStatus = [videoInfoItem itemStatus];

//	NSLog(@"videoId=%@", videoId);

	// fetch itemList_ with itemid
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"videoId == %@", videoId];
	NSArray *fetchedArray = [itemList_ filteredArrayUsingPredicate:pred];
//	NSLog(@"pred=%@", [pred description]);

	// if exists, set image item
	if([fetchedArray count] > 0){

		id record = [fetchedArray objectAtIndex:0];

		if(itemStatus == VIDEO_ITEM_SUCCESS){
			[record setValue:postUserId forKey:@"postUserId"];
			[record setValue:postedDate forKey:@"postedDate"];
			[record setValue:publishedDate forKey:@"publishedDate"];
			[record setValue:description forKey:@"description"];
			[record setValue:length forKey:@"length"];
			[record setValue:lastResBody forKey:@"lastResBody"];
			[record setValue:tagList forKey:@"tagList"];
			[record setValue:[NSNumber numberWithInt:viewCount] forKey:@"viewCount"];
			[record setValue:[NSNumber numberWithInt:resCount] forKey:@"resCount"];
			[record setValue:[NSNumber numberWithInt:mylistCount] forKey:@"mylistCount"];
			[record setValue:[NSNumber numberWithBool:YES] forKey:@"hasCountInfo"];

			// set image object
			if( hasImage == NO &&
				imageURL && ![imageURL isEqualToString:@""]
			){
				[record setValue:[[[ImageItem alloc] initWithImageURL:imageURL notifType:IMAGE_ITEM_NOTIF_RELATED] autorelease] forKey:@"imageItem"];
			}
		}

		[record setValue:[NSNumber numberWithInt:itemStatus] forKey:@"itemStatus"];
		[record setValue:errorDescription forKey:@"errorDescription"];

	}

	// reload
	[relatedlistArrayController rearrangeObjects];

}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// change review image
	if([keyPath isEqualToString:@"selection"]){
	}
}
//------------------------------------
// windowWillClose
//------------------------------------
- (void)windowWillClose
{
	// none
}
//------------------------------------
// setWindowTitle
//------------------------------------
- (void)setWindowTitle:(NSString*)searchTitle searchType:(int)searchType
{

	NSString *string = RESULT_TITLE_RELATED;

	if(![searchTitle isEqualToString:@""]){
		string = searchTitle;
		if([string length] > 20){
			string = [NSString stringWithFormat:@"%@..", [string substringToIndex:20]];
		}

		// related
		if(searchType == SEARCH_RELATED_WITH_VIDEO){
			string = [string stringByAppendingString:[self convertToShiftJISString:" ̊֘A"]];
		}
		// user
		else if(searchType == SEARCH_RELATED_WITH_POST_USER){
			string = [NSString stringWithFormat:@"%@ / %@", string, [self postUserId]];
		}
	}

	[self setResultTitle:string];
	[viewRelatedWindow setWindowTitle:string];
}
//------------------------------------
// connectionDataItem
//------------------------------------
- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem
{
	[connectionDataItem retain];
	[connectionDataItem_ release];
	connectionDataItem_ = connectionDataItem;
}
- (ConnectionDataItem*)connectionDataItem
{
	return connectionDataItem_;
}
//------------------------------------
// urlString
//------------------------------------
- (void)setUrlString:(NSString*)urlString
{
	[urlString retain];
	[urlString_ release];
	urlString_ = urlString;
}
- (NSString*)urlString
{
	return urlString_;
}
//------------------------------------
// videoId
//------------------------------------
- (void)setVideoId:(NSString*)videoId
{
	[videoId retain];
	[videoId_ release];
	videoId_ = videoId;
}
- (NSString*)videoId
{
	return videoId_;
}
//------------------------------------
// postUserId
//------------------------------------
- (void)setPostUserId:(NSString*)postUserId
{
	[postUserId retain];
	[postUserId_ release];
	postUserId_ = postUserId;
}
- (NSString*)postUserId
{
	return postUserId_;
}
//------------------------------------
// searchTitle
//------------------------------------
- (void)setSearchTitle:(NSString*)searchTitle
{
	[searchTitle retain];
	[searchTitle_ release];
	searchTitle_ = searchTitle;
}
- (NSString*)searchTitle
{
	return searchTitle_;
}
//------------------------------------
// resultTitle
//------------------------------------
- (void)setResultTitle:(NSString*)resultTitle
{
	[resultTitle retain];
	[resultTitle_ release];
	resultTitle_ = resultTitle;
}
- (NSString*)resultTitle
{
	return resultTitle_;
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}

//------------------------------------
// searchType
//------------------------------------
- (void)setSearchType:(int)searchType
{
	searchType_ = searchType;
}
- (int)searchType
{
	return searchType_;
}

//------------------------------------
// startIndex
//------------------------------------
- (void)setStartIndex:(int)startIndex
{
	startIndex_ = startIndex;
}
- (int)startIndex
{
	return startIndex_;
}
//------------------------------------
// lastIndex
//------------------------------------
- (void)setLastIndex:(int)lastIndex
{
	lastIndex_ = lastIndex;
}
- (int)lastIndex
{
	return lastIndex_;
}
//------------------------------------
// totalResults
//------------------------------------
- (void)setTotalResults:(int)totalResults
{
	totalResults_ = totalResults;
}
- (int)totalResults
{
	return totalResults_;
}
//------------------------------------
// maxResults
//------------------------------------
- (void)setMaxResults:(int)maxResults
{
	maxResults_ = maxResults;
}
- (int)maxResults
{
	return maxResults_;
}
//------------------------------------
// pageIndex
//------------------------------------
- (void)setPageIndex:(int)pageIndex
{
	pageIndex_ = pageIndex;
}
- (int)pageIndex
{
	return pageIndex_;
}
//------------------------------------
// pageMaxIndex
//------------------------------------
- (void)setPageMaxIndex:(int)pageMaxIndex
{
	pageMaxIndex_ = pageMaxIndex;
}
- (int)pageMaxIndex
{
	return pageMaxIndex_;
}
//------------------------------------
// clearConnection
//------------------------------------
- (void)clearConnection
{

	ConnectionDataItem *connectionDataItem = [self connectionDataItem];
	// cancel connection
	if(connectionDataItem != nil){
		if([connectionDataItem connection] != nil){
//			NSLog(@"cancelConnection");
			[connectionDataItem cancelConnection];
		}
		[self setConnectionDataItem:nil];
	}

}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[relatedlistArrayController removeObserver:self forKeyPath:@"selection"];
	[urlString_ release];
	[videoId_ release];
	[postUserId_ release];
	[searchTitle_ release];
	[resultTitle_ release];
	[logString_ release];
	[itemList_ release];
	[connectionDataItem_ release];
	[super dealloc];
}

@end

