#import "ViewRelatedWindow.h"
#import "ViewRelatedSearch.h"
#import "ViewTagSearch.h"
#import "ViewPlayHistory.h"
#import "TBArrayController.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation ViewRelatedWindow

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window position
	[self setWindowRect:relatedWindow key:@"rectWindowRelated"];

	// delete focus ring
	[btnBarVideo setFocusRingType:NSFocusRingTypeNone];
	[btnBarTag setFocusRingType:NSFocusRingTypeNone];

	// set first responder
	[relatedWindow makeFirstResponder:tabViewMain];

	[self changeButtonBar:RESULT_INDEX_RELATED];
	[self changeWindowTitle:RESULT_INDEX_RELATED];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:relatedWindow];
}
//------------------------------------
// openRelatedWindow
//------------------------------------
- (IBAction)openRelatedWindow:(id)sender
{
	[relatedWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// changeTabViewMain
//------------------------------------
- (IBAction)changeTabViewMain:(id)sender
{
	int tag = [sender tag];

	[tabViewMain selectTabViewItemAtIndex:tag];
	[self changeButtonBar:tag];
	[self changeWindowTitle:tag];

}
//------------------------------------
// closeRelatedWindow
//------------------------------------
- (IBAction)closeRelatedWindow:(id)sender
{
	[relatedWindow performClose:nil];
}
//------------------------------------
// openRelatedSearch
//------------------------------------
- (IBAction)openRelatedSearch:(id)sender
{
	[tabViewMain selectTabViewItemAtIndex:RESULT_INDEX_RELATED];
	[self changeButtonBar:RESULT_INDEX_RELATED];
	[self changeWindowTitle:RESULT_INDEX_RELATED];

	[self openRelatedWindow:nil];
}
//------------------------------------
// openTagSearch
//------------------------------------
- (IBAction)openTagSearch:(id)sender
{
	[tabViewMain selectTabViewItemAtIndex:RESULT_INDEX_TAG];
	[self changeButtonBar:RESULT_INDEX_TAG];
	[self changeWindowTitle:RESULT_INDEX_TAG];

	[self openRelatedWindow:nil];
}
//------------------------------------
// openPlayHistory
//------------------------------------
- (IBAction)openPlayHistory:(id)sender
{
	[tabViewMain selectTabViewItemAtIndex:RESULT_INDEX_HISTORY];
	[self changeButtonBar:RESULT_INDEX_HISTORY];
	[self changeWindowTitle:RESULT_INDEX_HISTORY];

	[self openRelatedWindow:nil];
}

//------------------------------------
// changeButtonBar
//------------------------------------
- (void)changeButtonBar:(int)index
{

	BOOL enableVideo = YES;
	BOOL enableTag = YES;
	BOOL enableHistory = YES;

	if(index == RESULT_INDEX_RELATED){
		enableVideo = NO;
	}
	else if(index == RESULT_INDEX_TAG){
		enableTag = NO;
	}
	else if(index == RESULT_INDEX_HISTORY){
		enableHistory = NO;
	}

	[btnBarVideo setEnabled:enableVideo];
	[btnBarTag setEnabled:enableTag];
	[btnBarHistory setEnabled:enableHistory];

/*
	// image
	NSString *nameVideo = @"btn_bar_video_off";
	NSString *nameTag = @"btn_bar_tag_off";
	NSString *nameHistory = @"btn_bar_history_off";
	if(index == RESULT_INDEX_RELATED){
		nameVideo = @"btn_bar_video_on";
	}
	else if(index == RESULT_INDEX_TAG){
		nameTag = @"btn_bar_tag_on";
	}
	else if(index == RESULT_INDEX_HISTORY){
		nameHistory = @"btn_bar_history_on";
	}

	[btnBarVideo setImage:[NSImage imageNamed:nameVideo]];
	[btnBarTag setImage:[NSImage imageNamed:nameTag]];
	[btnBarHistory setImage:[NSImage imageNamed:nameHistory]];
*/

}
//------------------------------------
// changeWindowTitle
//------------------------------------
- (void)changeWindowTitle:(int)index
{
	NSString *title = @"";

	if(index == RESULT_INDEX_RELATED){
		title = [viewRelatedSearch resultTitle];
		if(title == nil){
			title = RESULT_TITLE_RELATED;
		}
	}
	else if(index == RESULT_INDEX_TAG){
		title = [viewTagSearch resultTitle];
		if(title == nil){
			title = RESULT_TITLE_TAG;
		}
	}
	else if(index == RESULT_INDEX_HISTORY){
		title = RESULT_TITLE_HISTORY;
	}

	[self setWindowTitle:title];

}
//------------------------------------
// setWindowTitle
//------------------------------------
- (void)setWindowTitle:(NSString*)title
{
	[relatedWindow setTitle:title];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	[self saveWindowRect:relatedWindow key:@"rectWindowRelated"];

//	[viewRelatedSearch windowWillClose];
	[viewTagSearch windowWillClose];
//	[viewPlayHistory windowWillClose];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
