/* ViewTagSearch */

#import <Cocoa/Cocoa.h>
#import "SearchTypes.h"
#import "SearchTagTypes.h"
#import "RelatedWindowStatus.h"
#import "ConnectionDataItem.h"

@interface ViewTagSearch : NSObject
{

	IBOutlet id viewRelatedWindow;
	IBOutlet id viewMainSearch;
	IBOutlet id logStatusController;
	IBOutlet NSArrayController *taglistArrayController;

	IBOutlet NSSearchField *txtSearchField;
	IBOutlet NSTextField *txtSearchResults;
	IBOutlet NSButton *btnPagePrev;
	IBOutlet NSButton *btnPageNext;
	IBOutlet NSButton *btnMajorTag;
	IBOutlet NSButton *btnSearch;
	IBOutlet NSButton *btnSearchCancel;

	IBOutlet NSProgressIndicator *indProc;

	ConnectionDataItem *connectionDataItem_;
	NSMutableArray *tagList_;

	NSString *urlString_;
	NSString *searchString_;
	NSString *contentsName_;
	NSString *videoId_;
	NSString *searchTitle_;
	NSString *resultTitle_;
	NSString *logString_;
	
	int startIndex_;
	int lastIndex_;
	int totalResults_;
	int maxResults_;

	int pageIndex_;
	int pageMaxIndex_;

	int searchType_;

}
- (IBAction)openTagSearch:(id)sender;
- (IBAction)searchWithKeyword:(id)sender;
- (IBAction)searchWithMajorTag:(id)sender;
- (IBAction)searchItemWithTag:(id)sender;

- (IBAction)changeSearchPage:(id)sender;
- (IBAction)moveSearchPage:(id)sender;
- (IBAction)reloadSearchPage:(id)sender;
- (IBAction)cancelSearch:(id)sender;

- (IBAction)searchItem:(id)sender;
- (IBAction)openDictPageWithBrowser:(id)sender;
- (IBAction)copyItemToPasteboard:(id)sender;

- (void)reloadWithPageIndex:(int)pageIndex;
- (void)searchWithItems:(NSArray*)items;
- (void)searchWithString:(NSString*)searchString pageIndex:(int)pageIndex searchType:(int)searchType;
- (void)searchWithContentsName:(NSString*)contentsName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType;
- (void)searchWithRelatedTag:(NSString*)videoId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType;

- (void)parseResultXML:(NSString*)xml params:(NSDictionary*)params;
- (int)parseTagXML:(NSString*)xml;
- (void)parseResultHTML:(NSString*)html params:(NSDictionary*)params;
- (void)parseRelatedTagHTML:(NSString*)html;
- (void)parseContentsHTML:(NSString*)html;
- (void)parsePageStatusHTML:(NSString*)html;
- (NSDictionary*)splitResultHTML:(NSString*)html splitString:(NSString*)splitString;

- (int)getFontSize:(NSString*)levelStr;

- (void)changePageButtonEnable;
- (void)changeMajorTagButtonState;

- (void)handleDataItemStatus:(NSNotification *)notification;
- (void)handleDataItemDidLoad:(NSNotification *)notification;
- (void)clearConnection;

- (void)removeTagList;
- (void)windowWillClose;

- (void)setSearchStatus:(NSDictionary*)params;
- (void)setWindowStatus:(NSDictionary*)params;
- (void)setWindowTitle:(NSString*)videoId searchTitle:(NSString*)searchTitle searchType:(int)searchType;

- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem;
- (ConnectionDataItem*)connectionDataItem;

- (void)setUrlString:(NSString*)urlString;
- (NSString*)urlString;
- (void)setSearchString:(NSString*)searchString;
- (NSString*)searchString;
- (void)setContentsName:(NSString*)contentsName;
- (NSString*)contentsName;
- (void)setVideoId:(NSString*)videoId;
- (NSString*)videoId;
- (void)setSearchTitle:(NSString*)searchTitle;
- (NSString*)searchTitle;
- (void)setResultTitle:(NSString*)resultTitle;
- (NSString*)resultTitle;

- (void)setLogString:(NSString *)logString;
- (NSString *)logString;
- (void)appendLogString:(NSString *)logString;

- (void)setSearchType:(int)searchType;
- (int)searchType;

- (void)setStartIndex:(int)startIndex;
- (int)startIndex;
- (void)setLastIndex:(int)lastIndex;
- (int)lastIndex;
- (void)setTotalResults:(int)totalResults;
- (int)totalResults;
- (void)setMaxResults:(int)maxResults;
- (int)maxResults;
- (void)setPageIndex:(int)pageIndex;
- (int)pageIndex;
- (void)setPageMaxIndex:(int)pageMaxIndex;
- (int)pageMaxIndex;

@end
