#import "ViewTagSearch.h"
#import "ViewRelatedWindow.h"
#import "ViewMainSearch.h"
#import "LogStatusController.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "NicoVideoExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewTagSearch

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// recent searches
	[self setSearchFieldRecentSearches:txtSearchField key:@"arrayRecentTagSearches"];

	// alloc tagList and set content
	tagList_ = [[NSMutableArray alloc] init];
	[taglistArrayController setContent:tagList_];

	[self setConnectionDataItem:nil];

	[self setUrlString:@""];
	[self setSearchString:@""];
	[self setContentsName:@""];
	[self setVideoId:@""];
	[self setSearchTitle:@""];
	[self setResultTitle:RESULT_TITLE_TAG];
	[self setSearchType:SEARCH_TAG_WITH_STRING];
	[self setLogString:@""];

	[self setStartIndex:0];
	[self setLastIndex:0];
	[self setTotalResults:0];
	[self setMaxResults:100];

	[self setPageIndex:1];
	[self setPageMaxIndex:1];

	[self changePageButtonEnable];
	[self changeMajorTagButtonState];

	[txtSearchResults setStringValue:@""];

	// set first responder
//	[tagWindow makeFirstResponder:txtSearchField];

	// set key equivalent
	[btnSearch setKeyEquivalent:@" "];
	[btnSearchCancel setEnabled:NO];

}
//=======================================================================
// Actions
//=======================================================================
//------------------------------------
// openTagSearch
//------------------------------------
- (IBAction)openTagSearch:(id)sender
{
	//	none
}
//------------------------------------
// searchWithKeyword
//------------------------------------
- (IBAction)searchWithKeyword:(id)sender
{

	[self searchWithString:[txtSearchField stringValue] pageIndex:1 searchType:SEARCH_TAG_WITH_STRING];

}
//------------------------------------
// searchWithMajorTag
//------------------------------------
- (IBAction)searchWithMajorTag:(id)sender
{
	NSString *title = [self convertToShiftJISString:TAG_CONTENTS_TITLE_MAJOR_TAG];
	[self searchWithContentsName:TAG_CONTENTS_NAME_MAJOR_TAG searchTitle:title pageIndex:1 searchType:SEARCH_TAG_WITH_CONTENTS];

}
//------------------------------------
// searchItemWithTag
//------------------------------------
- (IBAction)searchItemWithTag:(id)sender
{

	// no select
	if([[taglistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[taglistArrayController selectedObjects] objectAtIndex:0];
	NSString *tagName = [record objectForKey:@"tagName"]; 

	[self searchWithString:tagName pageIndex:1 searchType:SEARCH_TAG_WITH_STRING];

}

//------------------------------------
// changeSearchPage
//------------------------------------
- (IBAction)changeSearchPage:(id)sender
{
	int tag = [sender tag];

	int pageIndex = [self pageIndex];
	int pageMaxIndex = [self pageMaxIndex];

	// prev
	if(tag == 0){
		if(pageIndex > 1){
			pageIndex--;
		}else{
			pageIndex = 1;
		}
	}
	// next
	else{
		if(pageIndex < pageMaxIndex){
			pageIndex++;
		}
	}

	// reload
	[self reloadWithPageIndex:pageIndex];

}
//------------------------------------
// moveSearchPage
//------------------------------------
- (IBAction)moveSearchPage:(id)sender
{
	int pageIndex = [[sender representedObject] intValue];

	// reload
	[self reloadWithPageIndex:pageIndex];

}
//------------------------------------
// reloadSearchPage
//------------------------------------
- (IBAction)reloadSearchPage:(id)sender
{

	// reload
	[self reloadWithPageIndex:1];

}

//------------------------------------
// cancelSearch
//------------------------------------
- (IBAction)cancelSearch:(id)sender
{
	[self clearConnection];
}
//------------------------------------
// searchItem
//------------------------------------
- (IBAction)searchItem:(id)sender
{
	// no select
	if([[taglistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[taglistArrayController selectedObjects] objectAtIndex:0];
	NSString *tagName = [record objectForKey:@"tagName"]; 

	// change default
	[self setDefaultIntValue:SEARCH_WITH_STRING_TAG key:@"optSearchKeywordType"];

//	[viewMainSearch openMainWindow:nil];	
	[viewMainSearch searchWithString:tagName pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:NO];

}
//------------------------------------
// openDictPageWithBrowser
//------------------------------------
- (IBAction)openDictPageWithBrowser:(id)sender
{

	// no select
	if([[taglistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[taglistArrayController selectedObjects] objectAtIndex:0];
	NSString *tagName = [record objectForKey:@"tagName"]; 

	// open dict url
	[self openDictionaryURL:tagName];

}
//------------------------------------
// copyItemToPasteboard
//------------------------------------
- (IBAction)copyItemToPasteboard:(id)sender
{
	// no select
	if([[taglistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[taglistArrayController selectedObjects] objectAtIndex:0];

	NSString *tagName = [record valueForKey:@"tagName"];

	[self copyStringToPasteboard:tagName];

}
//------------------------------------
// changeQueryButton
//------------------------------------
- (IBAction)changeQueryButton:(id)sender
{
	[self reloadSearchPage:nil];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// reloadWithPageIndex
//------------------------------------
- (void)reloadWithPageIndex:(int)pageIndex
{

	if([self searchType] == SEARCH_TAG_WITH_STRING){
		[self searchWithString:[self searchString] pageIndex:pageIndex searchType:[self searchType]];
	}
	else if([self searchType] == SEARCH_TAG_WITH_CONTENTS){
		[self searchWithContentsName:[self contentsName] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType]];
	}
	else if([self searchType] == SEARCH_TAG_WITH_VIDEO){
		[self searchWithRelatedTag:[self videoId] searchTitle:[self searchTitle] pageIndex:pageIndex searchType:[self searchType]];
	}

}
//------------------------------------
// searchWithItems
//------------------------------------
- (void)searchWithItems:(NSArray*)items
{
	// no item
	if([items count] <= 0){
		return;
	}

	id record = [items objectAtIndex:0];
	NSString *videoId = [record valueForKey:@"videoId"]; 
	NSString *title = [record valueForKey:@"title"]; 

	[viewRelatedWindow openTagSearch:nil];
	[self searchWithRelatedTag:videoId searchTitle:title pageIndex:1 searchType:SEARCH_TAG_WITH_VIDEO];

}
//------------------------------------
// searchWithString
//------------------------------------
- (void)searchWithString:(NSString*)searchString pageIndex:(int)pageIndex searchType:(int)searchType
{

	if ((searchString != nil) && ([searchString length] > 0))
	{
		// clear connection
		[self clearConnection];

		BOOL isAdded = NO;

		// search url
		NSString *urlString = @"";

		urlString = [self convertToRelatedTagURL:searchString];

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];			
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
												urlString, @"urlString" ,
												searchString, @"searchTitle" ,
												searchString, @"searchString" ,
												[self contentsName], @"contentsName" ,
												[self videoId],  @"videoId" ,
												[NSNumber numberWithInt:searchType], @"searchType" ,
												[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
												nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithContentsName
//------------------------------------
- (void)searchWithContentsName:(NSString*)contentsName searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType
{

	if ((contentsName != nil) && ([contentsName length] > 0))
	{

		BOOL isAdded = NO;

		// search url
		NSString *urlString = [self convertToContentsURL:contentsName withRss:NO rssType:PARSE_RSS_TYPE_NONE];

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];			
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
												urlString, @"urlString" ,
												searchTitle, @"searchTitle" ,
												[self searchString], @"searchString" ,
												[self videoId],  @"videoId" ,
												contentsName, @"contentsName" ,
												[NSNumber numberWithInt:searchType], @"searchType" ,
												[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
												nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// searchWithRelatedTag
//------------------------------------
- (void)searchWithRelatedTag:(NSString*)videoId searchTitle:(NSString*)searchTitle pageIndex:(int)pageIndex searchType:(int)searchType
{

	if ((videoId != nil) && ([videoId length] > 0))
	{

		BOOL isAdded = NO;

		// search url
		NSString *urlString = [self convertToThumbInfoURL:videoId];

		// page string
		if(pageIndex > 1){
			NSString *pageString = [NSString stringWithFormat:@"page=%d", pageIndex];
			urlString = [self appendStringToURL:urlString string:pageString isAdded:&isAdded];			
		}

//		NSLog(@"urlString=%@", urlString);

		// set link params
		NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
												urlString, @"urlString" ,
												searchTitle, @"searchTitle" ,
												[self searchString], @"searchString" ,
												videoId,  @"videoId" ,
												[self contentsName], @"contentsName" ,
												[NSNumber numberWithInt:searchType], @"searchType" ,
												[NSNumber numberWithInt:pageIndex], @"pageIndex" ,
												nil
								];

		// get html data
		[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:urlString
																	userInfo:nil
																	notifyTarget:self
																	params:params
									] autorelease]
		];

	}

}
//------------------------------------
// handleDataItemStatus
//------------------------------------
- (void)handleDataItemStatus:(NSNotification *)notification
{

//	NSLog(@"handleDataItemStatus");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	//  indicator
	int status = [connectionDataItem status];

	if(status == CONNECTION_INIT || status == CONNECTION_STARTED){
		[indProc startAnimation:nil];
		[btnSearchCancel setEnabled:YES];
	}
	else{
		[indProc stopAnimation:nil];
		[btnSearchCancel setEnabled:NO];
	}
}
//------------------------------------
// handleDataItemDidLoad
//------------------------------------
- (void)handleDataItemDidLoad:(NSNotification *)notification
{

//	NSLog(@"handleDataItemDidLoad");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	NSData *htmlData = [connectionDataItem data];
	NSError *error = [connectionDataItem error];
	int result;

	// link params
	NSDictionary *params = [connectionDataItem params];
	NSString *urlString = [params objectForKey:@"urlString"];
	int searchType = [[params objectForKey:@"searchType"] intValue];

	if(error != nil){
		result = [self displayMessageAlertGetData];
		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController openLogWindowWithError:urlString error:error];
		}
		return;
	}

	NSString *html = [[[NSString alloc] initWithData:htmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"html=%@", html);
	if(html == nil){
		[self displayMessage:@"alert"
				messageText:[NSString stringWithFormat:@"Can not init html from = %@", urlString]
				infoText:@"May be encoding error."
				btnList:@"Cancel"
		];
		return;
	}

	if([html length] <= 0){
		[self displayMessage:@"alert"
				messageText:@"Can not get html"
				infoText:@"May be server is busy."
				btnList:@"Cancel"
		];
		return;
	}

	// parse html
	if(searchType == SEARCH_TAG_WITH_STRING || searchType == SEARCH_TAG_WITH_CONTENTS){
		[self parseResultHTML:html params:params];
	}
	// parse xml
	else if(searchType == SEARCH_TAG_WITH_VIDEO){
		[self parseResultXML:html params:params];
	}


}
//------------------------------------
// parseResultXML
//------------------------------------
- (void)parseResultXML:(NSString*)xml params:(NSDictionary*)params
{

	// params
	int searchType = [[params objectForKey:@"searchType"] intValue];

	// set status
	[self setSearchStatus:params];

	// remove
	[self removeTagList];
	// clear log
	[self setLogString:@""];

//	if(xml != nil && [xml length] > 0){
		if(searchType == SEARCH_TAG_WITH_VIDEO){

			if([self parseTagXML:xml] == TAG_ITEM_FAILED){
				int result = [self displayMessage:@"alert"
									messageText:@"Can not parse tag info XML."
									infoText:@""
									btnList:@"Cancel,Log"
							];
				// show error log
				if(result == NSAlertSecondButtonReturn){
					[logStatusController setTitle:@"Error Log"];
					[logStatusController setLogString:[self logString]];
					[logStatusController openLogWindow:nil];
				}
				return;
			}
		}
//	}

	// set status
	[self setWindowStatus:params];

	// result string
	NSString *resultString = [self convertToResultString:[self startIndex]
												lastIndex:[self lastIndex]
												totalResults:[self totalResults]
												totalString:@""
												showTotal:1
							];
	[txtSearchResults setStringValue:resultString];

	// reload
	[taglistArrayController rearrangeObjects];


	// page button
//	[self changePageButtonEnable];
//	[self changeMajorTagButtonState];

}
//------------------------------------
// parseTagXML
//------------------------------------
- (int)parseTagXML:(NSString*)xml
{

	NSEnumerator *e;
	NSXMLNode *childNode;

	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xml options:nil error:&error] autorelease];
	if(error != nil){
		[self setLogString:@"Can not parse tag info XML\n"];
		[self appendLogString:[NSString stringWithFormat:@"url=[%@]\n", [self urlString]]];
		[self appendLogString:[NSString stringWithFormat:@"error=[%@]\n", error]];
		return TAG_ITEM_FAILED;
	}

	// parse xml document
	NSXMLNode *rootNode = [xmlDoc rootElement];
//	NSLog(@"rootNode=%@", [rootNode name]);
	if(rootNode == nil){
		[self setLogString:@"Can not parse tag info XML\n"];
		[self appendLogString:[NSString stringWithFormat:@"url=[%@]\n", [self urlString]]];
		[self appendLogString:@"error=[rootNode is null]\n"];
		return TAG_ITEM_FAILED;
	}

	// get status
	NSString *status = [[(NSXMLElement*)rootNode attributeForName:@"status"] stringValue];
	// failed
	if([status isEqualToString:@"fail"]){
		NSXMLNode *errorNode = [self getChildNode:rootNode name:@"error"];
		NSString *errorDescription = @"";
		if(errorNode != nil){
			e = [[errorNode children] objectEnumerator];
			while (childNode = [e nextObject]){
				errorDescription = [errorDescription stringByAppendingFormat:@"%@=%@\n", [childNode name], [childNode stringValue]];
			}
		}
		[self setLogString:@"Failed tag info XML\n"];
		[self appendLogString:[NSString stringWithFormat:@"url=[%@]\n", [self urlString]]];
		[self appendLogString:errorDescription];
		return TAG_ITEM_FAILED;
	}

	// get detail
	NSXMLNode *detailsNode = [self getChildNode:rootNode name:@"thumb"];
//	NSLog(@"detailsNode=%@", [detailsNode name]);
	if(detailsNode == nil){
		[self setLogString:@"Can not parse tag info XML\n"];
		[self appendLogString:[NSString stringWithFormat:@"url=[%@]\n", [self urlString]]];
		[self appendLogString:@"error=[detailsNode is null]\n"];
		return TAG_ITEM_FAILED;
	}

	int index = 0;
	int startIndex = index + 1;
	int lastIndex = startIndex;
	int itemCount = 0;

	NSString *tagName;
	int tagCount;
	int fontSize;

	NSXMLNode *tagsNode;
	NSString *domain;

	e = [[detailsNode children] objectEnumerator];
	while (childNode = [e nextObject]){

//		NSLog(@"name=%@", [childNode name]);

		// tags
		if([[childNode name] isEqualToString:@"tags"]){

			// domain
			domain = [[(NSXMLElement*)childNode attributeForName:@"domain"] stringValue];
//			NSLog(@"domain=%@", domain);

			if([domain isEqualToString:@"jp"]){

				NSEnumerator *e2 = [[childNode children] objectEnumerator];
				while (tagsNode = [e2 nextObject]){

//					NSLog(@"tagsNode=%@", [tagsNode name]);
					// tag
					if(![[tagsNode name] isEqualToString:@"tag"]){
						continue;
					}
					tagName = [tagsNode stringValue];
					if([tagName isEqualToString:@""]){
						continue;
					}

					index++;
					itemCount++;
					tagName = [self replaceWebCharacters:tagName];
					tagCount = 0;
					fontSize = 12;
		
					[tagList_ addObject:
						[NSMutableDictionary dictionaryWithObjectsAndKeys:
							[NSNumber numberWithInt:index], @"index" ,
							tagName, @"tagName" ,
							[NSNumber numberWithInt:tagCount], @"tagCount" ,
							[NSNumber numberWithInt:fontSize], @"fontSize" ,
							[NSNumber numberWithBool:NO], @"changeFontSize" ,
							[NSNumber numberWithBool:NO], @"showTagCount" ,
							nil
						]
					];
				}
				break;
			}
			break;
		}
	}

	// set index
	lastIndex = index;

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:itemCount];

	return TAG_ITEM_SUCCESS;
}
//------------------------------------
// parseResultHTML
//------------------------------------
- (void)parseResultHTML:(NSString*)html params:(NSDictionary*)params
{

	// params
	int searchType = [[params objectForKey:@"searchType"] intValue];

	NSString *splitString = @"";

	if([self searchType] == SEARCH_TAG_WITH_STRING){
		splitString = @"<div id=\"PAGEBODY\">";
	}
	else if([self searchType] == SEARCH_TAG_WITH_CONTENTS){
		splitString = @"<div id=\"PAGEBODY\">";
	}

	// split each html
	NSDictionary *htmlObject = [self splitResultHTML:html splitString:splitString];
	NSString *headHTML = [htmlObject objectForKey:@"headHTML"];
	NSString *bodyHTML = [htmlObject objectForKey:@"bodyHTML"];
//	NSLog(@"headHTML=%@", headHTML);
//	NSLog(@"bodyHTML=%@", bodyHTML);

	// check access error
	if([self checkAccessErrorStatus:headHTML bodyHTML:bodyHTML] == YES){
		return;
	}

	// set status
	[self setSearchStatus:params];

	// remove
	[self removeTagList];

	// parse html
	if(searchType == SEARCH_TAG_WITH_STRING){
		[self parseRelatedTagHTML:bodyHTML];
		// parse page & total status
		[self parsePageStatusHTML:bodyHTML];
	}
	else if(searchType == SEARCH_TAG_WITH_CONTENTS){
		[self parseContentsHTML:bodyHTML];
	}

	// set status
	[self setWindowStatus:params];

	// result string
	NSString *resultString = [self convertToResultString:[self startIndex]
												lastIndex:[self lastIndex]
												totalResults:[self totalResults]
												totalString:@""
												showTotal:1
							];
	[txtSearchResults setStringValue:resultString];

	// reload
	[taglistArrayController rearrangeObjects];

	// page button
//	[self changePageButtonEnable];
//	[self changeMajorTagButtonState];

}

//------------------------------------
// parseRelatedTagHTML
//------------------------------------
- (void)parseRelatedTagHTML:(NSString*)html
{

	NSString *keyTag = @"<a href=\"tag/";

	NSString *tagName = @"";
	NSString *tagCountStr = @"";
	int tagCount;
	int fontSize;

	int index = ([self pageIndex] - 1) * 100;
	int startIndex = index + 1;
	int lastIndex = startIndex;
	int itemCount = 0;

	NSString *line;
	NSRange rng;
	int i;

	html = [self replaceCharacter:html str1:@"><" str2:@">\n<"];

	NSArray *lines = [html componentsSeparatedByString:@"\n"];
//	NSLog(@"lines=%d", [lines count]);

	//
	// parse tag list
	//
	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];

		tagCountStr = @"0";
		tagCount = 0;
		fontSize = 12;

		// tag
		rng = [line rangeOfString:[keyTag lowercaseString]];
		if(rng.length > 0){
			// tagName
			tagName = [self getEnclosedString:line
										beginString:@">"
										endString:@"<"
										withEndSet:NO
					];
			if([tagName isEqualToString:@""]){
				continue;
			}

			// tagCount
			[self getTagCountStatusString:line
						countStr:&tagCountStr
						beginString:@">"
						endString:@"<"
			];
			tagCountStr = [self getEnclosedString:tagCountStr
										beginString:@"("
										endString:@")"
										withEndSet:NO
					];
			// trim space
			tagCountStr = [tagCountStr stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
			if([tagCountStr isEqualToString:@""]){
				continue;
			}
//			NSLog(@"tagName=%@", tagName);

			index++;
			itemCount++;
			tagCount = [tagCountStr intValue];
			tagName = [self replaceWebCharacters:tagName];

			[tagList_ addObject:
				[NSMutableDictionary dictionaryWithObjectsAndKeys:
					[NSNumber numberWithInt:index], @"index" ,
					tagName, @"tagName" ,
					[NSNumber numberWithInt:tagCount], @"tagCount" ,
					[NSNumber numberWithInt:fontSize], @"fontSize" ,
					[NSNumber numberWithBool:NO], @"changeFontSize" ,
					[NSNumber numberWithBool:YES], @"showTagCount" ,
					nil
				]
			];
		}
	}


	// set index
	lastIndex = index;

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:itemCount];

}
//------------------------------------
// parseContentsHTML
//------------------------------------
- (void)parseContentsHTML:(NSString*)html
{

	NSString *keyTag = @"<a rel=\"tag\"";

	NSString *tagName = @"";
	NSString *fontSizeLevelStr = @"";
	int tagCount;
	int fontSize;

	int index = ([self pageIndex] - 1) * 100;
	int startIndex = index + 1;
	int lastIndex = startIndex;
	int itemCount = 0;

	NSString *line;
	NSRange rng;
	int i;

	NSArray *lines = [html componentsSeparatedByString:@"\n"];
//	NSLog(@"lines=%d", [lines count]);

	//
	// parse tag list
	//
	for(i = 0; i < [lines count]; i++){


		line = [lines objectAtIndex:i];
//		NSLog(@"line=%@", line);

		fontSizeLevelStr = @"12";
		tagCount = 0;
		fontSize = 12;

		// tag
		rng = [line rangeOfString:[keyTag lowercaseString]];
		if(rng.length > 0){
			// tagName
			tagName = [self getEnclosedString:line
										beginString:@">"
										endString:@"<"
										withEndSet:NO
					];
//			NSLog(@"tagName=%@", tagName);
			if([tagName isEqualToString:@""]){
				continue;
			}
			// fontSizeLevel
			fontSizeLevelStr = [self getEnclosedString:line
										beginString:@"class=\""
										endString:@"\""
										withEndSet:NO
					];

			index++;
			itemCount++;

			tagName = [self replaceWebCharacters:tagName];
			fontSize = [self getFontSize:fontSizeLevelStr];
			tagCount = fontSize;

			[tagList_ addObject:
				[NSMutableDictionary dictionaryWithObjectsAndKeys:
					[NSNumber numberWithInt:index], @"index" ,
					tagName, @"tagName" ,
					[NSNumber numberWithInt:tagCount], @"tagCount" ,
					[NSNumber numberWithInt:fontSize], @"fontSize" ,
					[NSNumber numberWithBool:YES], @"changeFontSize" ,
					[NSNumber numberWithBool:NO], @"showTagCount" ,
					nil
				]
			];
		}
	}


	// set index
	lastIndex = index;

	[self setStartIndex:startIndex];
	[self setLastIndex:lastIndex];
	[self setTotalResults:itemCount];

}

//------------------------------------
// parsePageStatusHTML
//------------------------------------
- (void)parsePageStatusHTML:(NSString*)html
{

//	NSString *keyResults = @"class=\"result_total\">";
	NSString *keyResults = [self convertToShiftJISString:"o^F"];

	NSString *keyPageStart = @"class=\"pager\"";
	NSString *keyPageEnd = @"</p>";
	NSString *keyPageCurrent = @"<span>";
	NSString *keyPageLink = @"<a href=\"";

	NSString *totalResultsStr = @"";
	NSString *pageIndexStr = @"";

	int totalResults = [self totalResults];
	int maxResults = [self maxResults];

	int pageIndexCurrent = 1;
	int pageIndexLink = 1;
	int pageMaxIndex = 1;

	BOOL getPageLink;
	BOOL getPageEnd;
	BOOL getResults;

	NSString *line;
	NSString *lowLine;
	NSRange rng;
	int i;

	//
	// parse status
	//
	NSArray *lines = [self createLineBreakArray:html];

	getResults = NO;
	getPageLink = NO;
	getPageEnd = NO;

	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];
//		NSLog(@"line=%@", line);
		lowLine = [line lowercaseString];

		// results
		rng = [lowLine rangeOfString:[keyResults lowercaseString]];
		if(rng.length > 0 && getResults == NO){
			totalResultsStr = [self getEnclosedString:line
										beginString:@"<strong>"
										endString:@"</strong>"
										withEndSet:NO
							];
			getResults = YES;
			continue;
		}
		// pageLink
		rng = [lowLine rangeOfString:[keyPageStart lowercaseString]];
		if(rng.length > 0){
			getPageLink = YES;
			continue;
		}
		// pageEnd
		if(getPageLink == YES){
			rng = [lowLine rangeOfString:[keyPageEnd lowercaseString]];
			if(rng.length > 0){
				getPageEnd = YES;
				continue;
			}
		}
		// pageIndex
		if(getPageLink == YES && getPageEnd == NO){
			// keyPageCurrent
			rng = [lowLine rangeOfString:[keyPageCurrent lowercaseString]];
			if(rng.length > 0){
				pageIndexStr = [self getEnclosedString:line
											beginString:@">"
											endString:@"<"
											withEndSet:NO
							];
				if(pageIndexStr && [self checkIsNumberString:pageIndexStr] == YES){
					pageIndexCurrent = [pageIndexStr intValue];
					if(pageIndexCurrent > pageMaxIndex){
						pageMaxIndex = pageIndexCurrent;
					}
				}
				continue;
			}
			// keyPageLink
			rng = [lowLine rangeOfString:[keyPageLink lowercaseString]];
			if(rng.length > 0){
				pageIndexStr = [self getEnclosedString:line
											beginString:@">"
											endString:@"<"
											withEndSet:NO
							];
				if(pageIndexStr && [self checkIsNumberString:pageIndexStr] == YES){
					pageIndexLink = [pageIndexStr intValue];
					if(pageIndexLink > pageMaxIndex){
						pageMaxIndex = pageIndexLink;
					}
				}
				continue;
			}
		}

	}

	// set results
	if(![totalResultsStr isEqualToString:@""]){
		// replace
		totalResultsStr = [self replaceCharacter:totalResultsStr str1:[self convertToShiftJISString:""] str2:@""];
		totalResults = [self convertStringToIntValue:totalResultsStr];
		[self setTotalResults:totalResults];
		// calc from totalResults
		if(maxResults > 0){
			pageMaxIndex = totalResults / maxResults;
			if(totalResults % maxResults > 0){
				pageMaxIndex++;
			}
		}
	}

	// not used pageIndexCurrent
//	[self setPageIndex:pageIndexCurrent];
	[self setPageMaxIndex:pageMaxIndex];

}
//------------------------------------
// splitResultHTML
//------------------------------------
- (NSDictionary*)splitResultHTML:(NSString*)html splitString:(NSString*)splitString
{

	NSString *headHTML = @"";
	NSString *bodyHTML = @"";

	NSArray *lines = [html componentsSeparatedByString:splitString];

	headHTML = [lines objectAtIndex:0];
	if([lines count] > 1){
		bodyHTML = [lines objectAtIndex:1];
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				headHTML, @"headHTML" ,
				bodyHTML, @"bodyHTML" ,
				nil
			];

}
//------------------------------------
// getFontSize
//------------------------------------
- (int)getFontSize:(NSString*)levelStr
{
	int fontSize = 12;

	if([levelStr isEqualToString:@"level_1"]){
		fontSize = 30;
	}
	else if([levelStr isEqualToString:@"level_2"]){
		fontSize = 24;
	}
	else if([levelStr isEqualToString:@"level_3"]){
		fontSize = 18;
	}
	else if([levelStr isEqualToString:@"level_4"]){
		fontSize = 12;
	}

	return fontSize;
}
//------------------------------------
// changePageButtonEnable
//------------------------------------
- (void)changePageButtonEnable
{

	int pageIndex = [self pageIndex];
	int pageMaxIndex = [self pageMaxIndex];

	// page button
	// prev
	if(pageIndex > 1){
		[btnPagePrev setEnabled:YES];
	}else{
		[btnPagePrev setEnabled:NO];
	}
	// next
	if(pageIndex < pageMaxIndex){
		[btnPageNext setEnabled:YES];
	}else{
		[btnPageNext setEnabled:NO];
	}

}
//------------------------------------
// changeMajorTagButtonState
//------------------------------------
- (void)changeMajorTagButtonState
{

	NSImage *image = [NSImage imageNamed:@"btn_nicovideo"];

	if([self searchType] == SEARCH_TAG_WITH_CONTENTS){
		if([[self contentsName] isEqualToString:TAG_CONTENTS_NAME_MAJOR_TAG]){
			image = [NSImage imageNamed:@"btn_nicovideo_blue"];
		}
	}

	[btnMajorTag setImage:image];
}
//------------------------------------
// clearConnection
//------------------------------------
- (void)clearConnection
{

	ConnectionDataItem *connectionDataItem = [self connectionDataItem];
	// cancel connection
	if(connectionDataItem != nil){
		if([connectionDataItem connection] != nil){
//			NSLog(@"cancelConnection");
			[connectionDataItem cancelConnection];
		}
		[self setConnectionDataItem:nil];
	}

}

//------------------------------------
// removeTagList 
//------------------------------------
- (void)removeTagList
{

	[tagList_ removeAllObjects];
	// reload
	[taglistArrayController rearrangeObjects];

}
//------------------------------------
// windowWillClose
//------------------------------------
- (void)windowWillClose
{

	// recent searches
	[self saveSearchFieldRecentSearches:txtSearchField key:@"arrayRecentTagSearches"];

}
//------------------------------------
// setSearchStatus
//------------------------------------
- (void)setSearchStatus:(NSDictionary*)params
{

	// set status
	[self setUrlString:[params objectForKey:@"urlString"]];
	[self setSearchTitle:[params objectForKey:@"searchTitle"]];
	[self setSearchString:[params objectForKey:@"searchString"]];
	[self setContentsName:[params objectForKey:@"contentsName"]];
	[self setVideoId:[params objectForKey:@"videoId"]];

	[self setSearchType:[[params objectForKey:@"searchType"] intValue]];
	[self setPageIndex:[[params objectForKey:@"pageIndex"] intValue]];
	[self setPageMaxIndex:[[params objectForKey:@"pageIndex"] intValue]];

}
//------------------------------------
// setWindowStatus
//------------------------------------
- (void)setWindowStatus:(NSDictionary*)params
{
	NSString *videoId = [params objectForKey:@"videoId"];	
	NSString *searchTitle = [params objectForKey:@"searchTitle"];	
	int searchType = [[params objectForKey:@"searchType"] intValue];

	// window title
	[self setWindowTitle:videoId searchTitle:searchTitle searchType:searchType];

	[self changePageButtonEnable];
	[self changeMajorTagButtonState];

}
//------------------------------------
// setWindowTitle
//------------------------------------
- (void)setWindowTitle:(NSString*)videoId searchTitle:(NSString*)searchTitle searchType:(int)searchType
{

	NSString *string = RESULT_TITLE_TAG;

	if(![searchTitle isEqualToString:@""]){
		string = searchTitle;
		if([string length] > 20){
			string = [NSString stringWithFormat:@"%@..", [string substringToIndex:20]];
		}
		if(searchType == SEARCH_TAG_WITH_STRING){
			string = [string stringByAppendingString:[self convertToShiftJISString:" - ^O"]];
		}
		else if(searchType == SEARCH_TAG_WITH_CONTENTS){
//			string = [string stringByAppendingString:[self convertToShiftJISString:" - ^O"]];
		}
		else if(searchType == SEARCH_TAG_WITH_VIDEO){
//			string = [NSString stringWithFormat:@"%@(%@)", string, videoId];
			string = [string stringByAppendingString:[self convertToShiftJISString:" ̊֘A^O"]];
		}
	}

	[self setResultTitle:string];
	[viewRelatedWindow setWindowTitle:string];
}
//------------------------------------
// connectionDataItem
//------------------------------------
- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem
{
	[connectionDataItem retain];
	[connectionDataItem_ release];
	connectionDataItem_ = connectionDataItem;
}
- (ConnectionDataItem*)connectionDataItem
{
	return connectionDataItem_;
}
//------------------------------------
// urlString
//------------------------------------
- (void)setUrlString:(NSString*)urlString
{
	[urlString retain];
	[urlString_ release];
	urlString_ = urlString;
}
- (NSString*)urlString
{
	return urlString_;
}
//------------------------------------
// searchString
//------------------------------------
- (void)setSearchString:(NSString*)searchString
{
	[searchString retain];
	[searchString_ release];
	searchString_ = searchString;
}
- (NSString*)searchString
{
	return searchString_;
}
//------------------------------------
// contentsName
//------------------------------------
- (void)setContentsName:(NSString*)contentsName
{
	[contentsName retain];
	[contentsName_ release];
	contentsName_ = contentsName;
}
- (NSString*)contentsName
{
	return contentsName_;
}
//------------------------------------
// contentsName
//------------------------------------
- (void)setVideoId:(NSString*)videoId
{
	[videoId retain];
	[videoId_ release];
	videoId_ = videoId;
}
- (NSString*)videoId
{
	return videoId_;
}
//------------------------------------
// searchTitle
//------------------------------------
- (void)setSearchTitle:(NSString*)searchTitle
{
	[searchTitle retain];
	[searchTitle_ release];
	searchTitle_ = searchTitle;
}
- (NSString*)searchTitle
{
	return searchTitle_;
}
//------------------------------------
// resultTitle
//------------------------------------
- (void)setResultTitle:(NSString*)resultTitle
{
	[resultTitle retain];
	[resultTitle_ release];
	resultTitle_ = resultTitle;
}
- (NSString*)resultTitle
{
	return resultTitle_;
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}
//------------------------------------
// searchType
//------------------------------------
- (void)setSearchType:(int)searchType
{
	searchType_ = searchType;
}
- (int)searchType
{
	return searchType_;
}

//------------------------------------
// startIndex
//------------------------------------
- (void)setStartIndex:(int)startIndex
{
	startIndex_ = startIndex;
}
- (int)startIndex
{
	return startIndex_;
}
//------------------------------------
// lastIndex
//------------------------------------
- (void)setLastIndex:(int)lastIndex
{
	lastIndex_ = lastIndex;
}
- (int)lastIndex
{
	return lastIndex_;
}
//------------------------------------
// totalResults
//------------------------------------
- (void)setTotalResults:(int)totalResults
{
	totalResults_ = totalResults;
}
- (int)totalResults
{
	return totalResults_;
}
//------------------------------------
// maxResults
//------------------------------------
- (void)setMaxResults:(int)maxResults
{
	maxResults_ = maxResults;
}
- (int)maxResults
{
	return maxResults_;
}
//------------------------------------
// pageIndex
//------------------------------------
- (void)setPageIndex:(int)pageIndex
{
	pageIndex_ = pageIndex;
}
- (int)pageIndex
{
	return pageIndex_;
}
//------------------------------------
// pageMaxIndex
//------------------------------------
- (void)setPageMaxIndex:(int)pageMaxIndex
{
	pageMaxIndex_ = pageMaxIndex;
}
- (int)pageMaxIndex
{
	return pageMaxIndex_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[urlString_ release];
	[searchString_ release];
	[contentsName_ release];
	[videoId_ release];
	[searchTitle_ release];
	[resultTitle_ release];
	[logString_ release];
	[tagList_ release];
	[connectionDataItem_ release];
	[super dealloc];
}

@end

