/* ViewVideoContent */

#import <Cocoa/Cocoa.h>
#import "ContentImageView.h"
#import "ImageItem.h"
#import "ContentDescTextView.h"
#import "VideoInfoItem.h"
#import "VideoItemStatus.h"
#import "SearchTypes.h"
#import "BindObjectTypes.h"

@interface NSAttributedString (Hyperlink)
+(id)hyperlinkFromString:(NSString*)inString withURL:(NSURL*)aURL;
@end

@interface ViewVideoContent : NSObject
{

	IBOutlet id viewMainSearch;
	IBOutlet id viewPlayer;
	IBOutlet NSArrayController *searchlistArrayController;
	IBOutlet NSArrayController *relatedlistArrayController;

	IBOutlet NSWindow *videoContentWindow;
	IBOutlet ContentImageView *imgPreview;
	IBOutlet NSTextView *txtInformation;
	IBOutlet ContentDescTextView *txtDescription;
	IBOutlet NSButton *btnPlay;
	IBOutlet NSButton *btnPrev;
	IBOutlet NSButton *btnNext;
	IBOutlet NSButton *btnClose;
	IBOutlet NSButton *btnLocked;
	IBOutlet NSButton *btnAutoClose;
	IBOutlet NSSplitView *spvContentMain;
	IBOutlet NSSplitView *spvContentInfo;

	NSArrayController *targetArrayController_;

	NSDictionary *params_;
	int arrayNo_;
	int refViewNo_;
	BOOL isLocked_;

}
- (IBAction)openVideoContentWindow:(id)sender;
- (IBAction)closeVideoContentWindow:(id)sender;
- (IBAction)changeContentItem:(id)sender;
- (IBAction)playItem:(id)sender;
- (IBAction)getVideoInfo:(id)sender;

- (IBAction)changeContentIsLocked:(id)sender;

- (IBAction)searchWithKeyword:(id)sender;
- (IBAction)searchWithMylist:(id)sender;
- (IBAction)searchWithPostUser:(id)sender;
- (IBAction)playItemWithSelectedString:(id)sender;
- (IBAction)openURLWithSelectedString:(id)sender;
- (IBAction)copySelectedStringToPasteboard:(id)sender;

- (void)createVideoContentWindow:(NSDictionary*)params arrayNo:(int)arrayNo refViewNo:(int)refViewNo open:(BOOL)open;
- (void)createVideoContent:(NSDictionary*)params;
- (void)clearVideoContent;
- (void)getVideoInfoItem:(NSString*)videoId;
- (void)setVideoContentView;
- (void)setTextDescription:(NSString*)title
					comment:(NSString*)comment
					lastResBody:(NSString*)lastResBody
					tagString:(NSString*)tagString;
- (void)autoCloseWindow;

- (void)setContentIsLocked:(BOOL)isLocked;
- (void)setButtonLockedState:(BOOL)isLocked;

- (void)setTargetArrayController:(int)arrayNo;
- (void)changeControlButtonBind:(BOOL)isLocked;
- (void)changePlayButtonEnable;
- (void)selectContentItem:(int)tag;

- (NSDictionary*)getParamsFromArray;
- (void)setParams:(NSDictionary*)params;
- (NSDictionary*)params;
- (void)setRefViewNo:(int)refViewNo;
- (int)refViewNo;
- (void)setArrayNo:(int)arrayNo;
- (int)arrayNo;
- (void)setIsLocked:(BOOL)isLocked;
- (BOOL)isLocked;
- (BOOL)hasVideoId;
- (BOOL)windowIsVisible;

- (BOOL)selectedStringInDescriptionIsNull;
- (BOOL)selectedStringInDescriptionIsNotNull;
- (BOOL)selectedStringInDescriptionIsAlphabet;
- (BOOL)selectedStringInDescriptionIsNumber;
- (BOOL)selectedStringInDescriptionIsAlphabetAndNumberOrNumber;
- (BOOL)selectedStringInDescriptionIsMylistNo;
- (BOOL)selectedStringInDescriptionIsNotMylistNo;
- (BOOL)selectedStringInDescriptionIsPostUserId;
- (BOOL)selectedStringInDescriptionIsNotPostUserId;
- (BOOL)selectedStringInDescriptionIsURL;
- (BOOL)selectedStringInDescriptionIsNotURL;

@end
