#import "ViewVideoContent.h"
#import "ViewMainSearch.h"
#import "ViewPlayer.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "HelperExtension.h"
#import "DialogExtension.h"

@implementation NSAttributedString (Hyperlink)

+(id)hyperlinkFromString:(NSString*)inString withURL:(NSURL*)aURL
{
	NSMutableAttributedString* attrString = [[NSMutableAttributedString alloc] initWithString:inString];
	NSRange range = NSMakeRange(0, [attrString length]);
	[attrString beginEditing];
	[attrString addAttribute:NSLinkAttributeName value:[aURL absoluteString] range:range];
	[attrString endEditing];
	return [attrString autorelease];
}
/*
+(id)hyperlinkFromString:(NSString*)inString withURL:(NSURL*)aURL
{

	NSMutableAttributedString* attrString = [[NSMutableAttributedString alloc]
											initWithString:inString
											attributes:[NSDictionary dictionaryWithObjectsAndKeys:
															[aURL absoluteString], NSLinkAttributeName,
//															[NSColor whiteColor], NSForegroundColorAttributeName,
//															[NSNumber numberWithInt:NSSingleUnderlineStyle], NSUnderlineStyleAttributeName,
//															[NSCursor pointingHandCursor], NSCursorAttributeName, 
															nil
														]
											];

	return [attrString autorelease];
}
*/
@end


@implementation ViewVideoContent

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rect
	[self setWindowRect:videoContentWindow key:@"rectWindowVideoContent"];
 	// split view rect
	[self setSplitViewRect:spvContentMain key:@"rectSplitViewContentMain"];
	[self setSplitViewRect:spvContentInfo key:@"rectSplitViewContentInfo"];

	[txtInformation setDelegate:self];

 	// transparent
//	[videoContentWindow setBackgroundColor:[NSColor colorWithCalibratedWhite:0.05 alpha:0.9]];
//	[videoContentWindow setAlphaValue:0.8];

 	// transparent
	// same values of iLifeControls
	[txtInformation setBackgroundColor:[NSColor colorWithCalibratedWhite:32/255.0 alpha:213/255.0]];
	[txtInformation setTextColor:[NSColor whiteColor]];
	[txtInformation setFont:[NSFont systemFontOfSize:11.0]];
	[txtInformation setLinkTextAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
											[NSColor whiteColor], NSForegroundColorAttributeName,
											[NSNumber numberWithInt:NSSingleUnderlineStyle], NSUnderlineStyleAttributeName,
											[NSCursor pointingHandCursor], NSCursorAttributeName, 
											nil
										]
	];

	[txtDescription setBackgroundColor:[NSColor colorWithCalibratedWhite:32/255.0 alpha:213/255.0]];
	[txtDescription setTextColor:[NSColor whiteColor]];
	[txtDescription setFont:[NSFont systemFontOfSize:13.0]];

	[imgPreview setImage:nil];
	[txtInformation setString:@"No Selection"];
	[txtDescription setString:@""];
	[self setParams:nil];
	[self setRefViewNo:REF_VIEW_NONE];

	// set button text color
	NSAttributedString *attrStringBtnLocked =
		[[NSAttributedString alloc] initWithString:[btnLocked title]
						attributes:[NSDictionary dictionaryWithObjectsAndKeys:
										[NSColor whiteColor], NSForegroundColorAttributeName,
										nil
									]
		];
	[btnLocked setAttributedTitle:attrStringBtnLocked];

	NSAttributedString *attrStringBtnAutoClose =
		[[NSAttributedString alloc] initWithString:[btnAutoClose title]
						attributes:[NSDictionary dictionaryWithObjectsAndKeys:
										[NSColor whiteColor], NSForegroundColorAttributeName,
										nil
									]
		];
	[btnAutoClose setAttributedTitle:attrStringBtnAutoClose];

	// set first responder
	[videoContentWindow makeFirstResponder:btnClose];

	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];
//	[btnClose setKeyEquivalent:@" "];
//	[[btnClose cell] setKeyEquivalent:@" "];
//	[[btnClose cell] setKeyEquivalentModifierMask:NSAlternateKeyMask];

	// delete focus ring
	[btnPlay setFocusRingType:NSFocusRingTypeNone];
	[btnPrev setFocusRingType:NSFocusRingTypeNone];
	[btnNext setFocusRingType:NSFocusRingTypeNone];
	[btnClose setFocusRingType:NSFocusRingTypeNone];

	// set binding array type
	[self setArrayNo:BIND_ARRAY_NONE];
//	[self setTargetArrayController:BIND_ARRAY_NONE];
//	[self changeControlButtonBind:NO];
	[self changePlayButtonEnable];

	// lock status
	[self setContentIsLocked:NO];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:videoContentWindow];
	[nc addObserver:self selector:@selector(handleImageItemLoad:) name:IMAGE_ITEM_NOTIF_NAME_ITEM object:nil];
}
//------------------------------------
// openVideoContentWindow
//------------------------------------
- (IBAction)openVideoContentWindow:(id)sender
{
	[videoContentWindow setLevel:NSModalPanelWindowLevel];
	[videoContentWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// closeVideoContentWindow
//------------------------------------
- (IBAction)closeVideoContentWindow:(id)sender
{
	[videoContentWindow performClose:nil];
}
//------------------------------------
// changeContentItem
//------------------------------------
- (IBAction)changeContentItem:(id)sender
{
	[self selectContentItem:[sender tag]];
}
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{

	NSDictionary *params = [self params]; 
	if(params == nil){
		return;
	}

	NSString *videoId = [params objectForKey:@"videoId"]; 
	NSString *title = [params objectForKey:@"title"]; 

	if([viewPlayer setPlayerView:videoId title:title params:params arrayNo:BIND_ARRAY_NONE] == YES){
		[viewPlayer orderFrontPlayerWindow:nil];
		// auto close
		[self autoCloseWindow];
	}

}
//------------------------------------
// getVideoInfo
//------------------------------------
- (IBAction)getVideoInfo:(id)sender
{

	NSDictionary *params = [self params]; 
	if(params == nil){
		return;
	}

	NSString *videoId = [params objectForKey:@"videoId"];
	if(!videoId || [videoId isEqualToString:@""]){
		return;
	}

	[self getVideoInfoItem:videoId];

}
//------------------------------------
// changeContentIsLocked
//------------------------------------
- (IBAction)changeContentIsLocked:(id)sender
{
	BOOL isLocked = [self isLocked];
	if(isLocked == YES){
		isLocked = NO;
	}else{
		isLocked = YES;
	}
	[self setContentIsLocked:isLocked];

	// createVideoContent
	if(isLocked == NO){
		int refViewNo = [self refViewNo];
		if( refViewNo == REF_VIEW_MAIN ||
			refViewNo == REF_VIEW_RELATED){
			[self createVideoContent:[self getParamsFromArray]];
		}
		else if(refViewNo == REF_VIEW_PLAYER){
			[self createVideoContent:[viewPlayer params]];
		}
	}

}
//------------------------------------
// searchWithKeyword
//------------------------------------
- (IBAction)searchWithKeyword:(id)sender
{

	int tag = [sender tag];

	NSString *keyword = [txtDescription selectedString];
//	NSLog(@"keyword=%@", keyword);

	// null
	if(!keyword || [keyword isEqualToString:@""]){
		return;
	}

	// lock
//	[self setContentIsLocked:YES];

	// change default
	[self setDefaultIntValue:tag key:@"optSearchKeywordType"];

	[viewMainSearch searchWithString:keyword pageIndex:1 searchType:SEARCH_WITH_STRING isKeywordSearch:YES];

	// auto close
	[self autoCloseWindow];

}
//------------------------------------
// searchWithMylist
//------------------------------------
- (IBAction)searchWithMylist:(id)sender
{

	NSString *keyword = [txtDescription selectedString];

	// null
	if(!keyword || [keyword isEqualToString:@""]){
		return;
	}
	// not numeric
	if([self checkIsNumberString:keyword] == NO){
		return;
	}

	// lock
//	[self setContentIsLocked:YES];

	[viewMainSearch searchWithMylist:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_MYLIST isKeywordSearch:YES];

	// auto close
	[self autoCloseWindow];

}
//------------------------------------
// searchWithMylist
//------------------------------------
- (IBAction)searchWithPostUser:(id)sender
{

	NSString *keyword = [txtDescription selectedString];

	// null
	if(!keyword || [keyword isEqualToString:@""]){
		return;
	}
	// not numeric
	if([self checkIsNumberString:keyword] == NO){
		return;
	}

	// lock
//	[self setContentIsLocked:YES];

	[viewMainSearch searchWithPostUser:keyword searchTitle:keyword pageIndex:1 searchType:SEARCH_WITH_POST_USER isKeywordSearch:YES];

	// auto close
	[self autoCloseWindow];

}
//------------------------------------
// playItemWithKeyword
//------------------------------------
- (IBAction)playItemWithSelectedString:(id)sender
{

	NSString *keyword = [txtDescription selectedString];

	// null
	if(!keyword || [keyword isEqualToString:@""]){
		return;
	}

	// get videoId (watch/smXXXXXX -> smXXXXXX)
	NSString *videoId = [self getVideoIdFromURL:keyword setDefault:YES];
	if([videoId isEqualToString:@""]){
		return;
	}

	// lock
//	[self setContentIsLocked:YES];

	[viewPlayer openPlayerWindow:nil];
	[viewPlayer setPlayerViewWithVideoId:videoId arrayNo:BIND_ARRAY_NONE];

	// auto close
	[self autoCloseWindow];

}
//------------------------------------
// openURLWithSelectedString
//------------------------------------
- (IBAction)openURLWithSelectedString:(id)sender
{

	NSString *string = [txtDescription selectedString];

	// ttp: -> http:
	if( [string length] >= 4 &&
		[[string substringToIndex:4] isEqualToString:@"ttp:"]){
		string = [NSString stringWithFormat:@"h%@",string]; 
	}

	// open url
	[self openURLWithBrowser:string];

	// auto close
	[self autoCloseWindow];

}
//------------------------------------
// copySelectedStringToPasteboard
//------------------------------------
- (IBAction)copySelectedStringToPasteboard:(id)sender
{
	NSString *string = [txtDescription selectedString];

	// null
	if(!string || [string isEqualToString:@""]){
		return;
	}

	[self copyStringToPasteboard:string];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createVideoContentWindow
//------------------------------------
- (void)createVideoContentWindow:(NSDictionary*)params arrayNo:(int)arrayNo refViewNo:(int)refViewNo open:(BOOL)open
{

	[self setArrayNo:arrayNo];
	[self setTargetArrayController:arrayNo];
	[self setRefViewNo:refViewNo];

	[self createVideoContent:params];

	if(open == YES){
		[self openVideoContentWindow:nil];
	}

}
//------------------------------------
// createVideoContent
//------------------------------------
- (void)createVideoContent:(NSDictionary*)params
{

	// locked
	BOOL isLocked = [self isLocked];
	if(isLocked == YES){
		return;
	}

	// null
	if(params == nil){
		return;
	}

	// same videoId
	if(params != nil && [self params] != nil){
		if([[params objectForKey:@"videoId"] isEqualToString:[[self params] objectForKey:@"videoId"]]){
			return;
		}
	}

	[self setParams:params];
	[self setVideoContentView];

	[self changeControlButtonBind:isLocked];
	[self changePlayButtonEnable];

}

//------------------------------------
// clearVideoContent
//------------------------------------
- (void)clearVideoContent
{
	[imgPreview setImage:nil];
	[txtInformation setString:@"No Selection"];
	[txtDescription setString:@""];
	[videoContentWindow setTitle:@"Information"];
	[self setParams:nil];
	[self changePlayButtonEnable];

	// remove observer
	[targetArrayController_ removeObserver:self forKeyPath:@"selection"];

}
//------------------------------------
// getVideoInfoItem
//------------------------------------
- (void)getVideoInfoItem:(NSString*)videoId
{

	// alloc video info object
	[[[VideoInfoItem alloc] initWithVideo:videoId
									title:@""
									target:self
									notifType:VIDEO_ITEM_NOTIF_INFO
									hasImage:YES
									params:nil
	] autorelease];

}
//------------------------------------
// handleItemObjectInfo
//------------------------------------
-(void)handleItemObjectInfo:(NSNotification *)notification
{

	VideoInfoItem *videoInfoItem = [notification object];
//	NSLog(@"status=%d", [videoInfoItem status]);

	NSString *title = @"";
	NSString *comment = @"";
	NSString *lastResBody = @"";
	NSString *tagString = @"";

	// success
	if(videoInfoItem && [videoInfoItem itemStatus] == VIDEO_ITEM_SUCCESS){

		title = [videoInfoItem title];
		comment = [videoInfoItem description];
		lastResBody = [videoInfoItem lastResBody];
		NSArray *tagList = [videoInfoItem tagList];
		if([tagList count] > 0){
			tagString = [tagList componentsJoinedByString:@"\n"];
		}
	}
	//failed
	else{

		// get from params
		NSDictionary *params = [self params];
		if(params){
			title = [params objectForKey:@"title"];
			comment = [params objectForKey:@"description"];
		}
		// set error description
		if(videoInfoItem && [videoInfoItem itemStatus] == VIDEO_ITEM_STATUS_FAILED){
			tagString = [NSString stringWithFormat:@"Failed getting video info.\n%@", [videoInfoItem errorDescription]];
		}else{
			tagString = @"Failed getting video info.";
		}
		lastResBody = @"Failed getting video info.";

	}

	[self setTextDescription:title
						comment:comment
						lastResBody:lastResBody
						tagString:tagString
	];

}
//------------------------------------
// setVideoContentView
//------------------------------------
- (void)setVideoContentView
{

	// params
	NSDictionary *params = [self params];
	if(params == nil){
		[imgPreview setImage:nil];
		[imgPreview setVideoType:0];
		[txtInformation setString:@"No Selection"];
		[txtDescription setString:@""];
		[videoContentWindow setTitle:@"Information"];
		return;
	}

	// image
	ImageItem *imageItem = [params objectForKey:@"imageItem"];
	NSImage *image = [imageItem image];
//	NSString *videoType = [params objectForKey:@"videoType"];

	// content
	NSString *title = [params objectForKey:@"title"];
	NSString *comment = [params objectForKey:@"description"];

	NSString *videoId = [params objectForKey:@"videoId"];
	NSString *length = [params objectForKey:@"length"];
	NSString *postUserId = [params objectForKey:@"postUserId"];
	NSString *postedDate = [params objectForKey:@"postedDate"];
	NSString *errorDescription = [params objectForKey:@"errorDescription"];
	NSString *lastResBody = [params objectForKey:@"lastResBody"];
	NSArray *tagList = [params objectForKey:@"tagList"];
	NSString *watchURL = [self convertToWatchURL:videoId];

	int viewCount = [[params objectForKey:@"viewCount"] intValue];
	int resCount = [[params objectForKey:@"resCount"] intValue];
	int mylistCount = [[params objectForKey:@"mylistCount"] intValue];
	BOOL hasCountInfo = [[params objectForKey:@"hasCountInfo"] boolValue];
	int itemStatus = [[params objectForKey:@"itemStatus"] intValue];
	BOOL getVideoInfo = [[params objectForKey:@"getVideoInfo"] boolValue];

	if(!videoId){videoId = @"";}
	if(!length){length = @"";}
	if(!postUserId){postUserId = @"";}
	if(!postedDate){postedDate = @"";}
	if(!errorDescription){errorDescription = @"";}
	if(!lastResBody){lastResBody = @"";}

	NSString *descVideoId = @"";
	NSString *descTime = @"";
	NSString *descDate = @"";
	NSString *descCountInfo = @"";
	NSString *descURL = @" URL : ";
	NSString *descPostUser = @"";
//	NSString *descURL = [NSString stringWithFormat:@" URL : %@", watchLinkURL];

	descVideoId = [NSString stringWithFormat:@" ID : %@", videoId];

	if(itemStatus == VIDEO_ITEM_SUCCESS){
		descTime = [NSString stringWithFormat:@" Time : %@", length];
		descDate = [NSString stringWithFormat:@" Date : %@", postedDate];
		if(hasCountInfo == YES){
			descCountInfo = [NSString stringWithFormat:@" Views : %@\n Res    : %@\n Mylist : %@",
												[self convertToComma:viewCount],
												[self convertToComma:resCount],
												[self convertToComma:mylistCount]
					];
		}
		if(![postUserId isEqualToString:@""]){
			descPostUser = [NSString stringWithFormat:@" User : %@", postUserId];
		}

	}
	else if(itemStatus == VIDEO_ITEM_STATUS_FAILED){
		descTime = [NSString stringWithFormat:@" %@", errorDescription];
	}

    NSMutableAttributedString *attrInfo = 
		[[[NSMutableAttributedString alloc] 
				initWithString:
					[NSString stringWithFormat:@"%@\n%@\n\n%@\n%@\n%@\n%@",
													descVideoId,
													descPostUser,
													descTime,
													descDate,
													descCountInfo,
													descURL
					]
				attributes:
					[NSDictionary dictionaryWithObjectsAndKeys:
						[NSColor whiteColor], NSForegroundColorAttributeName,
						nil
					]
			] autorelease];

	// append hyperlink string
	NSAttributedString *watchLinkURL = [NSAttributedString hyperlinkFromString:watchURL withURL:[NSURL URLWithString:watchURL]];
	[attrInfo appendAttributedString:watchLinkURL];

	// set content
	[imgPreview setImage:image];
	[imgPreview setVideoType:@""];

//	[txtInformation setString:information];
	[[txtInformation textStorage] setAttributedString:attrInfo];

//	[txtDescription setString:description];
	[videoContentWindow setTitle:title];

	// description
	if(getVideoInfo == YES){
		NSString *tagString = @"";
		if([tagList count] > 0){
			tagString = [tagList componentsJoinedByString:@"\n"];
		}
		[self setTextDescription:title
							comment:comment
							lastResBody:lastResBody
							tagString:tagString
		];
	}else{
		[self getVideoInfoItem:videoId];
	}

}
//------------------------------------
// setTextDescription
//------------------------------------
- (void)setTextDescription:(NSString*)title
					comment:(NSString*)comment
					lastResBody:(NSString*)lastResBody
					tagString:(NSString*)tagString
{

	NSString *tagDesc = @"";
	NSString *resDesc = @"";

	// tags
	if(tagString && ![tagString isEqualToString:@""]){
		tagDesc = [NSString stringWithFormat:@"\n\n[ Tags ]\n\n%@"
											, tagString
					];
	}
	// lastResBody
	if(lastResBody && ![lastResBody isEqualToString:@""]){
		resDesc = [NSString stringWithFormat:@"\n\n[ Last Comment ]\n\n%@"
											, lastResBody
					];
	}
	// all description
	NSString *description = [NSString stringWithFormat:@"\n%@\n\n%@%@%@"
																, title
																, comment
																, tagDesc
																, resDesc
							];
	[txtDescription setString:description];

}
//------------------------------------
// autoCloseWindow
//------------------------------------
- (void)autoCloseWindow
{
	// close
	if([self defaultBoolValue:@"optVideoContentAutoClose"] == YES){
		[self closeVideoContentWindow:nil];
	}
}
//------------------------------------
// setContentIsLocked
//------------------------------------
- (void)setContentIsLocked:(BOOL)isLocked
{
	// set default
	[self setIsLocked:isLocked];
	[self setButtonLockedState:isLocked];
	[self changeControlButtonBind:isLocked];

}

//------------------------------------
// setButtonLockedState
//------------------------------------
- (void)setButtonLockedState:(BOOL)isLocked
{
	if(isLocked == YES){
		[btnLocked setState:1];
	}else{
		[btnLocked setState:0];
	}
}
//------------------------------------
// setTargetArrayController
//------------------------------------
- (void)setTargetArrayController:(int)arrayNo
{

	// remove observer
	if(targetArrayController_ != nil){
		[targetArrayController_ removeObserver:self forKeyPath:@"selection"];
	}

	if(arrayNo == BIND_ARRAY_SEARCH){
		targetArrayController_ = searchlistArrayController;
	}
	else if(arrayNo == BIND_ARRAY_RELATED){
		targetArrayController_ = relatedlistArrayController;
	}

	// set observer
	if(targetArrayController_ != nil){
		[targetArrayController_ addObserver:self forKeyPath:@"selection" options:0 context:nil];
	}
}
//------------------------------------
// changeControlButtonBind
//------------------------------------
- (void)changeControlButtonBind:(BOOL)isLocked
{
	if(isLocked == NO && targetArrayController_ != nil){
		[btnPrev bind:@"enabled" toObject:targetArrayController_ withKeyPath:@"canSelectPrevious" options:nil];
		[btnNext bind:@"enabled" toObject:targetArrayController_ withKeyPath:@"canSelectNext" options:nil];
	//	[btnPrev bind:@"enabled2" toObject:targetArrayController_ withKeyPath:@"selectedObjects.@count" options:nil];
	//	[btnNext bind:@"enabled2" toObject:targetArrayController_ withKeyPath:@"selectedObjects.@count" options:nil];
	}else{
		[btnPrev unbind:@"enabled"];
		[btnNext unbind:@"enabled"];	
		[btnPrev setEnabled:NO];
		[btnNext setEnabled:NO];
	}
}
//------------------------------------
// changePlayButtonEnable
//------------------------------------
- (void)changePlayButtonEnable
{

	BOOL enabled = [self hasVideoId];

	[btnPlay setEnabled:enabled];

}
//------------------------------------
// selectContentItem
//------------------------------------
- (void)selectContentItem:(int)tag
{

	// locked
	if([self isLocked] == YES){
		return;
	}

	// no select
//	if([[targetArrayController_ selectedObjects] count] <= 0){
//		return;
//	}

	// previous
	if(tag == 0){
		if([targetArrayController_ canSelectPrevious] == YES){
			[targetArrayController_ selectPrevious:nil];
		}
	}
	// next
	else if(tag == 1){
		if([targetArrayController_ canSelectNext] == YES){
			[targetArrayController_ selectNext:nil];
		}
	}

}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// createVideoContent
	if([self windowIsVisible] == YES){
		if([keyPath isEqualToString:@"selection"]){
//			NSLog(@"selection");
			[self createVideoContent:[self getParamsFromArray]];
		}
	}
}
//------------------------------------
// handleImageItemLoad
//------------------------------------
-(void)handleImageItemLoad:(NSNotification *)notification
{
	ImageItem *imageItem = [notification object];

	if(imageItem == nil){
		return;
	}

	// image
	if([self isLocked] == NO){
		NSImage *image = [imageItem image];
		[imgPreview setImage:image];
	}
}
//------------------------------------
// getParamsFromArray
//------------------------------------
- (NSDictionary*)getParamsFromArray
{

	NSDictionary *params = nil;

	// get record
	if([[targetArrayController_ selectedObjects] count] > 0){
		id record = [[targetArrayController_ selectedObjects] objectAtIndex:0];

		// set params
		params = (NSDictionary*)record;

	}

	return params;
}
//------------------------------------
// clickedOnLink (txtInformation delegate)
//------------------------------------
- (BOOL)textView:(NSTextView *)textView clickedOnLink:(id)link atIndex:(unsigned)charIndex
{
	NSDictionary *dict = [[textView textStorage] attributesAtIndex:charIndex effectiveRange:nil];
	if(!dict){
		return NO;
	}

	NSString *urlString = (NSString*)[dict objectForKey:NSLinkAttributeName];

	// open url
	[self openURLWithBrowser:urlString];

	// auto close
	[self autoCloseWindow];

	return YES;
}
//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
    [params_ release];
    params_ = params;
}
- (NSDictionary*)params
{
    return params_;
}
//------------------------------------
// arrayNo
//------------------------------------
- (void)setArrayNo:(int)arrayNo
{
    arrayNo_ = arrayNo;
}
- (int)arrayNo
{
    return arrayNo_;
}
//------------------------------------
// refViewNo
//------------------------------------
- (void)setRefViewNo:(int)refViewNo
{
    refViewNo_ = refViewNo;
}
- (int)refViewNo
{
    return refViewNo_;
}
//------------------------------------
// isLocked
//------------------------------------
- (void)setIsLocked:(BOOL)isLocked
{
    isLocked_ = isLocked;
}
- (BOOL)isLocked
{
    return isLocked_;
}
//----------------------
// hasVideoId
//----------------------
- (BOOL)hasVideoId
{

	NSDictionary *params = [self params]; 
	if(params == nil){
		return NO;
	}

	NSString *videoId = [params objectForKey:@"videoId"]; 

	if(videoId && ![videoId isEqualToString:@""]){
		return YES;
	}else{
		return NO;
	}
}
//----------------------
// windowIsVisible
//----------------------
- (BOOL)windowIsVisible
{
	return [videoContentWindow isVisible];
}
//----------------------
// selectedStringInDescriptionIsNull
//----------------------
- (BOOL)selectedStringInDescriptionIsNull
{
	return [txtDescription selectedStringIsNull];
}
//----------------------
// selectedStringInDescriptionIsNotNull
//----------------------
- (BOOL)selectedStringInDescriptionIsNotNull
{
	return ![self selectedStringInDescriptionIsNull];
}
//----------------------
// selectedStringInDescriptionIsAlphabet
//----------------------
- (BOOL)selectedStringInDescriptionIsAlphabet
{
	return [txtDescription selectedStringIsAlphabet];
}
//----------------------
// selectedStringInDescriptionIsNumber
//----------------------
- (BOOL)selectedStringInDescriptionIsNumber
{
	return [txtDescription selectedStringIsNumber];
}
//----------------------
// selectedStringInDescriptionIsAlphabetAndNumberOrNumber
//----------------------
- (BOOL)selectedStringInDescriptionIsAlphabetAndNumberOrNumber;
{
	return [txtDescription selectedStringIsAlphabetAndNumberOrNumber];
}
//----------------------
// selectedStringInDescriptionIsMylistNo
//----------------------
- (BOOL)selectedStringInDescriptionIsMylistNo
{
	return [txtDescription selectedStringIsMylistNo];
}
//----------------------
// selectedStringInDescriptionIsNotMylistNo
//----------------------
- (BOOL)selectedStringInDescriptionIsNotMylistNo
{
	return ![self selectedStringInDescriptionIsMylistNo];
}
//----------------------
// selectedStringInDescriptionIsPostUserId
//----------------------
- (BOOL)selectedStringInDescriptionIsPostUserId
{
	return [txtDescription selectedStringIsPostUserId];
}
//----------------------
// selectedStringInDescriptionIsNotPostUserId
//----------------------
- (BOOL)selectedStringInDescriptionIsNotPostUserId
{
	return ![self selectedStringInDescriptionIsPostUserId];
}
//----------------------
// selectedStringInDescriptionIsURL
//----------------------
- (BOOL)selectedStringInDescriptionIsURL
{
	return [txtDescription selectedStringIsURL];
}
//----------------------
// selectedStringInDescriptionIsNotURL
//----------------------
- (BOOL)selectedStringInDescriptionIsNotURL
{
	return ![self selectedStringInDescriptionIsURL];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	[self saveWindowRect:videoContentWindow key:@"rectWindowVideoContent"];

	// split view rect
	[self saveSplitViewRect:spvContentMain key:@"rectSplitViewContentMain"];
	[self saveSplitViewRect:spvContentInfo key:@"rectSplitViewContentInfo"];

//	[self clearVideoContent];

}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	// remove observer
	[targetArrayController_ removeObserver:self forKeyPath:@"selection"];
	[targetArrayController_ release];

	[params_ release];
	[super dealloc];
}
@end
