/* WebPlayerView */

#import <Cocoa/Cocoa.h>
#import <WebKit/WebKit.h>
#import "PlayerTypes.h"
#import "ProcInfoStatus.h"
#import "ConnectionDataItem.h"
#import "VideoObjectStatus.h"

@interface WebPlayerView : WebView
{

	IBOutlet id viewPlayer;
	IBOutlet id logStatusController;

	IBOutlet NSTabView *tabViewController;

	IBOutlet NSTextField *txtVideoInfo;
	IBOutlet NSProgressIndicator *indProc;

	NSString *videoId_;
	NSString *videoURL_;
	NSString *jsTemplate_;
	NSString *title_;

	ConnectionDataItem *connectionDataItem_;

	int playerType_;
	int pageCount_;

	NSString *nicoPlayerHTML_;
	NSString *nicoPlayerQHTML_;
	NSString *extPlayerHTML_;
	NSString *iPadPlayerHTML_;
	NSString *swfobjectPath_;

	BOOL isNicoPlayerHTML_;
	BOOL isNicoPlayerQHTML_;
	BOOL isExtPlayerHTML_;
	BOOL isIPadPlayerHTML_;

}
- (IBAction)cancelSearch:(id)sender;
- (IBAction)reloadPlayer:(id)sender;
- (IBAction)changeShowInputArea:(id)sender;
- (IBAction)changeMaximizePlayer:(id)sender;

- (BOOL)setPlayerView:(NSString*)videoId title:(NSString*)title reload:(BOOL)reload;
- (BOOL)setPlayerInfo:(NSString*)videoId title:(NSString*)title;
- (BOOL)displayPlayerViewWithExternalPlayer:(NSString*)videoId title:(NSString*)title playerType:(int)playerType;
- (BOOL)displayPlayerViewWithIPadPlayer:(NSString*)videoId title:(NSString*)title playerType:(int)playerType;
- (BOOL)displayPlayerViewWithNicoPlayer:(NSString*)videoId title:(NSString*)title playerType:(int)playerType;
- (BOOL)displayPlayerViewWithNicoPlayerQ:(NSString*)videoId title:(NSString*)title playerType:(int)playerType;
- (BOOL)displayPlayerViewWithWatchURL:(NSString*)videoId title:(NSString*)title playerType:(int)playerType;

- (BOOL)resizePlayerView;
- (void)deletePlayerView;
- (BOOL)reloadPlayerWithJsTemplate;

- (NSString*)runJavaScriptWithString:(NSString*)string;

- (void)setWebPlayerWithJsTemplate:(NSString*)jsTemplate videoId:(NSString*)videoId;
- (void)setWebPlayerWithExternalPlayer:(NSString*)html params:(NSDictionary*)params;
- (void)setWebPlayerWithNicoPlayer:(NSString*)html params:(NSDictionary*)params;
- (void)setWebPlayerWithNicoPlayerQ:(NSString*)html params:(NSDictionary*)params;
- (void)setWebPlayerWithIPadPlayer:(NSString*)html params:(NSDictionary*)params;

- (void)setWebPlayerWithNicoPlayerHTML:(NSString*)html params:(NSDictionary*)params videoId:(NSString*)videoId;
- (void)setWebPlayerWithNicoPlayerQHTML:(NSString*)html params:(NSDictionary*)params videoId:(NSString*)videoId;
- (void)setWebPlayerWithExternalPlayerHTML:(NSString*)html params:(NSDictionary*)params videoId:(NSString*)videoId;

- (NSString*)replaceJsTemplateParams:(NSString*)jsTemplate;
- (NSString*)createExternalParamVarsString:(NSDictionary*)params;
- (NSDictionary*)getExternalPlayerParams:(NSString*)html;
- (NSDictionary*)getNicoPlayerParams:(NSString*)html;
- (NSMutableDictionary*)getNicoPlayerQParams:(NSString*)html;
- (NSDictionary*)getIPadPlayerParams:(NSString*)html;

- (NSDictionary*)getWatchPlayerStrings:(NSString*)html;
- (NSDictionary*)getWatchPlayerQStrings:(NSString*)html;
- (NSString*)getWatchVideoVarString:(NSString*)html;

- (BOOL)setWebPlayerWithURL:(NSString*)urlString;

 - (void)setWebPlayerWithHTML:(NSString*)html videoId:(NSString*)videoId;

- (void)setPlayerWindowStatus:(NSString*)videoId title:(NSString*)title playerType:(int)playerType;
- (void)setWindowTitle:(NSString*)title;
- (NSSize)calcWebPlayerSize:(NSSize)playerSize
					frameSize:(NSSize)frameSize
					paddingSize:(NSSize)paddingSize;

- (void)changeTabViewControllerIndex;

- (void)changeLiveResize;

- (void)changeNicoPlayerState:(NSString*)state;
- (void)setLoadIndicator:(int)state;
- (void)setMaximizePlayerEnable:(int)state;
- (void)setMaximizePlayerState:(int)state;
- (void)setPlayerIsPlayed:(int)state;
- (void)receiveLogMessage:(NSString*)string;

- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem;
- (ConnectionDataItem*)connectionDataItem;
- (void)clearConnection;

- (void)changePlayerInfoWithVideoId;

- (void)handleDataItemDidLoad:(NSNotification *)notification;
- (void)handleDataItemStatus:(NSNotification *)notification;
- (void)postVideoObjectChangedNotification;

- (void)setupHTMLPlayer;

- (void)setNicoPlayerHTML:(NSString*)nicoPlayerHTML;
- (NSString*)nicoPlayerHTML;
- (void)setIsNicoPlayerHTML:(BOOL)isNicoPlayerHTML;
- (BOOL)isNicoPlayerHTML;

- (void)setNicoPlayerQHTML:(NSString*)nicoPlayerQHTML;
- (NSString*)nicoPlayerQHTML;
- (void)setIsNicoPlayerQHTML:(BOOL)isNicoPlayerQHTML;
- (BOOL)isNicoPlayerQHTML;

- (void)setExtPlayerHTML:(NSString*)extPlayerHTML;
- (NSString*)extPlayerHTML;
- (void)setIsExtPlayerHTML:(BOOL)isExtPlayerHTML;
- (BOOL)isExtPlayerHTML;

- (void)setIPadPlayerHTML:(NSString*)iPadPlayerHTML;
- (NSString*)iPadPlayerHTML;
- (void)setIsIPadPlayerHTML:(BOOL)isIPadPlayerHTML;
- (BOOL)isIPadPlayerHTML;

- (void)setSwfobjectPath:(NSString*)swfobjectPath;
- (NSString*)swfobjectPath;

- (void)setVideoId:(NSString*)videoId;
- (NSString*)videoId;
- (void)setVideoURL:(NSString*)videoURL;
- (NSString*)videoURL;
- (void)setJsTemplate:(NSString*)jsTemplate;
- (NSString*)jsTemplate;
- (void)setTitle:(NSString*)title;
- (NSString*)title;
- (void)setPlayerType:(int)playerType;
- (int)playerType;
- (void)setPageCount:(int)pageCount;
- (int)pageCount;
- (BOOL)hasVideoId;

- (BOOL)canChangePlayerScale;
- (BOOL)canChangeMaximize;
- (BOOL)canChangeComment;
- (BOOL)canReloadPlayer;

- (BOOL)isMaximizedPlayer;

@end
