#import "WebPlayerView.h"
#import "ViewPlayer.h"
#import "LogStatusController.h"
#import "FullScreenWindow.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

static const int PLAYER_BASE_WIDTH = 485;
static const int PLAYER_BASE_HEIGHT = 385;
static const int PLAYER_PADDING_WIDTH = 0;
static const int PLAYER_PADDING_HEIGHT = 0;
//static const int PLAYER_REP_WIDTH = -999999;
//static const int PLAYER_REP_HEIGHT = -888888;

@implementation WebPlayerView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// super delegate
	[super setFrameLoadDelegate:self];

	// WebFrameView delegate
//	[[[self mainFrame] frameView] setDelegate:self];
//	[[[self mainFrame] frameView] setAllowsScrolling:YES];

	[self setVideoId:@""];
	[self setVideoURL:@""];
	[self setJsTemplate:@""];

	[self setNicoPlayerHTML:@""];
	[self setIsNicoPlayerHTML:NO];

	[self setNicoPlayerQHTML:@""];
	[self setIsNicoPlayerQHTML:NO];

	[self setExtPlayerHTML:@""];
	[self setIsExtPlayerHTML:NO];

	[self setIPadPlayerHTML:@""];
	[self setIsIPadPlayerHTML:NO];

	[self setSwfobjectPath:@""];

	// set up html player
	[self setupHTMLPlayer];

	[self setTitle:@""];
	[self setPlayerType:0];
	[self setPageCount:0];

	[txtVideoInfo setStringValue:@""];

	[self changeTabViewControllerIndex];

	// set notification
//	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
//	[nc addObserver:self selector:@selector(windowDidResizePlayer:) name:NSWindowDidResizeNotification object:[self window]];

	// add observer
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addObserver:self forKeyPath:@"optNicoPlayerLiveResize" options:0 context:nil];

}
//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// cancelSearch
//------------------------------------
- (IBAction)cancelSearch:(id)sender
{
	[self clearConnection];
}
//------------------------------------
// reloadPlayer
//------------------------------------
- (IBAction)reloadPlayer:(id)sender
{
	[self setPlayerView:[self videoId] title:[self title] reload:YES];
}
//----------------------
// changeShowInputArea
//----------------------
- (IBAction)changeShowInputArea:(id)sender
{
	BOOL showController = [self defaultBoolValue:@"optNicoPlayerShowController"];
	BOOL showInputArea = [self defaultBoolValue:@"optNicoPlayerShowInputArea"];
	int stateController = 0;
	int stateInputArea = 0;

	showInputArea = !showInputArea;

	// set state
	if(showController == YES){
		stateController = 1;
	}
	if(showInputArea == YES){
		stateInputArea = 1;
	}
	// run javascript
	[self runJavaScriptWithString:[NSString stringWithFormat:@"setShowController(%d)", stateController]];
	[self runJavaScriptWithString:[NSString stringWithFormat:@"setShowInputArea(%d)", stateInputArea]];
	[self runJavaScriptWithString:@"updatePlayer()"];

	// set default
	[self setDefaultBoolValue:showController key:@"optNicoPlayerShowController"];
	[self setDefaultBoolValue:showInputArea key:@"optNicoPlayerShowInputArea"];

	// change button
//	[self changeCommentButtonImage];

}
//----------------------
// changeMaximizePlayer
//----------------------
- (IBAction)changeMaximizePlayer:(id)sender
{

	[self runJavaScriptWithString:@"changeMaximizePlayer()"];

}

//=======================================================================
// methods
//=======================================================================
//----------------------
// setPlayerView
//----------------------
- (BOOL)setPlayerView:(NSString*)videoId title:(NSString*)title reload:(BOOL)reload
{

	// null
	if(!videoId || [videoId isEqualToString:@""]){
		return NO;
	}

	int playerType = [self defaultPlayerType];

	// same object
	if(reload == NO){
		if(playerType == [self playerType] &&
			[videoId isEqualToString:[self videoId]]){
			return YES;
		}
	}

	// clear connection
	[self clearConnection];

	// clear page count
	[self setPageCount:0];

	// nico player
	if(playerType == PLAYER_TYPE_NICO_PLAYER){
		return [self displayPlayerViewWithNicoPlayer:videoId title:title playerType:playerType];
	}
	// nico player Q
	else if(playerType == PLAYER_TYPE_NICO_PLAYER_Q){
		return [self displayPlayerViewWithNicoPlayerQ:videoId title:title playerType:playerType];
	}
	// ext player
	else if(playerType == PLAYER_TYPE_EXT_PLAYER){
		return [self displayPlayerViewWithExternalPlayer:videoId title:title playerType:playerType];
	}
	// ipad player
//	else if(playerType == PLAYER_TYPE_IPAD_PLAYER){
//		return [self displayPlayerViewWithIPadPlayer:videoId title:title playerType:playerType];
//	}
	// nico watch
	else if(playerType == PLAYER_TYPE_WATCH_VIDEO){
		return [self displayPlayerViewWithWatchURL:videoId title:title playerType:playerType];
	}

	return NO;

}
//----------------------
// setPlayerInfo
//----------------------
- (BOOL)setPlayerInfo:(NSString*)videoId title:(NSString*)title
{

	// null
	if(!videoId || [videoId isEqualToString:@""]){
		return NO;
	}

	// same object
	if([videoId isEqualToString:[self videoId]]){
		return YES;
	}

	// set window status
	[self setPlayerWindowStatus:videoId title:title playerType:[self playerType]];

	return YES;

}


//----------------------
// displayPlayerViewWithExternalPlayer
//----------------------
- (BOOL)displayPlayerViewWithExternalPlayer:(NSString*)videoId title:(NSString*)title playerType:(int)playerType
{

	NSSize frameSize = [self frame].size;
	NSString *thumbWatchURL = [self convertToExternalThumbWatchURL:videoId width:frameSize.width height:frameSize.height];
//	NSString *thumbWatchURL = [self convertToExternalThumbWatchURL:videoId width:PLAYER_REP_WIDTH height:PLAYER_REP_HEIGHT];
//	NSLog(@"thumbWatchURL=%@", thumbWatchURL);

	// stop playing
	[self runJavaScriptWithString:@"setVideoPlay(0)"];
	// stop loading
	[[self mainFrame] stopLoading];

	// set link params
	NSDictionary *linkParams = [NSDictionary dictionaryWithObjectsAndKeys:
											[NSNumber numberWithInt:playerType], @"playerType" ,
											thumbWatchURL, @"urlString" ,
											videoId, @"videoId" ,
											title, @"title" ,
											nil
							];

	// get html data
	NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
											[self convertToBaseURL], @"referer" ,
											@"", @"userAgent" ,
											nil
							];
	[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:thumbWatchURL
																userInfo:userInfo
																notifyTarget:self
																params:linkParams
								] autorelease]
	];

	return YES;
}
//----------------------
// displayPlayerViewWithIPadPlayer
//----------------------
- (BOOL)displayPlayerViewWithIPadPlayer:(NSString*)videoId title:(NSString*)title playerType:(int)playerType
{

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return NO;
	}

	// stop playing
	[self runJavaScriptWithString:@"setVideoPlay(0)"];
	// stop loading
	[[self mainFrame] stopLoading];

	NSString *watchURL = [self convertToWatchURL:videoId];

	// eco mode
//	if([self defaultEcoMode] == YES){
//		watchURL = [self appendEcoModeStringToURL:watchURL];
//	}
//	NSLog(@"watchURL=%@", watchURL);

	// set link params
	NSDictionary *linkParams = [NSDictionary dictionaryWithObjectsAndKeys:
											[NSNumber numberWithInt:playerType], @"playerType" ,
											watchURL, @"urlString" ,
											videoId, @"videoId" ,
											title, @"title" ,
											nil
							];

	// get html data
	NSString *userAgent = @"Mozilla/5.0 (iPad; U; CPU OS 3_2_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B500 Safari/531.21.10";
	NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
											[self convertToBaseURL], @"referer" ,
											userAgent, @"userAgent" ,
											nil
							];
	[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:watchURL
																userInfo:userInfo
																notifyTarget:self
																params:linkParams
								] autorelease]
	];

	return YES;
}
//----------------------
// displayPlayerViewWithNicoPlayer
//----------------------
- (BOOL)displayPlayerViewWithNicoPlayer:(NSString*)videoId title:(NSString*)title playerType:(int)playerType
{

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return NO;
	}

	// stop playing
	[self runJavaScriptWithString:@"setVideoPlay(0)"];
	// stop loading
	[[self mainFrame] stopLoading];

	NSString *watchURL = [self convertToWatchURL:videoId];
	// eco mode
	if([self defaultEcoMode] == YES){
		watchURL = [self appendEcoModeStringToURL:watchURL];
	}
//	NSLog(@"watchURL=%@", watchURL);

	// set link params
	NSDictionary *linkParams = [NSDictionary dictionaryWithObjectsAndKeys:
											[NSNumber numberWithInt:playerType], @"playerType" ,
											watchURL, @"urlString" ,
											videoId, @"videoId" ,
											title, @"title" ,
											nil
							];

	// get html data
//	NSString *userAgent = @"Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
	NSString *userAgent = @"";
	NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
											[self convertToBaseURL], @"referer" ,
											userAgent, @"userAgent" ,
											nil
							];
	[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:watchURL
																userInfo:userInfo
																notifyTarget:self
																params:linkParams
								] autorelease]
	];

	return YES;
}
//----------------------
// displayPlayerViewWithNicoPlayer
//----------------------
- (BOOL)displayPlayerViewWithNicoPlayerQ:(NSString*)videoId title:(NSString*)title playerType:(int)playerType
{

	// check user session
	if([self checkUserSession] == NO){
		int result = [self displayMessageAlertNeedsLogin];
		// open login page
		if(result == NSAlertFirstButtonReturn){
//			[self openLoginURLWithBrowser];
			[self openLoginURLWithPlayer];
		}
		return NO;
	}

	// stop playing
//	[self runJavaScriptWithString:@"setVideoPlay(0)"];
	// stop loading
	[[self mainFrame] stopLoading];

	NSString *watchURL = [self convertToWatchQPlayerURL:videoId];
	// eco mode
	if([self defaultEcoMode] == YES){
		watchURL = [self appendEcoModeStringToURL:watchURL];
	}
//	NSLog(@"watchURL=%@", watchURL);

	// set link params
	NSDictionary *linkParams = [NSDictionary dictionaryWithObjectsAndKeys:
											[NSNumber numberWithInt:playerType], @"playerType" ,
											watchURL, @"urlString" ,
											videoId, @"videoId" ,
											title, @"title" ,
											nil
							];

	// get html data
//	NSString *userAgent = @"Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0 Safari/533.16";
	NSString *userAgent = @"";
	NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
											[self convertToBaseURL], @"referer" ,
											userAgent, @"userAgent" ,
											nil
							];
	[self setConnectionDataItem:[[[ConnectionDataItem alloc] initWithURL:watchURL
																userInfo:userInfo
																notifyTarget:self
																params:linkParams
								] autorelease]
	];

	return YES;
}
//----------------------
// displayPlayerViewWithWatchURL
//----------------------
- (BOOL)displayPlayerViewWithWatchURL:(NSString*)videoId title:(NSString*)title playerType:(int)playerType
{

	NSString *videoURL = [self convertToWatchURL:videoId];
	// eco mode
	if([self defaultEcoMode] == YES){
		videoURL = [self appendEcoModeStringToURL:videoURL];
	}

	if([self setWebPlayerWithURL:videoURL] == YES){

		[self setVideoURL:videoURL];

		// set window status
		[self setPlayerWindowStatus:videoId title:title playerType:playerType];

		return YES;

	}else{
		return NO;
	}

	return NO;

}

//------------------------------------
// handleDataItemDidLoad
//------------------------------------
- (void)handleDataItemDidLoad:(NSNotification *)notification
{

//	NSLog(@"handleDataItemDidLoad");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	NSData *htmlData = [connectionDataItem data];
	NSError *error = [connectionDataItem error];
	int result;

	// link params
	NSDictionary *linkParams = [connectionDataItem params];
	NSString *urlString = [linkParams objectForKey:@"urlString"];

	if(error != nil){
		result = [self displayMessageAlertGetData];
		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController openLogWindowWithError:urlString error:error];
		}
		return;
	}

	NSString *videoId = [linkParams objectForKey:@"videoId"];
	NSString *title = [linkParams objectForKey:@"title"];
	int playerType = [[linkParams objectForKey:@"playerType"] intValue];

	// create html
	NSStringEncoding encoding = [self getStringEncoding:htmlData];
	NSString *html = [[[NSString alloc] initWithData:htmlData encoding:encoding] autorelease];
//	NSString *html = [[[NSString alloc] initWithData:htmlData encoding:NSUTF8StringEncoding] autorelease];

//	NSLog(@"html=%@", html);

	if(html == nil){
		// set error log
		result = [self displayMessageAlertGetData];
		// show error log
		if(result == NSAlertSecondButtonReturn){
			[logStatusController setTitle:@"Error Log"];
			[logStatusController setLogString:[NSString stringWithFormat:@"Can not init html from url=%@\n", urlString]];
			[logStatusController openLogWindow:nil];
		}
		return;
	}
//	NSLog(@"html=%@", html);

	//
	// set player
	//
	// nico player
	if(playerType == PLAYER_TYPE_NICO_PLAYER){
		[self setWebPlayerWithNicoPlayer:html params:linkParams];
	}
	// nico player Q
	else if(playerType == PLAYER_TYPE_NICO_PLAYER_Q){
		[self setWebPlayerWithNicoPlayerQ:html params:linkParams];
	}
	// ext player
	else if(playerType == PLAYER_TYPE_EXT_PLAYER){
		[self setWebPlayerWithExternalPlayer:html params:linkParams];
	}
	// ipad player
//	else if(playerType == PLAYER_TYPE_IPAD_PLAYER){
//		[self setWebPlayerWithIPadPlayer:html params:linkParams];
//	}

	// set window status
	[self setPlayerWindowStatus:videoId title:title playerType:playerType];

	[self setVideoURL:urlString];


}
//------------------------------------
// handleDataItemStatus
//------------------------------------
- (void)handleDataItemStatus:(NSNotification *)notification
{

//	NSLog(@"handleDataItemStatus");

	ConnectionDataItem *connectionDataItem = [notification object];

	if(connectionDataItem == nil){
		return;
	}

	//  indicator
	int status = [connectionDataItem status];

	if(status == CONNECTION_INIT || status == CONNECTION_STARTED){
		[self setLoadIndicator:PROC_IND_STATUS_START];
	}
	else{
		[self setLoadIndicator:PROC_IND_STATUS_STOP];
	}
}
//------------------------------------
// postVideoObjectChangedNotification
//------------------------------------
- (void)postVideoObjectChangedNotification
{
	BOOL hasVideoId = [self hasVideoId];
	BOOL canChangeMaximize = [self canChangeMaximize];
	BOOL canChangeComment = [self canChangeComment];

	NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:
							[NSNumber numberWithBool:hasVideoId], @"hasVideoId" ,
							[NSNumber numberWithBool:canChangeMaximize], @"canChangeMaximize" ,
							[NSNumber numberWithBool:canChangeComment], @"canChangeComment" ,
							nil
						];

	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_OBJECT_DID_CHANGED object:params];
}
//----------------------
// resizePlayerView
//----------------------
- (BOOL)resizePlayerView
{
//	NSLog(@"resizePlayer");

//	[self runJavaScriptWithString:@"resizePlayer();"];
	[self runJavaScriptWithString:@"adjustBlank();"];
	return YES;
}

//----------------------
// deletePlayerView
//----------------------
- (void)deletePlayerView
{
	[[self mainFrame] stopLoading];
	[[self mainFrame] loadRequest:nil];
	[[self mainFrame] loadHTMLString:nil baseURL:nil];

	[self setVideoId:@""];
	[self setVideoURL:@""];
	[self setJsTemplate:@""];
	[self setTitle:@""];
	[self setPageCount:0];

	[self changeTabViewControllerIndex];

	[self clearConnection];

	[[self window] setTitle:@"Player"];
	[txtVideoInfo setStringValue:@""];

	// post notification
	[self postVideoObjectChangedNotification];

}
//----------------------
// reloadPlayerWithJsTemplate
//----------------------
- (BOOL)reloadPlayerWithJsTemplate
{

	NSString *videoId = [self videoId];
	NSString *jsTemplate = [self jsTemplate];

	// null
	if(!videoId || [videoId isEqualToString:@""]){
		return NO;
	}
	// null
	if(!jsTemplate || [jsTemplate isEqualToString:@""]){
		return NO;
	}

	// stop playing
	[self runJavaScriptWithString:@"setVideoPlay(0)"];
	// stop loading
	[[self mainFrame] stopLoading];

	// clear page count
	[self setPageCount:0];

	// get params
	NSDictionary *playerParams = nil;
	BOOL getParams = NO;
	if([self isExtPlayerHTML] == YES){
		playerParams = [self getExternalPlayerParams:jsTemplate];
		getParams = [[playerParams objectForKey:@"getParams"] boolValue];
	}
	// html player
	if(getParams == YES){
		[self setWebPlayerWithExternalPlayerHTML:[self extPlayerHTML] params:playerParams videoId:videoId];
	}
	// javascript player
	else{
		[self setWebPlayerWithJsTemplate:jsTemplate videoId:videoId];
	}

	return YES;

}
//------------------------------------
// runJavaScriptWithString
//------------------------------------
- (NSString*)runJavaScriptWithString:(NSString*)string
{

	// run javascript
	return [self stringByEvaluatingJavaScriptFromString:string];

}

//----------------------
// setWebPlayerWithJsTemplate
//----------------------
- (void)setWebPlayerWithJsTemplate:(NSString*)jsTemplate videoId:(NSString*)videoId
{

	// replace params
	jsTemplate = [self replaceJsTemplateParams:jsTemplate];

	NSString *html = [self convertToPlayerHTML:jsTemplate];
//	NSLog(@"html=%@", html);

	[self setWebPlayerWithHTML:html videoId:videoId];

}

//------------------------------------
// setWebPlayerWithExternalPlayer
//------------------------------------
- (void)setWebPlayerWithExternalPlayer:(NSString*)html params:(NSDictionary*)params
{

	// link params
	NSString *videoId = [params objectForKey:@"videoId"];

	// set template
	[self setJsTemplate:html];

	// get params
	NSDictionary *playerParams = nil;
	BOOL getParams = NO;
	if([self isExtPlayerHTML] == YES){
		playerParams = [self getExternalPlayerParams:html];
		getParams = [[playerParams objectForKey:@"getParams"] boolValue];
	}
	// html player
	if(getParams == YES){
		[self setWebPlayerWithExternalPlayerHTML:[self extPlayerHTML] params:playerParams videoId:videoId];
	}
	// javascript player
	else{
		[self setWebPlayerWithJsTemplate:html videoId:videoId];
	}

}
//------------------------------------
// setWebPlayerWithNicoPlayer
//------------------------------------
- (void)setWebPlayerWithNicoPlayer:(NSString*)html params:(NSDictionary*)params
{

	// link params
	NSString *videoId = [params objectForKey:@"videoId"];
	NSString *title = [params objectForKey:@"title"];
	int playerType = [[params objectForKey:@"playerType"] intValue];

	// get player string
	NSDictionary *dictPlayers = [self getWatchPlayerStrings:html];
	NSString *playerString = [dictPlayers objectForKey:@"playerString"];

	// change external player
	if([playerString isEqualToString:@""]){
		[self displayPlayerViewWithExternalPlayer:videoId title:title playerType:PLAYER_TYPE_EXT_PLAYER];
		return;
	}

//	NSLog(@"playerString=%@", playerString);

	// get player params
	NSDictionary *playerParams = nil;
	BOOL getPlayerUrl = NO;
	BOOL getVars = NO;
	if( (playerType == PLAYER_TYPE_NICO_PLAYER && [self isNicoPlayerHTML] == YES)
	){
		playerParams = [self getNicoPlayerParams:playerString];
		getPlayerUrl = [[playerParams objectForKey:@"getPlayerUrl"] boolValue];
		getVars = [[playerParams objectForKey:@"getVars"] boolValue];
	}

	// html player
	if(getPlayerUrl == YES && getVars == YES){
		if(playerType == PLAYER_TYPE_NICO_PLAYER){
			[self setWebPlayerWithNicoPlayerHTML:[self nicoPlayerHTML] params:playerParams videoId:videoId];
		}
	}
	// javascript player
	else{
		NSString *playerHTML = [NSString stringWithFormat:@"%@\n%@\n%@\n%@",
											 @"<html><body bgcolor=black text=white style=\"margin:0; text-align:center;\">",
											 @"<script type=\"text/javascript\" src=\"http://res.nicovideo.jp/js/swfobject.js?4\"></script>",
											 playerString,
											 @"--></script></body></html>"
								];
		[self setWebPlayerWithHTML:playerHTML videoId:videoId];
	}

}
//------------------------------------
// setWebPlayerWithNicoPlayerQ
//------------------------------------
- (void)setWebPlayerWithNicoPlayerQ:(NSString*)html params:(NSDictionary*)params
{

//	NSLog(@"html=%@", html);

	// link params
	NSString *videoId = [params objectForKey:@"videoId"];
	NSString *title = [params objectForKey:@"title"];
	int playerType = [[params objectForKey:@"playerType"] intValue];

	NSMutableDictionary *playerParams = nil;
	BOOL getPlayerUrl = NO;
	BOOL getVars = NO;

	// get player & api string
	NSDictionary *dictPlayers = [self getWatchPlayerQStrings:html];
	NSString *playerString = [dictPlayers objectForKey:@"playerString"];
	NSString *APIString = [dictPlayers objectForKey:@"APIString"];

//	NSLog(@"APIString=%@", APIString);
//	NSLog(@"playerString=%@", playerString);

	// change external player
	if([playerString isEqualToString:@""] || [APIString isEqualToString:@""]){
		[self displayPlayerViewWithExternalPlayer:videoId title:title playerType:PLAYER_TYPE_EXT_PLAYER];
		return;
	}

	// get player params
	if( (playerType == PLAYER_TYPE_NICO_PLAYER_Q && [self isNicoPlayerQHTML] == YES)
	){
		playerParams = [self getNicoPlayerQParams:playerString];
		getPlayerUrl = [[playerParams objectForKey:@"getPlayerUrl"] boolValue];
		getVars = [[playerParams objectForKey:@"getVars"] boolValue];
		[playerParams setValue:APIString forKey:@"APIString"];
	}

	// html player
	if(getPlayerUrl == YES && getVars == YES){
		if(playerType == PLAYER_TYPE_NICO_PLAYER_Q){
			[self setWebPlayerWithNicoPlayerQHTML:[self nicoPlayerQHTML] params:playerParams videoId:videoId];
		}
	}
	// javascript player
	else{
		NSString *playerHTML = [NSString stringWithFormat:@"%@\n%@\n%@\n%@",
											 @"<html><body bgcolor=black text=white style=\"margin:0; text-align:center;\">",
											 @"<script type=\"text/javascript\" src=\"http://res.nicovideo.jp/js/swfobject.js?4\"></script>",
											 playerString,
											 @"--></script></body></html>"
								];
		[self setWebPlayerWithHTML:playerHTML videoId:videoId];
	}

}

//------------------------------------
// setWebPlayerWithNicoPlayerHTML
//------------------------------------
- (void)setWebPlayerWithNicoPlayerHTML:(NSString*)html params:(NSDictionary*)params videoId:(NSString*)videoId
{

	// get value
	NSString *playerUrl = [params objectForKey:@"playerUrl"];
	NSString *playerVars = [[params objectForKey:@"lineVars"] componentsJoinedByString:@"\n"];
//	playerUrl = [self replaceCharacter:playerUrl str1:@"/player/" str2:@"/player_ps3/"];

/*
	// test zero player
	// http://res.nimg.jp/swf/player_zero/nicoplayer.swf
	// http://res.nimg.jp/swf/player/nicoplayer.swf?ts=1344497605
	playerUrl = [self replaceCharacter:playerUrl str1:@"/player/" str2:@"/player_zero/"];
	NSString *ts = [params objectForKey:@"ts"];
	NSLog(@"playerUrl=%@", playerUrl);
	NSLog(@"ts=%@", ts);
*/

	// player defaults
	int autoPlay = 0;
	int autoMaximize = 0;
	int noBanner = 0;
	int noMarquee = 0;
	int noNicowari = 0;

	if([self defaultBoolValue:@"optNicoPlayerAutoMaximize"] == YES){
		autoMaximize = 1;
	}
	if([self defaultBoolValue:@"optAutoPlay"] == YES){
		autoPlay = 1;
	}

	if([self defaultBoolValue:@"optNicoPlayerNoBanner"] == YES){
		noBanner = 1;
	}
	if([self defaultBoolValue:@"optNicoPlayerNoMarquee"] == YES){
		noMarquee = 1;
	}
	if([self defaultBoolValue:@"optNicoPlayerNoNicowari"] == YES){
		noNicowari = 1;
	}

//	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('thumbWatch', '%d');\n", 1]];
	if(noBanner == 1){
		playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('noBanner', '%d');\n", noBanner]];
	}
	if(noMarquee == 1){
		playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('noMarquee', '%d');\n", noMarquee]];
	}
	if(noNicowari == 1){
		playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('forceNicowariOff', '%d');\n", noNicowari]];
	}
	playerVars = [playerVars stringByAppendingString:@"\n"];

	// replace parameters
	html = [self replaceCharacter:html str1:@"..swfobject_path" str2:[self swfobjectPath]];
	html = [self replaceCharacter:html str1:@"..playerUrl" str2:playerUrl];
	html = [self replaceCharacter:html str1:@"..playerVars" str2:playerVars];

	html = [self replaceCharacter:html str1:@"..autoMaximize" str2:[NSString stringWithFormat:@"%d", autoMaximize]];
	html = [self replaceCharacter:html str1:@"..autoPlay" str2:[NSString stringWithFormat:@"%d", autoPlay]];

//	NSLog(@"html=%@", html);

	[self setWebPlayerWithHTML:html videoId:videoId];

}
//------------------------------------
// setWebPlayerWithNicoPlayerQHTML
//------------------------------------
- (void)setWebPlayerWithNicoPlayerQHTML:(NSString*)html params:(NSDictionary*)params videoId:(NSString*)videoId
{

	// get value
	NSString *playerUrl = [params objectForKey:@"playerUrl"];
	NSString *APIString = [params objectForKey:@"APIString"];
	NSString *playerVars = [[params objectForKey:@"lineVars"] componentsJoinedByString:@"\n"];

//	APIString = [self replaceCharacter:APIString str1:@"\"eco\":1" str2:@"\"eco\":0"];
//	APIString = [self replaceCharacter:APIString str1:@"\"isPremium\":false" str2:@"\"isPremium\":true"];
//	APIString = [self replaceCharacter:APIString str1:@"\"isPrivileged\":false" str2:@"\"isPrivileged\":true"];

	// player defaults
	int autoPlay = 0;
	int noBanner = 0;
	int noMarquee = 0;
	int noNicowari = 0;

	if([self defaultBoolValue:@"optAutoPlay"] == YES){
		autoPlay = 1;
	}

	if([self defaultBoolValue:@"optNicoPlayerNoBanner"] == YES){
		noBanner = 1;
	}
	if([self defaultBoolValue:@"optNicoPlayerNoMarquee"] == YES){
		noMarquee = 1;
	}
	if([self defaultBoolValue:@"optNicoPlayerNoNicowari"] == YES){
		noNicowari = 1;
	}

//	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('thumbWatch', '%d');\n", 1]];
	if(noBanner == 1){
//		playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('noBanner', '%d');\n", noBanner]];
	}
	if(noMarquee == 1){
//		playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('noMarquee', '%d');\n", noMarquee]];
	}
	if(noNicowari == 1){
//		playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('forceNicowariOff', '%d');\n", noNicowari]];
	}
	playerVars = [playerVars stringByAppendingString:@"\n"];

	// replace parameters
	html = [self replaceCharacter:html str1:@"..watch_api_data_string" str2:APIString];
	html = [self replaceCharacter:html str1:@"..playerUrl" str2:playerUrl];
	html = [self replaceCharacter:html str1:@"..playerVars" str2:playerVars];

	html = [self replaceCharacter:html str1:@"..autoPlay" str2:[NSString stringWithFormat:@"%d", autoPlay]];

//	NSLog(@"html=%@", html);

	[self setWebPlayerWithHTML:html videoId:videoId];

}

//------------------------------------
// setWebPlayerWithExternalPlayerHTML
//------------------------------------
- (void)setWebPlayerWithExternalPlayerHTML:(NSString*)html params:(NSDictionary*)params videoId:(NSString*)videoId
{

	NSString *playerVars = @"";
	NSString *externalVars = [self createExternalParamVarsString:[params objectForKey:@"dictVars"]];

	NSSize frameSize = [self frame].size;
	int frameWidth = frameSize.width;
	int frameHeight = frameSize.height;

	// player defaults
	int autoPlay = 0;
	if([self defaultBoolValue:@"optAutoPlay"] == YES){
		autoPlay = 1;
	}
	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('thumbWatch', '%d');\n", 1]];
	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('fv_autoplay', '%d');\n", autoPlay]];
	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('fv_no_link', '%d');\n", 0]];
	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('noAdSense', '%d');\n", 1]];

	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('v', '%@');\n", videoId]];
//	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('id', '%@');\n", videoId]];
	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('wv_id', '%@');\n", videoId]];
	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('thumbWidth', '%d');\n", frameWidth]];
	playerVars = [playerVars stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('thumbHeight', '%d');\n", frameHeight]];

	playerVars = [playerVars stringByAppendingString:externalVars];

//	NSLog(@"playerVars=%@", playerVars);

	// get value
	NSString *playerUrl = [params objectForKey:@"playerUrl"];
//	NSString *playerUrl = [NSString stringWithFormat:@"%@?ts=%@", [self convertToExternalNicoPlayerSwfURL], [params objectForKey:@"ts"]];

	// replace params
	html = [self replaceCharacter:html str1:@"..swfobject_path" str2:[self swfobjectPath]];
	html = [self replaceCharacter:html str1:@"..playerUrl" str2:playerUrl];
	html = [self replaceCharacter:html str1:@"..playerWidth" str2:[NSString stringWithFormat:@"%d", frameWidth]];
	html = [self replaceCharacter:html str1:@"..playerHeight" str2:[NSString stringWithFormat:@"%d", frameHeight]];
	html = [self replaceCharacter:html str1:@"..playerVars" str2:playerVars];

//	NSLog(@"html=%@", html);

	[self setWebPlayerWithHTML:html videoId:videoId];

}
//------------------------------------
// setWebPlayerWithIPadPlayer
//------------------------------------
- (void)setWebPlayerWithIPadPlayer:(NSString*)html params:(NSDictionary*)params
{

	// link params
	NSString *videoId = [params objectForKey:@"videoId"];

	// get player params
	NSDictionary *playerParams = nil;
	NSString *articleString = @"";
	NSString *sectionString = @"";
	NSString *playerString = @"";
	BOOL getArticleString = NO;
	BOOL getSectionString = NO;
	BOOL getPlayerString = NO;

	NSString *playerHTML = html;
	if([self isIPadPlayerHTML] == YES){
		playerHTML = [self iPadPlayerHTML];
	}

	playerParams = [self getIPadPlayerParams:html];
	getArticleString = [[playerParams objectForKey:@"getArticleString"] boolValue];
	getSectionString = [[playerParams objectForKey:@"getSectionString"] boolValue];
	getPlayerString = [[playerParams objectForKey:@"getPlayerString"] boolValue];

	if(getArticleString == YES){
		articleString = [playerParams valueForKey:@"articleString"];
	}
	if(getSectionString == YES){
		sectionString = [playerParams valueForKey:@"sectionString"];
	}
	if(getPlayerString == YES){
		playerString = [playerParams valueForKey:@"playerString"];
	}

	playerHTML = [self replaceCharacter:playerHTML str1:@"..article_string" str2:articleString];
//	playerHTML = [self replaceCharacter:playerHTML str1:@"..section_nicoplayer_string" str2:sectionString];
//	playerHTML = [self replaceCharacter:playerHTML str1:@"..player_string" str2:playerString];

//	NSLog(@"playerHTML=%@", playerHTML);

	[self setWebPlayerWithHTML:playerHTML videoId:videoId];

}

//----------------------
// replaceJsTemplateParams
//----------------------
- (NSString*)replaceJsTemplateParams:(NSString*)jsTemplate
{

/*
	// calc player size
	NSSize playerSize = [self calcWebPlayerSize:NSMakeSize(PLAYER_BASE_WIDTH, PLAYER_BASE_HEIGHT)
										frameSize:[self frame].size
										paddingSize:NSMakeSize(PLAYER_PADDING_WIDTH, PLAYER_PADDING_HEIGHT)
						];
*/												
//	NSSize frameSize = [self frame].size;

	// add parameter
	NSString *addParams = @"";
	// scale / not enabled
//	addParams = [addParams stringByAppendingString:@"\n\t\t\t fv_no_scale: 1,"];
	// auto play
	if([self defaultBoolValue:@"optAutoPlay"] == YES){
		addParams = [addParams stringByAppendingString:@"\n\t\t\t fv_autoplay: 1,"];
	}

	// replace parameter
	if(![addParams isEqualToString:@""]){
		jsTemplate = [self replaceCharacter:jsTemplate
										str1:@"defaultVariables: {"
										str2:[NSString stringWithFormat:@"defaultVariables: {%@", addParams]
						];
	}

	// replace size
//	jsTemplate = [self replaceSizeCharacter:jsTemplate size1:PLAYER_REP_WIDTH size2:frameSize.width];
//	jsTemplate = [self replaceSizeCharacter:jsTemplate size1:PLAYER_REP_HEIGHT size2:frameSize.height];
//	jsTemplate = [self replaceCharacter:jsTemplate str1:[NSString stringWithFormat:@"%d", PLAYER_REP_WIDTH] str2:@"100%"];
//	jsTemplate = [self replaceCharacter:jsTemplate str1:[NSString stringWithFormat:@"%d", PLAYER_REP_HEIGHT] str2:@"100%"];

//	NSLog(@"jsTemplate=%@", jsTemplate);

	return jsTemplate;

}
//------------------------------------
// createParamVarsString
//------------------------------------
- (NSString*)createExternalParamVarsString:(NSDictionary*)params
{

	NSString *varString = @"";
	NSArray *keys = [params allKeys];
	int i;
	id key, value;

	for(i = 0; i < [keys count]; i++){
		
		key = [keys objectAtIndex:i];
		value = [params objectForKey:key];
//		NSLog(@"key=%@, value=%@", key, value);
		varString = [varString stringByAppendingString:[NSString stringWithFormat:@"so.addVariable('%@', '%@');\n", key, value]];
	}

	return varString;

}
//------------------------------------
// getExternalPlayerParams
//------------------------------------
- (NSDictionary*)getExternalPlayerParams:(NSString*)html
{

	NSMutableDictionary *dictVars = [[[NSMutableDictionary alloc] init] autorelease];

	NSString *playerUrl = @"";
	NSString *ts = @"";
	NSString *value = @"";

	BOOL getParams = NO;

	BOOL getPlayerUrl = NO;
	BOOL getTs = NO;
	BOOL getPlayerTimestamp = NO;
	BOOL getPlayerVersionXML = NO;
	BOOL getPlayerInfoXML = NO;
	BOOL getTitle = NO;
	BOOL getDescription = NO;
	BOOL getThumbnail = NO;
	BOOL getPostedAt = NO;
	BOOL getThumbPlayKey = NO;
	BOOL getMovieType = NO;
	BOOL getMylistCount = NO;
	BOOL getIsDeleted = NO;
	BOOL getIsMymemory = NO;
	BOOL getHasOwnerThread = NO;
	BOOL getIee = NO;

	NSString *sep = @"'";
	NSString *endSet = @" \t\r\n";
	NSString *line;
	NSRange rng;
	int i;
	NSArray *lines = [html componentsSeparatedByString:@"\n"];

//	NSLog(@"html=%@", html);

	//
	// lines
	//
	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];

		// playerUrl
		rng = [line rangeOfString:@"playerUrl ="];
		if(rng.length > 0 && getPlayerUrl == NO){
			playerUrl = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			getPlayerUrl = YES;
		}

		// ts
		rng = [line rangeOfString:@"ts="];
		if(rng.length > 0 && getTs == NO){
			ts = [self getEnclosedString:line beginString:@"ts=" endString:sep withEndSet:NO];
			getTs = YES;
		}

		// playerTimestamp
		rng = [line rangeOfString:@"playerTimestamp:"];
		if(rng.length > 0 && getPlayerTimestamp == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"playerTimestamp"];
			getPlayerTimestamp = YES;
		}
		// player_version_xml
		rng = [line rangeOfString:@"player_version_xml:"];
		if(rng.length > 0 && getPlayerVersionXML == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"player_version_xml"];
			getPlayerVersionXML = YES;
		}
		// player_info_xml
		rng = [line rangeOfString:@"player_info_xml:"];
		if(rng.length > 0 && getPlayerInfoXML == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"player_info_xml"];
			getPlayerInfoXML = YES;
		}

		// title
		rng = [line rangeOfString:@"title:"];
		if(rng.length > 0 && getTitle == NO){
			// replace \' -> ..<apos>
			line = [self replaceCharacter:line str1:@"\\'" str2:@"..<apos>"];
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			value = [self replaceCharacter:value str1:@"..<apos>" str2:@"\\'"];
/*
			NSLog(@"title=%@", value);
			NSLog(@"len=%d", [value length]);
			if([value length] > 60){
				value = [value substringToIndex:60];
				value = [value stringByAppendingString:@"..."];
			}
*/
			[dictVars setObject:value forKey:@"thumbTitle"];
			[dictVars setObject:value forKey:@"wv_title"];
			getTitle = YES;
		}

		// description
		rng = [line rangeOfString:@"description:"];
		if(rng.length > 0 && getDescription == NO){
			// replace \' -> "..<apos>"
			line = [self replaceCharacter:line str1:@"\\'" str2:@"..<apos>"];
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			value = [self replaceCharacter:value str1:@"..<apos>" str2:@"\\'"];
/*
			NSLog(@"description=%@", value);
			NSLog(@"len=%d", [value length]);
			if([value length] > 60){
				value = [value substringToIndex:60];
				value = [value stringByAppendingString:@"..."];
			}
*/
			[dictVars setObject:value forKey:@"thumbDescription"];
			getDescription = YES;
		}

		// thumbnail
		rng = [line rangeOfString:@"thumbnail:"];
		if(rng.length > 0 && getThumbnail == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"thumbImage"];
			getThumbnail = YES;
		}

		// postedAt
		rng = [line rangeOfString:@"postedAt:"];
		if(rng.length > 0 && getPostedAt == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"thumbPostedAt"];
			getPostedAt = YES;
		}

		// thumbPlayKey
		rng = [line rangeOfString:@"thumbPlayKey:"];
		if(rng.length > 0 && getThumbPlayKey == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"thumbPlayKey"];
			getThumbPlayKey = YES;
		}

		// movieType
		rng = [line rangeOfString:@"movieType:"];
		if(rng.length > 0 && getMovieType == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"movie_type"];
			getMovieType = YES;
		}

		// mylistCount
		rng = [line rangeOfString:@"mylistCount:"];
		if(rng.length > 0 && getMylistCount == NO){
			value = [self getEnclosedString:line beginString:@"mylistCount: " endString:endSet withEndSet:YES];
			[dictVars setObject:value forKey:@"mylist_counter"];
			getMylistCount = YES;
		}

		// isDeleted
		rng = [line rangeOfString:@"isDeleted:"];
		if(rng.length > 0 && getIsDeleted == NO){
			value = [self getEnclosedString:line beginString:@"isDeleted: " endString:endSet withEndSet:YES];
			[dictVars setObject:value forKey:@"isDeleted"];
			getIsDeleted = YES;
		}

		// isMymemory
		rng = [line rangeOfString:@"isMymemory:"];
		if(rng.length > 0 && getIsMymemory == NO){
			value = [self getEnclosedString:line beginString:@"isMymemory: " endString:endSet withEndSet:YES];
			[dictVars setObject:value forKey:@"isMymemory"];
			getIsMymemory = YES;
		}

		// has_owner_thread
		rng = [line rangeOfString:@"has_owner_thread:"];
		if(rng.length > 0 && getHasOwnerThread == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"has_owner_thread"];
			getHasOwnerThread = YES;
		}

		// iee
		rng = [line rangeOfString:@"iee:"];
		if(rng.length > 0 && getIee == NO){
			value = [self getEnclosedString:line beginString:sep endString:sep withEndSet:NO];
			[dictVars setObject:value forKey:@"iee"];
			getIee = YES;
		}

	}

	// sucsess
	if(getPlayerUrl == YES && getThumbPlayKey == YES){
		getParams = YES;
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithBool:getParams], @"getParams" ,
				playerUrl, @"playerUrl" ,
				ts, @"ts" ,
				dictVars, @"dictVars" ,
				nil
			];

}
//------------------------------------
// getNicoPlayerParams
//------------------------------------
- (NSDictionary*)getNicoPlayerParams:(NSString*)html
{

	NSString *playerUrl = @"";
	NSString *ts = @"";
	NSMutableArray *lineVars = [[[NSMutableArray alloc] init] autorelease];
	NSMutableDictionary *dictVars = [[[NSMutableDictionary alloc] init] autorelease];
	BOOL getPlayerUrl = NO;
	BOOL getVars = NO;

	NSString *keyUrl = @"SWFObject";
	NSString *keyVar = @"addVariable";
	NSString *key = @"";
	NSString *value = @"";
	NSString *sep = @"\"";
	NSDictionary *dict;

	NSString *line;
	NSRange rng;
	int i;
	NSArray *lines = [html componentsSeparatedByString:@"\r"];

//	NSLog(@"html=%@", html);

	//
	// lines
	//
	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];
//		NSLog(@"line=%@", line);

		// url
		rng = [line rangeOfString:keyUrl];
		if(rng.length > 0){

			playerUrl = [self getEnclosedString:line
									beginString:sep
									endString:sep
									withEndSet:NO
						];
			ts = [self getEnclosedString:line beginString:@"ts=" endString:sep withEndSet:NO];
			getPlayerUrl = YES;
//			NSLog(@"playerUrl=%@ ts=%@", playerUrl, ts);
		}

		// addVariable line
		rng = [line rangeOfString:keyVar];
		if(rng.length > 0){

			[lineVars addObject:line];

			dict = [self getKeyAndValueString:line beginString:sep endString:sep];
			key = [dict objectForKey:@"key"];
			value = [dict objectForKey:@"value"];
			if( (key && ![key isEqualToString:@""]) &&
				(value && ![value isEqualToString:@""])
			){
				[dictVars setObject:value forKey:key];
			}

			getVars = YES;
		}

	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithBool:getPlayerUrl], @"getPlayerUrl" ,
				[NSNumber numberWithBool:getVars], @"getVars" ,
				playerUrl, @"playerUrl" ,
				ts, @"ts" ,
				lineVars, @"lineVars" ,
				dictVars, @"dictVars" ,
				nil
			];

}
//------------------------------------
// getNicoPlayerQParams
//------------------------------------
- (NSMutableDictionary*)getNicoPlayerQParams:(NSString*)html
{

	NSString *playerUrl = @"";
	NSString *ts = @"";
	NSMutableArray *lineVars = [[[NSMutableArray alloc] init] autorelease];
	NSMutableDictionary *dictVars = [[[NSMutableDictionary alloc] init] autorelease];
	BOOL getPlayerUrl = NO;
	BOOL getVars = NO;

	NSString *keyUrl = @"swf:";
	NSString *keyVar = @"flashVars.";
//	NSString *key = @"";
//	NSString *value = @"";
//	NSDictionary *dict;
	NSString *sepPlayer = @"'";

	NSString *line;
	NSRange rng;
	int i;
	NSArray *lines = [html componentsSeparatedByString:@"\n"];

//	NSLog(@"html=%@", html);

	//
	// lines
	//
	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];

		// url
		rng = [line rangeOfString:keyUrl];
		if(rng.length > 0){

			playerUrl = [self getEnclosedString:line
									beginString:sepPlayer
									endString:sepPlayer
									withEndSet:NO
						];
			ts = [self getEnclosedString:line beginString:@"ts=" endString:sepPlayer withEndSet:NO];
			getPlayerUrl = YES;
//			NSLog(@"playerUrl=%@ ts=%@", playerUrl, ts);
		}

		// addVariable line
		rng = [line rangeOfString:keyVar];
		if(rng.length > 0){

			[lineVars addObject:line];

/*
			dict = [self getKeyAndValueString:line beginString:@"\"" endString:@"\""];
			key = [dict objectForKey:@"key"];
			value = [dict objectForKey:@"value"];
			if( (key && ![key isEqualToString:@""]) &&
				(value && ![value isEqualToString:@""])
			){
				[dictVars setObject:value forKey:key];
			}
*/
			getVars = YES;
		}

	}

	return [NSMutableDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithBool:getPlayerUrl], @"getPlayerUrl" ,
				[NSNumber numberWithBool:getVars], @"getVars" ,
				playerUrl, @"playerUrl" ,
				ts, @"ts" ,
				lineVars, @"lineVars" ,
				dictVars, @"dictVars" ,
				nil
			];

}
//------------------------------------
// getIPadPlayerParams
//------------------------------------
- (NSDictionary*)getIPadPlayerParams:(NSString*)html
{

	NSString *keyArticleStart = @"<article>";
	NSString *keyArticleEnd = @"</article>";
	NSString *keySectionStart = @"section id=\"nicoplayer\"";
	NSString *keySectionEnd = @"</section>";
	NSString *keyPlayerStart = @"MarkuPlayer.setFlashvars({";
	NSString *keyPlayerEnd = @");";

	NSString *articleString = @"";
	NSString *sectionString = @"";
	NSString *playerString = @"";

	BOOL getArticleString = NO;
	BOOL getSectionString = NO;
	BOOL getPlayerString = NO;
	NSRange rng;

	// article
	rng = [html rangeOfString:keyArticleStart];
	if(rng.length > 0){
		articleString = [self getEnclosedString:html
								beginString:keyArticleStart
								endString:keyArticleEnd
								withEndSet:NO
					];
		articleString = [NSString stringWithFormat:@"%@\n%@\n%@", keyArticleStart, articleString, keyArticleEnd];
		getArticleString = YES;
	}

	// section
	rng = [html rangeOfString:keySectionStart];
	if(rng.length > 0){
		sectionString = [self getEnclosedString:html
								beginString:keySectionStart
								endString:keySectionEnd
								withEndSet:NO
					];
		sectionString = [NSString stringWithFormat:@"<%@%@\n%@", keySectionStart, sectionString, keySectionEnd];
		getSectionString = YES;
	}

	// player
	rng = [html rangeOfString:keyPlayerStart];
	if(rng.length > 0){
		playerString = [self getEnclosedString:html
								beginString:keyPlayerStart
								endString:keyPlayerEnd
								withEndSet:NO
					];
		playerString = [NSString stringWithFormat:@"%@\n%@\n%@", keyPlayerStart, playerString, keyPlayerEnd];
		getPlayerString = YES;
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithBool:getSectionString], @"getArticleString" ,
				[NSNumber numberWithBool:getSectionString], @"getSectionString" ,
				[NSNumber numberWithBool:getPlayerString], @"getPlayerString" ,
				articleString, @"articleString" ,
				sectionString, @"sectionString" ,
				playerString, @"playerString" ,
				nil
			];

}
//------------------------------------
// getWatchPlayerStrings
//------------------------------------
- (NSDictionary*)getWatchPlayerStrings:(NSString*)html
{

	NSString *playerString = @"";

	NSString *keyStart1 = @"SWFObject";
	NSString *keyStart2 = @"nicoplayer";
	NSString *keyEnd = @"so.write";
	BOOL getStart = NO;

	NSString *line;
	NSRange rng1, rng2;
	int i;
	NSArray *lines = [html componentsSeparatedByString:@"\n"];

	//
	// lines
	//
	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];

		// player params
		rng1 = [line rangeOfString:keyStart1];
		rng2 = [line rangeOfString:keyStart2];
		if(rng1.length > 0 && rng2.length > 0){
			getStart = YES;
		}

		if(getStart == YES){
			playerString = [playerString stringByAppendingString:line];

			rng1 = [line rangeOfString:keyEnd];
			if(rng1.length > 0){
				break;
			}
		}
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
				playerString, @"playerString" ,
				nil
			];

}
//------------------------------------
// getWatchPlayerQStrings
//------------------------------------
- (NSDictionary*)getWatchPlayerQStrings:(NSString*)html
{

	NSString *playerString = @"";
	NSString *APIString = @"";

	NSString *keyStart1 = @"var watchAPIData";
	NSString *keyStart2 = @"window.watchAPIData =";
	NSString *keyEnd1 = @")();";
	NSString *keyEnd2 = @"};";
	BOOL getStart1 = NO;
	BOOL getStart2 = NO;
	BOOL getEnd1 = NO;
	BOOL getEnd2 = NO;

	NSString *line;
	NSRange rng;
	int i;

	html = [self replaceCharacter:html str1:@"><" str2:@">\n<"];
	NSArray *lines = [html componentsSeparatedByString:@"\n"];

	//
	// lines
	//
	for(i = 0; i < [lines count]; i++){

		line = [lines objectAtIndex:i];
		line = [line stringByAppendingString:@"\n"];

		// player params
		if(getEnd1 == NO){
			rng = [line rangeOfString:keyStart1];
			if(rng.length > 0){
				getStart1 = YES;
			}
			if(getStart1 == YES){
				playerString = [playerString stringByAppendingString:line];

				// end
				rng = [line rangeOfString:keyEnd1];
				if(rng.length > 0){
					getEnd1 = YES;
				}
			}
		}

		// api params
		if(getEnd2 == NO){
			rng = [line rangeOfString:keyStart2];
			if(rng.length > 0){
				getStart2 = YES;
			}
			if(getStart2 == YES){
				APIString = [APIString stringByAppendingString:line];

				// end
				rng = [line rangeOfString:keyEnd2];
				if(rng.length > 0){
					getEnd2 = YES;
				}
			}
		}

		if(getEnd1 == YES && getEnd2 == YES){
			break;
		}

	}


	APIString = [self replaceCharacter:APIString str1:@"\n" str2:@""];
	APIString = [self replaceCharacter:APIString str1:@"\r" str2:@""];

	return [NSDictionary dictionaryWithObjectsAndKeys:
				playerString, @"playerString" ,
				APIString, @"APIString" ,
				nil
			];

}

//------------------------------------
// getWatchVideoVarString
//------------------------------------
- (NSString*)getWatchVideoVarString:(NSString*)html
{

	NSString *keyVideoVarStart = @"var Video = {";
	NSString *keyVideoVarEnd = @"};";
	NSString *videoVars = [self getEnclosedString:html beginString:keyVideoVarStart endString:keyVideoVarEnd withEndSet:NO];
	return [NSString stringWithFormat:@"%@\n%@\n%@\n", keyVideoVarStart, videoVars, keyVideoVarEnd];

}
//----------------------
// setWebPlayerWithURL
//----------------------
- (BOOL)setWebPlayerWithURL:(NSString*)urlString
{

	NSURL *url = [NSURL URLWithString:urlString];
	if(url == nil){
		return NO;
	}

	NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
	if(request == nil){
		return NO;
	}

	[[self mainFrame] loadRequest:request];

	return YES;

}
//----------------------
// setWebPlayerWithHTML
//----------------------
- (void)setWebPlayerWithHTML:(NSString*)html videoId:(NSString*)videoId
{
//	NSLog(@"html=%@", html);
	[[self mainFrame] loadHTMLString:html baseURL:[NSURL URLWithString:[self convertToBaseURL]]];
//	[[self mainFrame] loadHTMLString:html baseURL:[NSURL URLWithString:[self convertToWatchURL:videoId]]];
}
//------------------------------------
// setPlayerWindowStatus
//------------------------------------
- (void)setPlayerWindowStatus:(NSString*)videoId title:(NSString*)title playerType:(int)playerType
{

	// get params
	NSDictionary *params = [viewPlayer params];
	if(params == nil){
		return;
	}

	// set attribute
	[self setVideoId:videoId];
	[self setTitle:title];
	[self setPlayerType:playerType];

	// set window title
	[[self window] setTitle:title];

	//set video info
	int itemStatus = [[params objectForKey:@"itemStatus"] intValue];
	NSString *errorDescription = [params objectForKey:@"errorDescription"];
	int viewCount = [[params objectForKey:@"viewCount"] intValue];
	int resCount = [[params objectForKey:@"resCount"] intValue];
	int mylistCount = [[params objectForKey:@"mylistCount"] intValue];
	BOOL hasCountInfo = [[params objectForKey:@"hasCountInfo"] boolValue];

	if(!itemStatus){itemStatus = 0;}
	if(!errorDescription){errorDescription = @"";}
	if(!viewCount){viewCount = 0;}
	if(!mylistCount){resCount = 0;}
	if(!itemStatus){mylistCount = 0;}
	if(!hasCountInfo){hasCountInfo = NO;}

	NSString *statusString = @"";
	if(itemStatus == VIDEO_ITEM_SUCCESS){
		if(hasCountInfo == YES){
			statusString = [NSString stringWithFormat:@"Views:%@  Res:%@  Mylist:%@",
										[self convertToComma:viewCount],
										[self convertToComma:resCount],
										[self convertToComma:mylistCount]
							];
		}
	}
	else if(itemStatus == VIDEO_ITEM_STATUS_FAILED){
		statusString = errorDescription;
	}

	NSString *videoInfoString = [NSString stringWithFormat:@"%@  %@",
									videoId,
									statusString
								];
	[txtVideoInfo setStringValue:videoInfoString];

	[self changeTabViewControllerIndex];

	// post notification
	[self postVideoObjectChangedNotification];

}
//------------------------------------
// setWindowTitle
//------------------------------------
- (void)setWindowTitle:(NSString*)title
{
	int playerType = [self playerType];
	int playEcoMode = [self defaultBoolValue:@"optPlayEcoMode"];

	// window title
	NSString *windowTitle = title;
	int pageCount = [self pageCount];
	// eco mode
	if(pageCount <= 1){
		if(playerType == PLAYER_TYPE_WATCH_VIDEO &&
			playEcoMode == YES
		){
			windowTitle = [NSString stringWithFormat:@"[ECO] %@", windowTitle];
		}
	}
	[[self window] setTitle:windowTitle];

}
//------------------------------------
// calcWebPlayerSize
//------------------------------------
- (NSSize)calcWebPlayerSize:(NSSize)playerSize
					frameSize:(NSSize)frameSize
					paddingSize:(NSSize)paddingSize
{

	float playerWidth = playerSize.width;
	float playerHeight = playerSize.height;

	if(playerWidth < 1){playerWidth = 1;}
	if(playerHeight < 1){playerHeight = 1;}

	float frameWidth = frameSize.width - (paddingSize.width * 2);
	float frameHeight = frameSize.height - (paddingSize.height * 2);
	float scale = 1.0;

	// player size is smaller than frame size
	if(playerWidth < frameWidth && playerHeight < frameHeight){
		// calc scale horizontal and vertical
		float scaleH = frameWidth / playerWidth;
		float scaleV = frameHeight / playerHeight;
		// set smaller scale
		if(scaleH < scaleV){
			scale = scaleH;
		}else{
			scale = scaleV;
		}
		playerWidth = playerWidth * scale;
		playerHeight = playerHeight * scale;
	}
	else{
		// set player size by horizontal scale
		if(playerWidth > frameWidth){
			scale = frameWidth / playerWidth;
			playerWidth = playerWidth * scale;
			playerHeight = playerHeight * scale;
		}
		// still height is over,  set player size by vertical scale
		if(playerHeight > frameHeight){
			scale = frameHeight / playerHeight;
			playerWidth = playerWidth * scale;
			playerHeight = playerHeight * scale;
		}
	}

	if(playerWidth < 1){playerWidth = 1;}
	if(playerHeight < 1){playerHeight = 1;}

	return NSMakeSize((int)playerWidth, (int)playerHeight);
}
//------------------------------------
// closeWindow
//------------------------------------
- (void)closeWindow
{
	if([[self window] isVisible]){
		[[self window] close];
	}
}
//----------------------
// didFinishLoadForFrame
//----------------------
- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{

//	NSString *frameURLString = [[[[[self mainFrame] dataSource] request] URL] absoluteString];
//	NSLog(@"frameURLString=%@", frameURLString);

	// increase pageCount
	int pageCount = [self pageCount];
	if(pageCount <= 1){
		pageCount++;
		[self setPageCount:pageCount];
//		NSLog(@"pageCount=%d", pageCount);
	}
	// change player info
	else{
		[self changePlayerInfoWithVideoId];
//		[self setWindowTitle:[self title]];
	}

}
//------------------------------------
// changePlayerInfoWithVideoId
//------------------------------------
- (void)changePlayerInfoWithVideoId
{

	NSString *frameURLString = [[[[[self mainFrame] dataSource] request] URL] absoluteString];
//	NSLog(@"frameURLString=%@", frameURLString);

	// frame URL
	NSURL *frameURL = [NSURL URLWithString:frameURLString relativeToURL:nil];

	// base URL
	NSURL *baseURL = [NSURL URLWithString:[self convertToBaseURL] relativeToURL:nil];

	// same domain
	if([[frameURL host] isEqualToString:[baseURL host]]){

//		NSLog(@"same domain");

		// get videoId
		NSString *videoId = [self getVideoIdFromURL:frameURLString setDefault:NO];

		// set player info
		if( ![videoId isEqualToString:@""] &&
			![videoId isEqualToString:[self videoId]]){
//			NSLog(@"changed videoId=%@", videoId);
			[viewPlayer setPlayerInfoWithVideoId:videoId];
		}

	}
			
}
//------------------------------------
// viewDidEndLiveResize
//------------------------------------
- (void)viewDidEndLiveResize
{

//	NSString *frameURL = [[[[[self mainFrame] dataSource] request] URL] absoluteString];
//	NSLog(@"frameURL=%@", frameURL);

	// ext player
	if([self playerType] == PLAYER_TYPE_EXT_PLAYER){
		if([self isExtPlayerHTML] == YES){
//			if([self defaultBoolValue:@"optNicoPlayerLiveResize"] == NO){
				[self resizePlayerView];
//			}
		}else{
			// first page
			if([self pageCount] <= 1){
				[self reloadPlayerWithJsTemplate];
			}
		}
	}

}
/*
//------------------------------------
// windowDidResizePlayer
//------------------------------------
-(void)windowDidResizePlayer:(NSNotification *)notification
{
}
*/
//------------------------------------
// changeTabViewControllerIndex
//------------------------------------
- (void)changeTabViewControllerIndex
{
	int index = 0;
	int playerType = [self playerType];

	if(playerType == PLAYER_TYPE_NICO_PLAYER){
		index = 0;
	}
	else if(playerType == PLAYER_TYPE_NICO_PLAYER_Q){
		index = 0;
	}
	else if(playerType == PLAYER_TYPE_IPAD_PLAYER){
		index = 0;
	}
	else if(playerType == PLAYER_TYPE_EXT_PLAYER){
		index = 1;
	}
	else{
		return;
	}
	
	[tabViewController selectTabViewItemAtIndex:index];

}

//----------------------
// changeLiveResize
//----------------------
- (void)changeLiveResize
{
	int liveResize = 0;
	if([self defaultBoolValue:@"optNicoPlayerLiveResize"] == YES){
		liveResize = 1;
	}

	[self runJavaScriptWithString:[NSString stringWithFormat:@"setLiveResize(%d)", liveResize]];

}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqualToString:@"optNicoPlayerLiveResize"]){
		// change resize mode
		[self changeLiveResize];
	}
}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	
	BOOL isAction = NO;

	// control click
	if([theEvent modifierFlags] & NSControlKeyMask){
	}

	if(isAction == NO){
		[super mouseDown:theEvent];
	}
}
//=======================================================================
// drag & drop
//=======================================================================
//------------------------------------
// draggingEntered
//------------------------------------
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)info
{

	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard){
		return NSDragOperationNone;
	}

	// ignore self
	if (draggingSource == self) {
		return NSDragOperationNone;
	}

	// from sting
	if ([[pboard types] containsObject:NSStringPboardType]){
		return NSDragOperationCopy;
	}

	return NSDragOperationNone;

}
//------------------------------------
// draggingExited
//------------------------------------
- (void)draggingExited:(id <NSDraggingInfo>)info
{
	// none
}
//------------------------------------
// draggingUpdated
//------------------------------------
- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)info
{

	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard){
		return NSDragOperationNone;
	}

	// ignore self
	if (draggingSource == self) {
		return NSDragOperationNone;
	}

	// from sting
	if ([[pboard types] containsObject:NSStringPboardType]){
		return NSDragOperationCopy;
	}

	return NSDragOperationNone;

}
//------------------------------------
// prepareForDragOperation
//------------------------------------
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)info
{
	return YES;
}
//------------------------------------
// performDragOperation
//------------------------------------
- (BOOL)performDragOperation:(id <NSDraggingInfo>)info
{

	NSPasteboard *pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard) {
        return NO;
	}

	// ignore self
	if (draggingSource == self) {
		return NO;
	}

	// Check pboard type
	// from string
	if ([[pboard types] containsObject:NSStringPboardType]){
		NSString *urlString = [pboard stringForType:NSStringPboardType];
		if(!urlString || [urlString isEqualToString:@""]) {
			return NO;
		}
		// open player
		if([self checkIsWatchURL:urlString] == YES){
			[viewPlayer setPlayerViewWithURL:urlString];
			return YES;
		}
	}

	return NO;

}
//=======================================================================
// web scripts
//=======================================================================
//------------------------------------
// didClearWindowObject(available later 10.4.11)
//------------------------------------
- (void)webView:(WebView *)sender didClearWindowObject:(WebScriptObject *)wso forFrame:(WebFrame *)frame
{
//	NSLog(@"didClearWindowObject");
	[wso setValue:self forKey:@"webPlayerView"];
}
//------------------------------------
// windowScriptObjectAvailable(available before 10.4.10)
//------------------------------------
- (void)webView:(WebView *)sender windowScriptObjectAvailable:(WebScriptObject *)wso
{
//	NSLog(@"windowScriptObjectAvailable");
	[wso setValue:self forKey:@"webPlayerView"];
}
//------------------------------------
// webScriptNameForSelector
//------------------------------------
+ (NSString *)webScriptNameForSelector:(SEL)sel
{
	NSString *name = nil;

	if(sel == @selector(changeNicoPlayerState:)){
		name = @"changeNicoPlayerState";
	}
	else if(sel == @selector(setLoadIndicator:)){
		name = @"setLoadIndicator";
	}
	else if(sel == @selector(setMaximizePlayerEnable:)){
		name = @"setMaximizePlayerEnable";
	}
	else if(sel == @selector(setMaximizePlayerState:)){
		name = @"setMaximizePlayerState";
	}
	else if(sel == @selector(setPlayerIsPlayed:)){
		name = @"setPlayerIsPlayed";
	}
	else if(sel == @selector(receiveLogMessage:)){
		name = @"receiveLogMessage";
	}

	return name;
}
//----------------------
// isSelectorExcludedFromWebScript
//----------------------
+ (BOOL)isSelectorExcludedFromWebScript:(SEL)aSelector 
{
	return NO;
}
//------------------------------------
// changeNicoPlayerState
//------------------------------------
- (void)changeNicoPlayerState:(NSString*)state
{

//	NSLog(@"state=%@", state);

}
//------------------------------------
// setLoadIndicator
//------------------------------------
- (void)setLoadIndicator:(int)state
{
	if(state == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}

//------------------------------------
// setMaximizePlayerEnable
//------------------------------------
- (void)setMaximizePlayerEnable:(int)state
{
	BOOL enable;
	if(state == 1){
		enable = YES;
	}else{
		enable = NO;
	}
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:PLAYER_NOTIF_MAXIMIZE_ENABLE_DID_CHANGED object:[NSNumber numberWithInt:state]];
}
//------------------------------------
// setMaximizePlayerState
//------------------------------------
- (void)setMaximizePlayerState:(int)state
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:PLAYER_NOTIF_MAXIMIZE_DID_CHANGED object:[NSNumber numberWithInt:state]];

}
//------------------------------------
// setPlayerIsPlayed
//------------------------------------
- (void)setPlayerIsPlayed:(int)state
{
//	NSLog(@"setPlayerIsPlayed=%d", state);
	if(state == 1){
		[viewPlayer setPlayerIsPlayed:[self videoId] isPlayed:YES];
	}

}

//------------------------------------
// receiveLogMessage
//------------------------------------
- (void)receiveLogMessage:(NSString*)string
{
//	NSLog(@"Script says: %@", string);
/*
	[self displayMessage:@"alert"
				messageText:@"An message occurred in player window."
				infoText:string
				btnList:@"Cancel"
	];
*/
}
//------------------------------------
// connectionDataItem
//------------------------------------
- (void)setConnectionDataItem:(ConnectionDataItem*)connectionDataItem
{
	[connectionDataItem retain];
	[connectionDataItem_ release];
	connectionDataItem_ = connectionDataItem;
}
- (ConnectionDataItem*)connectionDataItem
{
	return connectionDataItem_;
}
//------------------------------------
// clearConnection
//------------------------------------
- (void)clearConnection
{

	ConnectionDataItem *connectionDataItem = [self connectionDataItem];
	// cancel connection
	if(connectionDataItem != nil){
		if([connectionDataItem connection] != nil){
//			NSLog(@"cancelConnection");
			[connectionDataItem cancelConnection];
		}
		[self setConnectionDataItem:nil];
	}

}
//------------------------------------
// setupHTMLPlayer
//------------------------------------
- (void)setupHTMLPlayer
{

	NSString *swfobjectString = @"";
	NSString *prototypeString = @"";
//	NSString *watch_ipad_js_string = @"";
//	NSString *watch_ipad_css_string = @"";

	NSString *nicoPlayerHTML = @"";
	NSString *nicoPlayerQHTML = @"";
	NSString *extPlayerHTML = @"";
//	NSString *iPadPlayerHTML = @"";
//	NSString *swfobjectPath = @"";

	BOOL isNicoPlayerHTML = NO;
	BOOL isNicoPlayerQHTML = NO;
	BOOL isExtPlayerHTML = NO;
//	BOOL isIPadPlayerHTML = NO;
//	BOOL isErrorWatchIpad = NO;

	// read template files
	NSError *error = nil;

	// swfobject
	swfobjectString = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"swfobject_1_5" ofType:@"js"] encoding:NSUTF8StringEncoding error:&error];
	if(error != nil){
		[self displayMessage:@"alert"
					messageText:@"Can not read file \"swfobject_1_5.js\" in Application."
					infoText:@"Please check file."
					btnList:@"Cancel"
		];
		return;
	}
	// prototype.js
	prototypeString = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"prototype" ofType:@"js"] encoding:NSUTF8StringEncoding error:&error];
	if(error != nil){
		[self displayMessage:@"alert"
					messageText:@"Can not read file \"prototype.js\" in Application."
					infoText:@"Please check file."
					btnList:@"Cancel"
		];
	}else{
		// nico player
		nicoPlayerHTML = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"nico_player_watch" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error];
		if(error != nil){
			[self displayMessage:@"alert"
						messageText:@"Can not read file \"nico_player_watch.html\" in Application."
						infoText:@"Please check file."
						btnList:@"Cancel"
			];
		}else{
			nicoPlayerHTML = [self replaceCharacter:nicoPlayerHTML str1:@"..swfobject_string" str2:swfobjectString];
			nicoPlayerHTML = [self replaceCharacter:nicoPlayerHTML str1:@"..prototype_string" str2:prototypeString];
			isNicoPlayerHTML = YES;
		}
	}

	// nico player Q
	nicoPlayerQHTML = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"nico_player_watch_q" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error];
	if(error != nil){
		[self displayMessage:@"alert"
					messageText:@"Can not read file \"nico_player_watch_q.html\" in Application."
					infoText:@"Please check file."
					btnList:@"Cancel"
		];
	}else{
		isNicoPlayerQHTML = YES;
	}

	// ext player
	extPlayerHTML = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"nico_player_ext" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error];
	if(error != nil){
		[self displayMessage:@"alert"
					messageText:@"Can not read file \"nico_player_ext.html\" in Application."
					infoText:@"Please check file."
					btnList:@"Cancel"
		];
	}else{
		extPlayerHTML = [self replaceCharacter:extPlayerHTML str1:@"..swfobject_string" str2:swfobjectString];
		extPlayerHTML = [self replaceCharacter:extPlayerHTML str1:@"..prototype_string" str2:prototypeString];
		isExtPlayerHTML = YES;
	}

/*
	// ipad player
	watch_ipad_js_string = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"watch_ipad" ofType:@"js"] encoding:NSUTF8StringEncoding error:&error];
	if(error != nil){
		[self displayMessage:@"alert"
					messageText:@"Can not read file \"watch_ipad.js\" in Application."
					infoText:@"Please check file."
					btnList:@"Cancel"
		];
		isErrorWatchIpad = YES;
	}
	watch_ipad_css_string = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"watch_ipad" ofType:@"css"] encoding:NSUTF8StringEncoding error:&error];
	if(error != nil){
		[self displayMessage:@"alert"
					messageText:@"Can not read file \"watch_ipad.css\" in Application."
					infoText:@"Please check file."
					btnList:@"Cancel"
		];
		isErrorWatchIpad = YES;
	}

	if(isErrorWatchIpad == NO){
		iPadPlayerHTML = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"nico_player_ipad" ofType:@"html"] encoding:NSUTF8StringEncoding error:&error];
		if(error != nil){
			[self displayMessage:@"alert"
						messageText:@"Can not read file \"nico_player_ipad.html\" in Application."
						infoText:@"Please check file."
						btnList:@"Cancel"
			];
		}else{
//			watch_ipad_js_string = [self replaceCharacter:watch_ipad_js_string str1:@"200:0" str2:@"400:0"];
//			watch_ipad_js_string = [self replaceCharacter:watch_ipad_js_string str1:@"512" str2:@"800"];
//			watch_ipad_js_string = [self replaceCharacter:watch_ipad_js_string str1:@"640" str2:@"800"];
//			watch_ipad_js_string = [self replaceCharacter:watch_ipad_js_string str1:@"384" str2:@"500"];
//			watch_ipad_js_string = [self replaceCharacter:watch_ipad_js_string str1:@"-20px" str2:@"0px"];
			iPadPlayerHTML = [self replaceCharacter:iPadPlayerHTML str1:@"..watch_ipad_css_string" str2:watch_ipad_css_string];
			iPadPlayerHTML = [self replaceCharacter:iPadPlayerHTML str1:@"..watch_ipad_js_string" str2:watch_ipad_js_string];
			isIPadPlayerHTML = YES;
		}
	}
*/
	// set value
	if(isNicoPlayerHTML == YES){
		[self setNicoPlayerHTML:nicoPlayerHTML];
	}
	if(isNicoPlayerQHTML == YES){
		[self setNicoPlayerQHTML:nicoPlayerQHTML];
	}
	if(isExtPlayerHTML == YES){
		[self setExtPlayerHTML:extPlayerHTML];
	}
//	if(isIPadPlayerHTML == YES){
//		[self setIPadPlayerHTML:iPadPlayerHTML];
//	}

	[self setIsNicoPlayerHTML:isNicoPlayerHTML];
	[self setIsNicoPlayerQHTML:isNicoPlayerQHTML];
	[self setIsExtPlayerHTML:isExtPlayerHTML];
//	[self setIsIPadPlayerHTML:isIPadPlayerHTML];

/*
	// check swfobject.js
	if(isExtPlayerHTML == YES){

		swfobjectPath = [[NSBundle mainBundle] pathForResource:@"swfobject_1_5" ofType:@"js"];
		if([[NSFileManager defaultManager] fileExistsAtPath:swfobjectPath] == NO){
			[self displayMessage:@"alert"
						messageText:@"Can not read file \"swfobject_1_5.js\" in Application."
						infoText:@"Please check file."
						btnList:@"Cancel"
			];
			isExtPlayerHTML = NO;
		}else{
			swfobjectPath = [NSString stringWithFormat:@"file://%@", swfobjectPath];
			// create url
			NSString *urlString = [self decodeToPercentEscapesString:swfobjectPath];
			NSURL *url = [NSURL URLWithString:urlString];
			if(url != nil){
				swfobjectPath = [url absoluteString];
			}
//			else{
//				[self displayMessage:@"alert"
//							messageText:@"Can not create url from \"swfobject.js\" in Application."
//							infoText:[NSString stringWithFormat:@"url=%@", swfobjectPath]
//							btnList:@"Cancel"
//				];
//				isExtPlayerHTML = NO;
//			}
		}
	}
*/
//	NSLog(@"swfobjectPath=%@", swfobjectPath);

	// not used
//	[self setSwfobjectPath:swfobjectPath];

}
//------------------------------------
// nicoPlayerHTML
//------------------------------------
- (void)setNicoPlayerHTML:(NSString*)nicoPlayerHTML
{
	[nicoPlayerHTML retain];
    [nicoPlayerHTML_ release];
    nicoPlayerHTML_ = nicoPlayerHTML;
}
- (NSString*)nicoPlayerHTML
{
    return nicoPlayerHTML_;
}
//------------------------------------
// isNicoPlayerHTML
//------------------------------------
- (void)setIsNicoPlayerHTML:(BOOL)isNicoPlayerHTML
{
	isNicoPlayerHTML_ = isNicoPlayerHTML;
}
- (BOOL)isNicoPlayerHTML
{
	return isNicoPlayerHTML_;
}
//------------------------------------
// nicoPlayerQHTML
//------------------------------------
- (void)setNicoPlayerQHTML:(NSString*)nicoPlayerQHTML
{
	[nicoPlayerQHTML retain];
    [nicoPlayerQHTML_ release];
    nicoPlayerQHTML_ = nicoPlayerQHTML;
}
- (NSString*)nicoPlayerQHTML
{
    return nicoPlayerQHTML_;
}
//------------------------------------
// isNicoPlayerQHTML
//------------------------------------
- (void)setIsNicoPlayerQHTML:(BOOL)isNicoPlayerQHTML
{
	isNicoPlayerQHTML_ = isNicoPlayerQHTML;
}
- (BOOL)isNicoPlayerQHTML
{
	return isNicoPlayerQHTML_;
}
//------------------------------------
// extPlayerHTML
//------------------------------------
- (void)setExtPlayerHTML:(NSString*)extPlayerHTML
{
	[extPlayerHTML retain];
    [extPlayerHTML_ release];
    extPlayerHTML_ = extPlayerHTML;
}
- (NSString*)extPlayerHTML
{
    return extPlayerHTML_;
}
//------------------------------------
// isExtPlayerHTML
//------------------------------------
- (void)setIsExtPlayerHTML:(BOOL)isExtPlayerHTML
{
	isExtPlayerHTML_ = isExtPlayerHTML;
}
- (BOOL)isExtPlayerHTML
{
	return isExtPlayerHTML_;
}
//------------------------------------
// iPadPlayerHTML
//------------------------------------
- (void)setIPadPlayerHTML:(NSString*)iPadPlayerHTML
{
	[iPadPlayerHTML retain];
    [iPadPlayerHTML_ release];
    iPadPlayerHTML_ = iPadPlayerHTML;
}
- (NSString*)iPadPlayerHTML
{
    return iPadPlayerHTML_;
}
//------------------------------------
// isIPadPlayerHTML
//------------------------------------
- (void)setIsIPadPlayerHTML:(BOOL)isIPadPlayerHTML
{
	isIPadPlayerHTML_ = isIPadPlayerHTML;
}
- (BOOL)isIPadPlayerHTML
{
	return isIPadPlayerHTML_;
}
//------------------------------------
// swfobjectPath
//------------------------------------
- (void)setSwfobjectPath:(NSString*)swfobjectPath
{
	[swfobjectPath retain];
	[swfobjectPath_ release];
	swfobjectPath_ = swfobjectPath;
}
- (NSString*)swfobjectPath
{
	return swfobjectPath_;
}
//------------------------------------
// videoId
//------------------------------------
- (void)setVideoId:(NSString*)videoId
{
	[videoId retain];
	[videoId_ release];
	videoId_ = videoId;
}
- (NSString*)videoId
{
	return videoId_;
}
//------------------------------------
// videoURL
//------------------------------------
- (void)setVideoURL:(NSString*)videoURL
{
	[videoURL retain];
	[videoURL_ release];
	videoURL_ = videoURL;
}
- (NSString*)videoURL
{
	return videoURL_;
}
//------------------------------------
// jsTemplate
//------------------------------------
- (void)setJsTemplate:(NSString*)jsTemplate
{
	[jsTemplate retain];
	[jsTemplate_ release];
	jsTemplate_ = jsTemplate;
}
- (NSString*)jsTemplate
{
	return jsTemplate_;
}
//------------------------------------
// title
//------------------------------------
- (void)setTitle:(NSString*)title
{
	[title retain];
	[title_ release];
	title_ = title;
}
- (NSString*)title
{
	return title_;
}
//------------------------------------
// playerType
//------------------------------------
- (void)setPlayerType:(int)playerType
{
	playerType_ = playerType;
}
- (int)playerType
{
	return playerType_;
}
//------------------------------------
// pageCount
//------------------------------------
- (void)setPageCount:(int)pageCount
{
	pageCount_ = pageCount;
}
- (int)pageCount
{
	return pageCount_;
}
//----------------------
// hasVideoId
//----------------------
- (BOOL)hasVideoId
{
	if([self videoId] && ![[self videoId] isEqualToString:@""]){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// canChangePlayerScale
//------------------------------------
- (BOOL)canChangePlayerScale
{
	BOOL ret = NO;

	if([self hasVideoId] == YES && [self pageCount] <= 1){
		if([self playerType] == PLAYER_TYPE_NICO_PLAYER && [self isNicoPlayerHTML] == YES){
			ret = YES;
		}
		if([self playerType] == PLAYER_TYPE_NICO_PLAYER_Q && [self isNicoPlayerHTML] == YES){
			ret = YES;
		}
		else if([self playerType] == PLAYER_TYPE_IPAD_PLAYER && [self isIPadPlayerHTML] == YES){
			ret = YES;
		}
		else if([self playerType] == PLAYER_TYPE_EXT_PLAYER && [self isExtPlayerHTML] == YES){
			ret = YES;
		}
	}

	return ret;
}
//------------------------------------
// canChangeMaximize
//------------------------------------
- (BOOL)canChangeMaximize
{
	BOOL ret = NO;

	if([self hasVideoId] == YES && [self pageCount] <= 1){
		if([self playerType] == PLAYER_TYPE_NICO_PLAYER && [self isNicoPlayerHTML] == YES){
			ret = YES;
		}
	}

	return ret;
}

//------------------------------------
// canChangeComment
//------------------------------------
- (BOOL)canChangeComment
{
	BOOL ret = NO;

	if([self hasVideoId] == YES && [self pageCount] <= 1){
		if([self playerType] == PLAYER_TYPE_EXT_PLAYER && [self isExtPlayerHTML] == YES){
			ret = YES;
		}
	}

	return ret;
}
//------------------------------------
// canReloadPlayer
//------------------------------------
- (BOOL)canReloadPlayer
{
	BOOL ret = NO;

	if([self hasVideoId] == YES){
		ret = YES;
	}

	return ret;
}
//------------------------------------
// isMaximizedPlayer
//------------------------------------
- (BOOL)isMaximizedPlayer
{
	BOOL ret = NO;

	NSString *state = [self runJavaScriptWithString:@"getMaximizePlayerState()"];
	if(state && [state isEqualToString:@"1"]){
		ret = YES;
	}
	return ret;
}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{

//	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[NSUserDefaults standardUserDefaults] removeObserver:self forKeyPath:@"optPlayerLiveResize"];

	[videoId_ release];
	[videoURL_ release];
	[jsTemplate_ release];
	[nicoPlayerHTML_ release];
	[nicoPlayerQHTML_ release];
	[extPlayerHTML_ release];
	[iPadPlayerHTML_ release];
	
	[swfobjectPath_ release];
	[title_ release];
	[connectionDataItem_ release];

	[super dealloc];
}
@end
